/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.helper;

import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.mof.Model.Association;
import org.omg.mof.Model.Classifier;
import org.omg.mof.Model.Import;
import org.omg.mof.Model.MofAttribute;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

/**
 * @author Xavier Blanc
 *
 */
public interface JMICommon {

	
	/**
	 * Return the JMI name of a package
	 */
	public String jmiPackageName(org.omg.mof.Model.Package _package)
		throws MofError;

	/**
	 * Return the JMI name of a package (be carefull, look at the TAG or maybe print an import)
	 */
	public String jmiPackageQualifiedName(org.omg.mof.Model.Package _package)
		throws MofError, NotSet;

	/**
	 *  Return the JMI name of a class
	 */
	public String jmiClassName(org.omg.mof.Model.Class _class)
		throws MofError, NotSet;

	/**
	 *  Return the JMI qualified name of a class
	 */
	public String jmiClassQualifiedName(org.omg.mof.Model.Class _class)
		throws MofError, NotSet;

	/**
	 * Return the JMI name of an association
	 */
	public String jmiAssociationName(Association _association)
		throws MofError, NotSet;

	/**
	 * Return the JMI name of an association
	 */
	public String jmiAssociationQualifiedName(Association _association)
		throws MofError, NotSet;

	/**
	 * Return the JMI name of an attribute
	 */
	public String jmiAttributeName(MofAttribute _attribute) throws MofError;

	/**
	 * Return the JMI name of a DataType
	 */
	public String jmiTypeCodeName(TypeCode _type) throws BadKind;

	/**
	 * Return the JMI name of an Import
	 */
	public String jmiImportName(Import _import) throws MofError, NotSet;

	/**
	 * Return the JMI name of a Classifier (DataType or org.omg.mof.Model.org.omg.mof.Model.Class) 
	 */
	public String jmiClassifierName(Classifier _classifier)
		throws MofError, BadKind, NotSet;

	/**
	 * Return the JMI name of a Classifier (DataType or org.omg.mof.Model.org.omg.mof.Model.Class) 
	 */
	public String jmiClassifierQualifiedName(Classifier _classifier)
		throws MofError, BadKind, NotSet;

	/**
	 * 
	 */
	//public String jmiType(String type);

	/**
	 * 
	 */
	public String jmiFormat1(String name) throws MofError;

	/**
	 * 
	 */
	public String jmiNullableTypeCodeName(TypeCode _type) throws BadKind;

	/**
	 * Return a nullable type (i.e. Boolean instead of boolean)
	 */
	public String jmiNullableClassifierName(Classifier classifier)
		throws MofError, BadKind, NotSet;

	/**
	 * Return a nullable type (i.e. Boolean instead of boolean)
	 */
	public String jmiNullableClassifierQualifiedName(Classifier classifier)
		throws MofError, BadKind, NotSet;

	/**
	 * Return the accessor name of the attribute
	 * @param attribute
	 * @return
	 * @throws MofError
	 */
	public String jmiAccessorName(MofAttribute attribute) throws MofError;
	
	/**
	 * Return the mutator name of the attribute
	 * @param attribute
	 * @return
	 * @throws MofError
	 */
	public String jmiMutatorName(MofAttribute attribute) throws MofError;
	
	/**
	 * Give the value of a Constant 
	 */
	public String valueOfConstant(org.omg.CORBA.Any _value)
		throws org.omg.CORBA.TypeCodePackage.BadKind;
}