/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.helper;

import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

/**
 * 
 */
public interface MOFCommon {

	/**
	 * Give the value of a Constant 
	 */
	public String valueOfConstant(org.omg.CORBA.Any _value)
		throws org.omg.CORBA.TypeCodePackage.BadKind;

	/**
	 * Give the attributes of a Class
	 */
	public org.omg.mof.Model.MofAttribute[] attributesOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope,
		boolean isDerived)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give all the attributes of a Class (included the inherited ones)
	 */
	public org.omg.mof.Model.MofAttribute[] allAttributesOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope,
		boolean _test_of_derived)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the operations of a Class
	 */
	public org.omg.mof.Model.Operation[] operationsOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give all the operation of a Class (included the inherited ones)
	 */
	public org.omg.mof.Model.Operation[] allOperationsOfClass(
		org.omg.mof.Model.Class _class,
		org.omg.mof.Model.ScopeKind _scope)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the references of a Class
	 */
	public org.omg.mof.Model.Reference[] referencesOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give all the references of a Class (included the inherited ones)
	 */
	public org.omg.mof.Model.Reference[] allReferencesOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the constants of a Class
	 */
	public org.omg.mof.Model.Constant[] constantsOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the datatypes of a Class
	 */
	public org.omg.mof.Model.DataType[] datatypesOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the exceptions of a Class
	 */
	public org.omg.mof.Model.MofException[] exceptionsOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the constrains of a Class
	 */
	public org.omg.mof.Model.Constraint[] constraintsOfClass(
		org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the sub-classes of a Class
	 */
	public org.omg.mof.Model.Class[] subClassesOfClass(
		org.omg.mof.Model.Classifier _class)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet;
	
	/**
	 * Give the super-classes of a Class
	 */
	public java.util.Vector superClassesOrdered(org.omg.mof.Model.Class _class)
		  throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the associations ends of an Association
	 */
	public org.omg.mof.Model.AssociationEnd[] associationEndsOfAssociation(
		org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError;
		
	/**
	 * Give the opposite association end of an association end
	 */
	public org.omg.mof.Model.AssociationEnd oppositeAssociationEnd(
		org.omg.mof.Model.AssociationEnd _association_end)
		throws org.omg.mof.Reflective.MofError;
				
	/**
	 * Give the classes of a Package
	 */
	public org.omg.mof.Model.Class[] classesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;
		
	/**
	 * Give the classes of a Package
	 */
	public org.omg.mof.Model.Class[] classesOfPackageOrderedByInheritance(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;
		
		
	/**
	 * Give all the classes of a Package (included clustered ones)
	 */
	public org.omg.mof.Model.Class[] allClassesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;
		
	/**
	 * Give all the classes of a Package (included clustered ones)
	 */
	public org.omg.mof.Model.Class[] allClassesOfPackageOrderedByInheritance(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the packages of a Package
	 */
	public org.omg.mof.Model.Package[] packagesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the associations of a Package
	 */
	public org.omg.mof.Model.Association[] associationsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;
		
	/**
	 * Give the associations of a Package
	 */
	public org.omg.mof.Model.Association[] allAssociationsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the imports of a Package
	 */
	public org.omg.mof.Model.Import[] importsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the tags of a Package
	 */
	public org.omg.mof.Model.Tag[] tagsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the datatypes of a Package
	 */
	public org.omg.mof.Model.DataType[] datatypesOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the exception of a Package
	 */
	public org.omg.mof.Model.MofException[] exceptionsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the constraints of a Package
	 */
	public org.omg.mof.Model.Constraint[] constraintsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Give the constrants of a Package
	 */
	public org.omg.mof.Model.Constant[] constantsOfPackage(
		org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError;

	/**
	 * Return true is Namespace is an outermostPackage
	 */
	public boolean isOutermostPackage(
		org.omg.mof.Model.Namespace _package);

	/**
	 * Give the references of an Association
	 */
	public org.omg.mof.Model.Reference[] referencesOfAssociation(
		org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet;

	/**
	 * Give the outermost package that includes the ModelElement
	 */
	public org.omg.mof.Model.Package outermostPackageOfModelElement(
		org.omg.mof.Model.ModelElement _element)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet;
	
	/**
	 * Return the idl_prefix assigned to the package if any (return "" else)
	 */
	public String idlPrefixForPackage(org.omg.mof.Model.Package _package) throws MofError;
	
	/**
	 * Return the idl_prefix assigned to the package if any (return "" else)
	 */
	public String idlSubstituteIdentifierForClass(org.omg.mof.Model.Classifier clazz) throws MofError , NotSet;
	
	/**
	 * Return true if sub class is in the same extent of super class
	 */
	public boolean isInSameExtent(org.omg.mof.Model.Class innerExtent , org.omg.mof.Model.Class outerExtent) throws NotSet , MofError;
	
	
	/**
	 * Return the jmi_prefix assigned to the package if any (return "" else)
	 */
	public String jmiPrefixForPackage(org.omg.mof.Model.Package _package) throws MofError;

	/**
	 * 
	 */
	public String jmiSubstituteIdentifierForClass(org.omg.mof.Model.Class clazz) throws MofError , NotSet;

}
