/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.helper;

import java.util.StringTokenizer;

/**
 * This class provides several functions to manipulate Strings.
 */
public class StringHelper {

    /**
     * Replace the first occurence of the given string with the given replacement.
     * @param original The original String to modify.
     * @param to_replace The substring of the string to search and replace.
     * @param new_value The replacement String.
     * @return The modified String.
     */
    public static String replaceFirst(String original, String to_replace, String new_value) {
        int index = original.indexOf(to_replace);
        if (index != -1) {
            return original.substring(0, index) + new_value + original.substring(index+to_replace.length());
        } else {
            return original;
        }
    }

    /**
     * Replace all the occurences of the given string with the given replacement.
     * @param original The original String to modify.
     * @param to_replace The substring of the string to search and replace.
     * @param new_value The replacement String.
     * @return The modified String.
     */
    public static String replaceAll(String original, String to_replace, String new_value) {
        StringBuffer newString = new StringBuffer();
        int indexBegin = 0;
        int index = original.indexOf(to_replace, indexBegin);
        while (index != -1) {
            newString.append(original.substring(indexBegin, index));
            newString.append(new_value);
            indexBegin = index + to_replace.length();
            index = original.indexOf(to_replace, indexBegin);
        }
        newString.append(original.substring(indexBegin));
        return newString.toString();
    }
    
    /**
     * Parse a String and test if it corresponds to the value. This value can be multiple and separate by '|', for example,
     * matchesValues(to_parse, new String("value1|value2|value3")).
     * @param to_parse The substring of the string to search and replace.
     * @param value The replacement String.
     * @return TRUE if the parsed string corresponds to one of the values.
     */
    public static boolean matchesValues(String to_parse, String value) {
        StringTokenizer token = new StringTokenizer(value, "|");
        while (token.hasMoreTokens()) {
            if (to_parse.equals(token.nextToken()))
                return true;
        }
        return false;
    }
    
    /**
      * Parse a String and test if it ends with the value. This value can be multiple and separate by '|', for example,
      * matchesValues(to_parse, new String("end1|end2|end3")).
      * @param to_parse The substring of the string to search and replace.
      * @param value The replacement String.
      * @return TRUE if the parsed string ends with on of the values.
      */
    public static boolean endsWithValues(String to_parse, String value) {
        StringTokenizer token = new StringTokenizer(value, "|");
        while (token.hasMoreTokens()) {
            if (to_parse.endsWith(token.nextToken()))
                return true;
        }
        return false;
    }

}
