/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl;

import org.objectweb.modfact.corba.generator.ZipGenerator;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.logging.ModFactLogger;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import org.omg.mof.Reflective.*;

import org.omg.CORBA.TypeCodePackage.*;

/**
 * 
 */
public class MOFIDL1_3Generator extends ZipGenerator {

	private org.omg.mof.Model.Package[] input;

	private MOFCommon mofHelper;
	private IDLCommon idlHelper;

	private ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Package[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Package) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * Generate
	 */
	public void generate()
		throws
			MofError,
			NotSet,
			BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			IOException {

		for (int i = 0; i < input.length; i++) {
			org.omg.mof.Model.Package[] apackage =
				new org.omg.mof.Model.Package[1];
			apackage[0] = input[i];

			//Traitement du package
			PackageGenerator packageGenerator = new PackageGenerator();
			packageGenerator.setInput(apackage);
			packageGenerator.setLogger(logger);
			ByteArrayOutputStream entryOutput1 = new ByteArrayOutputStream();
			packageGenerator.setOutput(entryOutput1);
			packageGenerator.setIdlHelper(idlHelper);
			packageGenerator.setMofHelper(mofHelper);
			packageGenerator.generate();

			String entryName =
				idlHelper.format1(input[i].name()) + ".idl";
			writeEntry(entryName, entryOutput1);
			entryOutput1.close();
			//closeFile();
		}
		out.finish();

	}

	/**
	 * Returns the idlHelper.
	 * @return IDLCommon
	 */
	public IDLCommon getIdlHelper() {
		return idlHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the idlHelper.
	 * @param idlHelper The idlHelper to set
	 */
	public void setIdlHelper(IDLCommon idlHelper) {
		this.idlHelper = idlHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}
