/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;

import org.omg.mof.Model.*;
import org.omg.mof.Reflective.*;
import java.util.Vector;

import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TypeCodePackage.*;

/**
 * 
 */
public class PackageGenerator extends BracketGenerator {

	private org.omg.mof.Model.Package[] input;

	private MOFCommon mofHelper;
	private IDLCommon idlHelper;

	private ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Package[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Package) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * Generate
	 */
	public void generate()
		throws MofError, NotSet, BadKind, org.omg.CORBA.TypeCodePackage.Bounds {

		org.omg.mof.Model.Package _package =
			(org.omg.mof.Model.Package) input[0];
		String packageName = idlHelper.format2(_package.name());

		outputln("#ifndef " + packageName + "_idl");
		outputln("#define " + packageName + "_idl");
		outputln("#include <Reflective.idl>");

		org.omg.mof.Model.Tag[] _tags = mofHelper.tagsOfPackage(_package);
		//Apples des diffrentes Templates correspendants
		for (int i = 0; i < _tags.length; i++)
			tagTemplate(_tags[i]);

		org.omg.mof.Model.Import[] _imports =
			mofHelper.importsOfPackage(_package);

		//si le package a des imports ( isCluetred=true or else ), rajouter les includes 
		for (int i = 0; i < _imports.length; i++) {
			Import _import = _imports[i];
			org.omg.mof.Model.Namespace importedPackage =
				_import.imported_namespace();
			outputln(
				"#include <"
					+ idlHelper.format1(importedPackage.name())
					+ ".idl>");
		}
		packageModuleTemplate(_package);

		outputln("#endif");
		flushFile();

	}

	public void packageModuleTemplate(org.omg.mof.Model.Package _package)
		throws MofError, BadKind, org.omg.CORBA.TypeCodePackage.Bounds, NotSet {

		logger.log(Level.FINE, "IDL packageModuleTemplate");
		if (_package.visibility().equals(VisibilityKind.public_vis)) {

			//MOFPackage packageTuple = helper.getContentsOfPackage(_package);

			org.omg.mof.Model.DataType[] _datatypes =
				mofHelper.datatypesOfPackage(_package);
			org.omg.mof.Model.Package[] _packages =
				mofHelper.packagesOfPackage(_package);
			org.omg.mof.Model.Class[] _classes =
				mofHelper.classesOfPackageOrderedByInheritance(_package);
			org.omg.mof.Model.Association[] _associations =
				mofHelper.associationsOfPackage(_package);
			org.omg.mof.Model.MofException[] _exceptions =
				mofHelper.exceptionsOfPackage(_package);
			org.omg.mof.Model.Constraint[] _constraints =
				mofHelper.constraintsOfPackage(_package);
			org.omg.mof.Model.Constant[] _constants =
				mofHelper.constantsOfPackage(_package);
			org.omg.mof.Model.Tag[] _tags = mofHelper.tagsOfPackage(_package);
			org.omg.mof.Model.Import[] _imports =
				mofHelper.importsOfPackage(_package);

			logger.log(Level.FINE, "Cache is Ok");

			//prefix!!
			//String idlPrefix = mofHelper.idlPrefixForPackage(_package);
			//if (idlPrefix.compareTo("")!=0) outputln("#pragma prefix\""+idlPrefix+"\"");

			//ecriture des annotations
			annotationTemplate(_package.annotation());
			String _packageName1 = idlHelper.format1(_package.name());
			//outputln("module " + _package.name() + " {");
			outputln("module " + _packageName1 + " {");

			//forward declaration
			outputln("interface " + _packageName1 + "Package ;");

			for (int i = 0; i < _classes.length; i++)
				classForwardDeclarationTemplate(_classes[i]);
			logger.log(Level.FINE, _classes.length + " ClassForward are Ok");

			//les package emboits 	
			for (int i = 0; i < _packages.length; i++)
				packageModuleTemplate(_packages[i]);
			logger.log(Level.FINE, _packages.length + " PackageModule are Ok");

			for (int i = 0; i < _datatypes.length; i++)
				dataTypeTemplate(_datatypes[i]);
			logger.log(Level.FINE, _datatypes.length + " DataTypes are Ok");

			for (int i = 0; i < _exceptions.length; i++)
				exceptionTemplate(_exceptions[i]);
			logger.log(Level.FINE, _exceptions.length + " Exceptions are Ok");

			for (int i = 0; i < _classes.length; i++)
				classTemplate(_classes[i]);
			logger.log(Level.FINE, _classes.length + " Classes are Ok");

			for (int i = 0; i < _associations.length; i++)
				associationTemplate(_associations[i]);
			logger.log(
				Level.FINE,
				_associations.length + " Associations are Ok");

			for (int i = 0; i < _constraints.length; i++)
				constraintTemplate(_constraints[i]);
			logger.log(Level.FINE, _constraints.length + " Constraints are Ok");

			for (int i = 0; i < _constants.length; i++)
				constantTemplate(_constants[i]);
			logger.log(Level.FINE, _constants.length + " Constants are Ok");

			//Generate the Package Factory interface
			packageFactoryTemplate(_package);
			logger.log(Level.FINE, "PackageFactories are Ok");

			// Generate the Model.Package interface
			packageTemplate(_package);

			outputln("}; //end of module " + _packageName1);
			outputln("");
		} else {
		} //do_nothing
	}

	//tag
	public void tagTemplate(Tag tag) throws MofError {
		if (tag.tag_id().compareTo("org.omg.mof.idl_prefix") == 0)
			outputln(
				"#pragma prefix \"" + tag.values()[0].extract_string() + "\"");
	}

	//annotation
	public void annotationTemplate(String _annotation) throws MofError {
		logger.log(Level.FINE, "IDL annotationTemplate");
		outputln("//" + _annotation);
	}

	public void classForwardDeclarationTemplate(org.omg.mof.Model.Class _class)
		throws MofError, NotSet {
		logger.log(Level.FINE, "IDL classForwardDeclarationTemplate");
		if (_class.visibility().equals(VisibilityKind.public_vis)) {
			String _classname = idlHelper.className(_class);
			outputln("interface " + _classname + "Class;");
			outputln("interface " + _classname + ";");
			outputln(
				"typedef sequence <"
					+ _classname
					+ "> "
					+ _classname
					+ "Set;");
			outputln(
				"typedef sequence <"
					+ _classname
					+ "> "
					+ _classname
					+ "Bag;");
			outputln(
				"typedef sequence <"
					+ _classname
					+ "> "
					+ _classname
					+ "List;");
			outputln(
				"typedef sequence <"
					+ _classname
					+ "> "
					+ _classname
					+ "UList;");
			outputln();
		} else {
		}; //rien  faire
	}

	public void dataTypeTemplate(DataType _datatype)
		throws MofError, BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		logger.log(Level.FINE, "IDL dataTypeTemplate");
		if (_datatype.visibility().equals(VisibilityKind.public_vis)) {
			annotationTemplate(_datatype.annotation());
			String _datatypename = _datatype.name();
			TypeCode tc = _datatype.type_code();

			if (idlHelper.isBaseType(tc)) {
				//_datatypename = _datatypename.substring(1);
				String tcName = getBaseTypeName(tc);
				String _idlbasename = idlBaseType(tcName);
				outputln(
					"typedef sequence <"
						+ tcName
						+ "> "
						+ _idlbasename
						+ "Set;");
				outputln(
					"typedef sequence <"
						+ tcName
						+ "> "
						+ _idlbasename
						+ "Bag;");
				outputln(
					"typedef sequence <"
						+ tcName
						+ "> "
						+ _idlbasename
						+ "List;");
				outputln(
					"typedef sequence <"
						+ tcName
						+ "> "
						+ _idlbasename
						+ "UList;");

			} else {
				String _datatypename1 = idlHelper.format1(_datatype.name());
				org.omg.CORBA.TypeCode _typecode = _datatype.type_code();
				typeCode2IDL(_typecode, 0, _datatypename);
				for (int i = 0; i < _datatype.constraints().length; i++) {
					constraintTemplate(_datatype.constraints()[i]);
				}
				outputln(
					"typedef sequence <"
						+ _datatypename1
						+ "> "
						+ _datatypename1
						+ "Set;");
				outputln(
					"typedef sequence <"
						+ _datatypename1
						+ "> "
						+ _datatypename1
						+ "Bag;");
				outputln(
					"typedef sequence <"
						+ _datatypename1
						+ "> "
						+ _datatypename1
						+ "List;");
				outputln(
					"typedef sequence <"
						+ _datatypename1
						+ "> "
						+ _datatypename1
						+ "UList;");

			}
		}
	}

	public void typeCode2IDL(
		org.omg.CORBA.TypeCode _typecode,
		int _level,
		String _datatypename)
		throws BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		logger.log(Level.FINE, "IDL typeCode2IDL");
		switch (_typecode.kind().value()) {
			case org.omg.CORBA.TCKind._tk_alias :
				{
					org.omg.CORBA.TypeCode _tc = _typecode.content_type();
					output("typedef ");
					//typeCode2IDL(_tc, 0, _tc.name());
					typeCode2IDL(_tc, 0, "");
					//outputln(" " + _tc.name() + " ;");
					outputln(" " + idlHelper.format1(_datatypename) + " ;");
					break;
				}
			case org.omg.CORBA.TCKind._tk_struct :
				{
					//output("struct " + helper.format1(_typecode.name()) + "{");
					output(
						"struct " + idlHelper.format1(_datatypename) + "{");
					int _temp = _level + 1;
					for (int i = 0; i < _typecode.member_count(); i++) {
						typeCode2IDL(
							_typecode.member_type(i),
							_temp,
							_typecode.member_name(i));
						//output(" " + _typecode.member_name(i) + "; ");
						output(" " + _typecode.member_name(i) + "; ");
					}
					output(" } ");
					if (_level == 0)
						outputln(";");
					break;
				}
			case org.omg.CORBA.TCKind._tk_enum :
				{
					//output("enum " + helper.format1(_typecode.name()) + "{");
					output(
						"enum " + idlHelper.format1(_datatypename) + "{");
					for (int i = 0; i < _typecode.member_count(); i++) {
						output(_typecode.member_name(i));
						if (!(i == (_typecode.member_count() - 1)))
							output(", ");
					}
					output(" } ");
					if (_level == 0)
						outputln(";");
					break;
				}
			case org.omg.CORBA.TCKind._tk_sequence :
				{
					org.omg.CORBA.TypeCode _tc = _typecode.content_type();
					output("typedef sequence <");
					int _temp = _level + 1;
					typeCode2IDL(_tc, _temp, "");
					if (_typecode.length() > 1)
						output(" ," + String.valueOf(_typecode.length()) + ">");
					else
						output(">");
					if (_level == 0)
						outputln(" " + idlHelper.format1(_datatypename) + ";");
					break;
				}
			default :
				typeCode2IDLBase(_typecode);
		}
	}

	public void typeCode2IDLBase(org.omg.CORBA.TypeCode _typecode)
		throws BadKind {
		logger.log(Level.FINE, "IDL typeCode2IDLBase");
		switch (_typecode.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				{
					output("boolean");
					break;
				}
			case org.omg.CORBA.TCKind._tk_octet :
				{
					output("octet");
					break;
				}
			case org.omg.CORBA.TCKind._tk_char :
				{
					output("char");
					break;
				}
			case org.omg.CORBA.TCKind._tk_wchar :
				{
					output("wchar");
					break;
				}
			case org.omg.CORBA.TCKind._tk_short :
				{
					output("short");
					break;
				}
			case org.omg.CORBA.TCKind._tk_ushort :
				{
					output("unsigned short");
					break;
				}
			case org.omg.CORBA.TCKind._tk_long :
				{
					output("long");
					break;
				}
			case org.omg.CORBA.TCKind._tk_ulong :
				{
					output("unsigned long");
					break;
				}
			case org.omg.CORBA.TCKind._tk_longlong :
				{
					output("long long");
					break;
				}
			case org.omg.CORBA.TCKind._tk_ulonglong :
				{
					output("unsigned long long");
					break;
				}
			case org.omg.CORBA.TCKind._tk_double :
				{
					output("double");
					break;
				}
			case org.omg.CORBA.TCKind._tk_float :
				{
					output("float");
					break;
				}
			case org.omg.CORBA.TCKind._tk_string :
				{
					output("string");
					break;
				}
			case org.omg.CORBA.TCKind._tk_wstring :
				{
					output("wstring");
					break;
				}
			case org.omg.CORBA.TCKind._tk_TypeCode :
				{
					output("CORBA::TypeCode");
					break;
				}
			case org.omg.CORBA.TCKind._tk_any :
				{
					output("any");
					break;
				}
			case org.omg.CORBA.TCKind._tk_objref :
				{
					output("Object");
					break;
				}
			default :
				throw new BadKind();
		}
	}

	public String getBaseTypeName(org.omg.CORBA.TypeCode _typecode)
		throws BadKind {
		logger.log(Level.FINE, "IDL typeCode2IDLBase");
		switch (_typecode.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				{
					return "boolean";
				}
			case org.omg.CORBA.TCKind._tk_octet :
				{
					return "octet";
				}
			case org.omg.CORBA.TCKind._tk_char :
				{
					return "char";
				}
			case org.omg.CORBA.TCKind._tk_wchar :
				{
					return "wchar";
				}
			case org.omg.CORBA.TCKind._tk_short :
				{
					return "short";
				}
			case org.omg.CORBA.TCKind._tk_ushort :
				{
					return "unsigned short";
				}
			case org.omg.CORBA.TCKind._tk_long :
				{
					return "long";
				}
			case org.omg.CORBA.TCKind._tk_ulong :
				{
					return "unsigned long";
				}
			case org.omg.CORBA.TCKind._tk_longlong :
				{
					return "long long";
				}
			case org.omg.CORBA.TCKind._tk_ulonglong :
				{
					return "unsigned long long";
				}
			case org.omg.CORBA.TCKind._tk_double :
				{
					return "double";
				}
			case org.omg.CORBA.TCKind._tk_float :
				{
					return "float";
				}
			case org.omg.CORBA.TCKind._tk_string :
				{
					return "string";
				}
			case org.omg.CORBA.TCKind._tk_wstring :
				{
					return "wstring";
				}
			case org.omg.CORBA.TCKind._tk_TypeCode :
				{
					return "CORBA::TypeCode";
				}
			case org.omg.CORBA.TCKind._tk_any :
				{
					return "any";
				}
			case org.omg.CORBA.TCKind._tk_objref :
				{
					return "Object";
				}
			default :
				throw new BadKind();
		}
	}

	//les types IDL de base

	public String idlBaseType(String _in) {
		logger.log(Level.FINE, "IDL idlBaseType");
		if (_in.compareToIgnoreCase("boolean") == 0)
			return "Boolean";
		else if (_in.compareToIgnoreCase("octet") == 0)
			return "Octet";
		else if (_in.compareToIgnoreCase("char") == 0)
			return "Char";
		else if (_in.compareToIgnoreCase("wchar") == 0)
			return "WChar";
		else if (_in.compareToIgnoreCase("short") == 0)
			return "Short";
		else if (_in.compareToIgnoreCase("unsigned short") == 0)
			return "UShort";
		else if (_in.compareToIgnoreCase("long") == 0)
			return "Long";
		else if (_in.compareToIgnoreCase("unsigned long") == 0)
			return "ULong";
		else if (_in.compareToIgnoreCase("long long") == 0)
			return "LLong";
		else if (_in.compareToIgnoreCase("unsigned long long") == 0)
			return "ULLong";
		else if (_in.compareToIgnoreCase("double") == 0)
			return "Double";
		else if (_in.compareToIgnoreCase("float") == 0)
			return "Float";
		else if (_in.compareToIgnoreCase("string") == 0)
			return "String";
		else if (_in.compareToIgnoreCase("wstring") == 0)
			return "WString";
		else if (_in.compareToIgnoreCase("any") == 0)
			return "Any";
		else if (_in.compareToIgnoreCase("CORBA::TypeCode") == 0)
			return "TypeCode";
		else if (_in.compareToIgnoreCase("Object") == 0)
			return "Object";
		else
			return _in;
	}

	public String idlType(Classifier _classifier)
		throws MofError, BadKind, Bounds, NotSet {
		logger.log(Level.FINE, "IDL idlType");
		String resu = null;
		if (_classifier._is_a(DataTypeHelper.id())) {
			//Base Type
			DataType dt = DataTypeHelper.narrow(_classifier);
			TypeCode tc = dt.type_code();
			if (idlHelper.isBaseType(tc))
				resu = getBaseTypeName(tc);
			else {
				//If container is a Class then the qualified is not applicable
				Namespace container = dt.container();
				if (container._is_a(ClassHelper.id())) { //What Else ???
					resu =
						idlHelper.qualifiedName(container)
							+ "Class::"
							+ idlHelper.format1(dt.name());
				} else
					resu = idlHelper.qualifiedName(_classifier);
			}
		} else {
			resu = idlHelper.qualifiedName(_classifier);
		}
		return resu;
	}

	public void exceptionTemplate(MofException _exception)
		throws MofError, BadKind, Bounds, NotSet {
		logger.log(Level.FINE, "IDL exceptionTemplate");
		if (_exception.visibility().equals(VisibilityKind.public_vis)) {
			String _exceptionname = idlHelper.format1(_exception.name());
			annotationTemplate(_exception.annotation());
			outputln("exception " + _exceptionname + " {");
			for (int i = 0; i < _exception.contents().length; i++) {
				Parameter _parameter =
					ParameterHelper.narrow(_exception.contents()[i]);
				output(idlBaseType(idlType(_parameter.type())));
				if ((_parameter.multiplicity().upper > 1)
					|| (_parameter.multiplicity().upper == -1)) {
					if (_parameter.multiplicity().isOrdered) {
						if (_parameter.multiplicity().isUnique) {
							output("UList");
						} else {
							output("List");
						}
					} else {
						if (_parameter.multiplicity().isUnique) {
							output("Set");
						} else {
							output("Bag");
						}
					}
				}
				output(" " + idlHelper.format1(_parameter.name()) + " ;");

			}
			outputln("} ;");
		}
	}

	//les classes
	public void classTemplate(org.omg.mof.Model.Class _class)
		throws MofError, BadKind, org.omg.CORBA.TypeCodePackage.Bounds, NotSet {

		logger.log(Level.FINE, "IDL classTemplate");
		if (_class.visibility().equals(VisibilityKind.public_vis)) {
			annotationTemplate(_class.annotation());
			classProxyTemplate(_class);
			instanceTemplate(_class);
		}
	}
	public void associationTemplate(Association _association)
		throws MofError, BadKind, Bounds, NotSet {

		logger.log(Level.FINE, "IDL associationTemplate");
		if (_association.visibility().equals(VisibilityKind.public_vis)) {
			String _associationformat1 = idlHelper.format1(_association.name());
			String _associationformat2 = idlHelper.format2(_association.name());

			AssociationEnd[] _ends =
				mofHelper.associationEndsOfAssociation(_association);

			//proprit de la premire associationEnd
			AssociationEnd _source = _ends[0];
			String _sourceformat1 = idlType(_source.type());
			String _sourceformat2 = idlHelper.format2(_source.name());
			String _sourceformat2IdlConflict = idlHelper.format2IdlConflict(_source.name());
			MultiplicityType _multiplicity_source = _source.multiplicity();
			int _upper_source = _multiplicity_source.upper;
			int _lower_source = _multiplicity_source.lower;
			boolean _is_ordered_source = _multiplicity_source.isOrdered;
			boolean _is_unique_source = _multiplicity_source.isUnique;
			boolean _is_changeable_source = _source.is_changeable();
			boolean _is_navigable_source = _source.is_navigable();

			//proprit de la deuxime associationEnd
			AssociationEnd _dest = _ends[1];
			String _destformat1 = idlType(_dest.type());
			String _destformat2 = idlHelper.format2(_dest.name());
			String _destformat2IdlConflict = idlHelper.format2IdlConflict(_dest.name());
			MultiplicityType _multiplicity_dest = _dest.multiplicity();
			int _upper_dest = _multiplicity_dest.upper;
			int _lower_dest = _multiplicity_dest.lower;
			boolean _is_ordered_dest = _multiplicity_dest.isOrdered;
			boolean _is_unique_dest = _multiplicity_dest.isUnique;
			boolean _is_changeable_dest = _dest.is_changeable();
			boolean _is_navigable_dest = _dest.is_navigable();

			//le code correspondant
			outputln("struct " + _associationformat1 + "Link {");
			outputln(_sourceformat1 + "  " + _sourceformat2IdlConflict + "; ");
			outputln(_destformat1 + "  " + _destformat2IdlConflict + "; ");
			outputln("};");

			outputln(
				"typedef sequence  "
					+ "<"
					+ _associationformat1
					+ "Link> "
					+ _associationformat1
					+ "LinkSet;");

			annotationTemplate(_association.annotation());
			outputln(
				"interface "
					+ _associationformat1
					+ ":Reflective::RefAssociation {");
			outputln(
					_associationformat1
					+ "LinkSet all_"
					+ _associationformat2
					+ "_Links () raises (Reflective::MofError) ;");
			output("boolean exists (");
			output("in " + _sourceformat1 + "  " + _sourceformat2IdlConflict + ", ");
			output("in " + _destformat1 + "  " + _destformat2IdlConflict + ")");
			outputln("raises (Reflective::MofError);");

			if (_is_navigable_source) {
				if ((_upper_source > 1) || (_upper_source == -1)) {
					//une  ligne rajoute
					output(idlBaseType(_sourceformat1));
					if ((_is_unique_source) && (_is_ordered_source)) {
						output("UList  ");
					} else {
						if (_is_ordered_source) {
							output("List  ");
						} else {
							if (_is_unique_source) {
								output("Set  ");
							} else
								output("Bag  ");
						}
					}
				} else {
					output(_sourceformat1 + "  ");
				}
				outputln(
					_sourceformat2IdlConflict
						+ "(in "
						+ _destformat1
						+ "  "
						+ _destformat2IdlConflict
						+ ") raises (Reflective::MofError);");
			}
			if (_is_navigable_dest) {
				if ((_upper_dest > 1) || (_upper_dest == -1)) {
					//une  ligne rajoute
					output(idlBaseType(_destformat1));
					if ((_is_unique_dest) && (_is_ordered_dest)) {
						output("UList  ");
					} else {
						if (_is_ordered_dest) {
							output("List  ");
						} else {
							if (_is_unique_dest) {
								output("Set  ");
							} else
								output("Bag  ");
						}
					}
				} else {
					output(_destformat1 + "  ");
				}
				outputln(
					_destformat2IdlConflict
						+ "(in "
						+ _sourceformat1
						+ "  "
						+ _sourceformat2IdlConflict
						+ ") raises (Reflective::MofError);");
			}

			if (_is_changeable_source && _is_changeable_dest) {
				outputln(
					"void add ( in "
						+ _sourceformat1
						+ "  "
						+ _sourceformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ "  "
						+ _destformat2IdlConflict
						+ ") raises (Reflective::MofError);");
			}
			if (_is_changeable_source
				&& _is_navigable_source
				&& _is_changeable_dest
				&& ((_upper_source > 1) || (_upper_source == -1))
				&& _is_ordered_source) {
				outputln(
					"void add_before_"
						+ _sourceformat2
						+ "( in "
						+ _sourceformat1
						+ "  "
						+ _sourceformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ "  "
						+ _destformat2IdlConflict
						+ ", in "
						+ _sourceformat1
						+ " before) raises (Reflective::NotFound, Reflective::MofError);");
			}
			if (_is_changeable_dest
				&& _is_navigable_dest
				&& _is_changeable_source
				&& ((_upper_dest > 1) || (_upper_dest == -1))
				&& _is_ordered_dest) {
				outputln(
					"void add_before_"
						+ _destformat2
						+ "( in "
						+ _sourceformat1
						+ "  "
						+ _sourceformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ "  "
						+ _destformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ " before) raises (Reflective::NotFound, Reflective::MofError);");
			}
			if (_is_navigable_source && _is_changeable_source) {
				outputln(
					"void modify_"
						+ _sourceformat2
						+ " ( in "
						+ _sourceformat1
						+ "  "
						+ _sourceformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ "  "
						+ _destformat2IdlConflict
						+ ", in "
						+ _sourceformat1
						+ " new_"
						+ _sourceformat2
						+ ") raises (Reflective::NotFound, Reflective::MofError);");
			}
			if (_is_navigable_dest && _is_changeable_dest) {
				outputln(
					"void modify_"
						+ _destformat2
						+ " ( in "
						+ _sourceformat1
						+ "  "
						+ _sourceformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ "  "
						+ _destformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ " new_"
						+ _destformat2
						+ ") raises (Reflective::NotFound, Reflective::MofError);");
			}
			if (_is_changeable_source && _is_changeable_dest) {
				outputln(
					"void remove ( in "
						+ _sourceformat1
						+ "  "
						+ _sourceformat2IdlConflict
						+ ", in "
						+ _destformat1
						+ "  "
						+ _destformat2IdlConflict
						+ ") raises (Reflective::NotFound, Reflective::MofError);");
			}
			outputln("}; // end of interface " + _associationformat1);
		}
		outputln();
	}

	public void constraintTemplate(Constraint _constraint) throws MofError {
		logger.log(Level.FINE, "IDL constraintTemplate");
		annotationTemplate(_constraint.annotation());
		outputln(
			"const string " + idlHelper.format3(_constraint.name()));
	}

	//la Constante
	public void constantTemplate(Constant _constant)
		throws BadKind, MofError, Bounds, NotSet {
		annotationTemplate(_constant.annotation());
		org.omg.CORBA.Any _value = _constant.value();
		int _tk = _value.type().kind().value();
		output(
			"const "
				+ idlType(_constant.type())
				+ " "
				+ idlHelper.format3(_constant.name())
				+ " = ");
		if ((_tk == org.omg.CORBA.TCKind._tk_char)
			|| (_tk == org.omg.CORBA.TCKind._tk_wchar)
			|| (_tk == org.omg.CORBA.TCKind._tk_string)
			|| (_tk == org.omg.CORBA.TCKind._tk_wstring)) {
			output("\"" + mofHelper.valueOfConstant(_value) + "\"");
		} else
			output(mofHelper.valueOfConstant(_value));
		outputln(" ;");
	}

	public void packageFactoryTemplate(org.omg.mof.Model.Package _package)
		throws MofError, BadKind, Bounds, NotSet {
		logger.log(Level.FINE, "IDL packageFactoryTemplate");

		String _packageformat1 = idlHelper.format1(_package.name());
		String _packageformat2 = idlHelper.format2(_package.name());

		//ordonner toutes les classes 
		org.omg.mof.Model.Class[] orderedClasses =
			mofHelper.classesOfPackage(_package);

		//Hashtable orderedClasses = mofHelper.orderedClasses(disorderedClasses);

		Vector attributes = new Vector();

		for (int i = 0; i < orderedClasses.length; i++) {

			//String key = (String) e.nextElement();

			//MOFClass contensClass =
			//	mofHelper.getContentsOfClass(
			//		(org.omg.mof.Model.Class) orderedClasses.get(key));

			org.omg.mof.Model.MofAttribute[] attributes_temp =
				mofHelper.allAttributesOfClass(
					orderedClasses[i],
					ScopeKind.classifier_level,
					false);

			for (int j = 0; j < attributes_temp.length; j++) {
				attributes.addElement(attributes_temp[j]);
			}
		}

		outputln("interface " + _packageformat1 + "PackageFactory {");
		output(
				_packageformat1
				+ "Package  create_"
				+ _packageformat2
				+ "_package(");

		//calculer les attributs statiques des classes dans le mtamodles
		for (int i = 0; i < attributes.size(); i++) {
			MofAttribute _current = (MofAttribute) attributes.elementAt(i);
			String _currentformat1 = idlType(_current.type());
			String _attributeqname = " ";
			String[] _qualified_name = _current.qualified_name();
			for (int j = 0; j < _qualified_name.length; j++) {
				if (j > 0)
					_attributeqname =
						idlHelper.format2(_qualified_name[j])
							+ "_"
							+ _attributeqname;
				else
					_attributeqname = idlHelper.format2(_qualified_name[j]);
			}

			if (!((i != 0) || (i != (attributes.size() - 1))))
				outputln(", ");

			if ((_current.multiplicity().upper > 1)
				|| (_current.multiplicity().upper == -1)) {

				output(idlBaseType(_currentformat1));
				if ((_current.multiplicity().isUnique)
					&& (_current.multiplicity().isOrdered)) {
					output("UList  " + _attributeqname);
				} else {
					if (_current.multiplicity().isOrdered) {
						output("List  " + _attributeqname);
					} else {
						if (_current.multiplicity().isUnique) {
							output("Set  " + _attributeqname);
						} else
							output("Bag  " + _attributeqname);
					}
				}
			} else {
				output(_currentformat1 + "  " + _attributeqname);
			}
		}
		outputln(") raises (Reflective::MofError);");
		outputln("}; ");
	}

	//package Template
	public void packageTemplate(org.omg.mof.Model.Package _package)
		throws MofError, NotSet {

		logger.log(Level.FINE, "IDL packageTemplate");
		String _pformat1 = idlHelper.format1(_package.name());
		String _pformat2 = idlHelper.format2(_package.name());

		output("interface  " + _pformat1 + "Package :");

		if (_package.all_supertypes().length == 0)
			outputln("Reflective::RefPackage {");

		//rcuprer le contenu du Package
		//MOFPackage contantsPackage =
		//	(MOFPackage) mofHelper.getContentsOfPackage(_package);

		org.omg.mof.Model.Import[] imports =
			mofHelper.importsOfPackage(_package);
		org.omg.mof.Model.Package[] packages =
			mofHelper.packagesOfPackage(_package);
		org.omg.mof.Model.Class[] classes =
			mofHelper.classesOfPackage(_package);
		org.omg.mof.Model.Association[] associations =
			mofHelper.associationsOfPackage(_package);

		for (int i = 0; i < imports.length; i++) {
			Import _import = imports[i];
			Namespace imported = _import.imported_namespace();
			if (imported._is_a(PackageHelper.id())) {
				org.omg.mof.Model.Package cluteredPackage =
					PackageHelper.narrow(imported);
				if ((_import.visibility().equals(VisibilityKind.public_vis))
					&& (_import.is_clustered())
					&& (cluteredPackage
						.visibility()
						.equals(VisibilityKind.public_vis)))
					outputln(
						"readonly attribute ::"
							+ idlHelper.qualifiedName(cluteredPackage)
							+ "::"
				            + idlHelper.format1(cluteredPackage.name())
							+ "Package  "
							+ idlHelper.format2(cluteredPackage.name())
							+ "_ref;");
			}
		}
		for (int i = 0; i < packages.length; i++) {
			org.omg.mof.Model.Package nestedPackage = packages[i];
			if (nestedPackage.visibility().equals(VisibilityKind.public_vis)) {
				String nestedName = idlHelper.format1(nestedPackage.name());
				outputln(
					"readonly attribute "
						+ idlHelper.qualifiedName(nestedPackage)
						+ "::"
						+ nestedName
						+ "Package  "
						+ idlHelper.format2(nestedPackage.name())
						+ "_ref;");

			}
		}
		for (int i = 0; i < classes.length; i++) {
			org.omg.mof.Model.Class _class = classes[i];
			if (_class.visibility().equals(VisibilityKind.public_vis))
				outputln(
					"readonly attribute "
						+ idlHelper.className(_class)
						+ "Class  "
						+ idlHelper.format2(idlHelper.className(_class))
						+ "_ref;");
		}
		for (int i = 0; i < associations.length; i++) {
			Association association = associations[i];
			if (association.visibility().equals(VisibilityKind.public_vis))
				outputln(
					"readonly attribute "
						+ idlHelper.format1(association.name())
						+ "  "
						+ idlHelper.format2(association.name())
						+ "_ref;");
		}

		outputln("};");
	}

	public void classProxyTemplate(org.omg.mof.Model.Class _class)
		throws MofError, BadKind, org.omg.CORBA.TypeCodePackage.Bounds, NotSet {

		logger.log(Level.FINE, "IDL classProxyTemplate");
		String _cformat1 = idlHelper.className(_class);
		String _cformat2 = idlHelper.format2(idlHelper.className(_class));

		outputln();
		output("interface  " + _cformat1 + "Class :");

		GeneralizableElement[] supertypes = _class.supertypes();

		if (supertypes.length == 0)
			output("Reflective::RefObject");
		else {
			for (int i = 0; i < supertypes.length; i++) {
				org.omg.mof.Model.Class _temp =
					ClassHelper.narrow(supertypes[i]);
				output(idlType(_temp) + "Class ");
				if (i != (supertypes.length - 1))
					output(", ");
			}
		}
		outputln("{ ");
		outputln(
			"readonly attribute "
				+ _cformat1
				+ "Set all_of_type_"
				+ _cformat2
				+ " ;");
		if (!_class.is_abstract()) {
			outputln(
				"readonly attribute "
					+ _cformat1
					+ "Set all_of_class_"
					+ _cformat2
					+ " ;");
		}

		//MOFClass contentsOfClass = (MOFClass) mofHelper.getContentsOfClass(_class);

		Constant[] _constants = mofHelper.constantsOfClass(_class);
		DataType[] _datatypes = mofHelper.datatypesOfClass(_class);
		MofException[] _exceptions = mofHelper.exceptionsOfClass(_class);
		Constraint[] _constraints = mofHelper.constraintsOfClass(_class);
		MofAttribute[] _attributes_cl =
			mofHelper.allAttributesOfClass(
				_class,
				ScopeKind.classifier_level,
				true);
		Operation[] _operations_cl =
			mofHelper.allOperationsOfClass(_class, ScopeKind.classifier_level);
		//Vector _constants = contentsOfClass.mofConstants;
		//Vector _datatypes = contentsOfClass.mofDatatypes;
		//Vector _exceptions = contentsOfClass.mofExcepion;
		//Vector _constraints = contentsOfClass.mofConstraints;
		//Vector _attributes_cl = contentsOfClass.attributesClass;
		//Vector _operations_cl = contentsOfClass.operationsClass;

		for (int i = 0; i < _constants.length; i++)
			constantTemplate(_constants[i]);

		for (int i = 0; i < _datatypes.length; i++)
			dataTypeTemplate(_datatypes[i]);

		for (int i = 0; i < _exceptions.length; i++)
			exceptionTemplate(_exceptions[i]);

		for (int i = 0; i < _constraints.length; i++)
			constraintTemplate(_constraints[i]);

		for (int i = 0; i < _attributes_cl.length; i++)
			attributeTemplate(_attributes_cl[i]);

		for (int i = 0; i < _operations_cl.length; i++)
			operationTemplate(_operations_cl[i]);

		// si la classe est abstraite, on appel la calsse create
		if (!_class.is_abstract())
			classCreateTemplate(_class);
		outputln("} ; // end of interface " + _cformat1 + "Class");
		outputln();
	}

	//instance Template	
	public void instanceTemplate(org.omg.mof.Model.Class _class)
		throws MofError, BadKind, Bounds, NotSet {

		logger.log(Level.FINE, "IDL instanceTemplate");
		//debut("InstanceTemplate", _class.name());
		String _cformat1 = idlHelper.className(_class);
		//String _cformat2 = idlHelper.format2(_cformat1);

		output("interface " + _cformat1 + ": " + _cformat1 + "Class");

		GeneralizableElement[] supertypes = _class.supertypes();

		for (int i = 0; i < supertypes.length; i++) {
			org.omg.mof.Model.Class _temp = ClassHelper.narrow(supertypes[i]);
			//if ((i != supertypes.length - 1) || (i == 0))
			output(", ");
			output(idlType(_temp));
		}

		outputln("{");

		//MOFClass contentsOfClass = (MOFClass) helper.getContentsOfClass(_class);

		MofAttribute[] _attributes_inst =
			mofHelper.attributesOfClass(_class, ScopeKind.instance_level, true);
		Operation[] _operations_inst =
			mofHelper.operationsOfClass(_class, ScopeKind.instance_level);
		Reference[] _references = mofHelper.referencesOfClass(_class);
		//Vector _attributes_inst = contentsOfClass.attributesInst;
		//Vector _operations_inst = contentsOfClass.operationsInst;
		//Vector _references = contentsOfClass.refrences;

		for (int i = 0; i < _attributes_inst.length; i++)
			attributeTemplate(_attributes_inst[i]);

		for (int i = 0; i < _operations_inst.length; i++)
			operationTemplate(_operations_inst[i]);

		for (int i = 0; i < _references.length; i++)
			referenceTemplate(_references[i]);

		outputln("} ; // end of interface " + _cformat1);
	}

	public void attributeTemplate(MofAttribute _attribute)
		throws MofError, BadKind, Bounds, NotSet {

		logger.log(Level.FINE, "IDL attributeTemplate");
		if (_attribute.visibility().equals(VisibilityKind.public_vis)) {
			annotationTemplate(_attribute.annotation());
			String _aformat1 = idlType(_attribute.type());
			String _aformat2 = idlHelper.format2(_attribute.name());
			String _aformat2IdlConflict = idlHelper.format2IdlConflict(_attribute.name());
			String _qualifiedName = idlHelper.qualifiedName(_attribute.type().container());						
			MultiplicityType _multiplicity = _attribute.multiplicity();
			int _upper = _multiplicity.upper;
			int _lower = _multiplicity.lower;
			boolean _is_ordered = _multiplicity.isOrdered;
			boolean _is_unique = _multiplicity.isUnique;
			boolean _is_changeable = _attribute.is_changeable();
			boolean _is_derived = _attribute.is_derived();
			
			String _aformat1_qn = idlHelper.format1(_aformat1);
			if (!_attribute.type().container()._is_equivalent(input[0])
				&& ((_upper > 1) || (_upper == -1))
				&& (_aformat1.indexOf("::") == -1))
				_aformat1_qn = _qualifiedName + "::" + idlHelper.format1(_aformat1);

			if ((_lower == 0) && (_upper == 1))
				outputln(
						_aformat1
						+ " "
						+ _aformat2IdlConflict
						+ "() raises (Reflective::NotSet, Reflective::MofError); ");
			if ((_lower == 1) && (_upper == 1))
				outputln(
						_aformat1
						+ " "
						+ _aformat2IdlConflict
						+ "() raises (Reflective::MofError); ");
			if ((_upper > 1) || (_upper == -1)) {
				// une ligne rajoute
				//output(idlBaseType(_aformat1));
				output(_aformat1_qn);
				if (_is_unique && _is_ordered) {
					output("UList  " + _aformat2IdlConflict);
				} else {
					if (_is_ordered) {
						output("List  " + _aformat2IdlConflict);
					} else {
						if (_is_unique) {
							output("Set  " + _aformat2IdlConflict);
						} else
							output("Bag  " + _aformat2IdlConflict);
					}
				}
				outputln("() raises (Reflective::MofError); ");
			}
			if ((_upper == 1) && (_is_changeable))
				outputln(
					"void set_"
						+ _aformat2
						+ " (in "
						+ _aformat1
						+ " new_value) raises (Reflective::MofError); ");
			if (((_upper > 1) || (_upper == -1)) && (_is_changeable)) {
				output("void set_" + _aformat2 + " (in ");
				// une ligne rajoute
				//output(idlBaseType(_aformat1));
				output(_aformat1_qn);
				if (_is_unique && _is_ordered) {
					output("UList  ");
				} else {
					if (_is_ordered) {
						output("List  ");
					} else {
						if (_is_unique) {
							output("Set  ");
						} else
							output("Bag  ");
					}
				}
				outputln(" new_value) raises (Reflective::MofError); ");
			}
			if ((_lower == 0) && (_upper == 1) && (_is_changeable))
				outputln(
					"void unset_"
						+ _aformat2
						+ " () raises (Reflective::MofError); ");
			if ((_upper > 1) || (_upper == -1) && (_is_changeable))
				outputln(
					"void add_"
						+ _aformat2
						+ " (in "
						+ _aformat1
						+ " new_element) raises (Reflective::MofError); ");
			if ((_upper > 1)
				|| (_upper == -1)
				&& (_is_changeable)
				&& (_is_ordered))
				outputln(
					"void add_"
						+ _aformat2
						+ "_before ( in "
						+ _aformat1
						+ " new_element, in "
						+ _aformat1
						+ " before_element) raises (Reflective::NotFound, Reflective::MofError); ");
			if ((_upper > 1)
				|| (_upper == -1)
				&& (_is_changeable)
				&& (_is_ordered)
				&& (!_multiplicity.isUnique))
				outputln(
					"void add_"
						+ _aformat2
						+ "_at (in "
						+ _aformat1
						+ " new_element, in unsigned long position) raises (Reflective::BadPosition, Reflective::MofError); ");
			if ((_upper > 1) || (_upper == -1) && (_is_changeable))
				outputln(
					"void modify_"
						+ _aformat2
						+ " (in "
						+ _aformat1
						+ " old_element, in "
						+ _aformat1
						+ " new_element) raises (Reflective::NotFound, Reflective::MofError); ");
			if ((_upper > 1)
				|| (_upper == -1)
				&& (_is_changeable)
				&& (_is_ordered)
				&& (!_is_unique))
				outputln(
					"void modify_"
						+ _aformat2
						+ "_at (in "
						+ _aformat1
						+ " new_element, in unsigned long position) raises (Reflective::BadPosition, Reflective::MofError); ");
			if ((_upper > 1)
				|| (_upper == -1)
				&& (_upper != _lower)
				&& (_is_changeable))
				outputln(
					"void remove_"
						+ _aformat2
						+ " (in "
						+ _aformat1
						+ " old_element ) raises (Reflective::NotFound, Reflective::MofError); ");
			if ((_upper > 1)
				|| (_upper == -1)
				&& (_upper != _lower)
				&& (_is_changeable)
				&& (_is_ordered)
				&& (!_is_unique))
				outputln(
					"void remove_"
						+ _aformat2
						+ "_at (in unsigned long position) raises (Reflective::BadPosition, Reflective::MofError); ");
		}
	}

	public void operationTemplate(Operation _operation)
		throws MofError, BadKind, Bounds, NotSet {

		logger.log(Level.FINE, "IDL operationTemplate");

		if (_operation.visibility().equals(VisibilityKind.public_vis)) {
			//String _oformat2 = idlHelper.format2(_operation.name());
			String _oformat2 = idlHelper.format2IdlConflict(_operation.name());
			annotationTemplate(_operation.annotation());

			//clacul du contenu
			ModelElement[] _contents = _operation.contents();

			// trouver le type de retour s'il y en a
			boolean bool = false;
			int j = 0;
			while (!bool && (j < _contents.length)) {
				Parameter _parameter = ParameterHelper.narrow(_contents[j]);
				if ((_parameter.direction())
					.equals(DirectionKind.return_dir)) {
					bool = true;
					String _typeparameter = idlType(_parameter.type());
					MultiplicityType _multiplicity = _parameter.multiplicity();
					boolean _is_unique = _multiplicity.isUnique;
					boolean _is_ordered = _multiplicity.isOrdered;
					if ((_multiplicity.upper > 1)
						|| (_multiplicity.upper == -1)) {
						output(idlBaseType(_typeparameter));
						if ((_is_unique) && (_is_ordered)) {
							output("UList  " + _oformat2 + " (");
						} else {
							if (_is_ordered) {
								output("List  " + _oformat2 + " (");
							} else {
								if (_is_unique) {
									output("Set  " + _oformat2 + " (");
								} else
									output("Bag  " + _oformat2 + " (");
							}
						}
					} else
						output(
							_typeparameter + "  " + _oformat2 + " (");
				}
				j++;
			}
			if (!bool) {
				output("void " + _oformat2 + " (");
			}

			// trouver le type et les nom des paramtres restants
			for (int i = 0; i < _contents.length; i++) {
				Parameter _parameter = ParameterHelper.narrow(_contents[i]);

				if (!(_parameter
					.direction()
					.equals(DirectionKind.return_dir))) {

					//Calculer les proprits du paramtre courant
					//String _pformat2 = idlHelper.format2(_parameter.name());
					String _pformat2 = idlHelper.format2IdlConflict(_parameter.name());
					String _typeparameter = idlType(_parameter.type());
					MultiplicityType _multiplicity = _parameter.multiplicity();
					int _upper = _multiplicity.upper;
					int _lower = _multiplicity.lower;
					boolean _is_unique = _multiplicity.isUnique;
					boolean _is_ordered = _multiplicity.isOrdered;
					DirectionKind _direction = _parameter.direction();

					//crire le paramtre
					if (_direction.equals(DirectionKind.in_dir))
						output("in ");
					else if (_direction.equals(DirectionKind.out_dir))
						output("out ");
					else if (_direction.equals(DirectionKind.inout_dir))
						output("inout ");

					if ((_upper > 1) || (_upper == -1)) {
						output(idlBaseType(_typeparameter));
						if ((_is_unique) && (_is_ordered)) {
							output("UList  " + _pformat2);
						} else {
							if (_is_ordered) {
								output("List  " + _pformat2);
							} else {
								if (_is_unique) {
									output("Set  " + _pformat2);
								} else
									output("Bag  " + _pformat2);
							}
						}
					} else
						output(_typeparameter + "  " + _pformat2);
					if (i != (_contents.length - 1))
						outputln(" ,");
				}
			}
			output(") raises (");

			//traitement des exceptions
			MofException[] _exceptions = _operation.exceptions();
			for (int i = 0; i < _exceptions.length; i++) {
				MofException _exception =
					MofExceptionHelper.narrow(_exceptions[i]);
				String _eformat1 = idlHelper.format1(_exception.name());
				output(_eformat1);
				outputln(" ,");
			}
			outputln("Reflective::MofError );");

			//traitements des contraintes
			Constraint[] _constraints = _operation.constraints();
			for (int i = 0; i < _constraints.length; i++) {
				Constraint _constraint =
					ConstraintHelper.narrow(_constraints[i]);
				constraintTemplate(_constraint);
			}
		}
	}

	public void classCreateTemplate(org.omg.mof.Model.Class _class)
		throws MofError, BadKind, Bounds, NotSet {

		logger.log(Level.FINE, "IDL classCreateTemplate");

		if (_class.visibility().equals(VisibilityKind.public_vis)) {
			String _typeclass = idlType(_class);
			String _cformat2 = idlHelper.format2(idlHelper.className(_class));
			output(_typeclass + " create_" + _cformat2 + " (");

			//Calcul des attributs
			MofAttribute[] attributes =
				mofHelper.allAttributesOfClass(
					_class,
					ScopeKind.instance_level,
					false);

			for (int i = 0; i < attributes.length; i++) {
				MofAttribute _current = attributes[i];
				String _aformat2 = idlHelper.format2(_current.name());
				String _typeattribute = idlType(_current.type());
				MultiplicityType _multiplicity = _current.multiplicity();
				boolean _is_unique = _multiplicity.isUnique;
				boolean _is_ordered = _multiplicity.isOrdered;
				if ((_multiplicity.upper > 1) || (_multiplicity.upper == -1)) {
					String _aformat1 = idlType(_current.type());
					String _aformat1_qn = idlHelper.format1(_aformat1);
					String _qualifiedName = idlHelper.qualifiedName(_current.type().container());
					if (!_current.type().container()._is_equivalent(input[0])
						&& ((_multiplicity.upper > 1) || (_multiplicity.upper == -1))
						&& (_aformat1.indexOf("::") == -1))
						_aformat1_qn = _qualifiedName + "::" + idlHelper.format1(_aformat1);
					if (_is_unique && _is_ordered) {
						output(
							"in "
								+ _aformat1_qn
								+ "UList  "
								+ _aformat2);
					} else {
						if (_is_ordered) {
							output(
								"in "
									+ _aformat1_qn
									+ "List  "
									+ _aformat2);
						} else {
							if (_is_unique) {
								output(
									"in "
										+ _aformat1_qn
										+ "Set  "
										+ _aformat2);
							} else
								output(
									"in "
										+ _aformat1_qn
										+ "Bag  "
										+ _aformat2);
						}
					}
				} else
					output("in " + _typeattribute + "  " + _aformat2);
				if (i != (attributes.length - 1))
					output(" ,");
			}
			outputln(") raises (Reflective::MofError);");
		}
	}

	public void referenceTemplate(Reference _reference)
		throws MofError, BadKind, Bounds, NotSet {

		logger.log(Level.FINE, "IDL referenceTemplate");
		if (_reference.visibility().equals(VisibilityKind.public_vis)) {
			annotationTemplate(_reference.annotation());
			String _rformat1 = idlHelper.format1(_reference.name());
			String _rformat2 = idlHelper.format2(_reference.name());
			String _rformat2IdlConflict = idlHelper.format2IdlConflict(_reference.name());
			String _typereference = idlType(_reference.type());
			MultiplicityType _multiplicity = _reference.multiplicity();
			int _upper = _multiplicity.upper;
			int _lower = _multiplicity.lower;

			if ((_lower == 0) && (_upper == 1))
				outputln(
						_typereference
						+ "  "
						+ _rformat2IdlConflict
						+ "() raises (Reflective::NotSet, Reflective::MofError);");

			if ((_lower == 1) && (_upper == 1))
				outputln(
						_typereference
						+ "  "
						+ _rformat2IdlConflict
						+ "() raises (Reflective::MofError);");

			if ((_upper > 1) || (_upper == -1)) {
				// une ligne rajoute
				output(idlBaseType(_typereference));
				if (_multiplicity.isUnique && _multiplicity.isOrdered) {
					output("UList  ");
				} else {
					if (_multiplicity.isOrdered) {
						output("List  ");
					} else {
						if (_multiplicity.isUnique) {
							output("Set  ");
						} else
							output("Bag  ");
					}
				}
				outputln(_rformat2IdlConflict + "() raises (Reflective::MofError);");
			}

			if ((_upper == 1) && (_reference.is_changeable()))
				outputln(
					"void set_"
						+ _rformat2
						+ " (in "
						+ _typereference
						+ " new_value ) raises (Reflective::MofError);");

			if (((_upper > 1) || (_upper == -1))
				&& (_reference.is_changeable())) {
				output("void set_" + _rformat2 + " ( ");
				if (_multiplicity.isUnique && _multiplicity.isOrdered) {
					output("in " + idlBaseType(_typereference) + "UList  ");
				} else {
					if (_multiplicity.isOrdered) {
						output(
							"in " + idlBaseType(_typereference) + "List  ");
					} else {
						if (_multiplicity.isUnique) {
							output(
								"in " + idlBaseType(_typereference) + "Set  ");
						} else
							output(
								"in " + idlBaseType(_typereference) + "Bag  ");
					}
				}
				outputln(" new_value ) raises (Reflective::MofError);");
			}

			if ((_lower == 0) && (_upper == 1) && (_reference.is_changeable()))
				outputln(
					"void unset_"
						+ _rformat2
						+ " () raises (Reflective::MofError);");

			if (((_upper > 1) || (_upper == -1))
				&& (_reference.is_changeable()))
				outputln(
					"void add_"
						+ _rformat2
						+ " ( in "
						+ _typereference
						+ " new_element ) raises (Reflective::MofError);");

			if (((_upper > 1) || (_upper == -1))
				&& (_multiplicity.isOrdered)
				&& (_reference.is_changeable())) {
				output("void add_" + _rformat2 + "_before ( ");
				output("in " + _typereference + " new_element, ");
				outputln(
					"in "
						+ _typereference
						+ " before_element ) raises (Reflective::NotFound, Reflective::MofError);");
			}

			if (((_upper > 1) || (_upper == -1))
				&& (_reference.is_changeable())) {
				output("void modify_" + _rformat2 + " ( ");
				output("in " + _typereference + " old_element, ");
				outputln(
					"in "
						+ _typereference
						+ " new_element ) raises (Reflective::NotFound, Reflective::MofError);");
			}

			if (((_upper > 1) || (_upper == -1))
				&& (_lower != _upper)
				&& (_reference.is_changeable())) {
				output("void remove_" + _rformat2 + " (");
				output("in " + _typereference + " old_element ");
				outputln(
					" ) raises (Reflective::NotFound, Reflective::MofError);");
			}
		}
	}

	/**
	 * Returns the idlHelper.
	 * @return IDLCommon
	 */
	public IDLCommon getIdlHelper() {
		return idlHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the idlHelper.
	 * @param idlHelper The idlHelper to set
	 */
	public void setIdlHelper(IDLCommon idlHelper) {
		this.idlHelper = idlHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}
