/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl2java;

import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.PackageHelper;
import org.omg.mof.Reflective.NotSet;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.logging.ModFactLogger;

public class ClassProxyImplementationGenerator extends BracketGenerator {

	ModFactLogger logger = null;

	org.omg.mof.Model.Class[] input;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Class[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Class) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	//Helper
	JavaCommon javaHelper;
	MOFCommon mofHelper;

	public void generate()
		throws
			org.omg.mof.Reflective.NotSet,
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds {
		outputln("// le Code gnr de la ProxyClass: " + input[0].name());
		classProxyImpl(input[0]);
		flushFile();
	}

	// La partie gnration du code
	public void allOfType(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));

		outputln(
			"public " + _cformat1 + "[] all_of_type_" + _cformat2 + "() {");

		// ce vecteur et tableau sont cres pour toutes les classes
		outputln(
			"java.util.Vector all_type_vector = new java.util.Vector();");
		org.omg.mof.Model.Class[] _sub_classes =
			mofHelper.subClassesOfClass(_class);

		for (int i = 0; i < _sub_classes.length; i++) {
			org.omg.mof.Model.Class _class_temp = _sub_classes[i];
			//Only if class is in the same extends
			if (!_class_temp.is_abstract()
				&& mofHelper.isInSameExtent(_class_temp, _class)) {
				String _sformat1 = javaHelper.javaQualifiedName(_class_temp);
				String _sformat2 = javaHelper.format2(javaHelper.javaClassName(_class_temp));
				/* If the class is contained by another package */
				String _sformat2_ref = getRefMethod(_class_temp, _class.container());
				
				//mettre tous les retours de la mthode all_classes des sous classes
				outputln(
					_sformat1
						+ "[] "
						+ "_"
						+ _sformat2
						+ " =_model_package."
						+ _sformat2_ref
						+ ".all_of_class_"
						+ _sformat2
						+ "();");
				outputln(
					"for(int i=0; i<"
						+ "_"
						+ _sformat2
						+ ".length ; i++) all_type_vector.addElement("
						+ "_"
						+ _sformat2
						+ "[i]);");
			}
		}
		if (!_class.is_abstract())
			outputln(
				"for(int i=0; i< all_class_vector_"
					+ _cformat2
					+ ".size() ; i++) all_type_vector.addElement(("
					+ _cformat1
					+ ")all_class_vector_"
					+ _cformat2
					+ ".elementAt(i));");
		outputln(
			_cformat1
				+ "[] all_type = new "
				+ _cformat1
				+ "[all_type_vector.size()];");
		outputln(
			"for (int i=0; i< all_type_vector.size(); i++) all_type[i]= ("
				+ _cformat1
				+ ")all_type_vector.elementAt(i);");
		outputln("return all_type; ");
		outputln("} ");

	}

	public void allOfClasses(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError , NotSet{
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));
		outputln(
			"java.util.Vector all_class_vector_"
				+ _cformat2
				+ " = new java.util.Vector();");
		outputln(
			"public " + _cformat1 + "[] all_of_class_" + _cformat2 + "() {");
		outputln(
			_cformat1
				+ "[] _"
				+ _cformat2
				+ " = new "
				+ _cformat1
				+ "[all_class_vector_"
				+ _cformat2
				+ ".size()];");
		outputln(
			"for (int i=0; i< all_class_vector_"
				+ _cformat2
				+ ".size(); i++) _"
				+ _cformat2
				+ "[i]= ("
				+ _cformat1
				+ ")all_class_vector_"
				+ _cformat2
				+ ".elementAt(i);");
		outputln("return _" + _cformat2 + "; ");
		outputln("} ");
	}

	public void createClass(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError, BadKind, Bounds , NotSet{
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));

		//if (_class.visibility().equals(org.omg.mof.Model.VisibilityKind.public_vis)){
		output("public " + _cformat1 + " create_" + _cformat2 + "(");

		// trouver toutes les attributs de cette classe
		org.omg.mof.Model.MofAttribute[] attributesND =
			mofHelper.allAttributesOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.instance_level,
				false);
		for (int i = 0; i < attributesND.length; i++) {
			String _type_format1 = javaHelper.javaType(attributesND[i].type());
			String _aformat2 = javaHelper.format2(attributesND[i].name());
			if ((attributesND[i].multiplicity().upper > 1)
				|| (attributesND[i].multiplicity().upper == -1)) {
				output(_type_format1 + "[]  " + _aformat2);
			} else
				output(_type_format1 + " " + _aformat2);
			if (i != (attributesND.length - 1))
				output(", ");
		}
		outputln(") throws org.omg.mof.Reflective.MofError {");
		output(
			_cformat1
				+ "Impl _"
				+ _cformat2
				+ " = new "
				+ _cformat1
				+ "Impl(");
		for (int i = 0; i < attributesND.length; i++) {
			String _aformat2 = javaHelper.format2(attributesND[i].name());
			output(_aformat2);
			output(", ");
		}
		outputln("_this());");
		outputln("try {");
		outputln("_poa().activate_object(_" + _cformat2 + ");");
		outputln(
			"org.omg.CORBA.Object obj = _poa().servant_to_reference(_"
				+ _cformat2
				+ ");");
		outputln(
			_cformat1
				+ " _ref = "
				+ _cformat1
				+ "Helper.narrow(obj);");
		outputln("all_class_vector_" + _cformat2 + ".addElement(_ref);");
		outputln("return _ref;");
		outputln("}");
		outputln("catch(java.lang.Exception ex){");
		outputln("ex.printStackTrace();");
		outputln("System.exit(0);");
		outputln("}");
		outputln("return null;");
		outputln("}");
	}

	public void constructeurClass(org.omg.mof.Model.Class _class)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.mof.Reflective.NotSet,
			BadKind,
			Bounds,
			Bounds {

		//String _nformat1 = helper.format1Idl2JavaConflict(_class.name());
		String _nformat1 = javaHelper.javaClassName(_class);
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));

		if (_class
			.visibility()
			.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
			output("public " + _nformat1 + "ClassImpl(");

			// trouver toutes les attributs classifier_level de cette classe
			org.omg.mof.Model.MofAttribute[] attributesND =
				mofHelper.allAttributesOfClass(
					_class,
					org.omg.mof.Model.ScopeKind.classifier_level,
					true);

			for (int i = 0; i < attributesND.length; i++) {
				String _type_format1 =
					javaHelper.javaType(attributesND[i].type());
				String _aformat2 = javaHelper.format2(attributesND[i].name());
				if ((attributesND[i].multiplicity().upper > 1)
					|| (attributesND[i].multiplicity().upper == -1))
					output(_type_format1 + "[]  " + _aformat2 + "s");
				else
					output(_type_format1 + " " + _aformat2);
				output(", ");
			}
			outputln(
				javaHelper.format1Idl2JavaConflict(
					_class.container().name() + "Package")
					+ " _model_package) {");
			for (int i = 0; i < attributesND.length; i++) {
				String _aformat2 = javaHelper.format2(attributesND[i].name());
				int _lower = attributesND[i].multiplicity().lower;
				int _upper = attributesND[i].multiplicity().upper;
				if ((_upper > 1) || (_upper == -1)) {
					output(
						"for(int i=0;i<" + _aformat2 + "s.length; i++) ");
					output(
						" _"
							+ _aformat2
							+ "s.addElement("
							+ _aformat2
							+ "s[i]);");
				} else {
					outputln("_" + _aformat2 + " = " + _aformat2 + " ;");
					if ((_lower == 0) && (_upper == 1))
						outputln("_exist_" + _aformat2 + " = true;");
				}
			}
			outputln("this._model_package = _model_package ;");
			outputln("}");
			outputln("}");

		}
	}

	public void classProxyImpl(org.omg.mof.Model.Class _class)
		throws
			org.omg.mof.Reflective.NotSet,
			org.omg.mof.Reflective.MofError,
			BadKind,
			Bounds {

		//les formats du nom de la classe
		//String _nformat1 = helper.format1Idl2JavaConflict(_class.name());
		String _nformat1 = javaHelper.javaClassName(_class);
		String _cformat1 = javaHelper.javaQualifiedName(_class);
		String _cformat2 = javaHelper.format2(javaHelper.javaClassName(_class));

		//Dcalration du Package
		output("package ");

		// Affichage du nom du package
		outputln(
			javaHelper.javaPackage(PackageHelper.narrow(_class.container()))
				+ ";");

		outputln(
			"public class "
				+ _nformat1
				+ "ClassImpl extends "
				+ _cformat1
				+ "ClassPOA {");
		outputln(
			javaHelper.format1Idl2JavaConflict(
				_class.container().name() + "Package")
				+ " _model_package;");

		//genrer du code pour RefBaseObject et RefObject
		output(javaHelper.refBaseObjectTemplate());
		outputln();
		output(javaHelper.refObjectTemplate());

		// Tous les attributs classifier_level
		org.omg.mof.Model.MofAttribute[] attributes =
			mofHelper.allAttributesOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.classifier_level,
				false);
		for (int i = 0; i < attributes.length; i++) {
			if (attributes[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _code_attribute =
					javaHelper.attributeTemplate(attributes[i]);
				outputln(_code_attribute);
			}
		}

		// Toutes les oprations classifier_level
		org.omg.mof.Model.Operation[] operations =
			mofHelper.allOperationsOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.classifier_level);
		for (int i = 0; i < operations.length; i++) {
			if (operations[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _code_operation =
					javaHelper.operationTemplate(operations[i]);
				outputln(_code_operation);
			}
		}

		// Traitement des CreateClass, all_of_type et all_of_classes de la classe et de ses superclasses
		//org.omg.mof.Model.GeneralizableElement[] _all_superclasses =
		//	_class.all_supertypes();
		//java.util.Vector _sup_classes = new java.util.Vector();
		//for (int i = 0; i < _all_superclasses.length; i++)
		//	_sup_classes.addElement(
		//		org.omg.mof.Model.ClassHelper.narrow(_all_superclasses[i]));
		java.util.Vector _sup_classes = mofHelper.superClassesOrdered(_class);
		java.util.Iterator it = _sup_classes.iterator();
		while (it.hasNext()) {
			org.omg.CORBA.Object obj = (org.omg.CORBA.Object)it.next();
			if (obj._is_equivalent(_class))
				it.remove();
		}

		for (int i = 0; i < _sup_classes.size(); i++) {
			org.omg.mof.Model.Class _current =
				(org.omg.mof.Model.Class) _sup_classes.elementAt(i);
			if (_current
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
				String _cu_format1 = javaHelper.javaQualifiedName(_current);
				String _cu_format2 = javaHelper.format2(javaHelper.className(_current));
				/* If the class is contained by another package */
				String _cu_format2_ref = getRefMethod(_current, _class.container());

				//Gnrer le code de la mthode Create et de la mthode all_of_classes
				if (!_current.is_abstract()) {

					//CreateClass des SuperClasses
					output(
						"public "
							+ _cu_format1
							+ " create_"
							+ _cu_format2
							+ "(");

					// trouver toutes les attributs de cette classe
					org.omg.mof.Model.MofAttribute attributesND[] =
						mofHelper.allAttributesOfClass(
							_current,
							org.omg.mof.Model.ScopeKind.instance_level,
							false);
					for (int j = 0; j < attributesND.length; j++) {
						String _type_format1 =
							javaHelper.javaType(attributesND[j].type());
						String _aformat2 =
							javaHelper.format2(attributesND[j].name());
						if ((attributesND[j].multiplicity().upper > 1)
							|| (attributesND[j].multiplicity().upper == -1)) {
							output(_type_format1 + "[]  " + _aformat2);
						} else
							output(_type_format1 + " " + _aformat2);
						if (j != (attributesND.length - 1))
							output(", ");
					}
					outputln(") throws org.omg.mof.Reflective.MofError {");
					if (mofHelper.isInSameExtent(_current, _class)) {
						//supertype closure rule
						output(
							_cu_format1
								+ " _"
								+ _cu_format2
								+ " = _model_package."
								+ _cu_format2_ref
								+ ".create_"
								+ _cu_format2
								+ "(");
						for (int j = 0; j < attributesND.length; j++) {
							String _aformat2 =
								javaHelper.format2(attributesND[j].name());
							output(_aformat2);
							if (j != (attributesND.length - 1))
								output(", ");
						}
						outputln(");");
						outputln("return _" + _cu_format2 + "; ");
					} else {
						outputln(
							"throw new org.omg.mof.Reflective.MofError();");
					}
					outputln("} ");

					//AllofClasses des SuperClasses
					outputln(
						"public "
							+ _cu_format1
							+ "[] all_of_class_"
							+ _cu_format2
							+ "() {");
					outputln(
						_cu_format1 + "[] _" + _cu_format2 + ";");
					if (mofHelper.isInSameExtent(_current, _class)) {
						outputln(
							"_"
								+ _cu_format2
								+ " = _model_package."
								+ _cu_format2_ref
								+ ".all_of_class_"
								+ _cu_format2
								+ "();");
					} else {
						outputln(
							"_"
								+ _cu_format2
								+ " = new "
								+ _cu_format1
								+ "[0];");
					}
					outputln("return _" + _cu_format2 + "; ");
					outputln("} ");
					//END allOfClass 
				}

				//Gnrer le code de la mthode AllOfType de la classe courante
				outputln(
					"public "
						+ _cu_format1
						+ "[] all_of_type_"
						+ _cu_format2
						+ "() {");
				outputln(_cu_format1 + "[] _" + _cu_format2 + ";");
				if (mofHelper.isInSameExtent(_current, _class)) {
					outputln(
						"_"
							+ _cu_format2
							+ " = _model_package."
							+ _cu_format2_ref
							+ ".all_of_type_"
							+ _cu_format2
							+ "();");
				} else {
					outputln(
						"_"
							+ _cu_format2
							+ " = new "
							+ _cu_format1
							+ "[0];");
				}
				outputln("return _" + _cu_format2 + "; ");
				outputln("} ");
				//END allOfType
			}
		}

		//Gnrer le code de la mthode AllOfType et AllofClasses et create Class  Ade la classe courante
		if (!_class.is_abstract()) {
			createClass(_class);
			allOfClasses(_class);
		}

		allOfType(_class);

		constructeurClass(_class);
	}
	
	/**
	 * Returns the ref method of a class contained in a package.
	 * And tests if the ref method is present in the package or in an imported package.
	 * @param clazz The class
	 * @param parent The pacakge
	 * @return The string representation of the ref method calling
	 */
	private String getRefMethod(org.omg.mof.Model.Class clazz, org.omg.mof.Model.Namespace parent)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		String _format2_ref = javaHelper.format2(javaHelper.javaClassName(clazz)) + "_ref()";
		try {
			if (!clazz.container()._is_equivalent(parent)) {
				_format2_ref = javaHelper.format2(clazz.container().name()) + "_ref()." + _format2_ref;
			}
		} catch (org.omg.mof.Reflective.NotSet notSet) {
			// Nothing to do
		}
		return _format2_ref;
	}
				
	/**
	 * Returns the javaHelper.
	 * @return JavaCommon
	 */
	public JavaCommon getJavaHelper() {
		return javaHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the javaHelper.
	 * @param javaHelper The javaHelper to set
	 */
	public void setJavaHelper(JavaCommon javaHelper) {
		this.javaHelper = javaHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}
