/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl2java;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.objectweb.modfact.corba.generator.ZipGenerator;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;

public class IDL2JavaMOF1_3Generator extends ZipGenerator {

	private org.omg.mof.Model.Package[] input;

	private MOFCommon mofHelper;

	private JavaCommon javaHelper;

	private ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Package[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Package) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * Generate
	 */
	public void generate()
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			org.omg.mof.Reflective.NotSet,
			java.io.FileNotFoundException,
			IOException {

		//mofHelper = new MOFCachedProvider(input);
		logger.log(Level.FINE, "IDL2Java Start Generation of "+input.length+" packages");
		for (int i = 0; i < input.length; i++) {

			if (input[i]
				.visibility()
				.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {

				org.omg.mof.Model.Package[] apackage =
					new org.omg.mof.Model.Package[1];
				apackage[0] = input[i];
				
				logger.log(Level.FINE , "package "+apackage[0].name());

				//Traitemement du Package
				PackageImplementationGenerator packageImpl =
					new PackageImplementationGenerator();
				packageImpl.setInput(apackage);
				ByteArrayOutputStream entryOutput1 =
					new ByteArrayOutputStream();
				packageImpl.setOutput(entryOutput1);
				packageImpl.setMofHelper(mofHelper);
				packageImpl.setJavaHelper(javaHelper);
				packageImpl.generate();

				String entryName =
					javaHelper.format1Idl2JavaConflict(
						input[i].name() + "Package")
						+ "Impl.java";
				writeEntry(entryName, entryOutput1);
				entryOutput1.close();

				PackageFactoryImplementationGenerator packageFactoryImpl =
					new PackageFactoryImplementationGenerator();
				packageFactoryImpl.setInput(apackage);
				//PipedOutputStream entryOutput2 = new PipedOutputStream();
				ByteArrayOutputStream entryOutput2 =
					new ByteArrayOutputStream();
				packageFactoryImpl.setOutput(entryOutput2);
				packageFactoryImpl.setMofHelper(mofHelper);
				packageFactoryImpl.setJavaHelper(javaHelper);
				packageFactoryImpl.generate();
				entryName =
					javaHelper.format1Idl2JavaConflict(
						input[i].name() + "PackageFactory")
						+ "Impl.java";
				writeEntry(entryName, entryOutput2);
				entryOutput2.close();

				//packageFactoryImpl.closeFile();

				//Traitement des Classes
				org.omg.mof.Model.Class[] classes =
					mofHelper.classesOfPackage(input[i]);

				for (int j = 0; j < classes.length; j++) {
					logger.log(Level.FINE, "Class "+classes[j].name());
					if (classes[0].visibility()
						== org.omg.mof.Model.VisibilityKind.public_vis) {

						org.omg.mof.Model.Class[] aclass =
							new org.omg.mof.Model.Class[1];
						aclass[0] = classes[j];

						// Cette Partie gnre toutes les ClasseProxy

						ClassProxyImplementationGenerator classProxyImpl =
							new ClassProxyImplementationGenerator();
						classProxyImpl.setInput(aclass);
						//PipedOutputStream entryOutputCP =
						//	new PipedOutputStream();
						ByteArrayOutputStream entryOutputCP =
							new ByteArrayOutputStream();
						classProxyImpl.setOutput(entryOutputCP);
						classProxyImpl.setMofHelper(mofHelper);
						classProxyImpl.setJavaHelper(javaHelper);
						classProxyImpl.generate();
						String entryNameCP =
							javaHelper.format1Idl2JavaConflict(
								javaHelper.javaClassName(classes[j]) + "Class")
								+ "Impl.java";
						writeEntry(entryNameCP, entryOutputCP);
						entryOutputCP.close();
						logger.log(Level.FINER , "Entry "+entryNameCP+" has been added");

						// Cette partie gnre toutes les Classes non abstraites
						if (!(classes[j].is_abstract())) {
							ClassImplementationGenerator classImpl =
								new ClassImplementationGenerator();
							classImpl.setInput(aclass);
							//PipedOutputStream entryOutputC = new PipedOutputStream();
							ByteArrayOutputStream entryOutputC =
								new ByteArrayOutputStream();
							classImpl.setOutput(entryOutputC);
							classImpl.setMofHelper(mofHelper);
							classImpl.setJavaHelper(javaHelper);
							classImpl.generate();

							String entryNameC =
								javaHelper.format1Idl2JavaConflict(
									javaHelper.javaClassName(classes[j]))
									+ "Impl.java";
							writeEntry(entryNameC, entryOutputC);
							entryOutputC.close();
							logger.log(Level.FINER , "Entry "+entryNameC+" has been added");
						}
					}
				}

				//Traitement des Associations
				org.omg.mof.Model.Association[] associations =
					mofHelper.associationsOfPackage(input[i]);
				for (int j = 0; j < associations.length; j++) {
					if (associations[j].visibility()
						== org.omg.mof.Model.VisibilityKind.public_vis) {
						org.omg.mof.Model.Association[] anasso =
							new org.omg.mof.Model.Association[1];
						anasso[0] = associations[j];

						AssociationImplementationGenerator associationImpl =
							new AssociationImplementationGenerator();
						associationImpl.setInput(anasso);
						//PipedOutputStream entryOutputA = new PipedOutputStream();
						ByteArrayOutputStream entryOutputA =
							new ByteArrayOutputStream();
						associationImpl.setOutput(entryOutputA);
						associationImpl.setMofHelper(mofHelper);
						associationImpl.setJavaHelper(javaHelper);
						associationImpl.generate();

						String entryNameA =
							javaHelper.format1Idl2JavaConflict(
								associations[j].name())
								+ "Impl.java";
						writeEntry(entryNameA, entryOutputA);
						entryOutputA.close();
					}
				}

				//Traitement des nested packages
				org.omg.mof.Model.Package[] packages =
					mofHelper.packagesOfPackage(input[i]);
				if (packages.length != 0) {
					IDL2JavaMOF1_3Generator javaGenerator =
						new IDL2JavaMOF1_3Generator();
					javaGenerator.setLogger(logger);
					javaGenerator.setInput(packages);
					javaGenerator.setMofHelper(mofHelper);
					javaGenerator.setJavaHelper(javaHelper);
					ByteArrayOutputStream entryOutputPackage =
						new ByteArrayOutputStream();
					javaGenerator.setOutput(entryOutputPackage);
					javaGenerator.generate();

					String entryNameP =
						javaHelper.format1Idl2JavaConflict(input[i].name())
							+ "NestedPackages.zip";
					writeEntry(entryNameP, entryOutputPackage);
					entryOutputPackage.close();
				}

			}
		}
		logger.log(Level.FINE, "IDL2Java Generation of "+input.length+" packages ended !!!");
		out.finish();
	}

	/**
	 * Returns the javaHelper.
	 * @return JavaCommon
	 */
	public JavaCommon getJavaHelper() {
		return javaHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the javaHelper.
	 * @param javaHelper The javaHelper to set
	 */
	public void setJavaHelper(JavaCommon javaHelper) {
		this.javaHelper = javaHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}