/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.idl2java;

import java.util.Vector;

import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.ScopeKind;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.logging.ModFactLogger;

public class PackageFactoryImplementationGenerator extends BracketGenerator {
	
	ModFactLogger logger = null;

	org.omg.mof.Model.Package[] input;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Package[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Package) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	//Helper
	JavaCommon javaHelper;
	MOFCommon mofHelper;	
	
	

	public  void generate()
		throws
			org.omg.mof.Reflective.NotSet,
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds {
		outputln(
			"// le Code gnr du PackageFactory: "
				+ input[0].name()
				+ "PackageFactory");
		packageFactoryImpl(input[0]);
		flushFile();
	}

	public  void packageFactoryImpl(org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.NotSet, org.omg.mof.Reflective.MofError , BadKind , Bounds{
		//Dcalration du Package
		output("package ");

		// les deux formats du nom du package
		String _nformat1 = javaHelper.format1Idl2JavaConflict(_package.name());
		String className = javaHelper.format1Idl2JavaConflict(_package.name()+"PackageFactory")+"Impl";
		String superClass = javaHelper.format1Idl2JavaConflict(_package.name()+"PackageFactory")+"POA";
		String itemType = javaHelper.format1Idl2JavaConflict(_package.name()+"Package");
		String itemClass = javaHelper.format1Idl2JavaConflict(_package.name()+"Package")+"Impl";
		String _pformat2 = javaHelper.format2(_package.name());

		// le nom du package
		outputln(javaHelper.javaPackage(_package) + " ;");

		outputln(
			"public class "
				+ className
				+ " extends "
				+ superClass
				+ " {");

		Vector _attributes = new Vector();
		org.omg.mof.Model.Class[] classes = mofHelper.classesOfPackage(_package);
		for (int i=0 ; i < classes.length ; i++) {
			org.omg.mof.Model.MofAttribute[] attributes = mofHelper.attributesOfClass(classes[i] , ScopeKind.classifier_level , true);
			for (int k=0 ; k<attributes.length ; k++) _attributes.addElement(attributes[k]);
			
		}

		//
		java.util.Vector _attributes_name =
			new java.util.Vector(_attributes.size());
		for (int i = 0; i < _attributes.size(); i++) {
			org.omg.mof.Model.MofAttribute _current =
				(org.omg.mof.Model.MofAttribute) _attributes.elementAt(i);
			String _attributeqname = " ";
			for (int j = 0; j < _current.qualified_name().length; j++) {
				if (j > 0)
					_attributeqname =
						javaHelper.format2(_current.qualified_name()[j])
							+ "_"
							+ _attributeqname;
				else
					_attributeqname =
						javaHelper.format2(_current.qualified_name()[j]);
			}
			_attributes_name.addElement(_attributeqname);
		}

		//
		output(
			"public "
				+ itemType
				+ " create_"
				+ _pformat2
				+ "_package(");
		for (int i = 0; i < _attributes.size(); i++) {
			org.omg.mof.Model.MofAttribute _current =
				(org.omg.mof.Model.MofAttribute) _attributes.elementAt(i);
			String _current_type = javaHelper.javaType(_current.type());
			String _attributeqname = (String) _attributes_name.elementAt(i);

			if (!((i != 0) || (i != (_attributes.size() - 1))))
				output(", ");

			if ((_current.multiplicity().upper > 1)
				|| (_current.multiplicity().upper == -1))
				output(_current_type + "[] " + _attributeqname);
			else
				output(_current_type + " " + _attributeqname);
		}
		outputln(") throws org.omg.mof.Reflective.MofError {");
		output(
			itemClass
				+ " _"
				+ _pformat2
				+ "_package = new "
				+ itemClass
				+ "(");
		for (int i = 0; i < _attributes_name.size(); i++) {
			String _attributeqname = (String) _attributes_name.elementAt(i);
			if (!((i != 0) || (i != (_attributes.size() - 1))))
				output(", ");
			output(_attributeqname);
		}
		outputln(");");

		outputln("try {");
		outputln("_poa().activate_object(_" + _pformat2 + "_package);");
		outputln(
			"org.omg.CORBA.Object obj = _poa().servant_to_reference(_"
				+ _pformat2
				+ "_package);");
		outputln("return " + itemType + "Helper.narrow(obj);");
		outputln("}");
		outputln("catch (java.lang.Exception ex) {");
		outputln("ex.printStackTrace();");
		outputln("System.exit(0);");
		outputln("}");
		outputln("return null;");
		outputln("}");
		outputln("}");
	}
	/**
	 * Returns the javaHelper.
	 * @return JavaCommon
	 */
	public JavaCommon getJavaHelper() {
		return javaHelper;
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Sets the javaHelper.
	 * @param javaHelper The javaHelper to set
	 */
	public void setJavaHelper(JavaCommon javaHelper) {
		this.javaHelper = javaHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

}