/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.io;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.mof.Model.*;
import org.omg.mof.Reflective.MofError;
import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;

public class Import {

	//Trace
	private ModFactLogger logger;

	//Dclaration des Helpers
	private IDLCommon idlHelper;

	//Dclaration des Factory
	private PackageClass _package_class;
	private ClassClass _class_class;
	private MofAttributeClass _attribute_class;
	private ReferenceClass _reference_class;
	private AssociationClass _association_class;
	private AssociationEndClass _association_end_class;
	private RefersTo _refers_to_class;
	private Exposes _exposes_class;
	private DataTypeClass _datatype_class;
	private ConstantClass _constant_class;
	private ConstraintClass _constraint_class;
	private TagClass _tag_class;
	private OperationClass _operation_class;
	private ParameterClass _parameter_class;
	private MofExceptionClass _exception_class;
	private ImportClass _import_class;

	// Dclaration
	private ORB _orb;
	private Element _document;

	// Hashtables pour typer  les attributs et les paramtre et rajouter les supertypes
	private Hashtable _types = new Hashtable();
	private Hashtable _elementsToType = new Hashtable();
	private Hashtable _elementsToSupertype = new Hashtable();

	//Hashtables pour les tags
	private Hashtable _tags = new Hashtable();
	private Hashtable _Id2element = new Hashtable();

	//Hashtables pour les Exception  lier aux oprations 
	private Hashtable _Id2exception = new Hashtable();
	private Hashtable _Exception4Operations = new Hashtable();

	//Hashtables pour les imports
	private Hashtable _Import2Imported = new Hashtable();

	// Hashtables pour les typer les rfrences et les lier aux associationEnd
	private Hashtable _OtherEnd = new Hashtable();
	private Hashtable _Id2End = new Hashtable();
	private Hashtable _referencedEndTable = new Hashtable();
	private Hashtable _constantValue = new Hashtable();
	
	// Hashtable for data types
	private java.util.Hashtable _dataTypes = new Hashtable(); // Key  / TypeCode

	// initialisation
	public void initiate(_ModelPackage _model_package, org.omg.CORBA.ORB orb)
		throws MofError {

		//creation des Factory
		_package_class = _model_package.package_ref();
		_class_class = _model_package.class_ref();
		_attribute_class = _model_package.mof_attribute_ref();
		_reference_class = _model_package.reference_ref();
		_association_class = _model_package.association_ref();
		_association_end_class = _model_package.association_end_ref();
		_refers_to_class = _model_package.refers_to_ref();
		_exposes_class = _model_package.exposes_ref();
		_datatype_class = _model_package.data_type_ref();
		_constant_class = _model_package.constant_ref();
		_constraint_class = _model_package.constraint_ref();
		_tag_class = _model_package.tag_ref();
		_operation_class = _model_package.operation_ref();
		_parameter_class = _model_package.parameter_ref();
		_exception_class = _model_package.mof_exception_ref();
		_import_class = _model_package.import_ref();
		_orb = orb;
		initPrimitiveTypes();
	}
	
   /**
	* Initialize the primitive types (i.e. String, Boolean, Integer, Long, Double, Float).
	*/
   protected void initPrimitiveTypes() throws org.omg.mof.Reflective.MofError {
	   org.omg.mof.Model.VisibilityKind pvis = org.omg.mof.Model.VisibilityKind.public_vis;
	   org.omg.mof.Model.DataType pt_integer = _datatype_class.create_data_type(
							"*long", "", pvis, true, true, false, _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_long));
	   org.omg.mof.Model.DataType pt_long = _datatype_class.create_data_type(
							"*long long", "", pvis, true, true, false, _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_longlong));
	   org.omg.mof.Model.DataType pt_float = _datatype_class.create_data_type(
							"*float", "", pvis, true, true, false, _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float));
	   org.omg.mof.Model.DataType pt_double = _datatype_class.create_data_type(
							"*double", "", pvis, true, true, false, _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_double));
	   org.omg.mof.Model.DataType pt_boolean = _datatype_class.create_data_type(
							"*boolean", "", pvis, true, true, false, _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_boolean));
	   org.omg.mof.Model.DataType pt_string = _datatype_class.create_data_type(
							"*string", "", pvis, true, true, false, _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string));

	   _types.put("Integer", pt_integer);
	   _types.put("Long", pt_long);
	   _types.put("Float", pt_float);
	   _types.put("Double", pt_double);
	   _types.put("Boolean", pt_boolean);
	   _types.put("String", pt_string);
   }

	//Rem changer Package par ModelElement 
	public org.omg.mof.Model.Package[] parse(String _fichier)
		throws
			SAXException,
			ParserConfigurationException,
			IOException,
			BadKind,
			MofError {

		// Charger le document
		FileInputStream _xml_input_file = new FileInputStream(_fichier);

		return parse(_xml_input_file);
	}

	public org.omg.mof.Model.Package[] parse(InputStream _xml_input_file)
		throws
			SAXException,
			ParserConfigurationException,
			IOException,
			BadKind,
			MofError {

		logger.log(Level.FINE, "Begin parsing");
		// le package
		Vector packagesV = new Vector();

		DocumentBuilderFactory _factory = DocumentBuilderFactory.newInstance();
		_factory.setIgnoringComments(true);
		DocumentBuilder _builder = _factory.newDocumentBuilder();

		// Charger le document
		Document doc = _builder.parse(_xml_input_file);
		logger.log(Level.FINE, "Document is valid");

		// Parser le document
		Element model = doc.getDocumentElement();

		//Enlever les espaces
		delSpaces(model);

		//rcuprer XMI.content
		Element xmiContent = null;
		NodeList modelContents = model.getChildNodes();
		for (int i = 0; i < modelContents.getLength(); i++) {
			Node node = modelContents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("XMI.content")) {
					xmiContent = contentElement;
				}
			}
		}
		if (xmiContent == null) {
			logger.log(Level.FATAL, "There is no XMI.content element !");
			return null;
		}

		//rcuprer les packages outermost contenus
		NodeList contents = xmiContent.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element element = (Element) node;
				String name = element.getNodeName().trim();

				if (name.toLowerCase().endsWith("package")) {
					packagesV.addElement(packageTemplate(element, null));
				}
			}
		}
		org.omg.mof.Model.Package[] packages =
			new org.omg.mof.Model.Package[packagesV.size()];
		for (int i = 0; i < packages.length; i++) {
			packages[i] = (org.omg.mof.Model.Package) packagesV.elementAt(i);
		}
		logger.log(Level.FINE, packagesV.size() + " Package(s) parsed");

		// Deuxime passe pour mettre les types, les tags, les imports  et les AssociationsEnds pour les rfrences
		typing();
		logger.log(Level.FINE, "typing is done");
		associationEnds4Reference();
		logger.log(Level.FINE, "associationEnds4Reference is done");
		putSupertypes();
		logger.log(Level.FINE, "putSupertypes is done");
		taggedElement();
		logger.log(Level.FINE, "taggedElement is done");
		importElements();
		logger.log(Level.FINE, "ImportElements is done");
		exceptions4Operations();
		logger.log(Level.FINE, "Exceptions4Operations is done");

		logger.log(Level.FINE, "Parsing is done");

		return packages;
	}

	// traitement du package
	public org.omg.mof.Model.Package packageTemplate(
		Element _package_element,
		Namespace _container)
		throws BadKind, MofError {
		//cration par dfault du package
		org.omg.mof.Model.Package _package =
			_package_class.create_package(
				"name",
				"",
				VisibilityKind.public_vis,
				false,
				false,
				false);

		//Properties
		String name = getValue(_package_element, "name");
		logger.log(Level.FINE, " Package " + name + " parsing");
		if (name.compareTo("") != 0)
			_package.set_name(name);
		String annotation = getValue(_package_element, "annotation");
		if (annotation.compareTo("") != 0)
			_package.set_annotation(annotation);
		String visibility = getValue(_package_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_package.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_package.set_visibility(VisibilityKind.private_vis);
		String isAbstract = getValue(_package_element, "isAbstract");
		if (isAbstract.endsWith("true"))
			_package.set_is_abstract(true);
		String isLeaf = getValue(_package_element, "isLeaf");
		if (isLeaf.endsWith("true"))
			_package.set_is_leaf(true);
		String isRoot = getValue(_package_element, "isRoot");
		if (isRoot.endsWith("true"))
			_package.set_is_root(true);

		//rcuprer le contenu du Package 
		NodeList contents = _package_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("Namespace.contents")) {
					NodeList namespaceContents = contentElement.getChildNodes();
					for (int j = 0; j < namespaceContents.getLength(); j++) {
						if (namespaceContents.item(j).getNodeType()
							== Node.ELEMENT_NODE) {
							Element _element =
								(Element) namespaceContents.item(j);
							String _element_name =
								_element.getNodeName().trim();
							if (_element_name.endsWith("DataType"))
								dataTypeTemplate(_element, _package);
							else if (_element_name.endsWith("PrimitiveType"))
								primitiveTypeTemplate(_element, _package);
							else if (_element_name.endsWith("EnumerationType"))
								enumerationTypeTemplate(_element, _package);
							else if (_element_name.endsWith("StructureType"))
								structureTypeTemplate(_element, _package);
							else if (_element_name.endsWith("AliasType"))
								aliasTypeTemplate(_element, _package);
							else if (_element_name.endsWith("CollectionType"))
								collectionTypeTemplate(_element, _package);
							else if (_element_name.endsWith("Association"))
								associationTemplate(_element, _package);
							else if (_element_name.endsWith("Class"))
								classTemplate(_element, _package);
							else if (_element_name.endsWith("Constant"))
								constantTemplate(_element, _package);
							else if (_element_name.endsWith("Tag"))
								tagTemplate(_element, _package);
							else if (_element_name.endsWith("Exception"))
								mofExceptionTemplate(_element, _package);
							else if (_element_name.endsWith("Import"))
								importTemplate(_element, _package);
							else if (_element_name.endsWith("Package"))
								packageTemplate(_element, _package);
						}
					}
				}
			}
		}

		// ranger le package dans la table des tags
		String _key = _package_element.getAttribute("xmi.id").trim();
		_Id2element.put(_key, _package);

		_package.set_container(_container);
		if (_container != null)
			_container.add_contents(_package);
		return _package;
	}

	//Traitement des classes
	public void classTemplate(Element _class_element, Namespace _container)
		throws MofError {
		// rcuprer l'identificateur
		String _key = _class_element.getAttribute("xmi.id").trim();

		// cration d'une classe par dfault
		org.omg.mof.Model.Class _class =
			_class_class.create_class(
				"name",
				"",
				VisibilityKind.public_vis,
				false,
				false,
				false,
				false);

		//Proprits de la class
		String name = getValue(_class_element, "name");
		logger.log(Level.FINE, " Class " + name + " parsing");
		if (name.compareTo("") != 0)
			_class.set_name(name);

		String annotation = getValue(_class_element, "annotation");
		if (annotation.compareTo("") != 0)
			_class.set_annotation(annotation);

		String visibility = getValue(_class_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_class.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_class.set_visibility(VisibilityKind.private_vis);

		String isAbstract = getValue(_class_element, "isAbstract");
		if (isAbstract.endsWith("true"))
			_class.set_is_abstract(true);

		String isLeaf = getValue(_class_element, "isLeaf");
		if (isLeaf.endsWith("true"))
			_class.set_is_leaf(true);

		String isRoot = getValue(_class_element, "isRoot");
		if (isRoot.endsWith("true"))
			_class.set_is_root(true);

		String isSingleton = getValue(_class_element, "isSingleton");
		if (isSingleton.endsWith("true"))
			_class.set_is_singleton(true);

		//to recover supertypes for XMI version 1.1
		String supertypes = _class_element.getAttribute("supertypes");
		if (supertypes.compareTo("") != 0) {
			Vector supers = new Vector();
			StringTokenizer supRefs = new StringTokenizer(supertypes, " 	");
			while (supRefs.hasMoreTokens()) {
				String part = supRefs.nextToken();
				supers.addElement(part);
			}
			_elementsToSupertype.put(_class, supers);
		}

		NodeList contents = _class_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();

				//to recover supertypes for XMI version 1.0
				if (currentName.endsWith("GeneralizableElement.supertypes")) {
					Vector supers = new Vector();
					NodeList supertypeNodes = contentElement.getChildNodes();
					for (int j = 0; j < supertypeNodes.getLength(); j++) {
						if (supertypeNodes.item(j).getNodeType()
							== Node.ELEMENT_NODE) {
							Element element = (Element) supertypeNodes.item(j);
							String _element_name = element.getNodeName().trim();
							if (_element_name
								.endsWith("GeneralizableElement")) {
								String _ref =
									element.getAttribute("xmi.idref").trim();
								supers.addElement(_ref);
							}
						}
					}
					_elementsToSupertype.put(_class, supers);
				} else if (currentName.endsWith("Namespace.contents")) {
					NodeList namespaceContents = contentElement.getChildNodes();
					for (int j = 0; j < namespaceContents.getLength(); j++) {
						if (namespaceContents.item(j).getNodeType()
							== Node.ELEMENT_NODE) {
							Element element =
								(Element) namespaceContents.item(j);
							String _element_name = element.getNodeName().trim();
							if (_element_name.endsWith("Attribute"))
								mofAttributeTemplate(element, _class);
							else if (_element_name.endsWith("Reference"))
								referenceTemplate(element, _class);
							else if (_element_name.endsWith("Operation"))
								operationTemplate(element, _class);
							else if (_element_name.endsWith("DataType"))
								dataTypeTemplate(element, _class);
							else if (_element_name.endsWith("PrimitiveType"))
								primitiveTypeTemplate(element, _class);
							else if (_element_name.endsWith("EnumerationType"))
								enumerationTypeTemplate(element, _class);
							else if (_element_name.endsWith("StructureType"))
								structureTypeTemplate(element, _class);
							else if (_element_name.endsWith("AliasType"))
								aliasTypeTemplate(element, _class);
							else if (_element_name.endsWith("CollectionType"))
								collectionTypeTemplate(element, _class);
							else if (_element_name.endsWith("Exception"))
								mofExceptionTemplate(element, _class);
							else if (_element_name.endsWith("Constant"))
								constantTemplate(element, _class);
						}
					}

				}
			}
		}
		// container et contents
		_class.set_container(_container);
		_container.add_contents(_class);

		// ranger la classe dans la tables des types et ventuellement pour les tags
		_types.put(_key, _class);
		_Id2element.put(_key, _class);
	}

	// les attributs d'une classe
	public void mofAttributeTemplate(
		Element _attribute_element,
		org.omg.mof.Model.Class _class)
		throws MofError {
		// cration d'un attribut par dfault
		MofAttribute _attribute =
			_attribute_class.create_mof_attribute(
				"name",
				"",
				VisibilityKind.public_vis,
				ScopeKind.instance_level,
				new MultiplicityType(0, 1, false, false),
				true,
				false);

		// rcuprer les proprits
		String name = getValue(_attribute_element, "name");
		if (name.compareTo("") != 0)
			_attribute.set_name(name);

		String annotation = getValue(_attribute_element, "annotation");
		if (annotation.compareTo("") != 0)
			_attribute.set_annotation(annotation);

		String visibility = getValue(_attribute_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_attribute.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_attribute.set_visibility(VisibilityKind.private_vis);

		String scope = getValue(_attribute_element, "scope");
		if (scope.endsWith("classifier_level"))
			_attribute.set_scope(ScopeKind.classifier_level);

		String isDerived = getValue(_attribute_element, "isDerived");
		if (isDerived.endsWith("true"))
			_attribute.set_is_derived(true);

		String changeable = getValue(_attribute_element, "isChangeable");
		if (changeable.endsWith("false"))
			_attribute.set_is_changeable(false);

		//mettre  jour la multiplicit de l'attribut
		_attribute.set_multiplicity(getMultiplicity(_attribute_element));

		// rcuprer le type 
		_elementsToType.put(_attribute, getType(_attribute_element));

		_attribute.set_container(_class);
		_class.add_contents(_attribute);
	}

	// les rfrences d'une classe
	public void referenceTemplate(
		Element _reference_element,
		org.omg.mof.Model.Class _class)
		throws MofError {
		// cration d'une rfrence  par dfault
		Reference _reference =
			_reference_class.create_reference(
				"name",
				"",
				VisibilityKind.public_vis,
				ScopeKind.instance_level,
				new MultiplicityType(0, 1, false, false),
				true);

		// rcuprer les valeurs de l'attribut
		String name = getValue(_reference_element, "name");
		if (name.compareTo("") != 0)
			_reference.set_name(name);

		String annotation = getValue(_reference_element, "annotation");
		if (annotation.compareTo("") != 0)
			_reference.set_annotation(annotation);

		String visibility = getValue(_reference_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_reference.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_reference.set_visibility(VisibilityKind.private_vis);

		String scope = getValue(_reference_element, "scope");
		if (scope.endsWith("classifier_level"))
			_reference.set_scope(ScopeKind.classifier_level);

		String changeable = getValue(_reference_element, "isChangeable");
		if (changeable.endsWith("false"))
			_reference.set_is_changeable(false);

		//mettre  jour la multiplicit de l'attribut
		_reference.set_multiplicity(getMultiplicity(_reference_element));

		// rcuprer le type 
		_elementsToType.put(_reference, getType(_reference_element));

		// rcuprer l'associationEnd associe 
		_referencedEndTable.put(
			_reference,
			getReferencedEnd(_reference_element));

		_reference.set_container(_class);
		_class.add_contents(_reference);
	}

	// les oprations d'une classe
	public void operationTemplate(
		Element _operation_element,
		org.omg.mof.Model.Class _class)
		throws MofError {

		// cration d'un attribut par dfault
		Operation _operation =
			_operation_class.create_operation(
				"name",
				"",
				VisibilityKind.public_vis,
				ScopeKind.instance_level,
				true);

		// rcuprer les valeurs de l'attribut
		String name = getValue(_operation_element, "name");
		if (name.compareTo("") != 0)
			_operation.set_name(name);

		String annotation = getValue(_operation_element, "annotation");
		if (annotation.compareTo("") != 0)
			_operation.set_annotation(annotation);

		String visibility = getValue(_operation_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_operation.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_operation.set_visibility(VisibilityKind.private_vis);

		String scope = getValue(_operation_element, "scope");
		if (scope.endsWith("classifier_level"))
			_operation.set_scope(ScopeKind.classifier_level);

		String isQuery = getValue(_operation_element, "isQuery");
		if (isQuery.endsWith("true"))
			_operation.set_is_query(true);

		// Rcuprer les exceptions associes
		NodeList list = _operation_element.getChildNodes();
		for (int i = 0; i < list.getLength(); i++) {
			Node node = list.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element exceptionsBloc = (Element) node;
				String currentName = exceptionsBloc.getNodeName().trim();
				if (currentName.endsWith("Operation.exceptions")) {
					Vector listExceptions = new Vector();
					NodeList exceptionsList = exceptionsBloc.getChildNodes();
					for (int j = 0; j < exceptionsList.getLength(); j++) {
						if (exceptionsList.item(j).getNodeType()
							== Node.ELEMENT_NODE) {
							Element _element = (Element) exceptionsList.item(i);
							String _element_name =
								_element.getNodeName().trim();
							if (_element_name.endsWith("Exception")) {
								String ref = _element.getAttribute("xmi.idref");
								listExceptions.addElement(ref);
							}
						}
					}
					_Exception4Operations.put(_operation, listExceptions);
				}
			}
		}

		// rcuprer les paramtres 
		NodeList contents = _operation_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("Namespace.contents")) {
					NodeList namespaceContents = contentElement.getChildNodes();
					for (int j = 0; j < namespaceContents.getLength(); j++) {
						if (namespaceContents.item(j).getNodeType()
							== Node.ELEMENT_NODE) {

							Element _element =
								(Element) namespaceContents.item(j);
							String _element_name =
								_element.getNodeName().trim();

							if (_element_name.endsWith("Parameter"))
								parameterTemplate(_element, _operation);
						}
					}
				}
			}
		}
		_operation.set_container(_class);
		_class.add_contents(_operation);
	}

	//traitement des paramtres d'oprations
	public void parameterTemplate(
		Element _parameter_element,
		Namespace _container)
		throws MofError {
		// cration d'un paramtre par dfault
		Parameter _parameter =
			_parameter_class.create_parameter(
				"name",
				"",
				new MultiplicityType(1, 1, false, false),
				DirectionKind.in_dir);

		// rcuprer les valeurs du paramtre
		String name = getValue(_parameter_element, "name");
		if (name.compareTo("") != 0)
			_parameter.set_name(name);

		String annotation = getValue(_parameter_element, "annotation");
		if (annotation.compareTo("") != 0)
			_parameter.set_annotation(annotation);

		String direction = getValue(_parameter_element, "direction");
		if (direction.endsWith("out_dir"))
			_parameter.set_direction(DirectionKind.out_dir);
		else if (direction.endsWith("inout_dir"))
			_parameter.set_direction(DirectionKind.inout_dir);
		else if (direction.endsWith("return_dir"))
			_parameter.set_direction(DirectionKind.return_dir);

		//mettre  jour la multiplicit de l'attribut
		_parameter.set_multiplicity(getMultiplicity(_parameter_element));

		// rcuprer le type 
		_elementsToType.put(_parameter, getType(_parameter_element));

		_container.add_contents(_parameter);
		_parameter.set_container(_container);
	}

	//traitement des exceptions
	public void mofExceptionTemplate(
		Element _exception_element,
		Namespace _container)
		throws MofError {
		// cration d'une exception  par dfault
		MofException _exception =
			_exception_class.create_mof_exception(
				"name",
				"",
				VisibilityKind.public_vis,
				ScopeKind.instance_level);

		// rcuprer les valeurs de l'attribut
		String name = getValue(_exception_element, "name");
		if (name.compareTo("") != 0)
			_exception.set_name(name);

		String annotation = getValue(_exception_element, "annotation");
		if (annotation.compareTo("") != 0)
			_exception.set_annotation(annotation);

		String visibility = getValue(_exception_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_exception.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_exception.set_visibility(VisibilityKind.private_vis);

		String scope = getValue(_exception_element, "scope");
		if (scope.endsWith("classifier_level"))
			_exception.set_scope(ScopeKind.classifier_level);

		// rcuprer les paramtres 
		NodeList contents = _exception_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("Namespace.contents")) {
					NodeList namespaceContents = contentElement.getChildNodes();
					for (int j = 0; j < namespaceContents.getLength(); j++) {
						if (namespaceContents.item(j).getNodeType()
							== Node.ELEMENT_NODE) {

							Element _element =
								(Element) namespaceContents.item(j);
							String _element_name =
								_element.getNodeName().trim();

							if (_element_name.endsWith("Parameter"))
								parameterTemplate(_element, _exception);
						}
					}
				}
			}
		}

		// Stocker les exception afin de les lier aux oprations
		String _key = _exception_element.getAttribute("xmi.id").trim();
		_Id2exception.put(_key, _exception);

		_container.add_contents(_exception);
		_exception.set_container(_container);
	}

	//Traitement des associations
	public void associationTemplate(
		Element _association_element,
		Namespace _container)
		throws MofError {

		// cration d'une classe par dfault
		Association _association =
			_association_class.create_association(
				"name",
				"",
				VisibilityKind.public_vis,
				false,
				false,
				false,
				false);

		//Proprits de la class
		String name = getValue(_association_element, "name");
		if (name.compareTo("") != 0)
			_association.set_name(name);

		String annotation = getValue(_association_element, "annotation");
		if (annotation.compareTo("") != 0)
			_association.set_annotation(annotation);

		String visibility = getValue(_association_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_association.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_association.set_visibility(VisibilityKind.private_vis);

		String isAbstract = getValue(_association_element, "isAbstract");
		if (isAbstract.endsWith("true"))
			_association.set_is_abstract(true);

		String isLeaf = getValue(_association_element, "isLeaf");
		if (isLeaf.endsWith("true"))
			_association.set_is_leaf(true);

		String isRoot = getValue(_association_element, "isRoot");
		if (isRoot.endsWith("true"))
			_association.set_is_root(true);

		String isDerived = getValue(_association_element, "isDerived");
		if (isRoot.endsWith("true"))
			_association.set_is_root(true);

		// rcuprer les associations ends 
		//Stocker les Id des associationEnds
		String[] _ends = new String[2];
		int k = 0;
		NodeList contents = _association_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("Namespace.contents")) {
					NodeList namespaceContents = contentElement.getChildNodes();
					for (int j = 0; j < namespaceContents.getLength(); j++) {
						if (namespaceContents.item(j).getNodeType()
							== Node.ELEMENT_NODE) {
							Element element =
								(Element) namespaceContents.item(j);
							String _element_name = element.getNodeName().trim();
							if (_element_name.endsWith("AssociationEnd")) {
								associationEndTemplate(element, _association);
								_ends[k] =
									element.getAttribute("xmi.id").trim();
								k++;
							}
						}
					}
				}
			}
		}

		// container et contents
		_association.set_container(_container);
		_container.add_contents(_association);

		// rajouer les association pour otherends
		_OtherEnd.put(_ends[0], _ends[1]);
		_OtherEnd.put(_ends[1], _ends[0]);

		//rajouter l'association dans la table des tags
		String _key = _association_element.getAttribute("xmi.id").trim();
		_types.put(_key, _association);
	}

	// les associations ends d'une classe
	public void associationEndTemplate(
		Element _end_element,
		Association _association)
		throws MofError {
		// cration de l'association end par dfault
		AssociationEnd _association_end =
			_association_end_class.create_association_end(
				"name",
				"",
				true,
				AggregationKind.none,
				new MultiplicityType(0, 1, false, false),
				true);

		//Proprits de la class
		String name = getValue(_end_element, "name");
		if (name.compareTo("") != 0)
			_association_end.set_name(name);

		String annotation = getValue(_end_element, "annotation");
		if (annotation.compareTo("") != 0)
			_association_end.set_annotation(annotation);

		String aggregation = getValue(_end_element, "aggregation");
		if (aggregation.endsWith("shared"))
			_association_end.set_aggregation(AggregationKind.shared);
		else if (aggregation.endsWith("composite"))
			_association_end.set_aggregation(AggregationKind.composite);

		String isChangeable = getValue(_end_element, "isChangeable");
		if (isChangeable.endsWith("false"))
			_association_end.set_is_changeable(false);

		String isNavigable = getValue(_end_element, "isNavigable");
		if (isNavigable.endsWith("false"))
			_association_end.set_is_navigable(false);

		//mettre  jour la multiplicit
		_association_end.set_multiplicity(getMultiplicity(_end_element));

		// rcuprer le type 
		_elementsToType.put(_association_end, getType(_end_element));

		// stocker
		String _key = _end_element.getAttribute("xmi.id").trim();
		_Id2End.put(_key, _association_end);

		// contents et container
		_association_end.set_container(_association);
		_association.add_contents(_association_end);
	}

	/**
	 * Datatypes abstract processing
	 */
	public org.omg.mof.Model.DataType dataTypeAbstractTemplate(org.w3c.dom.Element _datatype_element, org.omg.mof.Model.Namespace _namespace) throws org.omg.mof.Reflective.MofError {
		// Default DataType creation
		DataType _datatype =
			_datatype_class.create_data_type(
				"name",
				"",
				VisibilityKind.public_vis,
				false,
				false,
				false,
				_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_any));

		//Proprits de la class
		String name = getValue(_datatype_element, "name");
		if (name.compareTo("") != 0)
			_datatype.set_name(name);

		String annotation = getValue(_datatype_element, "annotation");
		if (annotation.compareTo("") != 0)
			_datatype.set_annotation(annotation);

		String visibility = getValue(_datatype_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_datatype.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_datatype.set_visibility(VisibilityKind.private_vis);

		String isAbstract = getValue(_datatype_element, "isAbstract");
		if (isAbstract.endsWith("true"))
			_datatype.set_is_abstract(true);

		String isLeaf = getValue(_datatype_element, "isLeaf");
		if (isLeaf.endsWith("true"))
			_datatype.set_is_leaf(true);

		String isRoot = getValue(_datatype_element, "isRoot");
		if (isRoot.endsWith("true"))
			_datatype.set_is_root(true);
			
		// rcuprer le TypeCode 
		NodeList contents = _datatype_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("DataType.typeCode")) {
					Element typecode = null;
					NodeList listCorbaTypecode =
						contentElement.getElementsByTagName(
							"XMI.CorbaTypeCode");
					if (listCorbaTypecode.getLength() != 0) {
						Element corbaTypecode =
							(Element) listCorbaTypecode.item(0);
						NodeList childrens = corbaTypecode.getChildNodes();
						int j = 0;
						boolean isTypecode = false;
						while (j < childrens.getLength() && !isTypecode) {
							Node children = childrens.item(j);
							if (children.getNodeType() == Node.ELEMENT_NODE) {
								typecode = (Element) children;
								isTypecode = true;
							}
							j++;
						}

						//calcul du Repository ID
						String RepositoryID = getValue(typecode, "xmi.tcId");
						if (RepositoryID == null)
							RepositoryID =
								"IDL:" + idlHelper.format1(_namespace.name());

						// calcul de _type code
						TypeCode _type_code =
							xml2TypeCode(typecode, RepositoryID);
						_datatype.set_type_code(_type_code);
					}
				}
			}
		}

		// Container and Contents
		_datatype.set_container(_namespace);
		_namespace.add_contents(_datatype);

		// Store the Class into Types tables and eventually for tags
		String _key = _datatype_element.getAttribute("xmi.id").trim();
		_types.put(_key, _datatype);
		_Id2element.put(_key, _datatype);

		return _datatype;
	}

	/**
	 * Datatypes processing
	 */
	public void dataTypeTemplate(org.w3c.dom.Element _datatype_element, org.omg.mof.Model.Namespace _namespace) throws org.omg.mof.Reflective.MofError {
		org.omg.mof.Model.DataType _datatype = dataTypeAbstractTemplate(_datatype_element, _namespace);
		_dataTypes.put(_datatype_element.getAttribute("xmi.id"), _datatype.type_code());
	}

	/**
	 * PrimitiveType processing
	 */
	public void primitiveTypeTemplate(org.w3c.dom.Element _datatype_element, org.omg.mof.Model.Namespace _namespace) throws org.omg.mof.Reflective.MofError {
		if (_datatype_element.hasAttribute("xmi.idref"))
			return;
		org.omg.mof.Model.DataType _datatype = dataTypeAbstractTemplate(_datatype_element, _namespace);
		org.omg.mof.Model.DataType dt = (org.omg.mof.Model.DataType) _types.get(_datatype.name());
		if (dt != null) {
			org.omg.CORBA.TypeCode typeCode = dt.type_code();
			_datatype.set_type_code(typeCode);
			_dataTypes.put(_datatype_element.getAttribute("xmi.id"), typeCode);
		}
	}
	
	

	public void typeElementTypeTemplate (org.w3c.dom.Element _element, org.omg.mof.Model.Namespace _namespace)
		throws org.omg.mof.Reflective.MofError {
		NodeList namespaceContents = _element.getChildNodes();
		for (int j = 0; j < namespaceContents.getLength(); j++) {
			if (namespaceContents.item(j).getNodeType() == Node.ELEMENT_NODE) {
				Element subElement = (Element) namespaceContents.item(j);
				String _sub_element_name = subElement.getNodeName().trim();
				if (_sub_element_name.endsWith("Association"))
					associationTemplate(subElement, _namespace);
				else if (_sub_element_name.endsWith("StructureType"))
					structureTypeTemplate(subElement, _namespace);
				else if (_sub_element_name.endsWith("CollectionType"))
					collectionTypeTemplate(subElement, _namespace);
				else if (_sub_element_name.endsWith("AliasType"))
					aliasTypeTemplate(subElement, _namespace);
				else if (_sub_element_name.endsWith("EnumerationType"))
					enumerationTypeTemplate(subElement, _namespace);
				else if (_sub_element_name.endsWith("PrimitiveType"))
					primitiveTypeTemplate(subElement, _namespace);
				else if (_sub_element_name.endsWith("DataType"))
					dataTypeTemplate(subElement, _namespace);
			}
		}		
	}
	

	/**
	 * AliasType processing
	 */
	public void aliasTypeTemplate(org.w3c.dom.Element _datatype_element, org.omg.mof.Model.Namespace _namespace) throws org.omg.mof.Reflective.MofError {
		if (_datatype_element.hasAttribute("xmi.idref"))
			return;
		org.omg.mof.Model.DataType _datatype = dataTypeAbstractTemplate(_datatype_element, _namespace);
		String repID = "IDL:" + this.idlHelper.format1(_namespace.name()) + "/" + _datatype.name() + ":1.0";
		org.w3c.dom.NodeList _contents = _datatype_element.getChildNodes();
		for (int i = 0; i < _contents.getLength(); i++) {
			org.w3c.dom.Element _element = (Element) _contents.item(i);
			String _element_name = _element.getNodeName().trim();
			if (_element_name.endsWith("TypedElement.type")) {
				String idref = new String();
				if (_element.hasAttribute("xmi.idref")) {
					idref = _element.getAttribute("xmi.idref");
				} else {
					typeElementTypeTemplate(_element, _namespace);
					if (_element.getFirstChild().getNodeType() == Node.ELEMENT_NODE) {
						idref = ((org.w3c.dom.Element)_element.getFirstChild()).getAttribute("xmi.id");
						if (idref == null || idref.length() == 0)
							idref = ((org.w3c.dom.Element)_element.getFirstChild()).getAttribute("xmi.idref");
					}
				}
				org.omg.CORBA.TypeCode tc = (org.omg.CORBA.TypeCode) _dataTypes.get(idref);
				if (tc != null) {
					org.omg.CORBA.TypeCode _typecode = _orb.create_alias_tc(repID, _datatype.name(), tc);
					_datatype.set_type_code(_typecode);
					_dataTypes.put(_datatype_element.getAttribute("xmi.id"), _typecode);
				}
			}
		}
	}

	/**
	 * CollectionType processing
	 */
	public void collectionTypeTemplate(org.w3c.dom.Element _datatype_element, org.omg.mof.Model.Namespace _namespace) throws org.omg.mof.Reflective.MofError {
		if (_datatype_element.hasAttribute("xmi.idref"))
			return;
		org.omg.mof.Model.DataType _datatype = dataTypeAbstractTemplate(_datatype_element, _namespace);
		String repID = "IDL:" + this.idlHelper.format1(_namespace.name()) + "/" + _datatype.name() + ":1.0";
		org.w3c.dom.NodeList _contents = _datatype_element.getChildNodes();
		for (int i = 0; i < _contents.getLength(); i++) {
			org.w3c.dom.Element _element = (Element) _contents.item(i);
			String _element_name = _element.getNodeName().trim();
			if (_element_name.endsWith("TypedElement.type")) {
				String idref = new String();
				if (_element.hasAttribute("xmi.idref")) {
					idref = _element.getAttribute("xmi.idref");
				} else {
					typeElementTypeTemplate(_element, _namespace);
					if (_element.getFirstChild().getNodeType() == Node.ELEMENT_NODE) {
						idref = ((org.w3c.dom.Element)_element.getFirstChild()).getAttribute("xmi.id");
						if (idref == null || idref.length() == 0)
							idref = ((org.w3c.dom.Element)_element.getFirstChild()).getAttribute("xmi.idref");
					}
				}
				org.omg.CORBA.TypeCode tc = (org.omg.CORBA.TypeCode) _dataTypes.get(idref);
				if (tc != null) {
					org.omg.CORBA.TypeCode _typecode = _orb.create_sequence_tc(0, tc);
					_datatype.set_type_code(_typecode);
					_dataTypes.put(_datatype_element.getAttribute("xmi.id"), _typecode);
				}
			}
		}
	}

	/**
	 * EnumerationType processing
	 */
	public void enumerationTypeTemplate(org.w3c.dom.Element _datatype_element, org.omg.mof.Model.Namespace _namespace) throws org.omg.mof.Reflective.MofError {
		if (_datatype_element.hasAttribute("xmi.idref"))
			return;
		org.omg.mof.Model.DataType _datatype = dataTypeAbstractTemplate(_datatype_element, _namespace);

		Vector _members = new Vector();
		org.w3c.dom.NodeList _labels = _datatype_element.getChildNodes();
		for (int i = 0; i < _labels.getLength(); i++) {
			org.w3c.dom.Element _element = (Element) _labels.item(i);
			String _element_name = _element.getNodeName().trim();
			if (_element_name.endsWith("EnumerationType.labels")) {
				_members.addElement(_element.getFirstChild().getNodeValue().trim());
			}
		}
		String[] _enum_members = new String[_members.size()];
		for (int i = 0; i < _members.size(); i++)
			_enum_members[i] = (String) _members.elementAt(i);
		String repID = "IDL:" + this.idlHelper.format1(_namespace.name()) + "/" + _datatype.name() + ":1.0";
		org.omg.CORBA.TypeCode typeCode = _orb.create_enum_tc(repID, _datatype.name(), _enum_members);
		_datatype.set_type_code(typeCode);
		_dataTypes.put(_datatype_element.getAttribute("xmi.id"), typeCode);
	}

	/**
	 * StructureType processing
	 */
	public void structureTypeTemplate(org.w3c.dom.Element _datatype_element, org.omg.mof.Model.Namespace _namespace) throws org.omg.mof.Reflective.MofError {
		if (_datatype_element.hasAttribute("xmi.idref"))
			return;
		org.omg.mof.Model.DataType _datatype = dataTypeAbstractTemplate(_datatype_element, _namespace);
		org.w3c.dom.NodeList _child = _datatype_element.getChildNodes();
		for (int i = 0; i < _child.getLength(); i++) {
			org.w3c.dom.Element _current = (Element) _child.item(i);
			readStructureTypeContents(_namespace, _datatype, _current);
		}
	}

	public void readStructureFieldType(org.omg.CORBA.StructMember _member, org.w3c.dom.Element _type) throws org.omg.mof.Reflective.MofError {
		org.w3c.dom.Element classifier = (Element) _type.getFirstChild();
		org.omg.CORBA.TypeCode _field_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_any);
		if (classifier.hasAttribute("xmi.idref")) {
			// Link to the reference
			if (_dataTypes.containsKey(classifier.getAttribute("xmi.idref"))) {
				_field_typecode = (org.omg.CORBA.TypeCode) _dataTypes.get(classifier.getAttribute("xmi.idref"));
			}
			if (_types.containsKey(classifier.getAttribute("xmi.idref"))) {
				org.omg.mof.Model.DataType type = (org.omg.mof.Model.DataType) _types.get(classifier.getAttribute("xmi.idref"));
				_field_typecode = type.type_code();
			}
		} else if (classifier.hasAttribute("href")) {
			// Import from the specified file
		}
		_member.type = _field_typecode;
	}

	public void readStructureTypeContents(org.omg.mof.Model.Namespace _namespace, org.omg.mof.Model.DataType _datatype, Element element) throws org.omg.mof.Reflective.MofError {
		String repID = "IDL:" + idlHelper.format1(_namespace.name()) + "/" + _datatype.name();
		Vector _struct_members = new Vector();
		org.w3c.dom.NodeList _fields = element.getChildNodes();
		for (int i = 0; i < _fields.getLength(); i++) {
			org.w3c.dom.Element _element = (Element) _fields.item(i);
			String _element_name = _element.getNodeName().trim();
			if (_element_name.endsWith("StructureField")) {
				org.omg.CORBA.StructMember _member = new org.omg.CORBA.StructMember();
				_member.name = _element.getAttribute("name").trim();
				org.w3c.dom.NodeList _types = _element.getChildNodes();
				for (int j = 0; j < _types.getLength(); j++) {
					org.w3c.dom.Element _type = (Element) _types.item(j);
					String _type_name = _type.getNodeName().trim();
					if (_type_name.endsWith("TypedElement.type")) {
						readStructureFieldType(_member, _type);
					}
				}
				_struct_members.addElement(_member);
			}
		}
		org.omg.CORBA.StructMember[] _members = new org.omg.CORBA.StructMember[_struct_members.size()];
		for (int j = 0; j < _struct_members.size(); j++)
			_members[j] = (org.omg.CORBA.StructMember) _struct_members.elementAt(j);
		_datatype.set_type_code(_orb.create_struct_tc(repID + ":1.0", _datatype.name(), _members));
		_dataTypes.put(element.getAttribute("xmi.id"), _datatype.type_code());
	}


	//Traitement des constants
	public void constantTemplate(
		Element _constant_element,
		Namespace _namespace)
		throws MofError {

		// cration d'un constant par dfault
		Constant _constant =
			_constant_class.create_constant("name", "", _orb.create_any());
		//la valeur 
		String _id = _constant_element.getAttribute("xmi.id").trim();

		// rcuprer les proprits
		String name = getValue(_constant_element, "name");
		if (name.compareTo("") != 0)
			_constant.set_name(name);

		String annotation = getValue(_constant_element, "annotation");
		if (annotation.compareTo("") != 0)
			_constant.set_annotation(annotation);

		// rcuprer le type 
		_elementsToType.put(_constant, getType(_constant_element));

		// rcuprer le type 
		NodeList contents = _constant_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("Constant.value")) {
					NodeList elements = contentElement.getChildNodes();
					for (int j = 0; j < elements.getLength(); j++) {
						if (elements.item(j).getNodeType()
							== Node.ELEMENT_NODE) {
							Element element = (Element) elements.item(j);
							String _element_name = element.getNodeName().trim();
							if (_element_name.endsWith("XMI.any")) {
								String value =
									element.getFirstChild().getNodeValue();
								_constantValue.put(_constant, value);
							}
						}
					}
				}
			}
		}
		_constant.set_container(_namespace);
		_namespace.add_contents(_constant);
	}

	// traitement du Tag
	public void tagTemplate(Element _tag_element, Namespace _namespace)
		throws MofError {

		// cration d'une classe par dfault
		org.omg.CORBA.Any[] _any_t = new org.omg.CORBA.Any[1];
		org.omg.CORBA.Any _mof_tag = _orb.create_any();
		_mof_tag.insert_string("");
		_any_t[0] = _mof_tag;
		Tag _tag = _tag_class.create_tag("name", "", "", _any_t);

		// rcuprer les proprits
		String name = getValue(_tag_element, "name");
		if (name.compareTo("") != 0)
			_tag.set_name(name);

		String annotation = getValue(_tag_element, "annotation");
		if (annotation.compareTo("") != 0)
			_tag.set_annotation(annotation);

		String tagId = getValue(_tag_element, "tagId");
		_tag.set_tag_id(tagId);

		//recover values : XMI version 1.1
		String taggedeElements = _tag_element.getAttribute("elements");
		if (taggedeElements.compareTo("") != 0) {
			Vector _tagged_elements = new Vector();
			StringTokenizer supRefs =
				new StringTokenizer(taggedeElements, " 	");
			while (supRefs.hasMoreTokens()) {
				String part = supRefs.nextToken();
				_tagged_elements.addElement(part);
			}
			_tags.put(_tag, _tagged_elements);
		}

		// rcuprer les valeurs et les lments asscocis
		NodeList children = _tag_element.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Element currentElement = (Element) children.item(i);
			String childrenName = currentElement.getNodeName().trim();
			if (childrenName.endsWith("Tag.values")) {
				NodeList anylist =
					currentElement.getElementsByTagName("XMI.any");
				if (anylist.getLength() != 0) {
					Element anyElement = (Element) anylist.item(0);
					String type = anyElement.getAttribute("xmi.type").trim();
					if (type.endsWith("string")) {
						org.omg.CORBA.Any[] any_tc = new org.omg.CORBA.Any[1];
						org.omg.CORBA.Any mof_tag_c = _orb.create_any();

						NodeList temp = anyElement.getChildNodes();
						for (int j = 0; j < temp.getLength(); j++) {
							Node current = temp.item(j);
							if (current.getNodeType() == Node.TEXT_NODE) {
								String tagValue = current.getNodeValue().trim();
								mof_tag_c.insert_string(tagValue);
								any_tc[0] = mof_tag_c;
								_tag.set_values(any_tc);
							}
						}
					}
				}
			} else
				//recover values : XMI version 1.O
				if (childrenName.endsWith("elements")) {
					Vector _tagged_elements = new Vector();
					NodeList elements = currentElement.getChildNodes();
					for (int j = 0; j < elements.getLength(); j++) {
						Node node = elements.item(j);
						if (node.getNodeType() == Node.ELEMENT_NODE) {
							Element contentElement = (Element) node;
							String currentName =
								contentElement.getNodeName().trim();
							if (currentName.endsWith("ModelElement")) {
								String _key_element =
									contentElement
										.getAttribute("xmi.idref")
										.trim();
								_tagged_elements.addElement(_key_element);

							}
						}
					}
					_tags.put(_tag, _tagged_elements);
				}
		}

		_tag.set_container(_namespace);
		_namespace.add_contents(_tag);
	}

	/**
	 * Method importPackage.
	 * @param _element
	 */
	public void importTemplate(Element _import_element, Namespace _container)
		throws MofError {

		//crer un objet import par dfaut 
		org.omg.mof.Model.Import _import =
			_import_class.create_import(
				"name",
				"",
				VisibilityKind.public_vis,
				false);

		// rcuprer l'identificateur
		String _key = _import_element.getAttribute("xmi.id").trim();

		//Proprits de la class
		String name = getValue(_import_element, "name");
		if (name.compareTo("") != 0)
			_import.set_name(name);

		String annotation = getValue(_import_element, "annotation");
		if (annotation.compareTo("") != 0)
			_import.set_annotation(annotation);

		String visibility = getValue(_import_element, "visibility");
		if (visibility.endsWith("protected_vis"))
			_import.set_visibility(VisibilityKind.protected_vis);
		else if (visibility.endsWith("private_vis"))
			_import.set_visibility(VisibilityKind.private_vis);

		String isClustered = getValue(_import_element, "isClustered");
		if (isClustered.endsWith("true"))
			_import.set_is_clustered(true);

		//to recover an imported namesapce : XMI version 1.1
		String refImported = _import_element.getAttribute("importedNamespace");
		if (refImported.compareTo("") != 0)
			_Import2Imported.put(_import, refImported);

		//to recover an imported namesapce : XMI version 1.0
		NodeList contents = _import_element.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("Import.importedNamespace")) {
					NodeList elements = contentElement.getChildNodes();
					for (int j = 0; j < elements.getLength(); j++) {
						if (elements.item(j).getNodeType()
							== Node.ELEMENT_NODE) {
							Element element = (Element) elements.item(j);
							String _element_name = element.getNodeName().trim();
							if (_element_name.endsWith("Namespace")) {

								String ref = element.getAttribute("xmi.idref");
								//mettre  jour les packages imports
								_Import2Imported.put(_import, ref);
							}
						}
					}
				}
			}
		}

		// container et contents
		_import.set_container(_container);
		_container.add_contents(_import);

	}

	// passer de l'XML au Type code CORBA
	public org.omg.CORBA.TypeCode xml2TypeCode(
		Element _element,
		String RepositoryID)
		throws org.omg.CORBA.BAD_TYPECODE {
		String _element_name = _element.getNodeName().trim();
		org.omg.CORBA.TypeCode _typecode = null;
		if (_element_name.startsWith("XMI.CorbaTcBoolean"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_boolean);
		else if (_element_name.startsWith("XMI.CorbaTcOctet"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_octet);
		else if (_element_name.startsWith("XMI.CorbaTcChar"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char);
		else if (_element_name.startsWith("XMI.CorbaTcWchar"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_wchar);
		else if (_element_name.startsWith("XMI.CorbaTcShort"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
		else if (_element_name.startsWith("XMI.CorbaTcUshort"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort);
		else if (_element_name.startsWith("XMI.CorbaTcLong"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
		else if (_element_name.startsWith("XMI.CorbaTcUlong"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulong);
		else if (_element_name.startsWith("XMI.CorbaTcLongLong"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_longlong);
		else if (_element_name.startsWith("XMI.CorbaTcULongLong"))
			_typecode =
				_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulonglong);
		else if (_element_name.startsWith("XMI.CorbaTcDouble"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_double);
		else if (_element_name.startsWith("XMI.CorbaTcFloat"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
		else if (_element_name.startsWith("XMI.CorbaTcTypeCode"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_TypeCode);
		else if (_element_name.startsWith("XMI.CorbaTcAny"))
			_typecode = _orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_any);
		else if (_element_name.startsWith("XMI.CorbaTcString")) {
			String _bound_value = _element.getAttribute("xmi.tcLength").trim();
			if (_bound_value.compareTo("") == 0)
				_typecode =
					_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
			else {
				int _bound = Integer.parseInt(_bound_value);
				_typecode = _orb.create_string_tc(_bound);
			}
		} else if (_element_name.startsWith("XMI.CorbaTcWstring")) {
			String _bound_value = _element.getAttribute("xmi.tcLength").trim();
			if (_bound_value.compareTo("") != 0)
				_typecode =
					_orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
			else {
				int _bound = Integer.parseInt(_bound_value);
				_typecode = _orb.create_string_tc(_bound);
			}
		} else if (_element_name.startsWith("XMI.CorbaTcStruct")) {

			// le nom de la structure
			String _name = _element.getAttribute("xmi.tcName").trim();
			String RepID = RepositoryID + "/" + _name + ":1.0";

			// ruprer les membres de la structure
			NodeList _fields = _element.getChildNodes();
			Vector _struct_members = new Vector();
			for (int i = 0; i < _fields.getLength(); i++) {
				Element _field = (Element) _fields.item(i);
				String _tag = _field.getNodeName();
				if (_tag.endsWith("CorbaTcField")) {
					Element _type_field =
						(Element) _field.getElementsByTagName(
							"XMI.CorbaTypeCode").item(
							0);
					//  construire les membres de la structure
					org.omg.CORBA.StructMember _member =
						new org.omg.CORBA.StructMember();
					// nom
					String _field_name =
						_field.getAttribute("xmi.tcName").trim();
					_member.name = _field_name;
					// le type
					Element _type_element =
						(Element) _field.getElementsByTagName(
							"XMI.CorbaTypeCode").item(
							0);
					Element _typecode_element =
						(Element) _type_element.getFirstChild();
					org.omg.CORBA.TypeCode _field_typecode =
						xml2TypeCode(
							_typecode_element,
							RepID.substring(0, RepID.lastIndexOf(":")));
					_member.type = _field_typecode;
					//
					_struct_members.addElement(_member);
				}
			} //
			org.omg.CORBA.StructMember[] _members =
				new org.omg.CORBA.StructMember[_struct_members.size()];
			for (int j = 0; j < _struct_members.size(); j++)
				_members[j] =
					(org.omg.CORBA.StructMember) _struct_members.elementAt(j);
			_typecode = _orb.create_struct_tc(RepID, _name, _members);
		} else if (_element_name.startsWith("XMI.CorbaTcSequence")) {
			//longueur de la sequence
			String _bound_value = _element.getAttribute("xmi.tcLength").trim();
			int _bound = 0;
			if (_bound_value.compareTo("") != 0)
				_bound = Integer.parseInt(_bound_value);
			//recuprer le type de la sequence
			Element _type_code =
				(Element) _element.getElementsByTagName(
					"XMI.CorbaTypeCode").item(
					0);
			Element _sequence_element = (Element) _type_code.getFirstChild();
			// rcuprer le type code de la squence
			org.omg.CORBA.TypeCode _sequence_type =
				xml2TypeCode(_sequence_element, "");
			_typecode = _orb.create_sequence_tc(_bound, _sequence_type);
		} else if (_element_name.startsWith("XMI.CorbaTcEnum")) {

			// le nom de l'numration
			String _name = _element.getAttribute("xmi.tcName").trim();
			String RepID = RepositoryID + "/" + _name + ":1.0";
			// calculer les membres de l'numration
			Vector members = new Vector();
			NodeList enumLabel =
				_element.getElementsByTagName("XMI.CorbaTcEnumLabel");
			for (int j = 0; j < enumLabel.getLength(); j++) {
				Element memeber = (Element) enumLabel.item(j);
				String value = memeber.getAttribute("xmi.tcName").trim();
				members.addElement(value);
			}
			String[] enumMembers = new String[members.size()];
			for (int i = 0; i < members.size(); i++)
				enumMembers[i] = (String) members.elementAt(i);
			_typecode = _orb.create_enum_tc(RepID, _name, enumMembers);
		} else if (_element_name.startsWith("XMI.CorbaTcAlias")) {
			// le nom de l'numration
			String _name = _element.getAttribute("xmi.tcName").trim();
			String RepID = RepositoryID + "/" + _name + ":1.0";
			//recuprer le type de l'alias
			Element _type_code =
				(Element) _element.getElementsByTagName(
					"XMI.CorbaTypeCode").item(
					0);
			Element _alias_element = (Element) _type_code.getFirstChild();
			// rcuprer le type code d'origine
			org.omg.CORBA.TypeCode _original_type =
				xml2TypeCode(
					_alias_element,
					RepID.substring(0, RepID.lastIndexOf(":")));
			_typecode = _orb.create_alias_tc(RepID, _name, _original_type);
		}
		return _typecode;
	}

	// typer les paramtres et les attributs et les constant
	public void typing() throws BadKind, MofError {
		Enumeration _keys = _elementsToType.keys();
		while (_keys.hasMoreElements()) {
			TypedElement _typed_element = (TypedElement) _keys.nextElement();

			// trouver la cl dans la deuxime table
			String _key = ((String) _elementsToType.get(_typed_element)).trim();

			// trouve le classifier
			Classifier _classifier = (Classifier) _types.get(_key);

			// typer
			_typed_element.set_type(_classifier);

			// affecter la valeur des constant
			if (_typed_element._is_a(ConstantHelper.id())) {
				Constant _constant = ConstantHelper.narrow(_typed_element);
				String _value = (String) _constantValue.get(_constant);
				putConstantValue(_constant, _value);

			}
		}
	}

	//mettre les valeurs des constants
	public void putConstantValue(Constant _constant, String _value)
		throws BadKind, MofError {

		Classifier _classifier = _constant.type();

		if (_classifier._is_a(DataTypeHelper.id())) {
			DataType _datatype = DataTypeHelper.narrow(_classifier);
			org.omg.CORBA.TypeCode _typecode = _datatype.type_code();
			org.omg.CORBA.Any _any = _orb.create_any();
			switch (_typecode.kind().value()) {
				case org.omg.CORBA.TCKind._tk_boolean :
					_any.insert_boolean(Boolean.valueOf(_value).booleanValue());
					break;
				case org.omg.CORBA.TCKind._tk_octet :

					_any.insert_octet(Byte.valueOf(_value).byteValue());
					break;
				case org.omg.CORBA.TCKind._tk_char :
					_any.insert_char(_value.charAt(0));
					break;
				case org.omg.CORBA.TCKind._tk_wchar :
					_any.insert_wchar(_value.charAt(0));
					break;

				case org.omg.CORBA.TCKind._tk_short :
					_any.insert_short(Short.valueOf(_value).shortValue());
					break;

				case org.omg.CORBA.TCKind._tk_ushort :
					_any.insert_ushort(Short.valueOf(_value).shortValue());
					break;

				case org.omg.CORBA.TCKind._tk_long :
					_any.insert_long(Integer.valueOf(_value).intValue());
					break;

				case org.omg.CORBA.TCKind._tk_ulong :
					_any.insert_ulong(Integer.valueOf(_value).intValue());
					break;

				case org.omg.CORBA.TCKind._tk_longlong :
					_any.insert_longlong(Long.valueOf(_value).longValue());
					break;

				case org.omg.CORBA.TCKind._tk_ulonglong :
					_any.insert_ulonglong(Long.valueOf(_value).longValue());
					break;

				case org.omg.CORBA.TCKind._tk_double :
					_any.insert_double(Double.valueOf(_value).doubleValue());
					break;

				case org.omg.CORBA.TCKind._tk_float :
					_any.insert_float(Float.valueOf(_value).floatValue());
					break;

				case org.omg.CORBA.TCKind._tk_string :
					_any.insert_string(_value.trim());
					break;

				case org.omg.CORBA.TCKind._tk_wstring :
					_any.insert_wstring(_value.trim());
					break;

				default :
					throw new BadKind();
			}
			_constant.set_value(_any);
		}
	}

	/**
	 * Mettre les supertypes  la fin
	 */
	private void putSupertypes() throws MofError {
		Enumeration _keys = _elementsToSupertype.keys();
		while (_keys.hasMoreElements()) {
			org.omg.mof.Model.Class _class =
				(org.omg.mof.Model.Class) _keys.nextElement();
			// trouver la cl dans la deuxime table
			Vector _supertypes = ((Vector) _elementsToSupertype.get(_class));
			for (int i = 0; i < _supertypes.size(); i++) {
				String _key = ((String) _supertypes.elementAt(i)).trim();
				org.omg.mof.Model.Class _super =
					(org.omg.mof.Model.Class) _types.get(_key);
				_class.add_supertypes(_super);
			}
		}
	}

	/**
	*affecter les tags aux lments
	* */
	public void taggedElement() throws MofError {
		Enumeration _keys = _tags.keys();
		while (_keys.hasMoreElements()) {
			Tag _tag = (Tag) _keys.nextElement();
			// trouver la cl dans la deuxime table
			Vector _tagged_elements = (Vector) _tags.get(_tag);
			for (int i = 0; i < _tagged_elements.size(); i++) {
				String _key = ((String) _tagged_elements.elementAt(i)).trim();
				ModelElement _element = (ModelElement) _Id2element.get(_key);
				_tag.add_elements(_element);
			}
		}
	}

	/**
	 * typage des rfrence et les lier aux associationsEnds corrrespondantes
	 */
	public void associationEnds4Reference() throws MofError {
		Enumeration _keys = _referencedEndTable.keys();

		while (_keys.hasMoreElements()) {
			Reference _reference = (Reference) _keys.nextElement();
			String _refend_id = (String) _referencedEndTable.get(_reference);
			AssociationEnd _ref_end = (AssociationEnd) _Id2End.get(_refend_id);
			_reference.set_type(_ref_end.type());
			_reference.set_referenced_end(_ref_end);

			// Mettre exposedEnd
			String _expend_id = (String) _OtherEnd.get(_refend_id);
			AssociationEnd _exp_end = (AssociationEnd) _Id2End.get(_expend_id);
			_reference.set_exposed_end(_exp_end);
		}
	}

	/**
	 * suprimer les noeuds qui contiennent du text vide
	 */
	public void delSpaces(Node _node)
		throws
			SAXException,
			ParserConfigurationException,
			FileNotFoundException,
			IOException {

		NodeList _fils = _node.getChildNodes();
		int i = 0;
		while (i < _fils.getLength()) {
			Node _temp = _fils.item(i);
			if (_temp.getNodeType() == 3) {
				String _temp1 = _temp.getNodeValue();
				if (_temp1.trim().compareTo("") == 0)
					_node.removeChild(_temp);
				else
					i++;
			} else {
				delSpaces(_temp);
				i++;
			}
		}
	}

	/**
	 * Affecter les exceptions pour les oprations
	 */
	public void exceptions4Operations() throws MofError {
		Enumeration _keys = _Exception4Operations.keys();
		while (_keys.hasMoreElements()) {
			Operation _operation = (Operation) _keys.nextElement();
			// trouver la cl dans la deuxime table
			Vector _exceptions = (Vector) _Exception4Operations.get(_operation);
			for (int i = 0; i < _exceptions.size(); i++) {
				String _idException =
					((String) _exceptions.elementAt(i)).trim();
				MofException _exception =
					(MofException) _Id2exception.get(_idException);
				_operation.add_exceptions(_exception);
				//_exception.set_scope(_operation.scope());

			}
		}
	}

	/**
	 * Method importElements.
	 */
	private void importElements() throws MofError {
		for (Enumeration keys = _Import2Imported.keys();
			keys.hasMoreElements();
			) {
			org.omg.mof.Model.Import importElement =
				(org.omg.mof.Model.Import) keys.nextElement();

			//rcuprer les lments imports
			String refOfImported = (String) _Import2Imported.get(importElement);

			//rcuprer le package importateur 
			Namespace _namespace = (Namespace) _Id2element.get(refOfImported);

			//mettre  jour la rfrence
			importElement.set_imported_namespace(_namespace);
		}
	}

	/**
	 * Sets the idlHelper.
	 * @param idlHelper The idlHelper to set
	 */
	public void setIdlHelper(IDLCommon idlHelper) {
		this.idlHelper = idlHelper;
	}

	/**
	 * Get value of a property of an element even if it is encoded as an attribute or as a child element
	 * Start by attribute
	 */
	public String getValue(Element elt, String propertyName) {
		String res = "";
		//XMI 1.1
		res = elt.getAttribute(propertyName);

		//XMI 1.0
		if ((res == null) || (res.compareTo("") == 0)) {
			NodeList children = elt.getChildNodes();
			for (int i = 0; i < children.getLength(); i++) {
				Node current = children.item(i);
				if (current.getNodeType() == Node.ELEMENT_NODE) {
					Element currentElement = (Element) current;
					if (currentElement
						.getNodeName()
						.toLowerCase()
						.endsWith(propertyName.toLowerCase())) {
						res = currentElement.getAttribute("xmi.value");
						if ((res == null) || (res.compareTo("") == 0)) {
							Node temp = currentElement.getFirstChild();
							if (temp != null)
								res = temp.getNodeValue().trim();
						}
					}
				}
			}
		}

		return res;
	}

	/**
	 * Get value of type
	 */
	public String getType(Element elt) {

		//XMI 1.1
		String type = elt.getAttribute("type");
		if (type.compareTo("") != 0)
			return type;
		//XMI 1.0
		NodeList contents = elt.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("TypedElement.type")) {

					NodeList children = contentElement.getChildNodes();

					for (int j = 0; j < children.getLength(); j++) {
						Node current = children.item(j);
						if (current.getNodeType() == Node.ELEMENT_NODE) {
							Element currentElement = (Element) current;
							String classifierType =
								currentElement.getNodeName().trim();

							if (classifierType.endsWith("Classifier")
								|| classifierType.endsWith("Class")
								|| classifierType.endsWith("Association")
								|| classifierType.endsWith("DataType"))
								type =
									currentElement
										.getAttribute("xmi.idref")
										.trim();

						}
					}
				}
			}
		}

		return type;
	}

	/**
	 * Get value of referencedEnd for Reference
	 */
	public String getReferencedEnd(Element elt) {

		//XMI 1.1
		String ref = elt.getAttribute("referencedEnd").trim();
		if (ref.compareTo("") != 0)
			return ref;

		//XMI 1.0
		NodeList contents = elt.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("Reference.referencedEnd")) {

					NodeList children = contentElement.getChildNodes();

					for (int j = 0; j < children.getLength(); j++) {
						Node current = children.item(j);
						if (current.getNodeType() == Node.ELEMENT_NODE) {
							Element currentElement = (Element) current;
							String referencedEndName =
								currentElement.getNodeName().trim();

							if (referencedEndName.endsWith("AssociationEnd"))
								ref =
									currentElement
										.getAttribute("xmi.idref")
										.trim();
						}
					}
				}
			}
		}

		return ref;
	}

	/**
	 * Get value of a Multiplicity
	 */
	public MultiplicityType getMultiplicity(Element elt) {

		NodeList contents = elt.getChildNodes();
		for (int i = 0; i < contents.getLength(); i++) {
			Node node = contents.item(i);
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				Element contentElement = (Element) node;
				String currentName = contentElement.getNodeName().trim();
				if (currentName.endsWith("multiplicity")) {
					NodeList list = contentElement.getChildNodes();
					for (int j = 0; j < list.getLength(); j++) {
						if (list.item(j).getNodeType() == Node.ELEMENT_NODE) {
							Element element = (Element) list.item(j);
							String _element_name = element.getNodeName().trim();

							//XMI version 1.1
							if (_element_name.endsWith("MultiplicityType"))
								return new MultiplicityType(
									Integer.parseInt(
										element.getAttribute("lower").trim()),
									Integer.parseInt(
										element.getAttribute("upper").trim()),
									Boolean.getBoolean(
										element
											.getAttribute("is_ordered")
											.trim()),
									Boolean.getBoolean(
										element
											.getAttribute("is_unique")
											.trim()));

							//XMI 1.0
							boolean is_unique = false, is_ordered = false;
							int lower = 0, upper = -1;
							NodeList properties =
								contentElement.getChildNodes();
							for (int k = 0; k < properties.getLength(); k++) {
								if (properties.item(k).getNodeType()
									== Node.ELEMENT_NODE) {
									Element item = (Element) properties.item(k);
									String value =
										item
											.getFirstChild()
											.getNodeValue()
											.trim();
									if (k == 0) {
										lower = Integer.parseInt(value);
									} else if (k == 1) {
										upper = Integer.parseInt(value);
									} else if (k == 2) {
										if (value.endsWith("true"))
											is_ordered = true;
									} else if (k == 3) {
										if (value.endsWith("true"))
											is_unique = true;
									}
								}
							}
							return new MultiplicityType(
								lower,
								upper,
								is_ordered,
								is_unique);
						}
					}
				}
			}
		}
		return null;
	}

	/**
	 * Sets the logger.
	 * @param logger The logger to set
	 */
	public void setLogger(ModFactLogger logger) {
		this.logger = logger;
	}

}