/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.provider;

import org.objectweb.modfact.corba.helper.CORBACommon;

/**
 * @author Pierre Carpentier
 *
 */
public class CORBAProvider implements CORBACommon {

	/**
	 * Reads an IOR from a File.
	 * @param orb The ORB.
	 * @param filename The name of the file which contains the IOR.
	 * @return The CORBA Object corresponding to the IOR read. 
	 */
	public org.omg.CORBA.Object readIOR(org.omg.CORBA.ORB orb, String filename) {
		try {
			System.err.println("Reading stringified IOR from file " + filename);
			java.io.BufferedReader input = new java.io.BufferedReader(new java.io.FileReader(filename));
			String line = input.readLine();
			return orb.string_to_object(line);
		} catch (java.io.IOException ex) {
			System.err.println("Problem opening/reading IOR file: " + ex);
		} catch (org.omg.CORBA.SystemException ex) {
			ex.printStackTrace();
			System.err.println("CORBA system exception in string_to_object: " + ex);
		}
		return null;
	}

	/**
	 * Writes an IOR to a File
	 * @param orb The ORB.
	 * @param obj The CORBA Object to write.
	 * @param filename The name of the file to write.
	 */
	public void writeIOR(org.omg.CORBA.ORB orb, org.omg.CORBA.Object obj, String filename) {
		try {
			System.err.println("Writing stringified IOR to file " + filename);
			java.io.FileWriter output = new java.io.FileWriter(filename);
			output.write(orb.object_to_string(obj) + "\n");
			output.close();
			return;
		} catch (java.io.IOException ex) {
			System.err.println("Problem opening/writing IOR file: " + ex);
		} catch (org.omg.CORBA.SystemException ex) {
			ex.printStackTrace();
			System.err.println("CORBA system exception in object_to_string: " + ex);
		}
	}

	/**
	 * Prints an IOR to a print stream
	 * @param orb The ORB.
	 * @param obj The CORBA Object to print.
	 * @param output The print stream which print the IOR (for example : System.out).
	 */
	public void printIOR(org.omg.CORBA.ORB orb, org.omg.CORBA.Object obj, java.io.PrintStream output) {
		try {
			output.println(orb.object_to_string(obj) + "\n");
			return;
		} catch (org.omg.CORBA.SystemException ex) {
			ex.printStackTrace();
			System.err.println("CORBA system exception in object_to_string: " + ex);
		}
	}
	
}
