/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.provider;

import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.mof.Model.Association;
import org.omg.mof.Model.ClassHelper;
import org.omg.mof.Model.Classifier;
import org.omg.mof.Model.DataType;
import org.omg.mof.Model.DataTypeHelper;
import org.omg.mof.Model.Import;
import org.omg.mof.Model.MofAttribute;
import org.omg.mof.Model.MultiplicityType;
import org.omg.mof.Model.Namespace;
import org.omg.mof.Model.PackageHelper;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

import org.objectweb.modfact.corba.helper.JMICommon;
import org.objectweb.modfact.corba.helper.MOFCommon;

/**
 * @author Xavier
 *
 */
public class JMIProvider implements JMICommon {

	MOFCommon mofHelper;

	public JMIProvider(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiPackageName(Package)
	 */
	public String jmiPackageName(org.omg.mof.Model.Package _package)
		throws MofError {
		return _package.name().toLowerCase();
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiQualifiedPackageName(Package)
	 */
	public String jmiPackageQualifiedName(org.omg.mof.Model.Package _package)
		throws MofError, NotSet {
		//String[] names = _package.qualified_name();
		String res = "";

		String[] names = _package.qualified_name();
		for (int i = 0; i < names.length; i++) {
			res = names[i].toLowerCase() + res;
			if (i != (names.length - 1))
				res = "." + res;
		}

		org.omg.mof.Model.Package outerPackage =
			mofHelper.outermostPackageOfModelElement(_package);
		//can be itself no matter
		String packagePrefix = mofHelper.jmiPrefixForPackage(_package);
		if (packagePrefix.compareTo("") != 0)
			res = packagePrefix + "." + res;

		return res;
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiClassName(Class)
	 */
	public String jmiClassName(org.omg.mof.Model.Class clazz)
		throws MofError, NotSet {
		if (mofHelper.jmiSubstituteIdentifierForClass(clazz).compareTo("")
			!= 0)
			return mofHelper.jmiSubstituteIdentifierForClass(clazz);
		char[] res = clazz.name().toCharArray();
		if (res.length != 0) {
			res[0] = Character.toUpperCase(res[0]);
		}
		return new String(res);
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiClassName(Class)
	 */
	public String jmiClassQualifiedName(org.omg.mof.Model.Class clazz)
		throws MofError, NotSet {
		String res = jmiClassName(clazz);

		org.omg.mof.Model.Package _package =
			PackageHelper.narrow(clazz.container());
		return jmiPackageQualifiedName(_package) + "." + res;
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiAssociationName(Association)
	 */
	public String jmiAssociationName(Association association)
		throws MofError, NotSet {
		char[] res = association.name().toCharArray();
		if (res.length != 0) {
			res[0] = Character.toUpperCase(res[0]);
		}
		return new String(res);
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiAssociationName(Association)
	 */
	public String jmiAssociationQualifiedName(Association association)
		throws MofError, NotSet {
		char[] res = association.name().toCharArray();
		if (res.length != 0) {
			res[0] = Character.toUpperCase(res[0]);
		}
		org.omg.mof.Model.Package _package =
			PackageHelper.narrow(association.container());
		return jmiPackageQualifiedName(_package) + "." + new String(res);
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiAttributeName(MofAttribute)
	 */
	public String jmiAttributeName(MofAttribute attribute) throws MofError {
		char[] res = attribute.name().toCharArray();
		if (res.length != 0) {
			res[0] = Character.toLowerCase(res[0]);
		}
		return new String(res);
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiTypeCodeName(TypeCode)
	 */
	public String jmiTypeCodeName(TypeCode _type) throws BadKind {
		switch (_type.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				{
					return "boolean";

				}
			case org.omg.CORBA.TCKind._tk_octet :
				{
					return "byte";

				}
			case org.omg.CORBA.TCKind._tk_char :
				{
					return "char";

				}
			case org.omg.CORBA.TCKind._tk_wchar :
				{
					return "char";

				}
			case org.omg.CORBA.TCKind._tk_short :
				{
					return "short";

				}
			case org.omg.CORBA.TCKind._tk_ushort :
				{
					return "short";

				}
			case org.omg.CORBA.TCKind._tk_long :
				{
					return "int";

				}
			case org.omg.CORBA.TCKind._tk_ulong :
				{
					return "int";

				}
			case org.omg.CORBA.TCKind._tk_longlong :
				{
					return "long";

				}
			case org.omg.CORBA.TCKind._tk_ulonglong :
				{
					return "long";

				}
			case org.omg.CORBA.TCKind._tk_double :
				{
					return "double";

				}
			case org.omg.CORBA.TCKind._tk_float :
				{
					return "float";

				}
			case org.omg.CORBA.TCKind._tk_string :
				{
					return "String";

				}
			case org.omg.CORBA.TCKind._tk_wstring :
				{
					return "String";

				}
			case org.omg.CORBA.TCKind._tk_TypeCode :
				{
					return "org.omg.CORBA.TypeCode";

				}
			case org.omg.CORBA.TCKind._tk_any :
				{
					return "org.omg.CORBA.Any";

				}
			case org.omg.CORBA.TCKind._tk_objref :
				{
					return "org.omg.CORBA.Object";

				}
		}
		return _type.name();
	}

	/**
	 * 
	 */
	public String jmiNullableTypeCodeName(TypeCode _type) throws BadKind {
		switch (_type.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				{
					return "Boolean";

				}
			case org.omg.CORBA.TCKind._tk_octet :
				{
					return "Byte";

				}
			case org.omg.CORBA.TCKind._tk_char :
				{
					return "Char";

				}
			case org.omg.CORBA.TCKind._tk_wchar :
				{
					return "Char";

				}
			case org.omg.CORBA.TCKind._tk_short :
				{
					return "Short";

				}
			case org.omg.CORBA.TCKind._tk_ushort :
				{
					return "Short";

				}
			case org.omg.CORBA.TCKind._tk_long :
				{
					return "Integer";

				}
			case org.omg.CORBA.TCKind._tk_ulong :
				{
					return "Integer";

				}
			case org.omg.CORBA.TCKind._tk_longlong :
				{
					return "Long";

				}
			case org.omg.CORBA.TCKind._tk_ulonglong :
				{
					return "Long";

				}
			case org.omg.CORBA.TCKind._tk_double :
				{
					return "Double";

				}
			case org.omg.CORBA.TCKind._tk_float :
				{
					return "Float";
				}
			case org.omg.CORBA.TCKind._tk_string :
				{
					return "String";

				}
			case org.omg.CORBA.TCKind._tk_wstring :
				{
					return "String";

				}
			case org.omg.CORBA.TCKind._tk_TypeCode :
				{
					return "org.omg.CORBA.TypeCode";

				}
			case org.omg.CORBA.TCKind._tk_any :
				{
					return "org.omg.CORBA.Any";

				}
			case org.omg.CORBA.TCKind._tk_objref :
				{
					return "org.omg.CORBA.Object";

				}
		}
		return _type.name();
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiImportName(Import)
	 */
	public String jmiImportName(Import _import) throws MofError, NotSet {
		return jmiPackageQualifiedName(
			PackageHelper.narrow(_import.imported_namespace()));
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiClassifierName(Classifier)
	 */
	public String jmiClassifierName(Classifier _classifier)
		throws MofError, BadKind, NotSet {
		if (_classifier._is_a(org.omg.mof.Model.ClassHelper.id()))
			return jmiClassName(
				(org.omg.mof.Model.Class) org.omg.mof.Model.ClassHelper.narrow(
					_classifier));
		else if (_classifier._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
			DataType dt = (DataType) DataTypeHelper.narrow(_classifier);
			String resu = jmiTypeCodeName(dt.type_code());
			return resu;
		} else
			return _classifier.name();
	}

	/**
	 * @see fr.lip6.modfact.helper.JMICommon#jmiClassifierName(Classifier)
	 */
	public String jmiClassifierQualifiedName(Classifier _classifier)
		throws MofError, BadKind, NotSet {

		if (_classifier._is_a(org.omg.mof.Model.ClassHelper.id()))
			return jmiClassQualifiedName(
				(org.omg.mof.Model.Class) org.omg.mof.Model.ClassHelper.narrow(
					_classifier));
		else if (_classifier._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
			DataType dt = (DataType) DataTypeHelper.narrow(_classifier);
			Namespace ns = dt.container();
			String resu = jmiTypeCodeName(dt.type_code());

			if ((dt.type_code().kind().value() == TCKind._tk_struct)
				|| (dt.type_code().kind().value() == TCKind._tk_enum)) {
				if (ns._is_a(PackageHelper.id())) {
					org.omg.mof.Model.Package _package =
						PackageHelper.narrow(ns);
					resu = jmiPackageQualifiedName(_package) + "." + resu;
				} else if (ns._is_a(ClassHelper.id())) {
					org.omg.mof.Model.Class clazz = ClassHelper.narrow(ns);
					org.omg.mof.Model.Package _package =
						PackageHelper.narrow(ns.container());
					resu = jmiPackageQualifiedName(_package) + "." + resu;
				}
			}
			return resu;
		} else
			return _classifier.name();
	}

	/**
												* @see fr.lip6.modfact.helper.JMICommon#jmiType(String)
												*/ //	public String jmiType(String _type) {
	//		if (_type.equalsIgnoreCase("boolean"))
	//			return "boolean";
	//		else if (_type.equalsIgnoreCase("octet"))
	//			return "byte";
	//		else if (_type.equalsIgnoreCase("char"))
	//			return "char";
	//		else if (_type.equalsIgnoreCase("wchar"))
	//			return "char";
	//		else if (_type.equalsIgnoreCase("short"))
	//			return "Short";
	//		else if (_type.equalsIgnoreCase("unsigned short"))
	//			return "Short";
	//		else if (_type.equalsIgnoreCase("long"))
	//			return "int";
	//		else if (_type.equalsIgnoreCase("unsigned long"))
	//			return "int";
	//		else if (_type.equalsIgnoreCase("long long"))
	//			return "long";
	//		else if (_type.equalsIgnoreCase("unsigned long long"))
	//			return "long";
	//		else if (_type.equalsIgnoreCase("double"))
	//			return "double";
	//		else if (_type.equalsIgnoreCase("float"))
	//			return "float";
	//		else if (_type.equalsIgnoreCase("string"))
	//			return "String";
	//		else if (_type.equalsIgnoreCase("wstring"))
	//			return "String";
	//		else if (_type.equalsIgnoreCase("CORBA::TypeCode"))
	//			return "TypeCode";
	//		else if (_type.equalsIgnoreCase("any"))
	//			return "Any";
	//		else if (_type.equalsIgnoreCase("Object"))
	//			return "Object";
	//		// je pourrais dclancher une exception  revoir la igne ci-dessous
	//		else {
	//			return _type;
	//		}
	//	}
	/**
	 * @see fr.lip6.modfact.helper.JMICommon#accessorName(String, TypeCode)
	 */
	public String jmiFormat1(String name) throws MofError {
		char[] res = name.toCharArray();
		if (res.length != 0) {
			res[0] = Character.toUpperCase(res[0]);
		}
		return new String(res);
	}

	/**
	* Return a nullable type (i.e. Boolean instead of boolean)
											 */
	public String jmiNullableClassifierName(Classifier classifier)
		throws MofError, BadKind, NotSet {
		if (classifier._is_a(org.omg.mof.Model.ClassHelper.id()))
			return jmiClassName(
				(org.omg.mof.Model.Class) org.omg.mof.Model.ClassHelper.narrow(
					classifier));
		else if (classifier._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
			DataType dt = (DataType) DataTypeHelper.narrow(classifier);
			String resu = jmiNullableTypeCodeName(dt.type_code());
			return resu;
		} else
			return classifier.name();
	} /**
																							 * Return a nullable type (i.e. Boolean instead of boolean)
																							 */
	public String jmiNullableClassifierQualifiedName(Classifier classifier)
		throws MofError, BadKind, NotSet {
		if (classifier._is_a(org.omg.mof.Model.ClassHelper.id()))
			return jmiClassName(
				(org.omg.mof.Model.Class) org.omg.mof.Model.ClassHelper.narrow(
					classifier));
		else if (classifier._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
			DataType dt = (DataType) DataTypeHelper.narrow(classifier);
			Namespace ns = dt.container();
			String resu = jmiNullableTypeCodeName(dt.type_code());

			if ((dt.type_code().kind().value() == TCKind._tk_struct)
				|| (dt.type_code().kind().value() == TCKind._tk_enum)) {
				if (ns._is_a(PackageHelper.id())) {
					org.omg.mof.Model.Package _package =
						PackageHelper.narrow(ns);
					resu = jmiPackageQualifiedName(_package) + "." + resu;
				} else if (ns._is_a(ClassHelper.id())) {
					org.omg.mof.Model.Class clazz = ClassHelper.narrow(ns);
					org.omg.mof.Model.Package _package =
						PackageHelper.narrow(ns.container());
					resu = jmiPackageQualifiedName(_package) + "." + resu;
				}
			}
			return resu;
		} else
			return classifier.name();
	}

	public String jmiAccessorName(MofAttribute attribute) throws MofError {
		String accessorName = jmiFormat1(attribute.name());
		MultiplicityType mult = attribute.multiplicity();
		//DataType
		boolean isBoolean = false;
		if (attribute.type()._is_a(DataTypeHelper.id())) {
			DataType type = DataTypeHelper.narrow(attribute.type());
			if (type.type_code().kind().equals(TCKind.tk_boolean))
				isBoolean = true;
		}
		if (mult.lower == 0 && mult.upper == 1 && isBoolean) {
			if (attribute.name().toLowerCase().startsWith("is"))
				return attribute.name();
			else
				return "is" + accessorName;
		} else
			return "get" + accessorName;
	}

	public String jmiMutatorName(MofAttribute attribute) throws MofError {
		String mutatorName = "";
		MultiplicityType mult = attribute.multiplicity();
		//DataType
		boolean isBoolean = false;
		if (attribute.type()._is_a(DataTypeHelper.id())) {
			DataType type = DataTypeHelper.narrow(attribute.type());
			if (type.type_code().kind().equals(TCKind.tk_boolean))
				isBoolean = true;
		}
		if (isBoolean
			&& (mult.lower == 0 || mult.lower == 1)
			&& mult.upper == 1
			&& attribute.name().toLowerCase().startsWith("is")) {
			return "set" + jmiFormat1(attribute.name().substring(2));
		} else
			return "set" + jmiFormat1(attribute.name());
	}
	
	public String valueOfConstant(org.omg.CORBA.Any _value)
		throws org.omg.CORBA.TypeCodePackage.BadKind {
			return mofHelper.valueOfConstant(_value);
		}

}
