/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.server;

import org.objectweb.modfact.corba.generator.BracketGenerator;
import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.omg.mof.Model.ModelElement;
import org.omg.mof.Reflective.MofError;

/**
 * The repository server generator.
 * This class allows the generation of the repository server (into Java).
 */
public class GenerationServer extends BracketGenerator {

	/** The IDL Helper .*/
	private IDLCommon idlHelper;

	/** The Java Helper. */
	private JavaCommon javaHelper;

	/** The logger. */
	private ModFactLogger logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.server.GenerationServer");

	/** True if the ORB is IDL to Java language mapping compliant */
	private boolean mappingValid = false;

	/**
	 * Default Constructor.
	 */
	public GenerationServer() {
		logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.server.GenerationServerApplication");
	}

	/**
	 * Set the flag to indicate if the ORB is IDL to Java language mapping compliant.
	 * @param mappingValid TRUE if it is compliant.
	 */
	public void setMappingValid(boolean mappingValid) {
		this.mappingValid = mappingValid;
	}

	/**
	 * Server generation.
	 * @param package_ The root package.
	 */
	public void generateServer(org.omg.mof.Model.Package package_, String class_name_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds, java.io.IOException {
		logger.log(Level.FINE, "Server - beginning of the generation");
		logger.log(Level.INFO, class_name_ + " generation ...");
		String packageName1 = idlHelper.format1(package_.name());
		header(package_, class_name_);
		constructor(class_name_, packageName1);
		runMethod(package_, packageName1);
		getObjectMethod(package_, packageName1);
		objectExistsMethod();
		outputln("} // end of class");
		flushFile();
	}

	/**
	 * Server generation for the header.
	 * @param package_ The package.
	 * @param className The application class name.
	 */
	public void header(org.omg.mof.Model.Package package_, String className) throws MofError {
		outputln("package " + javaHelper.javaPackage(package_) + ";");
		outputln();
		outputln("public class " + className + " extends Thread {");
		outputln();
	}

	/**
	 * Server generation for the constructor.
	 * @param className The application class name.
	 */
	public void constructor(String className, String package_name_) {
		String factoryImpl = package_name_ + "." + package_name_ + "PackageFactoryPOA";
		outputln("private org.omg.CORBA.ORB orb_;");
		outputln("private org.objectweb.modfact.corba.logging.ModFactLogger logger = org.objectweb.modfact.corba.logging.ModFactLogger.getLogger(\""
				+ package_name_
				+ "Server\");");
		outputln("private " + package_name_ + "PackageFactory _model_package;");
		outputln();
		outputln("public " + className + " (org.omg.CORBA.ORB orb) {");
		outputln("orb_ = orb;");
		outputln("}");
		outputln();
	}

	/**
	 * Server generation for the method run().
	 * @param package The package.
	 * @param packageName The package name.
	 */
	public void runMethod(org.omg.mof.Model.Package package_, String package_name_) throws MofError {
		outputln("public void run () {");
		outputln("try {");
		outputln("// Get reference to rootpoa & activate the POAManager");
		outputln("org.omg.PortableServer.POA rootpoa = org.omg.PortableServer.POAHelper.narrow(orb_.resolve_initial_references(\"RootPOA\"));");
		outputln("rootpoa.the_POAManager().activate();");
		outputln();
		outputln("// Create servant and register it with the ORB");
		String factoryImpl = package_name_ + "PackageFactoryImpl";
		outputln(factoryImpl + " _model_package_factory = new " + factoryImpl + "();");
		outputln();
		outputln("// Get object reference from the servant");
		outputln("org.omg.CORBA.Object ref = rootpoa.servant_to_reference(_model_package_factory);");
		String factory = javaHelper.javaPackage(package_) + "." + package_name_ + "PackageFactory";
		outputln("_model_package = " + factory + "Helper.narrow(ref);");
		outputln();
		outputln("// Get the root naming context");
		outputln("// NameService invokes the name service");
		outputln("org.omg.CORBA.Object objRef = orb_.resolve_initial_references(\"NameService\");");
		outputln("// Use NamingContextExt which is part of the Interoperable");
		outputln("// Naming Service (INS) specification.");
		outputln("org.omg.CosNaming.NamingContextExt ncRef = org.omg.CosNaming.NamingContextExtHelper.narrow(objRef);");
		outputln();
		outputln("// Bind the Object Reference in Naming");
		outputln("java.lang.String nameService = \"" + package_name_ + "FactoryServer\";");
		outputln("org.omg.CosNaming.NameComponent path[] = ncRef.to_name(nameService);");
		outputln("ncRef.rebind(path, _model_package);");
		outputln("logger.log(org.objectweb.modfact.corba.logging.Level.INFO, \"" + package_name_ + " Server ready and waiting ...\");");
		outputln();
		outputln("// Wait for invocations from clients");
		outputln("orb_.run();");
		outputln("} catch (java.lang.NullPointerException npe) {");
		outputln("System.err.println(\"[ERROR] The NameService is not started. \");");
		outputln("} catch (java.lang.Exception e) {");
		outputln("System.err.println(\"[ERROR] \" + e.getMessage());");
		outputln("}");
		outputln("logger.log(org.objectweb.modfact.corba.logging.Level.INFO, \"" + package_name_ + " Server Exiting ...\");");
		outputln("}");
		outputln();
	}

	/**
	 * Server generation for the method getObject().
	 * @param package_ The package.
	 * @param package_name_ The package name.
	 */
	public void getObjectMethod(org.omg.mof.Model.Package package_, String package_name_) throws MofError {
		String factoryImpl = javaHelper.javaPackage(package_) + "." + package_name_ + "PackageFactory";
		outputln("public " + factoryImpl + " getObject () {");
		outputln("return _model_package;");
		outputln("}");
		outputln();
	}

	/**
	 * Server generation for the method objectExists().
	 */
	public void objectExistsMethod() {
		outputln("public boolean objectExists () {");
		outputln("return _model_package != null;");
		outputln("}");
		outputln();
	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#generate()
	 */
	public void generate() throws Exception {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setInput(org.omg.mof.Model.ModelElement[])
	 */
	public void setInput(ModelElement[] elt) throws Exception {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setLogger(org.objectweb.modfact.corba.logging.ModFactLogger)
	 */
	public void setLogger(ModFactLogger log) throws Exception {
		logger = log;
	}

	/**
	 * @param common
	 */
	public void setIdlHelper(IDLCommon common) {
		idlHelper = common;
	}

	/**
	 * @param common
	 */
	public void setJavaHelper(JavaCommon common) {
		javaHelper = common;
	}

}
