/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.service;

import java.io.InputStream;
import java.io.OutputStream;

import org.omg.CosNaming.*;

// Imports for the logging
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;

// Imports for the generators
import org.objectweb.modfact.corba.io.Import;
import org.objectweb.modfact.corba.idl.MOFIDL1_3Generator;
import org.objectweb.modfact.corba.idl2java.IDL2JavaMOF1_3Generator;
import org.objectweb.modfact.corba.xmi.XMI1_1DTDGenerator;
import org.objectweb.modfact.corba.xmiio.exporter.XMIExportGenerator;
import org.objectweb.modfact.corba.xmiio.importer.XMIImportGenerator;
import org.objectweb.modfact.corba.server.ServerGenerator;

// Imports for the Helper API
import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.helper.XMICommon;
import org.objectweb.modfact.corba.helper.XMIImportCommon;

// Imports for the Helper Implementation
import org.objectweb.modfact.corba.provider.IDLProvider;
import org.objectweb.modfact.corba.provider.JavaProvider;
import org.objectweb.modfact.corba.provider.MOFCachedProvider;
import org.objectweb.modfact.corba.provider.XMIImportProvider;
import org.objectweb.modfact.corba.provider.XMIProvider;


public class Generation {;

	/**
	 * Generates all the selected items.
	 * @param idl TRUE to generate the IDL files.
	 * @param dtd TRUE to generate the DTD file.
	 * @param java TRUE to generate the Java files.
	 * @param xmiImport TRUE to generate the XMI Importer files.
	 * @param xmiExport TRUE to generate the XMI Exporter files.
	 * @param server TRUE to generate the Server files.
	 * @param mappingValid TRUE to indicate that the ORB is compliant with the IDL to Java mapping.
	 * @param input The input file (meta-model).
	 * @param output The output file (zip file).
	 * @param orb The ORB.
	 */
	public void execute(boolean idl, boolean dtd, boolean java, boolean xmiImport, boolean xmiExport, boolean server, boolean mappingValid, InputStream input, OutputStream output, org.omg.CORBA.ORB orb) {
		try {
			// get the root naming context
			org.omg.CORBA.Object objRef =
				orb.resolve_initial_references("NameService");

			// Use NamingContextExt instead of NamingContext. This is 
			// part of the Interoperable naming Service.  
			NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

			// resolve the Object Reference in Naming
			String name = "ModFactServer";
			org.omg.mof.Model.ModelPackageFactory _reference =
				org.omg.mof.Model.ModelPackageFactoryHelper.narrow(
					ncRef.resolve_str(name));

			// creation of Model 
			org.omg.mof.Model._ModelPackage modelPackage =
				_reference.create_model_package();

			// initialisation  
			//ImportFromRose imp = new ImportFromRose();
			//imp.initiate(modelPackage, orb);
			ModFactLogger loggerImport = ModFactLogger.getLogger("org.objectweb.modfact.corba.io.Import");
			Import imp = new Import();
			imp.setIdlHelper(new IDLProvider());
			imp.setLogger(loggerImport);
			imp.initiate(modelPackage, orb);

			// input XML model
			org.omg.mof.Model.Package[] outermostPackages =
				imp.parse(input);

			// Generation of IDL
			if (idl) {
				ModFactLogger logger =
					ModFactLogger.getLogger(
						"org.objectweb.modfact.corba.idl.MOFIDL1_3Generator");
				MOFIDL1_3Generator idlGenerator = new MOFIDL1_3Generator();
				idlGenerator.setLogger(logger);
				idlGenerator.setInput(outermostPackages);
				idlGenerator.setOutput(output);
				MOFCommon mofHelper = new MOFCachedProvider(outermostPackages);
				idlGenerator.setMofHelper(mofHelper);
				IDLCommon idlHelper = new IDLProvider(mofHelper);
				idlGenerator.setIdlHelper(idlHelper);
				idlGenerator.generate();
				//ch.close();

				//output.close();
			}

			//Generation of DTD
			if (dtd) {
				ModFactLogger logger =
					ModFactLogger.getLogger(
						"org.objectweb.modfact.corba.xmi.XMI1_1DTDGenerator");
				XMI1_1DTDGenerator dtdGenerator = new XMI1_1DTDGenerator();
				dtdGenerator.setLogger(logger);
				dtdGenerator.setInput(outermostPackages);
				dtdGenerator.setOutput(output);
				MOFCommon mofHelper = new MOFCachedProvider(outermostPackages);
				dtdGenerator.setMofHelper(mofHelper);
				XMICommon xmiHelper = new XMIProvider(mofHelper);
				dtdGenerator.setXmiHelper(xmiHelper);
				dtdGenerator.generate();
				//output.close();
			}

			//Generation of code
			if (java) {
				ModFactLogger logger =
					ModFactLogger.getLogger(
						"org.objectweb.modfact.corba.idl2java.IDL2JavaMOF1_3Generator");
				IDL2JavaMOF1_3Generator javaGenerator =
					new IDL2JavaMOF1_3Generator();
				javaGenerator.setLogger(logger);
				javaGenerator.setInput(outermostPackages);
				javaGenerator.setOutput(output);
				MOFCommon mofHelper = new MOFCachedProvider(outermostPackages);
				javaGenerator.setMofHelper(mofHelper);
				JavaCommon javaHelper = new JavaProvider(mofHelper);
				javaGenerator.setJavaHelper(javaHelper);
				javaGenerator.generate();
				//output.close();
			}

			//Generation of code
			if (xmiImport) {
				ModFactLogger logger =
					ModFactLogger.getLogger(
						"org.objectweb.modfact.corba.xmiio.importer.XMIImportGenerator");
				XMIImportGenerator xmiImpGenerator = new XMIImportGenerator();
				xmiImpGenerator.setLogger(logger);
				xmiImpGenerator.setInput(outermostPackages);
				xmiImpGenerator.setOutput(output);
				MOFCommon mofHelper = new MOFCachedProvider(outermostPackages);
				xmiImpGenerator.setMofHelper(mofHelper);
				JavaCommon javaHelper = new JavaProvider(mofHelper);
				xmiImpGenerator.setJavaHelper(javaHelper);
				IDLCommon idlHelper = new IDLProvider(mofHelper);
				xmiImpGenerator.setIdlHelper(idlHelper);
				XMIImportCommon xmiImpHelper = new XMIImportProvider(idlHelper, mofHelper, javaHelper);
				xmiImpGenerator.setXmiImportHelper(xmiImpHelper);
				xmiImpGenerator.setMappingValid(mappingValid);
				xmiImpGenerator.generate();
				//output.close();
			}

			//Generation of code
			if (xmiExport) {
				ModFactLogger logger =
					ModFactLogger.getLogger(
						"org.objectweb.modfact.corba.xmiio.exporter.XMIExportGenerator");
				XMIExportGenerator xmiExpGenerator = new XMIExportGenerator();
				xmiExpGenerator.setLogger(logger);
				xmiExpGenerator.setInput(outermostPackages);
				xmiExpGenerator.setOutput(output);
				MOFCommon mofHelper = new MOFCachedProvider(outermostPackages);
				xmiExpGenerator.setMofHelper(mofHelper);
				JavaCommon javaHelper = new JavaProvider(mofHelper);
				xmiExpGenerator.setJavaHelper(javaHelper);
				IDLCommon idlHelper = new IDLProvider(mofHelper);
				xmiExpGenerator.setIdlHelper(idlHelper);
				xmiExpGenerator.setMappingValid(mappingValid);
				xmiExpGenerator.generate();
				//output.close();
			}

			//Generation of code
			if (server) {
				ModFactLogger logger =
					ModFactLogger.getLogger(
						"org.objectweb.modfact.corba.server.ServerGenerator");
				ServerGenerator serverGenerator = new ServerGenerator();
				serverGenerator.setLogger(logger);
				serverGenerator.setInput(outermostPackages);
				serverGenerator.setOutput(output);
				MOFCommon mofHelper = new MOFCachedProvider(outermostPackages);
				IDLCommon idlHelper = new IDLProvider(mofHelper);
				serverGenerator.setIdlHelper(idlHelper);
				JavaCommon javaHelper = new JavaProvider(mofHelper);
				serverGenerator.setJavaHelper(javaHelper);
				serverGenerator.setMappingValid(mappingValid);
				serverGenerator.generate();
				//output.close();
			}

		} catch (Exception e) {
			ModFactLogger logger = ModFactLogger.getLogger("org.objectweb.modfact.corba.service.Generation");
			logger.log(Level.ERROR, e.toString());
		}
	}
	
}
