/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.service;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

public class GenerationService
	extends org.objectweb.modfact.corba.service.cmdline.common.ModfactApplicationBase
	implements
		org.objectweb.modfact.corba.service.cmdline.io.ImportApplication,
		org.objectweb.modfact.corba.service.cmdline.io.ExportApplication {

	/** The IDL flag. */
	private boolean idl = false;

	/** The Java flag. */
	private boolean java = false;

	/** The DTD flag. */
	private boolean dtd = false;

	/** The XMI Import flag. */
	private boolean xmiImport = false;

	/** The XMI Export flag. */
	private boolean xmiExport = false;

	/** The server generation flag. */
	private boolean server = false;

	/** True if the ORB is IDL to Java language mapping compliant */
	private boolean mappingValid = false;

	/** The input file. */
	private String input = null;

	/** The output file. */
	private String output = null;

	/**
	 * Constructor of a GenerationService.
	 */
	public GenerationService() {
		super(
			new org.objectweb.util.cmdline.lib.DefaultCommandLine(
				new String[] { "GenerationService" },
				new String[0],
				new String[] { "Tool to manage the MOF Repository" },
				true));
		// Creates the GenerationService options
		getCommandLine().addOption(
			new org.objectweb.modfact.corba.service.cmdline.client.OptionIDL(
				this));
		getCommandLine().addOption(
			new org.objectweb.modfact.corba.service.cmdline.client.OptionJava(
				this));
		getCommandLine().addOption(
			new org.objectweb.modfact.corba.service.cmdline.client.OptionDTD(
				this));
		getCommandLine().addOption(
			new org
				.objectweb
				.modfact
				.corba
				.service
				.cmdline
				.client
				.OptionXMIImport(
				this));
		getCommandLine().addOption(
			new org
				.objectweb
				.modfact
				.corba
				.service
				.cmdline
				.client
				.OptionXMIExport(
				this));
		getCommandLine().addOption(
			new org
				.objectweb
				.modfact
				.corba
				.service
				.cmdline
				.client
				.OptionServer(
				this));
		getCommandLine().addOption(
			new org
				.objectweb
				.modfact
				.corba
				.service
				.cmdline
				.client
				.OptionIDLtoJavaMappingValid(this));
		getCommandLine().addOption(
			new org.objectweb.modfact.corba.service.cmdline.io.OptionInput(
				this));
		getCommandLine().addOption(
			new org.objectweb.modfact.corba.service.cmdline.io.OptionOutput(
				this));
	}

	/**
	 * Set the DTD flag.
	 * 
	 * @param dtd
	 *            TRUE to generate the DTD File.
	 */
	public void setDTD(boolean dtd) {
		this.dtd = dtd;
	}

	/**
	 * Set the IDL flag.
	 * 
	 * @param idl
	 *            TRUE to generate the IDL File.
	 */
	public void setIDL(boolean idl) {
		this.idl = idl;
	}

	/**
	 * Set the java flag.
	 * 
	 * @param java
	 *            TRUE to generate the Java Files.
	 */
	public void setJava(boolean java) {
		this.java = java;
	}

	/**
	 * Set the XMI Import flag.
	 * 
	 * @param xmiImport
	 *            TRUE to generate the XMI Importer.
	 */
	public void setXMIImport(boolean xmiImport) {
		this.xmiImport = xmiImport;
	}

	/**
	 * Set the XMI Export flag.
	 * 
	 * @param xmiExport
	 *            TRUE to generate the XMI Exporter.
	 */
	public void setXMIExport(boolean xmiExport) {
		this.xmiExport = xmiExport;
	}

	/**
	 * Set the Server generation flag.
	 * 
	 * @param server
	 *            TRUE to generate the repository server.
	 */
	public void setServer(boolean server) {
		this.server = server;
	}

	/**
	 * Set the flag to indicate if the ORB is IDL to Java language mapping
	 * compliant.
	 * 
	 * @param mappingValid
	 *            TRUE if it is compliant.
	 */
	public void setMappingValid(boolean mappingValid) {
		this.mappingValid = mappingValid;
	}

	/**
	 * Set the output file.
	 * 
	 * @param output
	 *            The output file.
	 */
	public void setOutput(String output) {
		this.output = output;
	}

	/**
	 * Set the input file.
	 * 
	 * @param input
	 *            The input filename.
	 */
	public void setInputFile(String input) {
		this.input = input;
	}

	/**
	 * Starts the main function.
	 * 
	 * @param args
	 *            The command line arguments.
	 */
	public int start(String[] args) {
		try {
			Generation client = new Generation();
			client.execute(
				idl,
				dtd,
				java,
				xmiImport,
				xmiExport,
				server,
				mappingValid,
				new FileInputStream(input),
				new FileOutputStream(output),
				orb_);
			return 0;
		} catch (FileNotFoundException ex) {
			report_exception(ex);
			return -1;
		}
	}

	/**
	 * The main method.
	 * 
	 * @param args
	 *            The command line arguments.
	 */
	public static void main(String[] args) {
		GenerationService application = new GenerationService();
		application.runMain(args);
	}

}
