/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmi;

import org.omg.mof.Model.Association;
import org.omg.mof.Model.MofAttribute;
import org.omg.mof.Model.Reference;
import org.omg.mof.Reflective.NotSet;

import org.objectweb.modfact.corba.generator.PrintGenerator;
import org.objectweb.modfact.corba.helper.XMICommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;

/**
 * @author Xavier Blanc
 *
 */
public class PackageGenerator extends PrintGenerator {

	private org.omg.mof.Model.Package[] input;

	private MOFCommon mofHelper;
	private XMICommon xmiHelper;

	private ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(org.omg.mof.Model.ModelElement[] elt) {
		input = new org.omg.mof.Model.Package[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (org.omg.mof.Model.Package) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * Generate
	 */
	public void generate() throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			org.omg.mof.Reflective.NotSet {
		generate(input[0]);
		out.flush();
	}


	// 7.2 Rules Set 1: Simple DTD

	// GetAllInstanceAttributes: tous les attributs d'une classes et ses superclasses confondies 7.2.3
	public String getAllInstanceAttributes(
		org.omg.mof.Model.Class cls,
		String previousCls)
		throws org.omg.mof.Reflective.MofError , NotSet{
		logger.log(Level.FINER, "DTD getAllInstanceAttributes of a class");
		String _nclass = cls.ref_mof_id();
		if (previousCls.indexOf(_nclass) != -1)
			return "";
		else {
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			String parentAtts = "";
			for (int i = 0; i < parent.length; i++) {
				String temp =
					getAllInstanceAttributes(
						org.omg.mof.Model.ClassHelper.narrow(parent[i]),
						previousCls);
				if ((parentAtts.length() > 0) && (temp.length() > 0)) {
					parentAtts = parentAtts + ", ";
				}
				parentAtts = parentAtts + temp;
			}
			String atts = getAttributes(cls, "instance");
			if ((parentAtts.length() > 0) && (atts.length() > 0)) {
				parentAtts = parentAtts + ", ";
			}
			previousCls = previousCls + " " + _nclass;
			return parentAtts + atts;
		}
	}

	// GetAttributes :les attributs Instance_level ou Classifier_level d'une class

	public String getAttributes(org.omg.mof.Model.Class cls, String type)
		throws org.omg.mof.Reflective.MofError , NotSet {
		logger.log(Level.FINER, "DTD getAttrinbutes of a class");
		String rslt = "";
		org.omg.mof.Model.MofAttribute[] attributes;
		if (type == "instance")
			attributes =
				mofHelper.attributesOfClass(
					cls,
					org.omg.mof.Model.ScopeKind.instance_level,
					true);
		else
			attributes =
				mofHelper.attributesOfClass(
					cls,
					org.omg.mof.Model.ScopeKind.classifier_level,
					true);
		for (int i = 0; i < attributes.length; i++) {
			String name = xmiHelper.qualifiedName(attributes[i]);
			String m;
			int _upper = attributes[i].multiplicity().upper;
			int _lower = attributes[i].multiplicity().lower;
			if ((_lower == 1) && ((_upper > 1) || (_upper == -1)))
				m = "*"; /*ou + pour  DTD non relax */
			else if ((_lower == 0) && (_upper == 1))
				m = "?";
			else if ((_lower != 1) && (_upper != 1))
				m = "*";
			else
				m = "?"; /*ou "" pour  DTD non relax */
			if (rslt.length() > 0)
				rslt = rslt + ", ";
			rslt = rslt + name + m;
		}
		return rslt;
	}

	//GetAllReferences : toutes les rfrences d'une classes
	public String getAllReferences(
		org.omg.mof.Model.Class cls,
		String previousCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		logger.log(Level.FINER, "DTD getAllReferences of a class");
		String _nclass = cls.name();
		if (previousCls.indexOf(_nclass) != -1)
			return "";
		else {
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			String parentRefs = "";
			for (int i = 0; i < parent.length; i++) {
				String temp =
					getAllReferences(
						org.omg.mof.Model.ClassHelper.narrow(parent[i]),
						previousCls);
				if ((parentRefs.length() > 0) && (temp.length() > 0)) {
					parentRefs = parentRefs + temp;
				}
			}
			String refs = getReferences(cls);
			if ((parentRefs.length() > 0) && (refs.length() > 0)) {
				parentRefs = parentRefs + ", ";
			}
			previousCls = previousCls + " " + _nclass;
			return parentRefs + refs;
		}
	}

	// GetReferences : toutes les references d'une classe
	public String getReferences(org.omg.mof.Model.Class cls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		logger.log(Level.FINER, "DTD getReferences of a class");
		String refs = "";
		Reference[] references = mofHelper.referencesOfClass(cls);
		for (int i = 0; i < references.length; i++) {
			org.omg.mof.Model.AssociationEnd _association_end =
				references[i].exposed_end();
			org.omg.mof.Model.AggregationKind _aggregation =
				_association_end.aggregation();

			//deux ligne rajoutes qui ne sont pas dans l'algorithme, pour que la DTD soit compatible
			org.omg.mof.Model.Association _association =
				org.omg.mof.Model.AssociationHelper.narrow(
					_association_end.container());
			if (!_association.is_derived()) {
				if (!_aggregation
					.equals(org.omg.mof.Model.AggregationKind.composite)) {
					String name = xmiHelper.qualifiedName(references[i]);
					String m = getReferenceMultiplicity(references[i]);
					String temp = name + m;
					if (refs.length() > 0)
						refs = refs + ", ";
					refs = refs + temp;
				}
			}
		}
		return refs;
	}

	//GetReferenceMultiplicity : donne la multiplicts d'une rfrence
	public String getReferenceMultiplicity(org.omg.mof.Model.Reference ref)
		throws org.omg.mof.Reflective.MofError {

		logger.log(Level.FINER, "DTD getReferenceMultiplicity of a reference");
		org.omg.mof.Model.MultiplicityType _mult =
			ref.referenced_end().multiplicity();
		int _upper = _mult.upper;
		int _lower = _mult.lower;
		String m;
		if (((_lower == 0) && (_upper == 1))
			|| (ref
				.referenced_end()
				.equals(org.omg.mof.Model.AggregationKind.composite)))
			m = "?";
		else if ((_lower == 1) && ((_upper > 1) || (_upper == -1)))
			m = "*"; /*ou + pour  DTD non relax */
		else if ((_lower != 1) && (_upper != 1))
			m = "*";
		else
			m = "?"; /*ou "" pour  DTD non relax */
		return m;
	}

	//GetContainedClasses : donne les classes contenues dans une classe dans le sens du namespace MOF
	public String getContainedClasses(
		org.omg.mof.Model.Class cls,
		String previousCls)
		throws org.omg.mof.Reflective.MofError , NotSet{
		logger.log(Level.FINER, "DTD getContainedClasses of a class");
		String _nclass = cls.name();
		if (previousCls.indexOf(_nclass) != -1)
			return "";
		else {
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			String parentClasses = "";
			for (int i = 0; i < parent.length; i++) {
				String temp =
					getContainedClasses(
						org.omg.mof.Model.ClassHelper.narrow(parent[i]),
						previousCls);
				if ((parentClasses.length() > 0) && (temp.length() > 0)) {
					parentClasses = parentClasses + ", ";
				}
				parentClasses = parentClasses + temp;
			}
			String classes = "";
			org.omg.mof.Model.ModelElement[] _contents = cls.contents();

			//rcuprer le contenu de la classe et voir les lments de type class
			for (int i = 0; i < _contents.length; i++) {
				org.omg.mof.Model.ModelElement _current = _contents[i];
				if (_current._is_a(org.omg.mof.Model.ClassHelper.id())) {
					String Temp = xmiHelper.qualifiedName(_current);
					if (classes.length() > 0) {
						classes = classes + "|";
					}
					classes = classes + Temp;
				}
			}
			if (classes.length() > 0) {
				if (parentClasses.length() > 0) {
					parentClasses = parentClasses + ", ";
				}
				classes = classes + "(" + classes + ")" + "*";
			}
			previousCls = previousCls + " " + _nclass;
			return parentClasses + classes;
		}
	}

	//GetAllComposedRoles : donne les rfrences qui ont rfrences dont l'ExposedEnd vers une AssociationEnd d'une association
	public String getAllComposedRoles(
		org.omg.mof.Model.Class cls,
		String previousCls)
		throws org.omg.mof.Reflective.MofError , NotSet{
		logger.log(Level.FINER, "DTD getAllComposedRoles of a class");
		String _nclass = cls.name();
		if (previousCls.indexOf(_nclass) != -1)
			return "";
		else {
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			String parentRoles = "";
			for (int i = 0; i < parent.length; i++) {
				String temp =
					getAllComposedRoles(
						org.omg.mof.Model.ClassHelper.narrow(parent[i]),
						previousCls);
				if ((parentRoles.length() > 0) && (temp.length() > 0)) {
					parentRoles = parentRoles + ", ";
				}
				parentRoles = parentRoles + temp;
			}
			String roles = getComposedRoles(cls);
			if ((roles.length() > 0) && (parentRoles.length() > 0)) {
				parentRoles = parentRoles + ", ";
			}
			previousCls = previousCls + " " + _nclass;
			return parentRoles + roles;
		}
	}

	// GetComposedRoles : donne les rfrences qui ont rfrences dont l'ExposedEnd vers une AssociationEnd d'une association
	public String getComposedRoles(org.omg.mof.Model.Class cls)
		throws org.omg.mof.Reflective.MofError , NotSet{
		logger.log(Level.FINER, "DTD getComposedRoles of a class");
		String rslt = "";
		Reference[] references = mofHelper.referencesOfClass(cls);
		for (int i = 0; i < references.length; i++) {
			org.omg.mof.Model.AggregationKind _aggregation =
				references[i].exposed_end().aggregation();
			if (_aggregation
				.equals(org.omg.mof.Model.AggregationKind.composite)) {
				String name = xmiHelper.qualifiedName(references[i]);
				String m = getReferenceMultiplicity(references[i]);
				if (rslt.length() > 0)
					rslt = rslt + ", ";
				rslt = rslt + name + m;
			}
		}
		return rslt;
	}

	// GetClasses : donne la classe ainsi que toutes les classe qui drivent d'elle
	public String getClasses(org.omg.mof.Model.Class Cls, String prevCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		logger.log(Level.FINER, "DTD getClasses of a class");
		//    String _nclass = Cls.name();
		//    if(prevCls.indexOf(_nclass)!= -1) return "";
		//    else{
		//      prevCls = prevCls+" "+_nclass;
		String rslt = xmiHelper.qualifiedName(Cls);
		org.omg.mof.Model.Class[] subclass = mofHelper.subClassesOfClass(Cls);
		for (int i = 0; i < subclass.length; i++) {
			//	String Temp = GetClasses(subclass[i], prevCls) ;
			String Temp = xmiHelper.qualifiedName(subclass[i]);
			if (Temp.length() > 0) {
				rslt = rslt + "| ";
			}
			rslt = rslt + Temp;
		}
		return rslt;

	}

	// GetClassLevelAttributes: tous les attributs ClassifierLevel dans les classes contenues dans le package
	public String getClassLevelAttributes(org.omg.mof.Model.Package pkg)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD getClassLevelAttributes of a package");
		// le traitement des packages parents et les package conteneur  ventiellement
		java.util.Vector _packages_c_p = new java.util.Vector();

		//le conteneur du package
		//org.omg.mof.Model.Namespace _container = pkg.container();
		//if(_container!=null) _packages_c_p.addElement(org.omg.mof.Model.PackageHelper.narrow(_container));

		//les packages dont il hrite
		org.omg.mof.Model.GeneralizableElement[] _sup_packages =
			pkg.supertypes();
		for (int i = 0; i < _sup_packages.length; i++) {
			_packages_c_p.addElement(
				org.omg.mof.Model.PackageHelper.narrow(_sup_packages[i]));
		}

		// dbut de l'algorithme
		String parentAtts = "";
		for (int i = 0; i < _packages_c_p.size(); i++) {
			parentAtts =
				getClassLevelAttributes(
					(org.omg.mof.Model.Package) _packages_c_p.elementAt(i));
		}
		String atts = "";
		//rcuprer les classes du package
		org.omg.mof.Model.Class[] classes = mofHelper.classesOfPackage(pkg);

		for (int i = 0; i < classes.length; i++) {
			String temp = getAttributes(classes[i], "classifier");
			if (temp.length() > 0) {
				atts = atts + "| ";
			}
			atts = atts + temp;
		}
		if (atts.length() > 0) {
			if (parentAtts.length() > 0)
				parentAtts = parentAtts + ", ";
			atts = "(" + atts + ")";
		}
		return parentAtts + atts;
	}

	/* GetNestedClassLevelAttributes : donne toutes les attributs non drivs
	des classes contenues dans un package ainsi que dans toutes les packages contenues */

	public String getNestedClassLevelAttributes(org.omg.mof.Model.Package pkg)
		throws org.omg.mof.Reflective.MofError , NotSet{
		logger.log(Level.FINER, "DTD getNestedClassLevelAttribut of a package");
		String rslt = "";
		//rcuprer les classes du package
		org.omg.mof.Model.Class[] classes = mofHelper.classesOfPackage(pkg);

		for (int i = 0; i < classes.length; i++) {
			String temp = getAttributes(classes[i], "classifier");
			if (temp.length() > 0) {
				if (rslt.length() > 0)
					rslt = rslt + "| ";
				temp = "(" + temp + ") ";
			}
			rslt = rslt + temp;
		}

		return rslt;
	}

	/*GetPackageClasses: donne toutes les classes contenues dans le pckage courant
	ainsi que toutes les pckages dont il hrite ou il est contenu */
	public String getPackageClasses(org.omg.mof.Model.Package pkg)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD getPackageClasses of a package");
		// le traitement des packages parents et les package conteneur  eventiellement
		java.util.Vector _packages_c_p = new java.util.Vector();

		//les packages dont il hrite
		org.omg.mof.Model.GeneralizableElement[] _sup_packages =
			pkg.supertypes();
		for (int i = 0; i < _sup_packages.length; i++) {
			_packages_c_p.addElement(
				org.omg.mof.Model.PackageHelper.narrow(_sup_packages[i]));
		}

		//le conteneur du package
		//org.omg.mof.Model.Namespace _container = pkg.container();
		//if(_container!=null) _packages_c_p.addElement(org.omg.mof.Model.PackageHelper.narrow(_container));

		// dbut de l'algorithme
		String parentClasses = "";
		for (int i = 0; i < _packages_c_p.size(); i++) {
			parentClasses =
				getPackageClasses(
					(org.omg.mof.Model.Package) _packages_c_p.elementAt(i));
		}
		String classes = "";
		//rcuprer les classes du package
		org.omg.mof.Model.Class[] _classes = mofHelper.classesOfPackage(pkg);

		for (int i = 0; i < _classes.length; i++) {
			String Temp = xmiHelper.qualifiedName(_classes[i]);
			if (classes.length() > 0) {
				classes = classes + "| ";
			}
			classes = classes + Temp;
		}
		if (parentClasses.length() > 0)
			parentClasses = parentClasses + "| ";
		return parentClasses + classes;
	}

	//GetContainedPackages : donne toutes les packages contenues dans le package courant ainsi que les packges contenus dans ses parents
	public String getContainedPackages(org.omg.mof.Model.Package pkg)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		logger.log(Level.FINER, "DTD getContainedPackages of a package");
		// dbut de l'algorithme
		String parentPkgs = "";
		//les packages dont il hrite
		org.omg.mof.Model.GeneralizableElement[] _sup_packages =
			pkg.supertypes();
		for (int i = 0; i < _sup_packages.length; i++) {
			parentPkgs =
				getContainedPackages(
					org.omg.mof.Model.PackageHelper.narrow(_sup_packages[i]));
		}
		//    //rcuprer les packages contenus dans le package
		//    java.util.Vector _packages = new java.util.Vector();
		//    _packages = mofHelper.PackagesOfPackage(pkg);

		String pkgs = "";
		//    for(int i=0; i< _packages.size() ; i++){
		//	org.omg.mof.Model.Package _sub_package = (org.omg.mof.Model.Package)_packages.elementAt(i);
		//	String Temp = mofHelper.QualifiedName(_sub_package);
		//	if(pkgs.length()>0) pkgs = pkgs+"|" ;
		//	pkgs = pkgs + Temp ;
		//    }
		if ((parentPkgs.length() > 0) && (pkgs.length() > 0))
			parentPkgs = pkgs + "|";
		return parentPkgs + pkgs;
	}

	/*GetUnreferencedAssociations : donne toutes les associations sans rfrences dans le package et ses packages parents  */
	public String getUnreferencedAssociations(org.omg.mof.Model.Package pkg)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD getUnreferencdAssociations of a package");
		// dbut de l'algorithme
		String parentAssns = "";

		//les packages dont il hrite
		org.omg.mof.Model.GeneralizableElement[] _sup_packages =
			pkg.supertypes();
		for (int i = 0; i < _sup_packages.length; i++) {
			parentAssns =
				getUnreferencedAssociations(
					org.omg.mof.Model.PackageHelper.narrow(_sup_packages[i]));
		}

		//rcuprer les associations contenus dans le package
		Association[] associations = mofHelper.associationsOfPackage(pkg);

		String assns = "";
		for (int i = 0; i < associations.length; i++) {
			if (!associations[i].is_derived()) {
				if (mofHelper.referencesOfAssociation(associations[i]).length == 0) {
					String temp = xmiHelper.qualifiedName(associations[i]);
					if (assns.length() > 0)
						assns = assns + "|";
					assns = assns + temp;
				}
			}
		}
		if ((parentAssns.length() > 0) && (assns.length() > 0))
			parentAssns = parentAssns + "|";
		return parentAssns + assns;
	}

	// 7.3 Rules Set 2: Grouped entities

	//7.3.4.1  OutputEntityDefs2
	public void outputEntityDefs2(org.omg.mof.Model.Package pkg)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD ouputEntityDes2 of a package");
		//rcuprer les classes du package
		org.omg.mof.Model.Class[] _classes_of_package = mofHelper.classesOfPackage(pkg);

		for (int i = 0; i < _classes_of_package.length; i++) {
			if (_classes_of_package[i].supertypes().length == 0) {
				outputPropertiesEntityDef2(_classes_of_package[i], ", ");
				outputRefsEntityDef2(_classes_of_package[i], ", ");
				outputCompsEntityDef2(_classes_of_package[i], ", ");
			}
		}
	}

	//7.3.4.1  OutputPropertiesEntityDef2
	public void outputPropertiesEntityDef2(
		org.omg.mof.Model.Class cls,
		String prevCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		logger.log(Level.FINER, "DTD outputPRopertiesEntityDef2 of a class");
		String _nclass = cls.name();
		if (prevCls.indexOf(_nclass) != -1) {
			String rslt = "";
		} else {
			String PropsEntityName = xmiHelper.qualifiedName(cls) + "Properties";
			String PropsList = getAttributes(cls, "instance");
			if (PropsList.length() > 0) {
				//pour la partie #8
			}
			prevCls = prevCls + " " + _nclass;

			org.omg.mof.Model.Class[] _sub_classes = mofHelper.subClassesOfClass(cls);
			for (int i = 0; i < _sub_classes.length; i++) {
				org.omg.mof.Model.Class _class = _sub_classes[i];
				outputPropertiesEntityDef2(_class, prevCls);
			}
		}
	}

	//7.3.4.1  OutputRefsEntityDef2
	public void outputRefsEntityDef2(
		org.omg.mof.Model.Class cls,
		String prevCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD ouputRefsEntityDef2 of a class");
		String _nclass = cls.name();
		if (prevCls.indexOf(_nclass) != -1) {
			String rslt = "";
		} else {
			String RefsEntityName = xmiHelper.qualifiedName(cls) + "Associations";
			String RefsList = getReferences(cls);
			if (RefsList.length() > 0) {
				RefsList = "(" + RefsList + ")";
				//pour la partie #9
			}
			prevCls = prevCls + " " + _nclass;

			org.omg.mof.Model.Class[] _sub_classes = mofHelper.subClassesOfClass(cls);
			for (int i = 0; i < _sub_classes.length; i++) {
				org.omg.mof.Model.Class _class = _sub_classes[i];
				outputRefsEntityDef2(_class, prevCls);
			}
		}
	}

	//7.3.4.1  OutputCompsEntityDef2
	public void outputCompsEntityDef2(
		org.omg.mof.Model.Class cls,
		String prevCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD outputCompsEntityDef2 of a class");
		String _nclass = cls.name();
		if (prevCls.indexOf(_nclass) != -1) {
			String rslt = "";
		} else {
			String CompsEntityName = xmiHelper.qualifiedName(cls) + "Compositions";
			String CompsList = getComposedRoles(cls);
			if (CompsList.length() > 0) {
				CompsList = "(" + CompsList + ")";
				//pour la partie #9
			}
			prevCls = prevCls + " " + _nclass;

			org.omg.mof.Model.Class[] _sub_classes = mofHelper.subClassesOfClass(cls);
			for (int i = 0; i < _sub_classes.length; i++) {
				org.omg.mof.Model.Class _class = _sub_classes[i];
				outputCompsEntityDef2(_class, prevCls);
			}
		}
	}

	//GetContainedClasses2 : donne les classes contenues dans une classe dans le sens du namespace MOF
	public String getContainedClasses2(org.omg.mof.Model.Class cls)
		throws org.omg.mof.Reflective.MofError, NotSet {

		logger.log(Level.FINER, "DTD getContainedClasses2 of a class");
		String classes = "";
		org.omg.mof.Model.ModelElement[] _contents = cls.contents();

		//rcuprer le contenu de la classe et voir les lments de type class
		for (int i = 0; i < _contents.length; i++) {
			org.omg.mof.Model.ModelElement _current = _contents[i];
			if (_current._is_a(org.omg.mof.Model.ClassHelper.id())) {
				String Temp = xmiHelper.qualifiedName(_current);
				if (classes.length() > 0) {
					classes = classes + "|";
				}
				classes = classes + Temp;
			}
		}

		return classes;
	}

	// GetPropertiesEntities2
	public String getPropertiesEntities2(
		org.omg.mof.Model.Class cls,
		String prevCls)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD getPropertiesEntities2");
		String _nclass = cls.name();
		if (prevCls.indexOf(_nclass) != -1)
			return "";
		else {
			String parentProps = "";
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			;
			for (int i = 0; i < parent.length; i++) {
				String temp =
					getPropertiesEntities2(
						org.omg.mof.Model.ClassHelper.narrow(parent[i]),
						prevCls);
				if (temp.length() > 0) {
					if (parentProps.length() > 0)
						parentProps = parentProps + ", ";
					parentProps = parentProps + temp;
				}
			}
			String ClassName = xmiHelper.qualifiedName(cls);
			String props = "";
			//ceci est gnr si la Properties ENTITY de la partie #8 a t gnr
			if (parentProps.length() > 0)
				parentProps = parentProps + ", ";
			props = "%" + ClassName + "Properties" + ";";
			prevCls = prevCls + " " + _nclass;
			return parentProps + props;
		}
	}

	// GetRefsEntities2
	public String getRefsEntities2(org.omg.mof.Model.Class cls, String prevCls)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD getRefsEntities2 of a class");
		String _nclass = cls.name();
		if (prevCls.indexOf(_nclass) != -1)
			return "";
		else {
			String parentRefs = "";
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			;
			for (int i = 0; i < parent.length; i++) {
				String temp =
					getRefsEntities2(
						org.omg.mof.Model.ClassHelper.narrow(parent[i]),
						prevCls);
				if (temp.length() > 0) {
					if (parentRefs.length() > 0)
						parentRefs = parentRefs + ", ";
					parentRefs = parentRefs + temp;
				}
			}
			String ClassName = xmiHelper.qualifiedName(cls);
			String refs = "";
			//ceci est gnr si la References ENTITY de la partie #9 a t gnr
			if (parentRefs.length() > 0)
				parentRefs = parentRefs + ", ";
			refs = "%" + ClassName + "Associations" + ";";
			prevCls = prevCls + " " + _nclass;
			return parentRefs + refs;
		}
	}

	// GetCompsEntities2
	public String getCompsEntities2(
		org.omg.mof.Model.Class cls,
		String prevCls)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD getCompsEntities2");
		String _nclass = cls.name();
		if (prevCls.indexOf(_nclass) != -1)
			return "";
		else {
			String parentComps = "";
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			;
			for (int i = 0; i < parent.length; i++) {
				String temp =
					getRefsEntities2(
						org.omg.mof.Model.ClassHelper.narrow(parent[i]),
						prevCls);
				if (temp.length() > 0) {
					if (parentComps.length() > 0)
						parentComps = parentComps + ", ";
					parentComps = parentComps + temp;
				}
			}
			String ClassName = xmiHelper.qualifiedName(cls);
			String comps = "";
			//ceci est gnr si la References ENTITY de la partie #9 a t gnr
			if (parentComps.length() > 0)
				parentComps = parentComps + ", ";
			comps = "%" + ClassName + "Compositions" + ";";
			prevCls = prevCls + " " + _nclass;
			return parentComps + comps;
		}
	}

	// 7.4 Rules Set 3: Hierarchical Grouped DTD

	//7.4.4.1
	public void outputEntityDefs3(org.omg.mof.Model.Package pkg)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD outputEntityDefs3 of a package");
		//rcuprer les classes du package
		org.omg.mof.Model.Class[] _classes_of_package = mofHelper.classesOfPackage(pkg);

		out.println();
		comment("The Entities of Properties");
		out.println();

		for (int i = 0; i < _classes_of_package.length; i++) {
			if (_classes_of_package[i].supertypes().length == 0) {
				outputPropertiesEntityDef3(_classes_of_package[i], "", "");
			}
		}

		out.println();
		comment("The Entities of References");
		out.println();

		for (int i = 0; i < _classes_of_package.length; i++) {
			if (_classes_of_package[i].supertypes().length == 0) {
				outputRefsEntityDef3(_classes_of_package[i], "", "");
			}
		}

		out.println();
		comment("The Entities of Compositions");
		out.println();

		for (int i = 0; i < _classes_of_package.length; i++) {
			if (_classes_of_package[i].supertypes().length == 0) {
				outputCompsEntityDef3(_classes_of_package[i], "", "");
			}
		}

		comment("");
		out.println();

	}

	//7.4.4.2
	private java.util.Hashtable _entity_props = new java.util.Hashtable();
	public void outputPropertiesEntityDef3(
		org.omg.mof.Model.Class cls,
		String prevCls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		String nm = cls.name();
		logger.log(Level.FINER, "DTD outputPropsertiesEntityDef3 of a class"+ nm);
		String _key = cls.ref_mof_id();
		if (_entity_props.containsKey(_key)) {
		} else {
			String _nclass = _key;
			if (prevCls.indexOf(_nclass) != -1) {
				String rslt = "";
			} else {
				String PropsEntityName =
					xmiHelper.qualifiedName(cls) + "Properties";
				String temp = baseCls;
				String PropsList = getAllInstanceAttributes3(cls, temp);
				if (PropsList.length() > 0) {
					PropsList = "(" + PropsList + ")";
					//pour la partie #8
					out.println(
						"<!ENTITY % "
							+ PropsEntityName
							+ " \""
							+ PropsList
							+ "\">");
					_entity_props.put(_key, "");
				}
				baseCls = baseCls + " " + _nclass;
				temp = baseCls;
				prevCls = prevCls + " " + _nclass;

				//gnrer les entits pour les autres classes
				org.omg.mof.Model.Class[] _sub_classes = mofHelper.subClassesOfClass(cls);
				for (int i = 0; i < _sub_classes.length; i++) {
					org.omg.mof.Model.Class _class = _sub_classes[i];
					baseCls = temp;
					outputPropertiesEntityDef3(_class, prevCls, baseCls);
				}
			}
		}
	}

	//7.4.4.3 OutputRefsEntityDef2
	private java.util.Hashtable _entity_refs = new java.util.Hashtable();
	public void outputRefsEntityDef3(
		org.omg.mof.Model.Class cls,
		String prevCls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD outputRefsEntityDef3 of a class");
		String _key = cls.ref_mof_id();
		if (_entity_refs.containsKey(_key)) {
		} else {
			String _nclass = _key;
			if (prevCls.indexOf(_nclass) != -1) {
				String rslt = "";
			} else {
				String RefsEntityName =
					xmiHelper.qualifiedName(cls) + "Associations";
				String temp = baseCls;
				String RefsList = getAllReferences3(cls, temp);
				if (RefsList.length() > 0) {
					RefsList = "(" + RefsList + ")";
					//pour la partie #9
					out.println(
						"<!ENTITY % "
							+ RefsEntityName
							+ " \""
							+ RefsList
							+ "\">");
					_entity_refs.put(_key, "");
				}
				baseCls = baseCls + " " + _nclass;
				temp = baseCls;
				prevCls = prevCls + " " + _nclass;

				//gnrer les entits pour les autres classes
				org.omg.mof.Model.Class[] _sub_classes = mofHelper.subClassesOfClass(cls);
				for (int i = 0; i < _sub_classes.length; i++) {
					org.omg.mof.Model.Class _class = _sub_classes[i];
					baseCls = temp;
					outputRefsEntityDef3(_class, prevCls, baseCls);
				}
			}
		}
	}

	//7.4.4.4
	private java.util.Hashtable _entity_comps = new java.util.Hashtable();
	public void outputCompsEntityDef3(
		org.omg.mof.Model.Class cls,
		String prevCls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD outputCompsEntiryDef3 of a class");
		String _key = cls.ref_mof_id();
		if (_entity_comps.containsKey(_key)) {
		} else {
			String _nclass = _key;
			if (prevCls.indexOf(_nclass) != -1) {
				String rslt = "";
			} else {
				String CompsEntityName =
					xmiHelper.qualifiedName(cls) + "Compositions";
				String temp = baseCls;
				String CompsList = getAllComposedRoles3(cls, temp);
				if (CompsList.length() > 0) {
					CompsList = "(" + CompsList + ")";
					//pour la partie #10
					out.println(
						"<!ENTITY % "
							+ CompsEntityName
							+ " \""
							+ CompsList
							+ "\">");
					_entity_comps.put(_key, "");
				}
				baseCls = baseCls + " " + _nclass;
				temp = baseCls;
				prevCls = prevCls + " " + _nclass;

				//gnrer les entits pour les autres classes
				org.omg.mof.Model.Class[] _sub_classes = mofHelper.subClassesOfClass(cls);
				for (int i = 0; i < _sub_classes.length; i++) {
					org.omg.mof.Model.Class _class = _sub_classes[i];
					baseCls = temp;
					outputCompsEntityDef3(_class, prevCls, baseCls);
				}
			}
		}
	}

	//7.4.4.5
	public String getAllInstanceAttributes3(
		org.omg.mof.Model.Class cls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD getAllInstanceAttributes3 of a class");
		String parentEntity = "";
		String parentContents = "";
		org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
		for (int i = 0; i < parent.length; i++) {
			org.omg.mof.Model.Class _supertype =
				org.omg.mof.Model.ClassHelper.narrow(parent[i]);
			String _key = _supertype.ref_mof_id();
			String _nclass = _key;
			if (baseCls.indexOf(_nclass) != -1) {

				// si une entit est gnre pour _supertype
				if (_entity_props.containsKey(_key)) {
					parentEntity =
						"%" + xmiHelper.qualifiedName(_supertype) + "Properties;";
				}
			} else {
				String temp = getParentAttributes3(_supertype, baseCls);
				if ((temp.length() > 0) && (parentContents.length() > 0)) {
					parentContents = parentContents + ", ";
				}
				parentContents = parentContents + temp;
			}
		}

		if ((parentEntity.length() > 0) && (parentContents.length() > 0))
			parentEntity = parentEntity + ", ";
		parentContents = parentEntity + parentContents;
		String temp = getAttributes(cls, "instance");
		if ((temp.length() > 0) && (parentContents.length() > 0))
			parentContents = parentContents + ", ";
		return parentContents + temp;
	}

	//7.4.4.6
	public String getAllReferences3(
		org.omg.mof.Model.Class cls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD getAllReferences3 of a class");
		String parentEntity = "";
		String parentContents = "";
		org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
		for (int i = 0; i < parent.length; i++) {
			org.omg.mof.Model.Class _supertype =
				org.omg.mof.Model.ClassHelper.narrow(parent[i]);
			String _key = _supertype.ref_mof_id();
			String _nclass = _key;
			if (baseCls.indexOf(_nclass) != -1) {
				// si une entit est gnre pour _supertype
				if (_entity_refs.containsKey(_key)) {
					parentEntity =
						"%"
							+ xmiHelper.qualifiedName(_supertype)
							+ "Associations;";
				}
			} else {
				String temp = getParentReferences3(_supertype, baseCls);
				if ((temp.length() > 0) && (parentContents.length() > 0)) {
					parentContents = parentContents + ", ";
				}
				parentContents = parentContents + temp;
			}
		}

		if ((parentEntity.length() > 0) && (parentContents.length() > 0))
			parentEntity = parentEntity + ", ";
		parentContents = parentEntity + parentContents;
		String temp = getReferences(cls);
		//      if (temp.length()>0){
		//	if(parentContents.length()> 0) parentContents = parentContents + ", " ;
		//	temp = "("+ temp +")" ;
		//      }
		if ((temp.length() > 0) && (parentContents.length() > 0)) {
			parentContents = parentContents + ", ";
			temp = "(" + temp + ")";
		}
		return parentContents + temp;
	}

	//7.4.4.7
	public String getAllComposedRoles3(
		org.omg.mof.Model.Class cls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD getAllComposedRoles3 of a class");
		String parentEntity = "";
		String parentContents = "";
		org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
		for (int i = 0; i < parent.length; i++) {
			org.omg.mof.Model.Class _supertype =
				org.omg.mof.Model.ClassHelper.narrow(parent[i]);
			String _key = _supertype.ref_mof_id();
			String _nclass = _key;
			if (baseCls.indexOf(_nclass) != -1) {
				// si une entit est gnre pour _supertype
				if (_entity_comps.containsKey(_key)) {
					parentEntity =
						"%"
							+ xmiHelper.qualifiedName(_supertype)
							+ "Compositions;";
				}
			} else {
				String temp = getParentCompositionRoles3(_supertype, baseCls);
				if ((temp.length() > 0) && (parentContents.length() > 0)) {
					parentContents = parentContents + ", ";
				}
				parentContents = parentContents + temp;
			}
		}

		if ((parentEntity.length() > 0) && (parentContents.length() > 0))
			parentEntity = parentEntity + ", ";
		parentContents = parentEntity + parentContents;
		String temp = getComposedRoles(cls);
		if ((temp.length() > 0) && (parentContents.length() > 0))
			parentContents = parentContents + ", ";
		return parentContents + temp;
	}

	//7.4.4.7
	public String getParentAttributes3(
		org.omg.mof.Model.Class cls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD getParentAttributes3 of a class");
		String _nclass = cls.ref_mof_id();
		if (baseCls.indexOf(_nclass) != -1) {
			return "";
		} else {
			String parentContents = "";
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			for (int i = 0; i < parent.length; i++) {
				org.omg.mof.Model.Class _supertype =
					org.omg.mof.Model.ClassHelper.narrow(parent[i]);
				String temp = getParentAttributes3(_supertype, baseCls);
				if ((temp.length() > 0) && (parentContents.length() > 0))
					parentContents = parentContents + ", ";
				parentContents = parentContents + temp;
			}
			String temp = getAttributes(cls, "instance");
			if ((temp.length() > 0) && (parentContents.length() > 0))
				parentContents = parentContents + ", ";
			return parentContents + temp;
		}
	}

	//7.4.4.8
	public String getParentReferences3(
		org.omg.mof.Model.Class cls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD getParentReferences3 of a class");
		String _nclass = cls.ref_mof_id();
		if (baseCls.indexOf(_nclass) != -1) {
			return "";
		} else {
			String parentContents = "";
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			for (int i = 0; i < parent.length; i++) {
				org.omg.mof.Model.Class _supertype =
					org.omg.mof.Model.ClassHelper.narrow(parent[i]);
				String temp = getParentAttributes3(_supertype, baseCls);
				if ((temp.length() > 0) && (parentContents.length() > 0))
					parentContents = parentContents + ", ";
				parentContents = parentContents + temp;
			}
			String temp = getReferences(cls);
			if ((temp.length() > 0) && (parentContents.length() > 0))
				parentContents = parentContents + ", ";
			return parentContents + temp;
		}
	}

	//7.4.4.9
	public String getParentCompositionRoles3(
		org.omg.mof.Model.Class cls,
		String baseCls)
		throws org.omg.mof.Reflective.MofError , NotSet {

		logger.log(Level.FINER, "DTD getParentCompositionRoles3 of a class");
		String _nclass = cls.name();
		if (baseCls.indexOf(_nclass) != -1) {
			return "";
		} else {
			String parentContents = "";
			org.omg.mof.Model.GeneralizableElement[] parent = cls.supertypes();
			for (int i = 0; i < parent.length; i++) {
				org.omg.mof.Model.Class _supertype =
					org.omg.mof.Model.ClassHelper.narrow(parent[i]);
				String temp = getParentCompositionRoles3(_supertype, baseCls);
				if ((temp.length() > 0) && (parentContents.length() > 0))
					parentContents = parentContents + ", ";
				parentContents = parentContents + temp;
			}
			String temp = getComposedRoles(cls);
			if ((temp.length() > 0) && (parentContents.length() > 0))
				parentContents = parentContents + ", ";
			return parentContents + temp;
		}
	}

	// Pseudo code pour gnrer la DTD
	// 7.3.3.1 rgle 1
	public void generate(org.omg.mof.Model.Package _package)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.mof.Reflective.NotSet,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds {

		logger.log(Level.FINE, "DTD generate the DTD");
		//Fixed DTD
		out.println(
			"<!-- ______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI is the top-level XML element for XMI transfer text -->");
		out.println(
			"<!-- ______________________________________________________________ -->");
		out.println(
			"<!ELEMENT XMI (XMI.header?, XMI.content?, XMI.difference*, XMI.extensions*)>");
		out.println("<!ATTLIST XMI");
		out.println("	xmi.version CDATA #FIXED \"1.0\"");
		out.println("	timestamp CDATA #IMPLIED");
		out.println("	verified (true | false) #IMPLIED");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.header contains documentation and identifies the model, -->");
		out.println("<!-- metamodel, and metametamodel -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println(
			"<!ELEMENT XMI.header (XMI.documentation?, XMI.model*, XMI.metamodel*, XMI.metametamodel*, XMI.import*)>");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println("<!-- documentation for transfer data -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println(
			"<!ELEMENT XMI.documentation (#PCDATA | XMI.owner | XMI.contact | XMI.longDescription | XMI.shortDescription | XMI.exporter | XMI.exporterVersion | XMI.notice)*>");
		out.println("<!ELEMENT XMI.owner ANY>");
		out.println("<!ELEMENT XMI.contact ANY>");
		out.println("<!ELEMENT XMI.longDescription ANY>");
		out.println("<!ELEMENT XMI.shortDescription ANY>");
		out.println("<!ELEMENT XMI.exporter ANY>");
		out.println("<!ELEMENT XMI.exporterVersion ANY>");
		out.println("<!ELEMENT XMI.exporterID ANY>");
		out.println("<!ELEMENT XMI.notice ANY>");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.element.att defines the attributes that each XML element -->");
		out.println(
			"<!-- that corresponds to a metamodel class must have to conform to -->");
		out.println("<!-- the XMI specification. -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println(
			"<!ENTITY % XMI.element.att \"xmi.id ID #IMPLIED xmi.label CDATA #IMPLIED xmi.uuid CDATA #IMPLIED \">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.link.att defines the attributes that each XML element that -->");
		out.println(
			"<!-- corresponds to a metamodel class must have to enable it to -->");
		out.println(
			"<!-- function as a simple XLink as well as refer to model -->");
		out.println("<!-- constructs within the same XMI file. -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println(
			"<!ENTITY % XMI.link.att \"href CDATA #IMPLIED xmi.idref IDREF #IMPLIED\">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.model identifies the model(s) being transferred -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.model ANY>");
		out.println("<!ATTLIST XMI.model");
		out.println("	%XMI.link.att; ");
		out.println("	xmi.name CDATA #REQUIRED");
		out.println("	xmi.version CDATA #IMPLIED");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.metamodel identifies the metamodel(s) for the transferred -->");
		out.println("<!-- data -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.metamodel ANY>");
		out.println(
			"<!ATTLIST XMI.metamodel %XMI.link.att;xmi.name CDATA #REQUIRED xmi.version CDATA #IMPLIED >");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.metametamodel identifies the metametamodel(s) for the -->");
		out.println("<!-- transferred data -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.metametamodel ANY>");
		out.println("<!ATTLIST XMI.metametamodel");
		out.println("	%XMI.link.att; ");
		out.println("	xmi.name CDATA #REQUIRED");
		out.println("	xmi.version CDATA #IMPLIED");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println("<!-- XMI.import identifies imported metamodel(s) -->");
		out.println("<!-- -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.import ANY>");
		out.println("<!ATTLIST XMI.import");
		out.println("	%XMI.link.att; ");
		out.println("	xmi.name CDATA #REQUIRED");
		out.println("	xmi.version CDATA #IMPLIED");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.content is the actual data being transferred -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.content ANY>");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.extensions contains data to transfer that does not conform -->");
		out.println("<!-- to the metamodel(s) in the header -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.extensions ANY>");
		out.println("<!ATTLIST XMI.extensions");
		out.println("	xmi.extender CDATA #REQUIRED");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- extension contains information related to a specific model -->");
		out.println(
			"<!-- construct that is not defined in the metamodel(s) in the -->");
		out.println("<!-- header -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.extension ANY>");
		out.println("<!ATTLIST XMI.extension");
		out.println("	%XMI.element.att; ");
		out.println("	%XMI.link.att; ");
		out.println("	xmi.extender CDATA #REQUIRED");
		out.println("	xmi.extenderID CDATA #IMPLIED");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.difference holds XML elements representing differences to -->");
		out.println("<!-- a base model -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println(
			"<!ELEMENT XMI.difference (XMI.difference | XMI.delete | XMI.add | XMI.replace)*>");
		out.println("<!ATTLIST XMI.difference");
		out.println("	%XMI.element.att; ");
		out.println("	%XMI.link.att; ");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.delete represents a deletion from a base model -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.delete EMPTY>");
		out.println("<!ATTLIST XMI.delete");
		out.println("	%XMI.element.att; ");
		out.println("	%XMI.link.att; ");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println("<!-- XMI.add represents an addition to a base model -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.add ANY>");
		out.println("<!ATTLIST XMI.add");
		out.println("	%XMI.element.att; ");
		out.println("	%XMI.link.att; ");
		out.println("	xmi.position CDATA \"-1\"");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.replace represents the replacement of a model construct -->");
		out.println("<!-- with another model construct in a base model -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.replace ANY>");
		out.println("<!ATTLIST XMI.replace");
		out.println("	%XMI.element.att; ");
		out.println("	%XMI.link.att; ");
		out.println("	xmi.position CDATA \"-1\"");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- XMI.reference may be used to refer to data types not defined -->");
		out.println("<!-- in the metamodel -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.reference ANY>");
		out.println("<!ATTLIST XMI.reference");
		out.println("	%XMI.link.att; ");
		out.println(">");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!-- -->");
		out.println(
			"<!-- This section contains the declaration of XML elements -->");
		out.println("<!-- representing data types -->");
		out.println(
			"<!-- _______________________________________________________________ -->");
		out.println("<!ELEMENT XMI.TypeDefinitions ANY>");
		out.println("<!ELEMENT XMI.field ANY>");
		out.println("<!ELEMENT XMI.seqItem ANY>");
		out.println("<!ELEMENT XMI.octetStream (#PCDATA)>");
		out.println("<!ELEMENT XMI.unionDiscrim ANY>");
		out.println("<!ELEMENT XMI.enum EMPTY>");
		out.println("<!ATTLIST XMI.enum");
		out.println("	xmi.value CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT XMI.any ANY>");
		out.println("<!ATTLIST XMI.any");
		out.println("	%XMI.link.att; ");
		out.println("	xmi.type CDATA #IMPLIED");
		out.println("	xmi.name CDATA #IMPLIED");
		out.println(">");
		out.println(
			"<!ELEMENT XMI.CorbaTypeCode (XMI.CorbaTcAlias | XMI.CorbaTcStruct | XMI.CorbaTcSequence | XMI.CorbaTcArray | XMI.CorbaTcEnum | XMI.CorbaTcUnion | XMI.CorbaTcExcept | XMI.CorbaTcString | XMI.CorbaTcWstring | XMI.CorbaTcShort | XMI.CorbaTcLong | XMI.CorbaTcUshort | XMI.CorbaTcUlong | XMI.CorbaTcFloat | XMI.CorbaTcDouble | XMI.CorbaTcBoolean | XMI.CorbaTcChar | XMI.CorbaTcWchar | XMI.CorbaTcOctet | XMI.CorbaTcAny | XMI.CorbaTcTypeCode | XMI.CorbaTcPrincipal | XMI.CorbaTcNull | XMI.CorbaTcVoid | XMI.CorbaTcLongLong | XMI.CorbaTcLongDouble)>");
		out.println("<!ATTLIST XMI.CorbaTypeCode");
		out.println("	%XMI.element.att; ");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcAlias (XMI.CorbaTypeCode)>");
		out.println("<!ATTLIST XMI.CorbaTcAlias");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println("	xmi.tcId CDATA #IMPLIED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcStruct (XMI.CorbaTcField)*>");
		out.println("<!ATTLIST XMI.CorbaTcStruct");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println("	xmi.tcId CDATA #IMPLIED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcField (XMI.CorbaTypeCode)>");
		out.println("<!ATTLIST XMI.CorbaTcField");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println(">");
		out.println(
			"<!ELEMENT XMI.CorbaTcSequence (XMI.CorbaTypeCode | XMI.CorbaRecursiveType)>");
		out.println("<!ATTLIST XMI.CorbaTcSequence");
		out.println("	xmi.tcLength CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaRecursiveType EMPTY>");
		out.println("<!ATTLIST XMI.CorbaRecursiveType");
		out.println("	xmi.offset CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcArray (XMI.CorbaTypeCode)>");
		out.println("<!ATTLIST XMI.CorbaTcArray");
		out.println("	xmi.tcLength CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcObjRef EMPTY>");
		out.println("<!ATTLIST XMI.CorbaTcObjRef");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println("	xmi.tcId CDATA #IMPLIED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcEnum (XMI.CorbaTcEnumLabel)>");
		out.println("<!ATTLIST XMI.CorbaTcEnum");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println("	xmi.tcId CDATA #IMPLIED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcEnumLabel EMPTY>");
		out.println("<!ATTLIST XMI.CorbaTcEnumLabel");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println(">");
		out.println(
			"<!ELEMENT XMI.CorbaTcUnionMbr (XMI.CorbaTypeCode, XMI.any)>");
		out.println("<!ATTLIST XMI.CorbaTcUnionMbr");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println(">");
		out.println(
			"<!ELEMENT XMI.CorbaTcUnion (XMI.CorbaTypeCode, XMI.CorbaTcUnionMbr*)>");
		out.println("<!ATTLIST XMI.CorbaTcUnion");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println("	xmi.tcId CDATA #IMPLIED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcExcept (XMI.CorbaTcField)*>");
		out.println("<!ATTLIST XMI.CorbaTcExcept");
		out.println("	xmi.tcName CDATA #REQUIRED");
		out.println("	xmi.tcId CDATA #IMPLIED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcString EMPTY>");
		out.println("<!ATTLIST XMI.CorbaTcString");
		out.println("	xmi.tcLength CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcWstring EMPTY>");
		out.println("<!ATTLIST XMI.CorbaTcWstring");
		out.println("	xmi.tcLength CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcFixed EMPTY>");
		out.println("<!ATTLIST XMI.CorbaTcFixed");
		out.println("	xmi.tcDigits CDATA #REQUIRED");
		out.println("	xmi.tcScale CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT XMI.CorbaTcShort EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcLong EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcUshort EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcUlong EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcFloat EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcDouble EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcBoolean EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcChar EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcWchar EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcOctet EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcAny EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcTypeCode EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcPrincipal EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcNull EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcVoid EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcLongLong EMPTY>");
		out.println("<!ELEMENT XMI.CorbaTcLongDouble EMPTY>");

		//# 15
		entityDTD(_package);
		//#2
		packageDTD(_package);
		logger.log(Level.FINE, "DTD DTD is Generated");
	}

	//7.3.3.2
	public void packageDTD(org.omg.mof.Model.Package _package)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.mof.Reflective.NotSet,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds {

		logger.log(Level.FINER, "DTD packageDTD of a package");
		// Packages DTD
		// Traitement des attributs classifier level des classes
		comment("The Classifier Attributs");
		out.println();
		org.omg.mof.Model.Class[] _classes_package = mofHelper.classesOfPackage(_package);
		for (int i = 0; i < _classes_package.length; i++) {
			MofAttribute[] _attributes = mofHelper.attributesOfClass(
					_classes_package[i],
					org.omg.mof.Model.ScopeKind.classifier_level,
					true);
			for (int j = 0; j < _attributes.length; j++) {
				//#4
				attributeElementDTD(_attributes[j]);
			}
		}
		comment("");
		out.println();

		// traitement des associations
		comment("The Associations");
		out.println();
		Association[] _associations_package = mofHelper.associationsOfPackage(_package);
		for (int i = 0; i < _associations_package.length; i++) {
			org.omg.mof.Model.AssociationEnd[] _ass_ends =
				mofHelper.associationEndsOfAssociation(_associations_package[i]);
			boolean _has_compo = false;
			int j = 0;
			while ((j < _ass_ends.length) && (!_has_compo)) {
				org.omg.mof.Model.AssociationEnd _ass_end = _ass_ends[j];
				if (_ass_end
					.aggregation()
					.equals(org.omg.mof.Model.AggregationKind.composite))
					_has_compo = true;
				j++;
			}
			if (mofHelper.referencesOfAssociation(_associations_package[i]).length == 0) {
							//#16
							associationDTD(_associations_package[i]);
			} else if(_has_compo) {
				//#12
				compositionDTD(_associations_package[i]);
			}
		}
		comment("");
		out.println();

		// traitement des classes
		comment("The Classes");
		out.println();
		for (int i = 0; i < _classes_package.length; i++) {
			//#3
			classDTD(_classes_package[i]);
		}

		//#14
		comment("");
		out.println();
		packageElementDef(_package);
	}

	//7.3.3.3
	public void classDTD(org.omg.mof.Model.Class _class)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.mof.Reflective.NotSet,
			org.omg.CORBA.TypeCodePackage.Bounds,
			org.omg.CORBA.TypeCodePackage.BadKind {

		logger.log(Level.FINER, "DTD classDTD of a class");

		//rcuprer les   attributs
		MofAttribute[] _attributes = mofHelper.attributesOfClass(
				_class,
				org.omg.mof.Model.ScopeKind.instance_level,
				true);
		comment(_class.name());
		out.println();
		for (int i = 0; i < _attributes.length; i++) {
			//#4
			attributeElementDTD(_attributes[i]);
		}

		//rcuprer les   rfrences
		Reference[] _references = mofHelper.referencesOfClass(_class);

		for (int i = 0; i < _references.length; i++) {
			org.omg.mof.Model.AssociationEnd _association_end =
				_references[i].exposed_end();
			org.omg.mof.Model.Association _association =
				org.omg.mof.Model.AssociationHelper.narrow(
					_association_end.container());

			if (!_association.is_derived()) {
				if (!_association_end
					.aggregation()
					.equals(org.omg.mof.Model.AggregationKind.composite)) {
					{
						//#7
						referenceElementDef(_references[i]);
					}
				}
			}
		}

		//#11
		classElementDef(_class);
	}

	//7.4.3.4
	private java.util.Hashtable _attribute_elementDTD =
		new java.util.Hashtable();
	public void attributeElementDTD(org.omg.mof.Model.MofAttribute _attribute)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD attributeElementDTD of an attribute");
		org.omg.mof.Model.Classifier _type = _attribute.type();
		if (_type._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
			org.omg.mof.Model.DataType _data_type =
				org.omg.mof.Model.DataTypeHelper.narrow(_type);
			org.omg.CORBA.TypeCode _type_code = _data_type.type_code();
			int _kind = _type_code.kind().value();
			if ((_kind == org.omg.CORBA.TCKind._tk_boolean)
				|| (_kind == org.omg.CORBA.TCKind._tk_enum)) {
				String _key = _type.ref_mof_id();
				if (!_attribute_elementDTD.containsKey(_key)) {
					//#5
					attributeEntityDef(_data_type);
					_attribute_elementDTD.put(_key, _type);
				}
			}
		}
		//#6
		attributeElementDef(_attribute);

	}

	//7.3.3.5
	public void attributeEntityDef(org.omg.mof.Model.DataType _data_type)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds , NotSet{

		logger.log(Level.FINER, "DTD attributeEntityDef of a DataType");
		org.omg.CORBA.TypeCode _type_code = _data_type.type_code();
		int _kind = _type_code.kind().value();
		String TypeName = xmiHelper.qualifiedName(_data_type);
		String enumvalues = "";
		if (_kind == org.omg.CORBA.TCKind._tk_enum) {
			for (int i = 0; i < _type_code.member_count(); i++) {
				if (enumvalues.length() > 0)
					enumvalues = enumvalues + " | ";
				enumvalues = enumvalues + _type_code.member_name(i);
			}
		} else {
			if (_kind == org.omg.CORBA.TCKind._tk_boolean) {
				enumvalues = "true | false ";
			}
		}
		//gnrer l'entit du datatype
		out.println(
			"<!ENTITY % "
				+ TypeName
				+ " \"xmi.value ("
				+ enumvalues
				+ ") #REQUIRED\">");

	}

	//7.4.3.6
	public void attributeElementDef(org.omg.mof.Model.MofAttribute _attribute)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD attributeElementDef of an attribute");

		String AttribName = xmiHelper.qualifiedName(_attribute);
		String AttribContents;
		String AttribAttList = "";
		String TypeName;
		boolean _tk_bool_enum = false;
		org.omg.mof.Model.Classifier _type = _attribute.type();
		if (_type._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
			org.omg.mof.Model.DataType _data_type =
				org.omg.mof.Model.DataTypeHelper.narrow(_type);
			org.omg.CORBA.TypeCode _type_code = _data_type.type_code();
			int _kind = _type_code.kind().value();
			if ((_kind == org.omg.CORBA.TCKind._tk_boolean)
				|| (_kind == org.omg.CORBA.TCKind._tk_enum)) {
				AttribContents = "EMPTY";
				TypeName = xmiHelper.qualifiedName(_data_type);
				AttribAttList = "\n\t\t%" + TypeName + ";";
				_tk_bool_enum = true;
			} else {
				if ((_kind == org.omg.CORBA.TCKind._tk_string)
					|| (_kind == org.omg.CORBA.TCKind._tk_wstring)
					|| (_kind == org.omg.CORBA.TCKind._tk_char)
					|| (_kind == org.omg.CORBA.TCKind._tk_wchar)) {
					AttribContents = "(#PCDATA | XMI.reference)*";
				} else {
					if (_kind == org.omg.CORBA.TCKind._tk_struct) {
						AttribContents = "(XMI.field | XMI.reference)*";
					} else {
						if (_kind == org.omg.CORBA.TCKind._tk_union) {
							AttribContents = "(XMI.unionDiscrim | XMI.field)*";
						} else {
							if ((_kind == org.omg.CORBA.TCKind._tk_sequence)
								|| (_kind == org.omg.CORBA.TCKind._tk_array)) {
								AttribContents =
									"(XMI.octetStream | XMI.seqltem | XMI.reference)*";
							} else {
								if (_kind == org.omg.CORBA.TCKind._tk_any) {
									AttribContents = "(XMI.any)";
								} else {
									if (_kind
										== org.omg.CORBA.TCKind._tk_objref) {
										AttribContents = "(XMI.reference)";
									} else {
										if (_kind
											== org
												.omg
												.CORBA
												.TCKind
												._tk_TypeCode) {
											AttribContents =
												"(XMI.CorbaTypeCode | XMI.reference)";
										} else {
											AttribContents =
												"(#PCDATA | XMI.reference)*";
										}
									}
								}
							}
						}
					}
				}
			}
		} else {
			// c'est une classe
			org.omg.mof.Model.Class _class =
				org.omg.mof.Model.ClassHelper.narrow(_type);
			AttribContents = "(" + getClasses(_class, "") + ")";
		}

		//gnrer les lments attributs
		out.println("<!ELEMENT " + AttribName + " " + AttribContents + ">");
		if (_tk_bool_enum) {
			out.println(
				"<!ATTLIST " + AttribName + " " + AttribAttList + "\n>");
		}
		out.println();
	}

	//7.4.3.7
	public void referenceElementDef(org.omg.mof.Model.Reference _reference)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD referenceElementDef of a reference");
		String RefName = xmiHelper.qualifiedName(_reference);
		org.omg.mof.Model.Class cls =
			org.omg.mof.Model.ClassHelper.narrow(_reference.type());
		String m = getReferenceMultiplicity(_reference);
		String RefContents = "(" + getClasses(cls, "") + ")" + m;

		//gnrer les lments references
		out.println("<!ELEMENT " + RefName + " " + RefContents + ">");
		out.println();
	}

	//7.4.3.8
	public void propertiesEntityDef(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		logger.log(Level.FINER, "DTD propertiesEntityDef of a class");
		// revoir
		outputPropertiesEntityDef3(_class, "", "");
	}

	//7.4.3.9
	public void refsEntityDef(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD refsEntityDef of a class");
		//
		outputRefsEntityDef3(_class, "", "");
	}

	//7.4.3.10
	public void compsEntityDef(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD compsEntityDef of a class");
		//
		outputCompsEntityDef3(_class, "", "");
	}

	//7.4.3.11
	public void classElementDef(org.omg.mof.Model.Class _class)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD classElementDef of a class");

		String ClassName = xmiHelper.qualifiedName(_class);
		String props = "";
		String _key = _class.ref_mof_id();

		//si les Properties Entity sont gnrs pour cette classe
		if (_entity_props.containsKey(_key))
			props = "%" + ClassName + "Properties" + ";";
		String refs = "";

		//si les References Entity sont gnrs pour cette classe
		if (_entity_refs.containsKey(_key))
			refs = ", " + "%" + ClassName + "Associations" + ";";
		refs = "(" + "XMI.extension" + "*" + refs + ")";

		String comps1 = "";
		//si les Compositions Entity sont gnrs pour cette classe
		if (_entity_comps.containsKey(_key))
			comps1 = "%" + ClassName + "Compositions" + ";";

		String comps2 = getContainedClasses(_class, "");
		String ClassContents =
			props + ", " + refs + ", " + comps1 + ", " + comps2;

		//Enlever les virgules succssives en cas d'lment vide
		ClassContents = removeDanglingCommas(ClassContents);

		//
		if (ClassContents.length() == 0)
			ClassContents = "EMPTY";
		else
			ClassContents = "(" + ClassContents + ")" + "?";
		String ClassAttlisttems =
			"\n\t\t%XMI.element.att; \n\t\t%XMI.link.att;";

		//gnrer les lments contenus dans la class
		out.println("<!ELEMENT " + ClassName + " " + ClassContents + ">");
		out.println("<!ATTLIST " + ClassName + " " + ClassAttlisttems + "\n>");
	}

	//7.4.3.12
	public void compositionDTD(org.omg.mof.Model.Association _composition)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD compositionDTD of an association");
		//#13
		comment(_composition.name());
		out.println();
		compositionElementDef(_composition);
	}

	//7.4.3.13
	public void compositionElementDef(
		org.omg.mof.Model.Association _composition)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD compositionElementDef of a composition");

		org.omg.mof.Model.Reference[] _references =
			mofHelper.referencesOfAssociation(_composition);
		boolean _find = false;
		int i = 0;
		org.omg.mof.Model.Reference _reference = null;
		while ((!_find) && (i < _references.length)) {
			_reference = _references[i];
			if (_reference
				.exposed_end()
				.aggregation()
				.equals(org.omg.mof.Model.AggregationKind.composite))
				_find = true;
			i++;
		}

		org.omg.mof.Model.Class Container =
			org.omg.mof.Model.ClassHelper.narrow(_reference.container());
		String RoleName = xmiHelper.qualifiedName(_reference);
		org.omg.mof.Model.Class Contained =
			org.omg.mof.Model.ClassHelper.narrow(
				_reference.referenced_end().type());
		String CompContents = getClasses(Contained, "");
		//la multiplicit rajouter qui n'existe pas dans la norme "m"
		String m = getReferenceMultiplicity(_reference);
		CompContents = "(" + CompContents + ")" + m;

		//gnrer l'lmnt de la composition
		out.println("<!ELEMENT " + RoleName + " " + CompContents + ">");

	}

	//7.4.3.14
	public void packageElementDef(org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {

		logger.log(Level.FINER, "DTD packageElementDef of package");
		String PkgName = xmiHelper.format1(_package.name());
		String atts = getClassLevelAttributes(_package);
		String atts2 = "";

		//    java.util.Vector _sub_packages = new java.util.Vector();
		//    _sub_packages = mofHelper.PackagesOfPackage(_package);
		//    for(int i= 0; i< _sub_packages.size(); i++) {
		//      org.omg.mof.Model.Package _sub_package = (org.omg.mof.Model.Package) _sub_packages.elementAt(i);
		//      String temp = GetNestedClassLevelAttributes(_sub_package) ;
		//      if(temp.length()>0) {
		//	if(atts2.length()>0) atts2 = "(" + atts2 +")" ;
		//	temp = atts2 + temp ;
		//      }
		//      atts2 = atts2 + temp ;
		//    }
		String classes = getPackageClasses(_package);
		String assns = getUnreferencedAssociations(_package);
		String pkgs = getContainedPackages(_package);
		String PkgContents = "(" + atts + ")" + "(" + atts2 + ")";
		if (assns.length() != 0)
			if (pkgs.length() != 0)
				PkgContents =
					PkgContents
						+ "("
						+ classes
						+ " | "
						+ assns
						+ "|"
						+ pkgs
						+ ")*";
			else
				PkgContents =
					PkgContents + "(" + classes + " | " + assns + ")*";
		else if (pkgs.length() != 0)
			PkgContents = PkgContents + "(" + classes + " | " + pkgs + ")*";
		else
			PkgContents = PkgContents + "(" + classes + ")*";

		//enlever les parenthses succssives en cas d'lment vide
		for (int j = 0; j < PkgContents.length() - 1; j++) {
			for (int i = 0; i < PkgContents.length() - 1; i++) {
				String _v1 = String.valueOf(PkgContents.charAt(i));
				String _v2 = String.valueOf(PkgContents.charAt(i + 1));
				if ((_v1.compareTo("(") == 0) && (_v2.compareTo(")") == 0)) {
					PkgContents =
						PkgContents.substring(0, i)
							+ PkgContents.substring(i + 2, PkgContents.length());
				}
			}
		}

		//enlever les virgules succssives en cas d'lment vide
		PkgContents = removeDanglingCommas(PkgContents);

		if (PkgContents.length() > 0)
			PkgContents = "(" + PkgContents + ")";
		else
			PkgContents = "EMPTY";
		String PkgAttlistItems = "\n\t\t%XMI.element.att; \n\t\t%XMI.link.att;";
		comment(_package.name());
		out.println();
		out.println();
		out.println("<!ELEMENT " + PkgName + " " + PkgContents + ">");
		out.println("<!ATTLIST " + PkgName + " " + PkgAttlistItems + "\n>");
	}

	//7.4.3.15
	public void entityDTD(org.omg.mof.Model.Package _package)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		logger.log(Level.FINER, "DTD entityDTD of a package");
		outputEntityDefs3(_package);
	}

	//7.4.3.16
	public void associationDTD(org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD associationDTD of an association");
		org.omg.mof.Model.AssociationEnd[] _association_ends =
			mofHelper.associationEndsOfAssociation(_association);
		comment(_association.name());
		out.println();
		associationEndDef(_association_ends[0]);
		associationEndDef(_association_ends[1]);
		associationDef(_association);
	}

	//7.4.3.17
	public void associationEndDef(
		org.omg.mof.Model.AssociationEnd _association_end)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD associationEndDef of an association end");
		String EndName = xmiHelper.qualifiedName(_association_end);
		String EndAtts = "\n\t\t%XMI.link.att;";
		out.println("<!ELEMENT " + EndName + " EMPTY >");
		out.println("<!ATTLIST " + EndName + " " + EndAtts + "\n>");

	}

	//7.4.3.18
	public void associationDef(org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError , NotSet{

		logger.log(Level.FINER, "DTD associationDef of an association");
		String AssnName = xmiHelper.qualifiedName(_association);
		String EndAtts = "\n\t\t%XMI.element.att;";
		out.println("<!ELEMENT " + AssnName + " EMPTY >");
		out.println("<!ATTLIST " + AssnName + " " + EndAtts + "\n>");
	}

	//Enlever les virgules succssives en cas d'lment vide
	public String removeDanglingCommas(String _string) {

		logger.log(Level.FINER, "DTD removeDanglingCommas");
		String rslt = _string;
		if (rslt.matches(" *,.*"))
			rslt = rslt.substring(rslt.indexOf(",") + 1);
		int i = 0;
		int j;
		while (i < rslt.length()) {
			String _v1 = String.valueOf(rslt.charAt(i));
			if ((_v1.compareTo(",") == 0) && (i < rslt.length() - 1)) {
				j = i;
				i++;
				_v1 = String.valueOf(rslt.charAt(i));
				while ((_v1.compareTo(" ") == 0) && (i < rslt.length() - 1)) {
					i++;
					_v1 = String.valueOf(rslt.charAt(i));
				}
				if (_v1.compareTo(",") == 0) {
					rslt =
						rslt.substring(0, j + 1)
							+ rslt.substring(i + 1, rslt.length());
					i = j;
				}
			} else
				i++;
		}
		rslt = rslt.trim();
		if (rslt.endsWith(","))
			rslt = rslt.substring(0, rslt.length() - 1);
		return rslt;
	}

	//Commentaire
	public void comment(String _string) {
		String _temp =
			"<!-- _______________________________________________________________ -->";
		if (_string.compareTo("") == 0)
			out.println(_temp);
		else {
			int i = _string.length();
			_temp =
				_temp.substring(0, 31)
					+ " "
					+ _string
					+ " "
					+ _temp.substring(33 + i, _temp.length());
			out.println(_temp);
		}
	}

	/**
	 * Returns the mofHelper.
	 * @return MOFCommon
	 */
	public MOFCommon getMofHelper() {
		return mofHelper;
	}

	/**
	 * Returns the xmiHelper.
	 * @return XMICommon
	 */
	public XMICommon getXmiHelper() {
		return xmiHelper;
	}

	/**
	 * Sets the mofHelper.
	 * @param mofHelper The mofHelper to set
	 */
	public void setMofHelper(MOFCommon mofHelper) {
		this.mofHelper = mofHelper;
	}

	/**
	 * Sets the xmiHelper.
	 * @param xmiHelper The xmiHelper to set
	 */
	public void setXmiHelper(XMICommon xmiHelper) {
		this.xmiHelper = xmiHelper;
	}

}
