/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.exporter;

import java.util.Vector;

import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.objectweb.modfact.corba.xmiio.common.AbstractXMIIOGenerator;
import org.omg.mof.Reflective.NotSet;

/**
 * The XMI Export generator.
 * This class allows the generation of the XMI exporter (into XMI).
 */
public class GenerationXMIExport extends AbstractXMIIOGenerator {

	/** The IOR output file name. */
	private String _ior_filename = new String("");

	/** The logger. */
	protected ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiio.exporter.GenerationXMIExport");

	/**
	 * Default Constructor.
	 */
	public GenerationXMIExport() {
		super();
		ModFactLogger logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiio.exporter.GenerationXMIExportApplication");
	}

	/**
	 * Set the IOR output file name.
	 * @param file_ The file name.
	 */
	public void setIOR(String file_) {
		_ior_filename = file_;
	}

	/**
	 * XMIExport generation.
	 * @param package_ The root package.
	 */
	public void generateXMIExport(org.omg.mof.Model.Package package_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds, java.io.IOException {
		_logger.log(Level.FINE, "XMIExport beginning of the generation");
		String className = getClassName(package_.name());
		_logger.log(Level.INFO, className + " generation ...");
		packageModuleTemplate(package_);
		flushFile();
	}

	/**
	 * Get the class name.
	 * @param package_name_ The root package name.
	 * @return The class name.
	 */
	protected String getClassName(String package_name_) {
		return super.getClassName(package_name_, "XMIExport");
	}

	/**
	 * Find all the classes of the package.
	 * @param package_ The package to parse.
	 * @return The found classes.
	 */
	protected java.util.Vector findAllClasses(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError {
		org.omg.mof.Model.ModelElement[] contents = package_.contents();
		java.util.Vector classes = new java.util.Vector();
		for (int i = 0; i < contents.length; i++) {
			if (contents[i]._is_a(org.omg.mof.Model.ClassHelper.id()))
				classes.addElement(org.omg.mof.Model.ClassHelper.narrow(contents[i]));
			else if (contents[i]._is_a(org.omg.mof.Model.ImportHelper.id())) {
				org.omg.mof.Model.Import imp = org.omg.mof.Model.ImportHelper.narrow(contents[i]);
				java.util.Vector call = findAllClasses(org.omg.mof.Model.PackageHelper.narrow(imp.imported_namespace()));
				classes.addAll(call);
			}
		}
		return classes;
	}

	/**
	 * Find all the DataType Attributes of the package.
	 * @param package_ The package to parse.
	 * @return The found DataType Attribute.
	 */
	protected java.util.Vector findAllDataTypeAttributes(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError {
		org.omg.mof.Model.ModelElement[] contents = package_.contents();
		java.util.Vector attributes = new java.util.Vector();
		for (int i = 0; i < contents.length; i++) {
			if (contents[i]._is_a(org.omg.mof.Model.DataTypeHelper.id()))
				attributes.add(org.omg.mof.Model.DataTypeHelper.narrow(contents[i]));
			else if (contents[i]._is_a(org.omg.mof.Model.ImportHelper.id())) {
				org.omg.mof.Model.Import imp = org.omg.mof.Model.ImportHelper.narrow(contents[i]);
				java.util.Vector call = findAllDataTypeAttributes(org.omg.mof.Model.PackageHelper.narrow(imp.imported_namespace()));
				java.util.Iterator iterator = call.iterator();
				while (iterator.hasNext()) {
					boolean found = false;
					org.omg.CORBA.Object next = (org.omg.CORBA.Object) iterator.next();
					for (int k = 0; k < attributes.size(); k++) {
						org.omg.CORBA.Object attr = (org.omg.CORBA.Object) attributes.elementAt(k);
						if (attr._is_equivalent(next))
							found = true;
					}
					if (!found)
						attributes.add(next);
				}
			}
		}
		return attributes;
	}

	/**
	 * Find all the Class Attributes of the package.
	 * @param package_ The package to parse.
	 * @return The found Class Attributes.
	 */
	protected java.util.Vector findAllClassAttributes(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError {
		org.omg.mof.Model.ModelElement[] contents = package_.contents();
		java.util.Vector attributes = new java.util.Vector();
		for (int i = 0; i < contents.length; i++) {
			if (contents[i]._is_a(org.omg.mof.Model.ClassHelper.id())) {
				org.omg.mof.Model.Class class_ = org.omg.mof.Model.ClassHelper.narrow(contents[i]);
				org.omg.mof.Model.Reference references[] = mofHelper.referencesOfClass(class_);
				for (int l = 0; l < references.length; l++) {
					org.omg.mof.Model.Classifier classifier = references[l].type();
					if (!attributes.contains(classifier))
						attributes.add(classifier);
				}
				org.omg.mof.Model.MofAttribute attr[] =
					mofHelper.allAttributesOfClass(class_, org.omg.mof.Model.ScopeKind.instance_level, true);
				for (int l = 0; l < attr.length; l++) {
					org.omg.mof.Model.Classifier classifier = attr[l].type();
					if (classifier._is_a(org.omg.mof.Model.ClassHelper.id()) && !attributes.contains(classifier))
						attributes.add(classifier);
				}
			} else if (contents[i]._is_a(org.omg.mof.Model.ImportHelper.id())) {
				org.omg.mof.Model.Import imp = org.omg.mof.Model.ImportHelper.narrow(contents[i]);
				java.util.Vector call = findAllClassAttributes(org.omg.mof.Model.PackageHelper.narrow(imp.imported_namespace()));
				java.util.Iterator iterator = call.iterator();
				while (iterator.hasNext()) {
					boolean found = false;
					org.omg.CORBA.Object next = (org.omg.CORBA.Object) iterator.next();
					for (int k = 0; k < attributes.size(); k++) {
						org.omg.CORBA.Object attr = (org.omg.CORBA.Object) attributes.elementAt(k);
						if (attr._is_equivalent(next))
							found = true;
					}
					if (!found)
						attributes.add(next);
				}
			}
		}
		return attributes;
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 */
	public void packageModuleTemplate(org.omg.mof.Model.Package package_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		_logger.log(Level.FINE, "XMIExport packageModuleTemplate");
		if (package_.visibility().equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
			org.omg.mof.Model.Class classes_ordered[] = mofHelper.allClassesOfPackageOrderedByInheritance(package_);

			String className = getClassName(package_.name());
			outputln("package " + javaHelper.javaPackage(package_) + ".xmi;");
			outputln();
			annotationTemplate(package_.annotation());
			outputln("public class " + className + " extends org.objectweb.modfact.corba.xmiio.exporter.XMIExport {");

			constructor(package_.name(), className);
			classAttributes(findAllClassAttributes(package_));
			classDataTypeAttributes(findAllDataTypeAttributes(package_));
			classTemplate(package_, classes_ordered);
			rootPackage(package_, classes_ordered);
			outputln("} // end of class " + className);
			outputln();
		}
	}

	/**
	 * XMIExport generation for constructor.
	 * @param package_name The package name.
	 * @param class_name The class name.
	 */
	public void constructor(String package_name, String class_name) {
		_logger.log(Level.FINE, "XMIExport constructor(" + package_name + ", " + class_name + ")");
		outputln("/**");
		outputln(" * Default " + class_name + " Constructor");
		outputln(" * @param dtd The DTD File.");
		outputln(" * @param namespace The XML Namespace.");
		outputln(" */");
		outputln("public " + class_name + "(java.lang.String dtd, java.lang.String namespace) {");
		outputln("this(dtd, namespace, org.objectweb.modfact.corba.logging.ModFactLogger.getLogger(\"" + package_name + "XMIExport\"));");
		outputln("}");
		outputln();
		outputln("/**");
		outputln(" * " + class_name + " Constructor");
		outputln(" * @param dtd The DTD File.");
		outputln(" * @param namespace The XML Namespace.");
		outputln(" * @param logger The logger.");
		outputln(" */");
		outputln(
			"public "
				+ class_name
				+ "(java.lang.String dtd, java.lang.String namespace, org.objectweb.modfact.corba.logging.ModFactLogger logger) {");
		outputln("super(dtd, namespace, \"" + idlHelper.format1(package_name) + "\", \"Model\");");
		outputln("_logger = logger;");
		outputln("}");
		outputln();
	}

	protected String refMethod(org.omg.mof.Model.Class cls, String package_name) throws org.omg.mof.Reflective.MofError {
		String method = idlHelper.format2(cls.name());
		try {
			method = idlHelper.format2(idlHelper.className(cls)) + "_ref()";
			org.omg.mof.Model.ModelElement clsTmp = cls;
			while (!clsTmp.container().name().equals(package_name)) {
				org.omg.mof.Model.Namespace ns = clsTmp.container();
				if (ns._is_a(org.omg.mof.Model.ClassifierHelper.id())) {
					org.omg.mof.Model.Classifier container = org.omg.mof.Model.ClassifierHelper.narrow(ns);
					method = idlHelper.format2(javaHelper.javaClassifierName(container)) + "_ref()." + method;
				} else {
					method = idlHelper.format2(clsTmp.container().name()) + "_ref()." + method;
				}
				clsTmp = clsTmp.container();
			}
		} catch (org.omg.CORBA.MARSHAL marshal) { /* The marshal exception contains the NotSet Exception */
			if (!marshal.getMessage().equals("IDL:mof.omg.org/Reflective/NotSet:1.0"))
				throw marshal;
		} catch (org.omg.mof.Reflective.NotSet notSet) {
			// Nothing to do
		}
		return method;
	}

	/**
	 * Generate the methods for the writing of complex type.
	 * @param attributes List of DataType attributes.
	 */
	public void classAttributes(Vector attributes) throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIExport classAttributeTemplate()");
		java.util.Iterator iterator = attributes.iterator();
		outputln("// ==================================================================");
		outputln("//  Method to write some Classes");
		outputln("// ==================================================================");
		while (iterator.hasNext()) {
			org.omg.mof.Model.Classifier classifier2 = (org.omg.mof.Model.Classifier) iterator.next();
			org.omg.mof.Model.Class classifier = org.omg.mof.Model.ClassHelper.narrow(classifier2);
			annotationTemplate(classifier.annotation());
			String classifierQualifiedName;
			try {
				classifierQualifiedName = javaHelper.javaQualifiedName(classifier);
			} catch (NotSet notSet) {
				classifierQualifiedName = idlHelper.format1(classifier.name());
			}
			outputln("public void write" + idlHelper.format1(classifier.name()) + "(" + classifierQualifiedName + " _toWrite)");
			outputln("    throws org.omg.mof.Reflective.MofError {");
			outputln("if (_toWrite == null)");
			outputln("    return;");
			outputln("outputln(\"<" + classifierQualifiedName + " xmi.id=\\\"\" + getXMIId(_toWrite) + \"\\\">\");");
			org.omg.mof.Model.MofAttribute attr[] =
				mofHelper.allAttributesOfClass(classifier, org.omg.mof.Model.ScopeKind.instance_level, true);
			for (int j = 0; j < attr.length; j++) {
				classAttributeTemplate(attr[j], "_toWrite");
			}
			org.omg.mof.Model.Reference ref[] = mofHelper.allReferencesOfClass(classifier);
			for (int j = 0; j < ref.length; j++) {
				classReferenceTemplate(ref[j], "_toWrite");
			}
			outputln("outputln(\"</" + classifierQualifiedName + ">\");");
			outputln("}");
			outputln();
		}
	}

	/**
	 * Generate the methods for the writing of complex type.
	 * @param attributes List of DataType attributes.
	 */
	public void classDataTypeAttributes(Vector attributes) throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIExport classAttributeTemplate()");
		java.util.Iterator iterator = attributes.iterator();
		outputln("// ==================================================================");
		outputln("//  Method to write the DataTypes");
		outputln("// ==================================================================");
		while (iterator.hasNext()) {
			org.omg.mof.Model.DataType dataType = (org.omg.mof.Model.DataType) iterator.next();
			org.omg.CORBA.TypeCode typeCode = dataType.type_code();
			switch (typeCode.kind().value()) {
				case org.omg.CORBA.TCKind._tk_enum :
					try {
						outputln(
							"public java.lang.String enumeration"
								+ idlHelper.format1(dataType.name())
								+ "ToString("
								+ javaHelper.javaQualifiedName(dataType)
								+ " value) {");
						for (int i = 0; i < typeCode.member_count(); i++) {
							outputln("if (value.equals(" + javaHelper.javaQualifiedName(dataType) + "." + typeCode.member_name(i) + "))");
							outputln("    return \"" + idlHelper.format2(typeCode.member_name(i)) + "\";");
						}
					} catch (org.omg.CORBA.TypeCodePackage.BadKind badKind) {
						// Nothing to do
					} catch (org.omg.CORBA.TypeCodePackage.Bounds bounds) {
						// Nothing to do
					} catch (org.omg.mof.Reflective.NotSet notSet) {
						// Nothing to do
					}
					outputln("return new java.lang.String();");
					outputln("}");
					outputln();
					break;
			}
		}
	}

	/**
	 * XMIExport generation for the class template.
	 * @param package_ The package.
	 * @param classes The classes of the package.
	 */
	public void classTemplate(org.omg.mof.Model.Package package_, org.omg.mof.Model.Class classes[])
		throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIExport classTemplate()");
		String package_name = package_.name();
		outputln("// ==================================================================");
		outputln("//  XXXTemplate with XXX an entity");
		outputln("// ==================================================================");
		// Classes
		for (int i = 0; i < classes.length; i++) {
			org.omg.mof.Model.Class cls = classes[i];
			annotationTemplate(cls.annotation());
			String model_package_type;
			if (mappingValid) {
				model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
			} else {
				model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
			}
			outputln("public void " + idlHelper.format1FirstMin(cls.name()) + "Template(" + model_package_type + " _model_package)");
			outputln("    throws org.omg.mof.Reflective.MofError {");
			String clsClass = idlHelper.format1FirstMin(cls.name()) + "Class";
			String clsTab = javaHelper.testJavaConflict(idlHelper.format1FirstMin(cls.name()) + "s");
			String clsName;
			String xmiQualifiedName;
			try {
				clsName = idlHelper.className(cls);
				xmiQualifiedName = javaHelper.javaQualifiedName(cls);
			} catch (NotSet notSet) {
				clsName = cls.name();
				xmiQualifiedName = clsName;
			}
			String javaQualifiedName;
			try {
				org.omg.mof.Model.Namespace ns = cls.container();
				if (ns._is_a(org.omg.mof.Model.PackageHelper.id())) {
					org.omg.mof.Model.Package pck = org.omg.mof.Model.PackageHelper.narrow(ns);
					javaQualifiedName = javaHelper.javaPackage(pck) + "." + clsName;
				} else {
					javaQualifiedName = javaHelper.javaQualifiedName(ns) + "." + clsName;
				}
			} catch (NotSet notSet) {
				javaQualifiedName = clsName;
			}
			outputln(
				"_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \""
					+ idlHelper.format1FirstMin(cls.name())
					+ "Template(\" + _model_package + \")\");");
			outputln(javaQualifiedName + "Class " + clsClass + " = _model_package." + refMethod(cls, package_name) + ";");
			try {
				outputln(
					javaQualifiedName
						+ "[] "
						+ clsTab
						+ " = "
						+ clsClass
						+ ".all_of_type_"
						+ idlHelper.format2(idlHelper.className(cls))
						+ "();");
			} catch (org.omg.mof.Reflective.NotSet notSet) {
				outputln(javaQualifiedName + "[] " + clsTab + " = " + clsClass + ".all_of_type_" + idlHelper.format2(cls.name()) + "();");
			}
			outputln("for (int i=0 ; i<" + clsTab + ".length ; i++) {");
			outputln("outputln(\"<" + xmiQualifiedName + " xmi.id=\\\"\" + getXMIId(" + clsTab + "[i]) + \"\\\">\");");
			org.omg.mof.Model.MofAttribute attributes[] =
				mofHelper.allAttributesOfClass(cls, org.omg.mof.Model.ScopeKind.instance_level, true);
			for (int j = 0; j < attributes.length; j++) {
				classAttributeTemplate(attributes[j], clsTab + "[i]");
			}
			org.omg.mof.Model.Reference references[] = mofHelper.allReferencesOfClass(cls);
			for (int j = 0; j < references.length; j++) {
				//classReferenceTemplate((org.omg.mof.Model.Reference)references.elementAt(j), clsTab + "[i]");
				org.omg.mof.Model.Reference ref = references[j];
				String variable = javaHelper.format2Idl2JavaConflict(ref.name());

				//if (ref.multiplicity().lower == 0)
				//    outputln("try {");

				if (ref.multiplicity().upper == -1) {
					try {
						outputln(javaHelper.javaQualifiedName(ref.type()) + " [] " + variable + " = " + clsTab + "[i]." + variable + "();");
						outputln("for (int k=0 ; k<" + variable + ".length ; k++) {");
						outputln("write" + idlHelper.format1(ref.type().name()) + "(" + variable + "[k]);");
						outputln("}");
					} catch (NotSet notSet) {
						// Nothing to do.
					}
				} else {
					String content = "write" + idlHelper.format1(ref.type().name()) + "(" + clsTab + "[i]." + variable + "());";
					if (ref.multiplicity().lower == 0) {
						outputln("try {");
						outputln(content);
						outputln("} catch (org.omg.mof.Reflective.NotSet notSet) { /* Nothing to do */ }");
					} else {
						outputln(content);
					}
				}

				//if (ref.multiplicity().lower == 0)
				//    outputln("} catch (org.omg.mof.Reflective.NotSet notSet) { /* Nothing to do */ }");

			}
			outputln("outputln(\"</" + xmiQualifiedName + ">\");");
			outputln("}");
			outputln("}");
			outputln();
		}
	}

	/**
	 * XMIExport generation for the class attribute template.
	 * @param attribute The class attribute.
	 * @param variable_name The variable name.
	 */
	public void classAttributeTemplate(org.omg.mof.Model.MofAttribute attribute, String variable_name)
		throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIExport classAttributeTemplate(" + attribute.name() + ")");
		try {
			String attrName = javaHelper.javaQualifiedName(attribute.container()) + "." + idlHelper.format1(attribute.name());
			String attrCall = idlHelper.format2(attribute.name()) + "()";
			if (attribute.type().name().equals("*CORBA::TypeCode")) {
				outputln("outputln(\"<" + attrName + ">\");");
				outputln(
					"outputln(\"<XMI.CorbaTypeCode xmi.id=\\\"\" + getTypeCodeId(" + variable_name + "." + attrCall + ") + \"\\\" >\");");
				outputln("outputln(\"<\" + typeCode2XML(" + variable_name + "." + attrCall + ") + \"/>\");");
				outputln("outputln(\"</XMI.CorbaTypeCode>\");");
				outputln("outputln(\"</" + attrName + ">\");");
			} else {
				String tag1 = "<" + attrName + ">";
				String tag2 = "</" + attrName + ">";
				String readAttribute = variable_name + "." + attrCall;
				String content = "outputln(\"" + tag1 + "\" + " + readAttribute + " + \"" + tag2 + "\");";
				org.omg.mof.Model.Classifier type = attribute.type();
				if (type._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
					org.omg.mof.Model.DataType dataType = org.omg.mof.Model.DataTypeHelper.narrow(type);
					switch (dataType.type_code().kind().value()) {
						case org.omg.CORBA.TCKind._tk_enum :
							readAttribute = "enumeration" + idlHelper.format1(dataType.name()) + "ToString(" + readAttribute + ")";
							content = "outputln(\"" + tag1 + "\" + " + readAttribute + " + \"" + tag2 + "\");";
					}
				} else {
					if (type._is_a(org.omg.mof.Model.ClassHelper.id())) {
						org.omg.mof.Model.Class class_ = org.omg.mof.Model.ClassHelper.narrow(type);
						content = "if (" + readAttribute + " != null) {\n";
						readAttribute = "write" + idlHelper.format1(class_.name()) + "(" + readAttribute + ")";
						content = content + "outputln(\"" + tag1 + "\");\n" + readAttribute + ";\n" + "outputln(\"" + tag2 + "\");" + "\n}";
					}
				}

				if (attribute.multiplicity().lower == 0 && attribute.multiplicity().upper != -1) {
					outputln("try {");
					outputln(content);
					outputln("} catch (org.omg.mof.Reflective.NotSet notSet) { /* Nothing to do */ }");
				} else if (attribute.multiplicity().upper == -1) {
					outputln("for (int k=0 ; k<" + readAttribute + ".length ; k++) {");
					outputln("if (k>0) output(\" \");");
					outputln("output(\"<" + attrName + ">\" + " + readAttribute + "[k] + \"</" + attrName + ">\");");
					outputln("}");
				} else {
					outputln(content);
				}
			}
		} catch (org.omg.mof.Reflective.NotSet notSet) {
			// Nothing to do
		}
	}

	/**
	 * XMIExport generation for the class reference template.
	 * @param reference The class reference.
	 * @param variable_name The variable name.
	 */
	public void classReferenceTemplate(org.omg.mof.Model.Reference reference, String variable_name)
		throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIExport classReferenceTemplate(" + reference.name() + ")");
		try {
			String refName = javaHelper.javaQualifiedName(reference.container()) + "." + idlHelper.format1(reference.name());
			String refTab = javaHelper.format2Idl2JavaConflict(reference.name());
			String refType = javaHelper.javaQualifiedName(reference.type());
			if (reference.multiplicity().upper == -1) {
				outputln(refType + "[] " + refTab + " = " + variable_name + "." + refTab + "();");
				outputln("if (" + refTab + ".length > 0) {");
				outputln("outputln(\"<" + refName + ">\");");
				outputln("for (int j=0 ; j<" + refTab + ".length ; j++)");
				outputln("    outputln(\"<" + refType + " xmi.idref=\\\"\" + getXMIId(" + refTab + "[j]) + \"\\\" />\");");
				outputln("outputln(\"</" + refName + ">\");");
				outputln("}");
			} else {
				if (reference.multiplicity().lower == 0)
					outputln("try {");
				outputln(refType + " " + refTab + " = " + variable_name + "." + refTab + "();");
				outputln("if (" + refTab + " != null) {");
				outputln("outputln(\"<" + refName + ">\");");
				outputln("outputln(\"<" + refType + " xmi.idref=\\\"\" + getXMIId(" + refTab + ") + \"\\\" />\");");
				outputln("outputln(\"</" + refName + ">\");");
				outputln("}");
				if (reference.multiplicity().lower == 0)
					outputln("} catch (org.omg.mof.Reflective.NotSet notSet) { /* Nothing to do */ }");
			}
		} catch (org.omg.mof.Reflective.NotSet notSet) {
			// Nothing to do
		}
	}

	/**
	 * XMIExport generation for the root package.
	 * @param package_ The package.
	 * @param classes The classes of the package.
	 */
	public void rootPackage(org.omg.mof.Model.Package package_, org.omg.mof.Model.Class classes[]) throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIExport rootPackage()");
		outputln("public void rootPackageTemplate(org.omg.CORBA.Object _root_package)");
		outputln("    throws org.omg.mof.Reflective.MofError {");
		outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"rootPackageTemplate(\" + _root_package + \")\");");
		String packageName = idlHelper.format1(package_.name());
		String model_package_type;
		if (mappingValid) {
			model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(packageName) + "Package";
		} else {
			model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(packageName) + "Package";
		}
		outputln(model_package_type + " _model_package = " + model_package_type + "Helper.narrow(_root_package);");
		for (int i = 0; i < classes.length; i++) {
			org.omg.mof.Model.Class cls = (org.omg.mof.Model.Class) classes[i];
			if (!cls.is_abstract())
				outputln(idlHelper.format1FirstMin(cls.name()) + "Template(_model_package);");
		}
		outputln("}");
		outputln();
	}

}
