/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.exporter;

import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.objectweb.modfact.corba.xmiio.common.AbstractXMIIOGenerator;
import org.omg.mof.Reflective.MofError;

/**
 * The XMI Export Application generator.
 * This class allows the generation of XMI exporter launcher.
 */
public class GenerationXMIExportApplication extends AbstractXMIIOGenerator {

	/** The logger. */
	protected ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiio.exporter.GenerationXMIExportApplication");

	/**
	 * XMIExport generation.
	 * @param package_ The root package.
	 * @param class_name_ The class name.
	 * @param class_name_application The application class name.
	 */
	public void generateXMIExportApplication(org.omg.mof.Model.Package package_, String class_name_, String class_name_application_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		_logger.log(Level.FINE, "XMIExport beginning of the generation");
		_logger.log(Level.INFO, class_name_application_ + " generation ...");
		String package_name = package_.name();
		head(package_, class_name_application_);
		constructors(package_name, class_name_application_);
		startMethod(package_, class_name_);
		mainMethod(class_name_application_);
		outputln("} // end class " + class_name_);
		flushFile();
	}

	/**
	 * XMIExport generation for the head.
	 * @param package_ The package.
	 * @param class_name_application_ The application class name.
	 */
	public void head(org.omg.mof.Model.Package package_, String class_name_application_)
		throws MofError {
		outputln("package " + javaHelper.javaPackage(package_) + ".xmi;");
		outputln();
		outputln("public class " + class_name_application_);
		outputln("     extends org.objectweb.modfact.corba.xmiio.exporter.XMIExportApplication {");
		outputln();
	}

	/**
	 * XMIExport generation for the constructors.
	 * @param package_name_ The package name.
	 * @param class_name_application_ The application class name.
	 */
	public void constructors(String package_name_, String class_name_application_) {
		outputln("public " + class_name_application_ + "() {");
		outputln("super(\"" + package_name_ + "\", \"" + class_name_application_ + "\");");
		outputln("}");
		outputln();
	}

	/**
	 * XMIExport generation for the main method.
	 * @param class_name_application_ The application class name.
	 */
	public void mainMethod(String class_name_application_) throws org.omg.mof.Reflective.MofError {
		outputln("/**");
		outputln(" * The main method");
		outputln(" */");
		outputln("public static void main (java.lang.String [] args) {");
		outputln(class_name_application_ + " application = new " + class_name_application_ + "();");
		outputln("application.runMain(args);");
		outputln("}");
		outputln();
	}

	/**
	 * XMIExport generation for the start method.
	 * @param package_ The package.
	 * @param class_name_ The class name.
	 */
	public void startMethod(org.omg.mof.Model.Package package_, String class_name_)
		throws MofError {
		String package_name = package_.name();
		String package_qualified_name = javaHelper.javaPackage(package_);
		outputln("/**");
		outputln(" * Starts the main function");
		outputln(" */");
		outputln("public int start(java.lang.String[] args) {");
		outputln("try {");
		outputln("// Get the root naming context");
		outputln("org.omg.CORBA.Object objRef = orb_.resolve_initial_references(\"NameService\");");
		outputln("// Use NamingContextExt instead of NamingContext. This is part of the Interoperable naming Service.");
		outputln("org.omg.CosNaming.NamingContextExt ncRef = org.omg.CosNaming.NamingContextExtHelper.narrow(objRef);");
		outputln();
		outputln("// Resolve the Object Reference in Naming");
		outputln("java.lang.String name = \"" + idlHelper.format1(package_name) + "FactoryServer\";");
		outputln(package_qualified_name	+ "." + idlHelper.format1(package_name)	+ "PackageFactory _reference = "
				+ package_qualified_name + "." + idlHelper.format1(package_name)
				+ "PackageFactoryHelper.narrow(ncRef.resolve_str(name));");
		outputln();
		outputln("// Creation of Model ");
		String model_package_type;
		if (mappingValid) {
			model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
		} else {
			model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
		}
		outputln(model_package_type + " _model_package = " + "_reference.create_" + idlHelper.format2(package_name) + "_package();");
		outputln();
		outputln("org.objectweb.modfact.corba.logging.ModFactLogger _logger;");
		outputln(TABULATION
				+ "_logger = org.objectweb.modfact.corba.logging.ModFactLogger.getLogger(\""
				+ package_name + "XMIExport\");");
		outputln(class_name_ + " xmiExport = new " + class_name_ + "(dtdFile_, \"\", _logger);");
		outputln("xmiExport.setORB(orb_);");
		outputln("java.io.FileOutputStream fileOutput = new java.io.FileOutputStream(new java.io.File(outputFile_));");
		outputln("xmiExport.setOutput(fileOutput);");
		outputln("if (iorFile_.length() > 0)");
		outputln("    xmiExport.parseIOR(iorFile_, printIOR_);");
		outputln("else if (nameService_.length() > 0)");
		outputln("    xmiExport.parseName(nameService_, printIOR_);");
		output("} ");
		outputCatchException("org.omg.CORBA.ORBPackage.InvalidName");
		outputCatchException("org.omg.CosNaming.NamingContextPackage.InvalidName");
		outputCatchException("org.omg.CosNaming.NamingContextPackage.NotFound");
		outputCatchException("org.omg.CosNaming.NamingContextPackage.CannotProceed");
		outputCatchException("org.omg.mof.Reflective.MofError");
		outputCatchException("java.io.FileNotFoundException");
		outputCatchException("java.io.IOException");
		outputln();
		outputln("return 0;");
		outputln("}");
		outputln();
	}

	/**
	 * XMIExport generation for a 'catch'.
	 * @param exception_ The exception to catch.
	 */
	public void outputCatchException(String exception_) {
		String exception_name;
		if (exception_.indexOf(".") != -1)
			exception_name = exception_.substring(exception_.lastIndexOf(".") + 1);
		else
			exception_name = exception_;
		outputln("catch (" + exception_ + " " + idlHelper.format1FirstMin(exception_name) + ") {");
		outputln(
			"System.err.println(\""
				+ idlHelper.format1(exception_name)
				+ " : \" + "
				+ idlHelper.format1FirstMin(exception_name)
				+ ".getMessage());");
		output("} ");
	}

}
