/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.importer;

import org.objectweb.modfact.corba.service.cmdline.common.ModfactApplicationBase;
import org.objectweb.modfact.corba.service.cmdline.io.OptionInput;
import org.objectweb.modfact.corba.service.cmdline.xmiio.OptionIORFile;
import org.objectweb.modfact.corba.service.cmdline.xmiio.OptionNameService;
import org.objectweb.modfact.corba.service.cmdline.xmiio.OptionPrintIOR;
import org.objectweb.modfact.corba.service.cmdline.xmiio.XMIImportApplication;

/**
 * An abstract implementation of an XMI Import application.
 * This class provides methods for all XMI importers.
 */
public abstract class AbstractXMIImportApplication
    extends ModfactApplicationBase 
    implements XMIImportApplication {

    /** The PrintIOR flag. */
    protected boolean printIOR_ = false;

    /** The IOR file. */
    protected String iorFile_ = new String();

    /** The input file. */
    protected String inputFile_ = new String();

    /** The name in the name service. */
    protected String nameService_ = new String();

    /**
     * XMIImportApplication constructor.
     * @param package_name The package name.
     * @param class_name The class name.
     * @param default_ior_file The default IOR file.
     */
    protected AbstractXMIImportApplication(String package_name, String class_name, String default_ior_file) {
        this(new org.objectweb.util.cmdline.lib.DefaultCommandLine(new String[] { class_name }, new String[0], new String[] { "Import an XMI file for the model " + package_name + "." }, true));
        setIORFile(default_ior_file);
    }

    /**
     * XMIImportApplication constructor.
     * @param commandLine The command line manager.
     */
    private AbstractXMIImportApplication(org.objectweb.util.cmdline.lib.DefaultCommandLine commandLine) {
        super(commandLine);
        // Creates the XMIImportApplication options
		commandLine.addOption(new OptionInput(this));
        commandLine.addOption(new OptionIORFile(this));
        commandLine.addOption(new OptionPrintIOR(this));
        commandLine.addOption(new OptionNameService(this));
    }

    /**
     * Set the IOR file.
     * @param filename The IOR filename.
     */
    public void setIORFile(String filename) {
        iorFile_ = filename;
    }

    /**
     * Set the input file.
     * @param filename The input filename.
     */
    public void setInputFile(String filename) {
        inputFile_ = filename;
    }

    /**
     * Set the PrintIOR flag.
     * @param print TRUE to display the IOR on the console.
     */
    public void setPrintIOR(boolean print) {
        printIOR_ = print;
    }

    /**
     * Set the name in the Name Service.
     * @param nameService The name in the Name Service.
     */
    public void setNameService(String nameService) {
        nameService_ = nameService;
    }

}
