/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.importer;

import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.objectweb.modfact.corba.xmiio.common.AbstractXMIIOGenerator;


/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImport extends AbstractXMIIOGenerator {

    /** The logger. */
    ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImport");

    /**
     * Default Constructor.
     */
    public GenerationXMIImport() {
        super();
		ModFactLogger logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImportApplication");
    }

    /**
     * XMIImport generation.
     * @param package_ The root package.
     */
    public void generateXMIImport(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds, java.io.IOException {
        _logger.log(Level.FINE, "XMIImport beginning of the generation");
        String className = getClassName(package_.name());
        _logger.log(Level.INFO, className + " generation ...");
        packageModuleTemplate(package_);
        flushFile();
    }

    /**
     * Get the class name.
     * @param package_name_ The root package name.
     * @return The class name.
     */
    protected String getClassName(String package_name_) {
        return super.getClassName(package_name_, "XMIImport");
    }

    /**
     * XMIImport generation for constructor.
     * @param class_name The class name.
     */
    public void constructor(String class_name, String package_name) {
        _logger.log(Level.FINE, "XMIImport constructor(" + class_name + ")");
        outputln("/** The XMIImportAssociations class. */");
        outputln(package_name + "XMIImportAssociations _xmi_import_associations;");
        outputln();
        outputln("/** The XMIImportAttributes class. */");
        outputln(package_name + "XMIImportAttributes _xmi_import_attributes;");
        outputln();
        outputln("/** The XMIImportClasses class. */");
        outputln(package_name + "XMIImportClasses _xmi_import_classes;");
        outputln();
		outputln("/** The logger. */");
		outputln("org.objectweb.modfact.corba.logging.ModFactLogger _logger;");
		outputln();
        outputln("/**");
        outputln(" * Default " + class_name + " Constructor");
        outputln(" */");
        outputln("public " + class_name + "() {");
        outputln("this(org.objectweb.modfact.corba.logging.ModFactLogger.getLogger(\"" + package_name + "XMIImport\"));");
        outputln("}");
        outputln();
        outputln("/**");
        outputln(" * " + class_name + " Constructor with quiet flag");
        outputln(" */");
        outputln("public " + class_name + "(boolean quiet) {");
        outputln("this(org.objectweb.modfact.corba.logging.ModFactLogger.getLogger(\"" + package_name + "XMIImport\"));");
        outputln("if (quiet)");
        outputln(TABULATION + "_logger = org.objectweb.modfact.corba.logging.ModFactLogger.getLogger(\"" + package_name + "XMIImport\");");
        outputln("}");
        outputln();
        outputln("/**");
        outputln(" * " + class_name + " Constructor");
        outputln(" * @param logger The logger.");
        outputln(" */");
        outputln("public " + class_name + "(org.objectweb.modfact.corba.logging.ModFactLogger logger) {");
        outputln("_logger = logger;");
        outputln("_xmi_import_associations = new " + package_name + "XMIImportAssociations (logger, _types);");
        outputln("_xmi_import_attributes = new " + package_name + "XMIImportAttributes (logger, _types);");
        outputln("_xmi_import_classes = new " + package_name + "XMIImportClasses (logger, _types);");
        outputln("_xmi_import_classes.setXMIImportAttributes(_xmi_import_attributes);");
        outputln("_xmi_import_associations.setXMIImportClasses(_xmi_import_classes);");
        outputln("_xmi_import_attributes.setXMIImportClasses(_xmi_import_classes);");
        outputln("}");
        outputln();
    }

    /**
     * Package processing.
     * @param package_ The package to process.
     */
    public void packageModuleTemplate(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
        _logger.log(Level.FINE, "XMIImport packageModuleTemplate(" + package_.name() + ")");
        if (package_.visibility().equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
			org.omg.mof.Model.Association associations[] = mofHelper.allAssociationsOfPackage(package_);
			org.omg.mof.Model.Class classes_ordered[] = mofHelper.allClassesOfPackageOrderedByInheritance(package_);
            _logger.log(Level.FINE, "XMIImport produce");

            String className = getClassName(package_.name());
			outputln("package " + javaHelper.javaPackage(package_) + ".xmi;");
            outputln();
            annotationTemplate(package_.annotation());
            outputln("public class " + className + " extends org.objectweb.modfact.corba.xmiio.importer.XMIImportParser {");
            classAttributes(package_);
            constructor(className, package_.name());
            associationAttributes(package_.name());
            initiateMethod(package_, classes_ordered);
            rootPackage(package_.name(), classes_ordered, associations);
            completeImportMethod(associations);
            outputln("} // end of class " + className);
            outputln();
        }
    }

    /**
    * XMIImport generation for the class attributes.
    * @param package_ The package.
     * @param associations_ The associations of the package.
    */
   public void classAttributes(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError {
   	   String package_name = package_.name();
       _logger.log(Level.FINE, "XMIImport classAttributes(" + package_name + ")");
       outputln("// Factories Declaration");
       String model_package_type;
       if (mappingValid) {
           model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
       } else {
           model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
       }
       outputln("private " + model_package_type + " _" + idlHelper.format2(package_name) + "_package = null;");
       outputln();
   }
   
    /**
     * XMIImport generation for the association attributes.
     * @param package_name_ The package name.
     * @param associations_ The associations of the package.
     */
    public void associationAttributes(String package_name_) throws org.omg.mof.Reflective.MofError {
        _logger.log(Level.FINE, "XMIImport associationAttributes(" + package_name_ + ")");
        outputln("// Hashtables for associations (Model / Key Vector)");
        outputln();
        outputln("/** Hashtables for types (Key / Model) */");
        outputln("private java.util.Hashtable _types = new java.util.Hashtable();");
        outputln();
    }

    /**
     * XMIImport generation for the initiate method.
     * @param package_ The package.
     * @param classes_ The classes of the package.
     * @param associations_ The associations of the package.
     */
    public void initiateMethod(org.omg.mof.Model.Package package_, org.omg.mof.Model.Class classes_ []) throws org.omg.mof.Reflective.MofError {
    	String package_name = package_.name();
        _logger.log(Level.FINE, "XMIImport initiateMethod(" + package_name + ", " + classes_.length + ")");
        String model_package_type;
        if (mappingValid) {
            model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
        } else {
            model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
        }
        outputln("public void initiate(" + model_package_type + " _model_package, org.omg.CORBA.ORB orb)");
        outputln("    throws org.omg.mof.Reflective.MofError {");
        outputln("super.setORB(orb);");
        outputln("// Factories creation");
        outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"initiate(\" + _model_package + \")\");");
        outputln("this._" + idlHelper.format2(package_name) + "_package = _model_package;");
        outputln("_xmi_import_classes.initiate(_model_package);");
        outputln("_xmi_import_associations.initiate(_model_package);");
        outputln("_xmi_import_attributes.initiate(_model_package);");
        outputln("}");
        outputln();
    }

    /**
     * XMIImport generation for the root package.
     * @param package_name_ The package name.
     * @param classes_ The classes of the package.
     * @param associations_ The associations of the package.
     */
    public void rootPackage(String package_name_, org.omg.mof.Model.Class classes_ [], org.omg.mof.Model.Association associations_ []) throws org.omg.mof.Reflective.MofError {
        _logger.log(Level.FINE, "XMIImport rootTemplate(" + package_name_ + ", " + classes_.length + ", " + associations_.length + ")");
        annotationTemplate(" The root package processing");
        outputln("public org.omg.CORBA.portable.IDLEntity rootPackageTemplate(org.w3c.dom.Element _root_package_element)");
        outputln("    throws org.omg.mof.Reflective.MofError {");
        outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"rootPackageTemplate(\" + _root_package_element + \")\");");
        outputln("java.lang.String prefix_element = new java.lang.String();");
        outputln("java.lang.String namespace = _root_package_element.getNamespaceURI();");
        outputln("if (namespace != null && namespace.length() > 0)");
        outputln("    prefix_element = _root_package_element.getPrefix() + \":\";");
        outputln("org.w3c.dom.Element _current = _root_package_element;");
        outputln("String _current_name = _root_package_element.getNodeName().trim();");

        for (int i = 0; i < classes_.length; i++) {
            org.omg.mof.Model.Class cls = (org.omg.mof.Model.Class) classes_[i];
            outputln("if (_current_name.endsWith(prefix_element + \"" + idlHelper.format1(cls.name()) + "\")) {");
            outputln("_xmi_import_classes." + idlHelper.format1FirstMin(cls.name()) + "Template(_current);");
            if (i == classes_.length - 1 && associations_.length == 0)
                outputln("}");
            else
                output("} else ");
        }
        for (int i = 0; i < associations_.length; i++) {
            org.omg.mof.Model.Association asso = (org.omg.mof.Model.Association) associations_[i];
            outputln("if (_current_name.endsWith(prefix_element + \"" + asso.name() + "\")) {");
            outputln("_xmi_import_associations." + idlHelper.format1FirstMin(asso.name()) + "Template(_current);");
            if (i == associations_.length - 1)
                outputln("}");
            else
                output("} else ");
        }
        outputln("return (org.omg.CORBA.portable.IDLEntity) _" + idlHelper.format2(package_name_) + "_package;");
        outputln("}");
        outputln();
    }

    /**
     * XMIImport generation for the completeImport() method.
     * @param associations_ The associations of the package.
     */
    public void completeImportMethod(org.omg.mof.Model.Association associations_ []) throws org.omg.mof.Reflective.MofError {
        outputln("public void completeImport () throws org.omg.mof.Reflective.MofError {");
        for (int i = 0; i < associations_.length; i++) {
            org.omg.mof.Model.Association asso = (org.omg.mof.Model.Association) associations_[i];
            try {
                org.omg.mof.Model.Reference[] refs = mofHelper.referencesOfAssociation(asso);
                for (int j = 0; j < refs.length; j++) {
                    org.omg.mof.Model.AssociationEnd assoEnd = refs[j].referenced_end();
                    if (assoEnd.aggregation().value() == org.omg.mof.Model.AggregationKind._none && assoEnd.is_changeable())
                        outputln("_xmi_import_attributes.put" + idlHelper.format1(refs[j].container().name()) + idlHelper.format1(refs[j].name()) + "();");
                }
            } catch (org.omg.mof.Reflective.NotSet notSet) {
                // Nothing to do
            }
        }
        outputln("}");
        outputln();
    }

}
