/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.importer;

import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.objectweb.modfact.corba.xmiio.common.AbstractXMIIOGenerator;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImportAssociations extends AbstractXMIIOGenerator {

	/** The logger. */
	protected ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImport");

	/**
	 * Default Constructor.
	 */
	public GenerationXMIImportAssociations() {
		super();
		ModFactLogger logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImportApplication");
	}

	/**
	 * XMIImport generation.
	 * @param package_ The root package.
	 * @param class_name_ The class name of the generated class.
	 */
	public void generateXMIImportAssociations(org.omg.mof.Model.Package package_, String class_name_)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			java.io.FileNotFoundException {
		_logger.log(Level.FINE, "XMIImport beginning of the generation");
		_logger.log(Level.INFO, class_name_ + " generation ...");
		packageModuleTemplate(package_, class_name_);
		flushFile();
	}

	/**
	 * XMIImport generation for constructor.
	 * @param class_name The class name.
	 * @param package_name The package name.
	 */
	public void constructor(String class_name, String package_name) {
		_logger.log(Level.FINE, "XMIImport constructor(" + class_name + ")");
		outputln("/** The logger. */");
		outputln("org.objectweb.modfact.corba.logging.ModFactLogger _logger;");
		outputln();
		outputln("/** The XMIImportClasses class. */");
		outputln(package_name + "XMIImportClasses _xmi_import_classes;");
		outputln();
		outputln("/** Hashtables for types (Key / Model) */");
		outputln("private java.util.Hashtable _types;");
		outputln();
		outputln("/**");
		outputln(" * Default " + class_name + " Constructor");
		outputln(" * @param logger The logger.");
		outputln(" * @param types The defined types.");
		outputln(" */");
		outputln("public " + class_name + "(org.objectweb.modfact.corba.logging.ModFactLogger logger, java.util.Hashtable types) {");
		outputln("_logger = logger;");
		outputln("_types = types;");
		outputln("}");
		outputln();
		outputln("public void setXMIImportClasses (" + package_name + "XMIImportClasses xmi_import_classes) {");
		outputln("_xmi_import_classes = xmi_import_classes;");
		outputln("}");
		outputln();
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 * @param class_name_ The class name of the generated class.
	 */
	public void packageModuleTemplate(org.omg.mof.Model.Package package_, String class_name_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		_logger.log(Level.FINE, "XMIImport packageModuleTemplate(" + package_.name() + ")");
		if (package_.visibility().equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
			org.omg.mof.Model.Association associations[] = mofHelper.allAssociationsOfPackage(package_);
			_logger.log(Level.FINE, "XMIImport produce");

			outputln("package " + javaHelper.javaPackage(package_) + ".xmi;");
			outputln();
			annotationTemplate(package_.annotation());
			outputln("public class " + class_name_ + " extends org.objectweb.modfact.corba.xmiio.importer.XMIImport {");
			classAttributes(package_, associations);
			constructor(class_name_, package_.name());
			initiateMethod(package_, associations);
			associationTemplate(associations);
			outputln("} // end of class " + class_name_);
			outputln();
		}
	}

	/**
	* XMIImport generation for the class attributes.
	* @param package_ The package.
	* @param associations_ The associations of the package.
	*/
	public void classAttributes(org.omg.mof.Model.Package package_, org.omg.mof.Model.Association associations_[])
		throws org.omg.mof.Reflective.MofError {
		String package_name = package_.name();
		_logger.log(Level.FINE, "XMIImport classAttributes(" + package_name + ")");
		outputln("// Factories Declaration");
		String model_package_type;
		if (mappingValid) {
			model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
		} else {
			model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
		}
		outputln("private " + model_package_type + " _" + idlHelper.format2(package_name) + "_package = null;");
		outputln();
		// Associations
		for (int i = 0; i < associations_.length; i++) {
			org.omg.mof.Model.Association asso = (org.omg.mof.Model.Association) associations_[i];
			try {
				String javaName = javaHelper.javaQualifiedName(asso);
				outputln("private " + javaName + " _" + idlHelper.format2(javaName) + "_class = null;");
			} catch (NotSet notSet) {
				// Nothing to do.
			}
		}
	}

	/**
	 * XMIImport generation for the initiate method.
	 * @param package_ The package.
	 * @param associations_ The associations of the package.
	 */
	public void initiateMethod(org.omg.mof.Model.Package package_, org.omg.mof.Model.Association associations_[]) throws MofError {
		String package_name = package_.name();
		_logger.log(Level.FINE, "XMIImport initiateMethod(" + package_name + ", " + associations_.length + ")");
		String model_package_type;
		if (mappingValid) {
			model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
		} else {
			model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
		}
		outputln("public void initiate(" + model_package_type + " _model_package)");
		outputln("    throws org.omg.mof.Reflective.MofError {");
		outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"initiate(\" + _model_package + \")\");");
		outputln("// Factories creation");
		outputln("this._" + idlHelper.format2(package_name) + "_package = _model_package;");
		// Associations
		initiateMethodAssociations(package_name, associations_);
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the "association attributes" of the initiate method.
	 * @param package_name_ The package name.
	 * @param classes_ The classes of the package.
	 */
	public void initiateMethodAssociations(String package_name_, org.omg.mof.Model.Association associations_[]) throws MofError {
		for (int i = 0; i < associations_.length; i++) {
			org.omg.mof.Model.Association asso = (org.omg.mof.Model.Association) associations_[i];
			String javaName;
			try {
				javaName = idlHelper.format2(javaHelper.javaQualifiedName(asso));
			} catch (NotSet notSet) {
				javaName = asso.name();
			}
			String method = idlHelper.format2(asso.name()) + "_ref()";
			try {
				org.omg.mof.Model.ModelElement assoTmp = asso;
				while (!assoTmp.container().name().equals(package_name_)) {
					method = idlHelper.format2(assoTmp.container().name()) + "_ref()." + method;
					assoTmp = assoTmp.container();
				}
			} catch (org.omg.CORBA.MARSHAL marshal) { /* !!! The marshal exception contains the NotSet Exception */
				if (!marshal.getMessage().equals("IDL:mof.omg.org/Reflective/NotSet:1.0"))
					throw marshal;
			} catch (org.omg.mof.Reflective.NotSet notSet) {
				// Nothing to do
			}
			outputln("_" + javaName + "_class = _model_package." + method + ";");
		}
	}

	/**
	 * XMIImport generation for the association template.
	 * @param associations_ The list of association of the package.
	 */
	public void associationTemplate(org.omg.mof.Model.Association associations_[]) throws org.omg.mof.Reflective.MofError {
		for (int i = 0; i < associations_.length; i++) {
			org.omg.mof.Model.Association asso = (org.omg.mof.Model.Association) associations_[i];
			String methodName = idlHelper.format1FirstMin(asso.name()) + "Template";
			String param = idlHelper.format2(asso.name()) + "_element";
			org.omg.mof.Model.AssociationEnd[] ends = mofHelper.associationEndsOfAssociation(asso);
			outputln("public void " + methodName + "(org.w3c.dom.Element " + param + ") throws org.omg.mof.Reflective.MofError {");
			for (int j = 0; j < ends.length; j++) {
				try {
					outputln(
						javaHelper.javaQualifiedName(ends[j].type())
							+ " "
							+ javaHelper.testJavaConflict(idlHelper.format1FirstMin(ends[j].name()))
							+ " = null;");
				} catch (NotSet notSet) {
					// Nothing to do.
				}
			}
			outputln("org.w3c.dom.NodeList _child_nodes = " + param + ".getChildNodes();");
			outputln("for (int i=0 ; i<_child_nodes.getLength() ; i++) {");
			outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child_nodes.item(i);");
			for (int j = 0; j < ends.length; j++) {
				String ends_variable = javaHelper.testJavaConflict(idlHelper.format1FirstMin(ends[j].name()));
				String type_variable;
				try {
					type_variable = javaHelper.javaQualifiedName(ends[j].type());
				} catch (NotSet notSet) {
					type_variable = ends[j].type().name();
				}
				String methodTemplate = "_xmi_import_classes." + idlHelper.format1FirstMin(ends[j].type().name()) + "Template(_current)";
				output("if (_current.getNodeName().endsWith(\"" + idlHelper.format1(ends[j].type().name()) + "\")");
				try {
					org.omg.mof.Model.Class[] subclasses = mofHelper.subClassesOfClass(ends[j].type());
					for (int k = 0; k < subclasses.length; k++) {
						output(" || _current.getNodeName().endsWith(\"" + idlHelper.format1(subclasses[k].name()) + "\")");
						if (k % 2 == 0 && k < subclasses.length - 1) {
							outputln();
							output(TABULATION);
						}
					}
				} catch (org.omg.mof.Reflective.NotSet notSet) {
					// Nothing to do.
				}
				outputln(") {");

				if (ends[j].type().is_abstract()) {
					outputln(
						"org.objectweb.modfact.corba.xmiio.mofobject.MOFObject "
							+ ends_variable
							+ "Obj = "
							+ methodTemplate
							+ ".getFirstObject();");
				} else {
					outputln(
						"org.objectweb.modfact.corba.xmiio.mofobject.MOFObject " + ends_variable + "Obj = " + methodTemplate + ";");
				}
				outputln("if (" + ends_variable + "Obj.isAnObject())");
				outputln(TABULATION + ends_variable + " = (" + type_variable + ")" + ends_variable + "Obj.get();");
				outputln("else");
				outputln(TABULATION + ends_variable + " = (" + type_variable + ") _types.get(" + ends_variable + "Obj.get());");
				outputln("}");
			}
			outputln("}");
			try {
				output("_" + idlHelper.format2(javaHelper.javaQualifiedName(asso)) + "_class.add(");
			} catch (NotSet notSet) {
				// Nothing to do.
			}
			for (int j = 0; j < ends.length; j++) {
				output(javaHelper.testJavaConflict(idlHelper.format1FirstMin(ends[j].name())));
				if (j < ends.length - 1)
					output(", ");
			}
			outputln(");");
			outputln("}");
			outputln();
		}
		outputln();
	}

}
