/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.importer;

import org.objectweb.modfact.corba.helper.StringHelper;
import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.objectweb.modfact.corba.xmiio.common.AbstractXMIIOGenerator;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImportAttributes extends AbstractXMIIOGenerator {

	/** The logger. */
	protected ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImport");

	/** The generated association attributes methods. */
	private java.util.List _associationAttributes;

	/**
	 * Default Constructor.
	 */
	public GenerationXMIImportAttributes() {
		super();
		ModFactLogger logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImportApplication");
		_associationAttributes = new java.util.ArrayList();
	}

	/**
	 * XMIImport generation.
	 * @param package_ The root package.
	 * @param class_name_ The class name of the generated class.
	 */
	public void generateXMIImportAttributes(org.omg.mof.Model.Package package_, String class_name_)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			java.io.FileNotFoundException {
		_logger.log(Level.FINE, "XMIImport beginning of the generation");
		_logger.log(Level.INFO, class_name_ + " generation ...");
		packageModuleTemplate(package_, class_name_);
		flushFile();
	}

	/**
	 * XMIImport generation for constructor.
	 * @param class_name The class name.
	 * @param package_name The package name.
	 */
	public void constructor(String class_name, String package_name) {
		_logger.log(Level.FINE, "XMIImport constructor(" + class_name + ")");
		outputln("/** The logger. */");
		outputln("org.objectweb.modfact.corba.logging.ModFactLogger _logger;");
		outputln("/** The XMIImportClasses class. */");
		outputln(package_name + "XMIImportClasses _xmi_import_classes;");
		outputln();
		outputln("/** Hashtables for types (Key / Model) */");
		outputln("private java.util.Hashtable _types;");
		outputln();
		outputln("/** The XMI Import helper. */");
		outputln("org.objectweb.modfact.corba.helper.XMIImportCommon _helper;");
		outputln();
		outputln("/**");
		outputln(" * Default " + class_name + " Constructor");
		outputln(" * @param logger The logger.");
		outputln(" */");
		outputln("public " + class_name + "(org.objectweb.modfact.corba.logging.ModFactLogger logger, java.util.Hashtable types) {");
		outputln("_logger = logger;");
		outputln("_types = types;");
		outputln("_helper = new org.objectweb.modfact.corba.provider.XMIImportProvider();");
		outputln("}");
		outputln();
		outputln("public void setXMIImportClasses (" + package_name + "XMIImportClasses xmi_import_classes) {");
		outputln("_xmi_import_classes = xmi_import_classes;");
		outputln("}");
		outputln();
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 * @param class_name_ The class name of the generated class.
	 */
	public void packageModuleTemplate(org.omg.mof.Model.Package package_, String class_name_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		_logger.log(Level.FINE, "XMIImport packageModuleTemplate(" + package_.name() + ")");
		if (package_.visibility().equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
			org.omg.mof.Model.Association associations[] = mofHelper.allAssociationsOfPackage(package_);
			org.omg.mof.Model.Class classes_ordered[] = mofHelper.allClassesOfPackageOrderedByInheritance(package_);
			_logger.log(Level.FINE, "XMIImport produce");

			outputln("package " + javaHelper.javaPackage(package_) + ".xmi;");
			outputln();
			annotationTemplate(package_.annotation());
			outputln("public class " + class_name_ + " extends org.objectweb.modfact.corba.xmiio.importer.XMIImport {");
			constructor(class_name_, package_.name());
			classAttributes(package_);
			associationAttributes(package_.name(), associations);
			initiateMethod(package_);
			readingAttributes(package_.name(), classes_ordered);
			putAssociationsValues(package_.name(), associations);
			outputln("} // end of class " + class_name_);
			outputln();
		}
	}

	/**
	 * XMIImport generation for the class attributes.
	 * @param package_ The package.
	 */
	public void classAttributes(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError {
		String package_name = package_.name();
		_logger.log(Level.FINE, "XMIImport classAttributes(" + package_name + ")");
		outputln("org.objectweb.modfact.corba.helper.IDLCommon idlHelper = new org.objectweb.modfact.corba.provider.IDLProvider();");
		outputln("// Factories Declaration");
		String model_package_type;
		if (mappingValid) {
			model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
		} else {
			model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
		}
		outputln("private " + model_package_type + " _" + idlHelper.format2(package_name) + "_package = null;");
		outputln();
	}

	/**
	 * XMIImport generation for the association attributes.
	 * @param package_name_ The package name.
	 * @param associations_ The associations of the package.
	 */
	public void associationAttributes(String package_name_, org.omg.mof.Model.Association associations_[])
		throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIImport associationAttributes(" + package_name_ + ", " + associations_.length + ")");
		outputln("// Hashtables for associations (Model / Key Vector)");
		for (int i = 0; i < associations_.length; i++) {
			org.omg.mof.Model.Association asso = (org.omg.mof.Model.Association) associations_[i];
			try {
				org.omg.mof.Model.Reference[] refs = xmiImpHelper.getReferences(asso);
				for (int j = 0; j < refs.length; j++) {
					outputln(
						"private java.util.Hashtable _"
							+ idlHelper.format2(asso.name())
							+ "_"
							+ idlHelper.format2(refs[j].name())
							+ " = new java.util.Hashtable();");
				}
			} catch (org.omg.CORBA.MARSHAL marshal) {
				if (!marshal.getMessage().equals("IDL:mof.omg.org/Reflective/NotSet:1.0"))
					throw marshal;
			} catch (org.omg.mof.Reflective.NotSet notSet) {
				// Nothing to do
			}
		}
		outputln();
	}

	/**
	 * XMIImport generation for the initiate method.
	 * @param package_ The package.
	 */
	public void initiateMethod(org.omg.mof.Model.Package package_) throws org.omg.mof.Reflective.MofError {
		String package_name = package_.name();
		_logger.log(Level.FINE, "XMIImport initiateMethod(" + package_name + ")");
		String model_package_type;
		if (mappingValid) {
			model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
		} else {
			model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
		}
		outputln("public void initiate(" + model_package_type + " _model_package)");
		outputln("    throws org.omg.mof.Reflective.MofError {");
		outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"initiate(\" + _model_package + \")\");");
		outputln("// Factories creation");
		outputln("this._" + idlHelper.format2(package_name) + "_package = _model_package;");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the attributes reading methods.
	 * @param package_name_ The package name.
	 * @param classes_ The classes of the package.
	 */
	public void readingAttributes(String package_name_, org.omg.mof.Model.Class classes_[])
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		_logger.log(Level.FINE, "XMIImport readingAttributes(" + package_name_ + ", " + classes_.length + ")");
		outputln("// ==================================================================");
		outputln("//  readXXXYYY with XXX an entity and YYY its property");
		outputln("// ==================================================================");
		outputln();
		// Classes
		for (int i = 0; i < classes_.length; i++) {
			org.omg.mof.Model.Class cls = (org.omg.mof.Model.Class) classes_[i];
			String clsName = cls.name();
			org.omg.mof.Model.ModelElement[] contents = cls.contents();
			for (int j = 0; j < contents.length; j++) {
				if (contents[j]._is_a(org.omg.mof.Model.MofAttributeHelper.id())) {
					org.omg.mof.Model.MofAttribute attr = org.omg.mof.Model.MofAttributeHelper.narrow(contents[j]);
					org.omg.mof.Model.Classifier type = attr.type();
					String attrName = attr.name();
					if (attr.is_changeable()) {
						if (type._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
							try {
								org.omg.mof.Model.DataType data_type = org.omg.mof.Model.DataTypeHelper.narrow(type);
								org.omg.CORBA.TypeCode type_code = data_type.type_code();
								int type_code_value = type_code.kind().value();
								if (type_code_value == org.omg.CORBA.TCKind._tk_TypeCode) {
									readClassTypeCodeAttributeMethod(package_name_, cls, attrName);
								} else if (type_code_value == org.omg.CORBA.TCKind._tk_enum) {
									java.util.List associatedTags = new java.util.ArrayList();
									org.omg.mof.Model.ModelElement clsContents[] = type.container().contents();
									for (int c = 0; c < clsContents.length; c++) {
										if (clsContents[c]._is_a(org.omg.mof.Model.TagHelper.id())) {
											org.omg.mof.Model.Tag tag = org.omg.mof.Model.TagHelper.narrow(clsContents[c]);
											org.omg.mof.Model.ModelElement[] elements = tag.elements();
											for (int e = 0; e < elements.length; e++) {
												if (elements[e]._is_equivalent(type))
													associatedTags.add(tag);
											}
										}
									}
									readClassEnumerationTypeAttributeMethod(package_name_, cls, attr, type_code, associatedTags);
								} else if (type_code_value == org.omg.CORBA.TCKind._tk_struct) {
									readClassStructureTypeAttributeMethod(package_name_, cls, attr, type_code);
								} else if (type_code_value == org.omg.CORBA.TCKind._tk_alias) {
									readClassAliasTypeMethod(package_name_, cls, attr, type_code.content_type());
								} else if (type_code_value == org.omg.CORBA.TCKind._tk_sequence) {
									_logger.log(
										Level.INFO,
										"SequenceType " + idlHelper.format1(clsName) + "." + idlHelper.format1(attrName) + " ignored");
									//throw new org.omg.CORBA.NO_IMPLEMENT("CollectionType reading method not implemented", 1,
									//				     org.omg.CORBA.CompletionStatus.COMPLETED_NO);
								} else
									readClassAttributeMethod(
										package_name_,
										cls,
										org.omg.mof.Model.MofAttributeHelper.narrow(contents[j]),
										type.name(),
										type_code);
							} catch (org.omg.mof.Reflective.NotSet notSet) {
								// Nothing to do
							}
						} else {
							try {
								readClassAttributeMethod(
									package_name_,
									cls,
									org.omg.mof.Model.MofAttributeHelper.narrow(contents[j]),
									type.name(),
									null);
							} catch (NotSet notSet) {
								// Nothing to do
							}
						}
					}
				} else if (contents[j]._is_a(org.omg.mof.Model.ReferenceHelper.id())) {
					try {
						org.omg.mof.Model.Reference ref = org.omg.mof.Model.ReferenceHelper.narrow(contents[j]);
						org.omg.mof.Model.AssociationEnd assoEnd = ref.referenced_end();
						org.omg.mof.Model.AssociationEnd otherEnd = mofHelper.oppositeAssociationEnd(assoEnd);
						boolean none = (assoEnd.aggregation().value() == org.omg.mof.Model.AggregationKind._none);
						boolean composite = (assoEnd.aggregation().value() == org.omg.mof.Model.AggregationKind._composite);

						//if (composite) {
						//	org.omg.mof.Model.Class[] subClasses = mofHelper.subClassesOfClass(cls);
						//	System.out.println("composite -> " + cls.name() + "-" + assoEnd.name());
						//	readCompositeAssociationAttributeMethod(package_name_, cls, subClasses, assoEnd.name());
						//} else
						if (none) {
							org.omg.mof.Model.Association asso = org.omg.mof.Model.AssociationHelper.narrow(assoEnd.container());
							org.omg.mof.Model.Reference[] refs = xmiImpHelper.getReferences(asso);
							for (int r = 0; r < refs.length; r++) {
								String hashtable = "_" + idlHelper.format2(asso.name()) + "_" + idlHelper.format2(refs[r].name());
								try {
									readAssociationAttributeMethod(package_name_, refs[r].container(), refs[r], hashtable);
								} catch (org.omg.mof.Reflective.NotSet notSet) {
									notSet.printStackTrace();
								}
							}
						}
					} catch (org.omg.mof.Reflective.NotSet notSet) {
						// Nothing to do
					}
				}
			}
		}
	}

	/**
	 * XMIImport generation for the class attribute reading method.
	 * @param package_name_ The package name.
	 * @param classes_name_ The class name.
	 * @param attribute_name_ The attribute name.
	 */
	public void readClassAttributeMethod(
		String package_name_,
		org.omg.mof.Model.Class class_,
		org.omg.mof.Model.MofAttribute attribute_,
		String type_name_,
		org.omg.CORBA.TypeCode type_)
		throws BadKind, Bounds, MofError, NotSet {
		_logger.log(
			Level.FINE,
			"XMIImport readClassAttributeMethod(" + package_name_ + ", " + class_.name() + ", " + attribute_.name() + ", " + type_ + ")");
		String type_name = type_name_;
		if (type_name_.startsWith("*"))
			type_name = idlHelper.format1(type_name_.substring(1));
		else
			type_name = idlHelper.format1(type_name_);
		String operation = "add_";
		if (attribute_.multiplicity().upper == 1)
			operation = "set_";
		String attrName = attribute_.name();
		String attrName4 = idlHelper.format1FirstMin(attrName);
		outputln("/**");
		outputln(" * Read the '" + idlHelper.format1(attrName) + "' attribute of the " + idlHelper.format1(class_.name()) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + idlHelper.format1(class_.name()) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + idlHelper.format1(class_.name()) + idlHelper.format1(attrName) + "(");
		outputln("org.w3c.dom.Element parent, " + javaHelper.javaQualifiedName(class_) + " model, org.w3c.dom.Element element)");
		outputln("throws org.omg.mof.Reflective.MofError {");
		//if (_helper.isPrimitiveType(type_name_)) {
		if (type_ != null && idlHelper.isBaseType(type_)) {
			String method = typeCode2ReadingMethod(type_);
			outputln("if (parent.hasAttribute(\"" + attrName4 + "\"))");
			outputln("    model." + operation + idlHelper.format2(attrName)
					+ "(" + method + "(parent.getAttribute(\""
					+ attrName4	+ "\")));");
			outputln("else if (element != null && nodeNameCorrespondsTo(element, \"" + idlHelper.format1(attrName) + "\")) {");
			outputln("if (element.getFirstChild() != null)");
			outputln("    model." + operation + idlHelper.format2(attrName)
					+ "(" + method + "(element.getFirstChild().getNodeValue().trim()));");
			outputln("else");
			outputln("    model." + operation + idlHelper.format2(attrName)
					+ "(" + method + "(element.getAttribute(\"xmi.value\").trim()));");
			outputln("}");
		} else {
			outputln(
				"if (element != null && element.getFirstChild() != null && nodeNameCorrespondsTo(element, \""
					+ idlHelper.format1(attrName) + "\"))");
			String paramType = javaHelper.javaQualifiedName(attribute_.type());
			outputln("    model." + operation + idlHelper.format2(attrName)
					+ "((" + paramType + ")_xmi_import_classes." + idlHelper.format1FirstMin(type_name)
					+ "Template((org.w3c.dom.Element)element.getFirstChild()).get());");
		}
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the class attribute reading method (for a TypeCode attribute).
	 * @param package_name_ The package name.
	 * @param classes_name_ The class name.
	 * @param attribute_name_ The attribute name.
	 */
	public void readClassTypeCodeAttributeMethod(String package_name_, org.omg.mof.Model.Class class_, String attribute_name_)
		throws MofError, NotSet {
		String class_name = class_.name();
		_logger.log(
			Level.FINE,
			"XMIImport readClassTypeCodeAttributeMethod(" + package_name_ + ", " + class_name + ", " + attribute_name_ + ")");
		outputln("/**");
		outputln(" * Read the '" + idlHelper.format1(attribute_name_) + "' attribute of the " + idlHelper.format1(class_name) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + idlHelper.format1(class_name) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + idlHelper.format1(class_name) + idlHelper.format1(attribute_name_) + "(");
		outputln("org.w3c.dom.Element parent, " + javaHelper.javaQualifiedName(class_) + " model, org.w3c.dom.Element element)");
		outputln("throws org.omg.mof.Reflective.MofError {");
		outputln("String repositoryID = \"IDL:\" + idlHelper.format1(model.name());");
		outputln("if (element != null && nodeNameCorrespondsTo(element, \"" + idlHelper.format1(attribute_name_) + "\")) {");
		outputln("org.w3c.dom.Element _corba_typecode = (org.w3c.dom.Element) element.getElementsByTagName(\"XMI.CorbaTypeCode\").item(0);");
		outputln("org.w3c.dom.Element _element = (org.w3c.dom.Element) _corba_typecode.getChildNodes().item(0);");
		outputln("model.set_" + idlHelper.format2(attribute_name_) + "(xml2TypeCode(_element, repositoryID));");
		outputln("}");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the class attribute reading method (for an EnumarationType attribute).
	 * @param package_name_ The package name.
	 * @param classes_ The class.
	 * @param attribute_ The attribute.
	 * @param type_code_ The attribute type code.
	 * @param associated_tags_ The tags associated to the Data Type.
	 */
	public void readClassEnumerationTypeAttributeMethod(
		String package_name_,
		org.omg.mof.Model.Class class_,
		org.omg.mof.Model.MofAttribute attribute_,
		org.omg.CORBA.TypeCode type_code_,
		java.util.List associated_tags_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		String attrName = attribute_.name();
		String attrName4 = idlHelper.format1FirstMin(attrName);
		_logger.log(
			Level.FINE,
			"XMIImport readClassEnumerationTypeAttributeMethod("
				+ package_name_
				+ ", "
				+ class_.name()
				+ ", "
				+ attrName
				+ ", "
				+ type_code_.kind().value()
				+ ")");
		outputln("/**");
		outputln(" * Read the '" + idlHelper.format1(attrName) + "' attribute of the " + idlHelper.format1(class_.name()) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + idlHelper.format1(class_.name()) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + idlHelper.format1(class_.name()) + idlHelper.format1(attrName) + "(");
		try {
			outputln("org.w3c.dom.Element parent, " + javaHelper.javaQualifiedName(class_) + " model, org.w3c.dom.Element element)");
		} catch (NotSet notSet) {
			outputln("org.w3c.dom.Element parent, " + idlHelper.format1(class_.name()) + " model, org.w3c.dom.Element element)");
		}
		outputln("throws org.omg.mof.Reflective.MofError {");
		outputln("java.lang.String " + attrName4 + " = new java.lang.String();");
		outputln("if (parent.hasAttribute(\"" + attrName4 + "\"))");
		outputln(TABULATION + attrName4 + " = parent.getAttribute(\"" + attrName4 + "\");");
		outputln("else if (element != null && nodeNameCorrespondsTo(element, \"" + idlHelper.format1(attrName) + "\")) {");
		outputln("if (element.getFirstChild() != null)");
		outputln(TABULATION + attrName4 + " = element.getFirstChild().getNodeValue().trim();");
		outputln("else");
		outputln(TABULATION + attrName4 + " = element.getAttribute(\"xmi.value\").trim();");
		outputln("}");
		outputln("if (" + attrName4 + " != null && " + attrName4 + ".length() > 0) {");
		String operation = "add_";
		if (attribute_.multiplicity().upper == 1)
			operation = "set_";

		// Get the prefix
		String prefix = new String();
		java.util.Iterator it = associated_tags_.iterator();
		while (it.hasNext()) {
			org.omg.mof.Model.Tag tag = (org.omg.mof.Model.Tag) it.next();
			if (tag.tag_id().equals("org.omg.xmi.enumerationUnprefix")) {
				org.omg.CORBA.Any[] values = tag.values();
				for (int v = 0; v < values.length; v++) {
					if (values[v].type().kind().value() == org.omg.CORBA.TCKind._tk_string)
						prefix = values[v].extract_string();
				}
			}
		}
		for (int i = 0; i < type_code_.member_count(); i++) {
			String member_name = StringHelper.replaceFirst(type_code_.member_name(i), prefix, "");
			outputln("if (" + attrName4 + ".endsWith(\"" + member_name + "\"))");
			try {
				outputln(
					TABULATION
						+ "model."
						+ operation
						+ idlHelper.format2(attrName)
						+ "("
						+ javaHelper.javaQualifiedName(attribute_.type())
						+ "."
						+ type_code_.member_name(i)
						+ ");");
			} catch (NotSet notSet) {
				outputln(TABULATION + "break;");
			}
		}
		outputln("}");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the class attribute reading method (for a StructureType attribute).
	 * @param package_name_ The package name.
	 * @param classes_name_ The class name.
	 * @param attribute_name_ The attribute name.
	 */
	public void readClassStructureTypeAttributeMethod(
		String package_name_,
		org.omg.mof.Model.Class class_,
		org.omg.mof.Model.MofAttribute attribute_,
		org.omg.CORBA.TypeCode type_code_)
		throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
		try {
			String attrName = attribute_.name();
			String classQualifiedName = javaHelper.javaQualifiedName(class_);
			String attributeQualifiedName = javaHelper.javaQualifiedName(attribute_.type());
			_logger.log(
				Level.FINE,
				"XMIImport readClassStructureTypeAttributeMethod("
					+ package_name_
					+ ", "
					+ class_.name()
					+ ", "
					+ attrName
					+ ", "
					+ type_code_.kind().value()
					+ ")");
			outputln("/**");
			outputln(" * Read the '" + idlHelper.format1(attrName) + "' attribute of the " + idlHelper.format1(class_.name()) + ".");
			outputln(" * @param parent The parent of the current element in the XMI file.");
			outputln(" * @param model The " + idlHelper.format1(class_.name()) + " to set.");
			outputln(" * @param element The current element in the XMI file.");
			outputln(" */");
			output("public void read" + idlHelper.format1(class_.name()) + idlHelper.format1(attrName) + "(");
			outputln("org.w3c.dom.Element parent, " + classQualifiedName + " model, org.w3c.dom.Element element)");
			outputln("throws org.omg.mof.Reflective.MofError {");
			outputln("if (element != null && nodeNameCorrespondsTo(element, \"" + idlHelper.format1(attrName) + "\")) {");
			for (int i = 0; i < type_code_.member_count(); i++) {
				outputln(
					xmiImpHelper.typeCode2Java(type_code_.member_type(i))
						+ " _"
						+ idlHelper.format2(type_code_.member_name(i))
						+ " = "
						+ xmiImpHelper.defaultAttributeValue(type_code_.member_type(i))
						+ ";");
			}
			outputln("org.w3c.dom.NodeList _" + idlHelper.format2(attrName) + " = element.getChildNodes();");
			outputln("for (int j = 0; j < _" + idlHelper.format2(attrName) + ".getLength(); j++) {");
			outputln("org.w3c.dom.Element _item = (org.w3c.dom.Element) _" + idlHelper.format2(attrName) + ".item(j);");
			outputln("if (_item.getFirstChild() != null) {");
			outputln("java.lang.String value = _item.getFirstChild().getNodeValue().trim();");
			for (int i = 0; i < type_code_.member_count(); i++) {
				outputln("if (j==" + i + ")");
				outputln(
					"   _"
						+ idlHelper.format2(type_code_.member_name(i))
						+ " = read"
						+ idlHelper.format1(xmiImpHelper.typeCode2Java(type_code_.member_type(i)))
						+ "Type(value);");
			}
			outputln("}");
			outputln("}");

			for (int i = 0; i < type_code_.member_count(); i++) {
				outputln("if (element.hasAttribute(\"" + idlHelper.format1FirstMin(type_code_.member_name(i)) + "\"))");
				outputln(
					"   _"
						+ idlHelper.format2(type_code_.member_name(i))
						+ " = read"
						+ idlHelper.format1(xmiImpHelper.typeCode2Java(type_code_.member_type(i)))
						+ "Type(element.getAttribute(\""
						+ idlHelper.format1FirstMin(type_code_.member_name(i))
						+ "\"));");
			}
			output("model.set_" + idlHelper.format2(attrName) + "(new " + attributeQualifiedName + "(");
			for (int i = 0; i < type_code_.member_count(); i++) {
				if (i > 0)
					output(", ");
				output("_" + idlHelper.format2(type_code_.member_name(i)));
			}
			outputln("));");
			outputln("}");
			outputln("}");
			outputln();
		} catch (NotSet notSet) {
			// Nothing to do.
		}
	}

	/**
	 * XMIImport generation for the class attribute reading method (for an AliasType attribute).
	 * @param package_name_ The package name.
	 * @param cls The Model class.
	 * @param attribute_ The AliasType attribute.
	 * @param content_type_ The Content Type of the AliasType.
	 */
	public void readClassAliasTypeMethod(
		String package_name_,
		org.omg.mof.Model.Class cls,
		org.omg.mof.Model.MofAttribute attribute_,
		org.omg.CORBA.TypeCode content_type_)
		throws MofError, NotSet {
		_logger.log(Level.FINE, "XMIImport readClassAliasType(" + package_name_ + ", " + attribute_.name() + ")");
		String attrName = attribute_.name();
		String attrName4 = idlHelper.format1FirstMin(attrName);
		String attrName2 = idlHelper.format2(attrName);
		outputln("/**");
		outputln(" * Read the '" + idlHelper.format1(attrName) + "' attribute of the " + idlHelper.format1(cls.name()) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + idlHelper.format1(cls.name()) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + idlHelper.format1(cls.name()) + idlHelper.format1(attrName) + "(");
		outputln("org.w3c.dom.Element parent, " + javaHelper.javaQualifiedName(cls) + " model, org.w3c.dom.Element element)");
		outputln("throws org.omg.mof.Reflective.MofError {");
		String operation = "add_";
		if (attribute_.multiplicity().upper == 1)
			operation = "set_";
		outputln("if (parent.hasAttribute(\"" + attrName4 + "\"))");

		String contentType = typeCode2ReadingMethod(content_type_);
		outputln("    model." + operation + attrName2 + "(" + contentType + "(parent.getAttribute(\"" + attrName4 + "\")));");
		outputln(
			"else if (element != null && element.getFirstChild() != null && nodeNameCorrespondsTo(element, \""
				+ idlHelper.format1(attrName)
				+ "\"))");
		outputln("    model." + operation + attrName2 + "(" + contentType + "(element.getFirstChild().getNodeValue().trim()));");
		outputln("}");
		outputln();
	}

	/**
	 * Pass from the TypeCode to the name of the associated reading method.
	 * @param type_code The TypeCode.
	 * @return The name of the reading method.
	 */
	public String typeCode2ReadingMethod(org.omg.CORBA.TypeCode type_code) {
		switch (type_code.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				return "readBooleanType";
			case org.omg.CORBA.TCKind._tk_short :
			case org.omg.CORBA.TCKind._tk_ushort :
			case org.omg.CORBA.TCKind._tk_long :
			case org.omg.CORBA.TCKind._tk_ulong :
				return "java.lang.Integer.parseInt";
			case org.omg.CORBA.TCKind._tk_longlong :
			case org.omg.CORBA.TCKind._tk_ulonglong :
				return "java.lang.Long.parseLong";
			case org.omg.CORBA.TCKind._tk_double :
				return "java.lang.Double.parseDouble";
			case org.omg.CORBA.TCKind._tk_float :
				return "java.lang.Float.parseFloat";
			case org.omg.CORBA.TCKind._tk_string :
			case org.omg.CORBA.TCKind._tk_wstring :
				return "java.lang.String.valueOf";
			default :
				return "java.lang.String.valueOf";
		}
	}

	/**
	 * XMIImport generation for the association attribute reading method.
	 * @param package_name_ The package name.
	 * @param class_ The class name.
	 * @param ref_ The reference attribute.
	 * @param hashtable_ The name of the hashtable.
	 */
	public void readAssociationAttributeMethod(
		String package_name_,
		org.omg.mof.Model.Namespace class_,
		org.omg.mof.Model.Reference ref_,
		String hashtable_)
		throws MofError, NotSet {
		String className = class_.name();
		String refName = ref_.name();
		String typeName = class_.name();
		_logger.log(
			Level.FINE,
			"XMIImport readAssociationAttributeMethod(" + package_name_ + ", " + className + ", " + refName + ", " + hashtable_ + ")");
		String signature = "public void read" + idlHelper.format1(className) + idlHelper.format1(refName) + "(";
		if (!_associationAttributes.contains(signature)) {
			outputln("/**");
			outputln(" * Read the '" + idlHelper.format1(refName) + "' attribute of the " + idlHelper.format1(className) + ".");
			outputln(" * @param parent The parent of the current element in the XMI file.");
			outputln(" * @param model The " + idlHelper.format1(class_.name()) + " to set.");
			outputln(" * @param element The current element in the XMI file.");
			outputln(" */");
			_associationAttributes.add(signature);
			output(signature);

			outputln("org.w3c.dom.Element parent, " + javaHelper.javaQualifiedName(class_) + " model, org.w3c.dom.Element element)");
			outputln("throws org.omg.mof.Reflective.MofError {");

			//// If generation problem with the '_' character : use 'idlhelper.format#(refName)' instead of 'refName'.
			outputln(
				"if (element != null && element.getNodeName().trim().endsWith(\"" + idlHelper.format1(typeName) + "." + refName + "\")) {");
			outputln("org.w3c.dom.NodeList _child = element.getChildNodes();");
			String operation = "add_";
			if (ref_.multiplicity().upper == 1)
				operation = "set_";
			if (ref_.type().is_abstract()) {
				outputln(
					"org.objectweb.modfact.corba.xmiio.mofobject.MOFObjectList list = _xmi_import_classes."
						+ idlHelper.format1FirstMin(ref_.type().name())
						+ "Template(element);");
				outputln("java.util.Iterator it = list.getObjects();");
				outputln("while (it.hasNext())");
				outputln(
					TABULATION
						+ "model."
						+ operation
						+ idlHelper.format2(refName)
						+ "(("
						+ javaHelper.javaQualifiedName(ref_.type())
						+ ")it.next());");
				outputln("it = list.getXmiIdRef();");
				outputln("while (it.hasNext())");
				outputln(TABULATION + "_helper.addHashtableValue(" + hashtable_ + ", model, (java.lang.String)it.next());");
			} else {
				outputln("int i = 0;");
				outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
				outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
				outputln("if (_current != null) {");
				String variableType = javaHelper.javaQualifiedName(ref_.type());
				outputln(
					"org.objectweb.modfact.corba.xmiio.mofobject.MOFObject mof_obj = _xmi_import_classes."
						+ idlHelper.format1FirstMin(ref_.type().name())
						+ "Template(_current);");
				outputln("if (mof_obj.isAnObject())");
				outputln(TABULATION + "model." + operation + idlHelper.format2(ref_.name()) + "((" + variableType + ")mof_obj.get());");
				outputln("else if (mof_obj.isValid())");
				outputln(TABULATION + "_helper.addHashtableValue(" + hashtable_ + ", model, mof_obj.get());");
				outputln("}");
				outputln("i++;");
				outputln("}");
			}
			outputln("}");
			outputln("}");
			outputln();
		}
	}

	/**
	 * XMIImport generation for the composite association attribute method.
	 * @param package_name_ The package name.
	 * @param classes_ The class.
	 * @param subclass_ The subclass.
	 * @param attribute_name_ The attribute name.
	 */
	public void readCompositeAssociationAttributeMethod(
		String package_name_,
		org.omg.mof.Model.Class class_,
		org.omg.mof.Model.Class[] subclass_,
		String attribute_name_)
		throws MofError, NotSet {
		_logger.log(
			Level.FINE,
			"XMIImport readCompositeAssociationAttributeMethod("
				+ package_name_
				+ ", "
				+ class_.name()
				+ ", "
				+ subclass_.length
				+ ", "
				+ attribute_name_
				+ ")");
		outputln("/**");
		outputln(" * Read the '" + idlHelper.format1(attribute_name_) + "' attribute of the " + idlHelper.format1(class_.name()) + ".");
		outputln(" * @param parent The parent of the current element in the XMI file.");
		outputln(" * @param model The " + idlHelper.format1(attribute_name_) + " to set.");
		outputln(" * @param element The current element in the XMI file.");
		outputln(" */");
		output("public void read" + idlHelper.format1(class_.name()) + idlHelper.format1(attribute_name_) + "(");
		outputln("org.w3c.dom.Element parent, " + javaHelper.javaQualifiedName(class_) + " model, org.w3c.dom.Element element)");
		outputln("throws org.omg.mof.Reflective.MofError {");
		outputln(
			"if (element != null && element.getNodeName().trim().endsWith(\""
				+ javaHelper.javaQualifiedName(class_)
				+ "."
				+ idlHelper.format1(attribute_name_)
				+ "\")) {");
		outputln("org.w3c.dom.NodeList _child = element.getChildNodes();");
		outputln("int i = 0;");
		outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
		outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
		outputln("java.lang.String _current_name = _current.getNodeName().trim();");
		for (int i = 0; i < subclass_.length; i++) {
			if (i > 0)
				output("else ");
			outputln("if (_current_name.endsWith(\"" + idlHelper.format1(subclass_[i].name()) + "\"))");
			outputln("   _xmi_import_classes." + idlHelper.format1FirstMin(subclass_[i].name()) + "Template(_current);");
		}
		outputln("i++;");
		outputln("}");
		outputln("}");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the "put" methods.
	 * @param package_name_ The package name.
	 * @param associations_ The associations of the package.
	 */
	public void putAssociationsValues(String package_name_, org.omg.mof.Model.Association associations_[])
		throws org.omg.mof.Reflective.MofError {
		_logger.log(Level.FINE, "XMIImport putAssociationsValue(" + package_name_ + ", " + associations_.length + ")");
		for (int i = 0; i < associations_.length; i++) {
			org.omg.mof.Model.Association asso = (org.omg.mof.Model.Association) associations_[i];
			try {
				org.omg.mof.Model.Reference[] refs = mofHelper.referencesOfAssociation(asso);
				for (int j = 0; j < refs.length; j++) {
					org.omg.mof.Model.AssociationEnd assoEnd = refs[j].referenced_end();
					if (assoEnd.aggregation().value() == org.omg.mof.Model.AggregationKind._none && assoEnd.is_changeable()) {
						String hashtable = "_" + idlHelper.format2(asso.name()) + "_" + idlHelper.format2(refs[j].name());
						String type = javaHelper.javaQualifiedName(refs[j].container());
						String typeName = refs[j].container().name();
						String typeClass = type;
						String ref = refs[j].name();
						String typeRef = javaHelper.javaQualifiedName(refs[j].referenced_end().type());
						String vector = idlHelper.format2(ref) + "_vector";
						String type2 = idlHelper.format2(type);
						String ref2 = idlHelper.format2(ref);
						annotationTemplate(asso.annotation());
						outputln("public void put" + idlHelper.format1(typeName) + idlHelper.format1(ref) + "()");
						outputln("    throws org.omg.mof.Reflective.MofError {");
						outputln(
							"_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"put"
								+ idlHelper.format1(typeName)
								+ idlHelper.format1(ref)
								+ "()\");");
						outputln("java.util.Enumeration _keys = " + hashtable + ".keys();");
						outputln("while (_keys.hasMoreElements()) {");
						outputln(typeClass + " _" + type2 + " = (" + typeClass + ") _keys.nextElement();");
						outputln("java.util.Vector _" + vector + " = (java.util.Vector) " + hashtable + ".get(_" + type2 + ");");
						outputln("for (int i=0 ; i<_" + vector + ".size() ; i++) {");
						outputln("java.lang.String _key = ((java.lang.String) _" + vector + ".elementAt(i)).trim();");
						outputln(typeRef + " _value = (" + typeRef + ") _types.get(_key);");
						if (refs[j].multiplicity().upper == 1) {
							if (refs[j].multiplicity().lower == 0) {
								outputln("try {");
								outputln("if (_value != null && !_value._is_equivalent(_" + type2 + "." + ref2 + "()))");
								outputln("    _" + type2 + ".set_" + ref2 + "(_value);");
								outputln("} catch (org.omg.mof.Reflective.NotSet notSet) {");
								outputln("_" + type2 + ".set_" + ref2 + "(_value);");
								outputln("}");
							} else {
								outputln(
									"if (_"
										+ type2
										+ "."
										+ ref2
										+ "() == null || (_value != null && !_value._is_equivalent(_"
										+ type2
										+ "."
										+ ref2
										+ "()))) {");
								outputln("_" + type2 + ".set_" + ref2 + "(_value);");
								outputln("}");
							}
						} else {
							outputln("if (_value != null && !_helper.arrayContains(_" + type2 + "." + ref2 + "(), _value)) {");
							outputln("_" + type2 + ".add_" + ref2 + "(_value);");
							outputln("}");
						}

						if (refs.length > 1
							&& refs[(j + 1) % 2].referenced_end().aggregation().value() != org.omg.mof.Model.AggregationKind._none) {
							// There is j=0 or j=1, so, to switch between the values, we can use: (j+1)%2
							if (refs[(j + 1) % 2].multiplicity().upper == 1)
								outputln(
									"_value.set_" + idlHelper.format2(refs[(j + 1) % 2].name()) + "(_" + idlHelper.format2(type) + ");");
							else
								outputln(
									"_value.add_" + idlHelper.format2(refs[(j + 1) % 2].name()) + "(_" + idlHelper.format2(type) + ");");
						}
						outputln("}");
						outputln("}");
						outputln("}");
						outputln();
					}
				}
			} catch (org.omg.mof.Reflective.NotSet notSet) {
				// Nothing to do
			}
		}
	}

}
