/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.importer;

import org.objectweb.modfact.corba.logging.Level;
import org.objectweb.modfact.corba.logging.ModFactLogger;
import org.objectweb.modfact.corba.xmiio.common.AbstractXMIIOGenerator;
import org.omg.mof.Reflective.MofError;
import org.omg.mof.Reflective.NotSet;

/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImportClasses extends AbstractXMIIOGenerator {

    /** The logger. */
    ModFactLogger _logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImport");

    /** The generated association attributes methods. */
    private java.util.List _associationAttributes;

    /**
     * Default Constructor.
     */
    public GenerationXMIImportClasses() {
        super();
		ModFactLogger logger = ModFactLogger.getLogger("fr.lip6.src.mdafactory.generation.xmiimport.GenerationXMIImportApplication");
        _associationAttributes = new java.util.ArrayList();
    }

    /**
     * XMIImport generation.
     * @param package_ The root package.
     * @param class_name_ The class name of the generated class.
     */
    public void generateXMIImportClasses(org.omg.mof.Model.Package package_, String class_name_) throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds, java.io.FileNotFoundException {
        _logger.log(Level.FINE, "XMIImport beginning of the generation");
        _logger.log(Level.INFO, class_name_ + " generation ...");
        packageModuleTemplate(package_, class_name_);
        flushFile();
    }

    /**
     * XMIImport generation for constructor.
     * @param class_name The class name.
     */
    public void constructor(String class_name, String package_name) {
        _logger.log(Level.FINE, "XMIImport constructor(" + class_name + ")");
        outputln("/** The logger. */");
        outputln("org.objectweb.modfact.corba.logging.ModFactLogger _logger;");
		outputln();
        outputln("/** The XMIImportAttributes class. */");
        outputln(package_name + "XMIImportAttributes _xmi_import_attributes;");
        outputln();
        outputln("/** Hashtables for types (Key / Model) */");
        outputln("private java.util.Hashtable _types;");
        outputln();
        outputln("/**");
        outputln(" * Default " + class_name + " Constructor");
        outputln(" * @param logger The logger.");
        outputln(" * @param types The defined types.");
        outputln(" */");
        outputln("public " + class_name + "(org.objectweb.modfact.corba.logging.ModFactLogger logger, java.util.Hashtable types) {");
        outputln("_logger = logger;");
        outputln("_types = types;");
        outputln("}");
        outputln();
        outputln("public void setXMIImportAttributes(" + package_name + "XMIImportAttributes xmi_import_attributes) {");
        outputln("_xmi_import_attributes = xmi_import_attributes;");
        outputln("}");
        outputln();
    }

    /**
     * Package processing.
     * @param package_ The package to process.
     * @param class_name_ The class name of the generated class.
     */
    public void packageModuleTemplate(org.omg.mof.Model.Package package_, String class_name_) throws org.omg.mof.Reflective.MofError, org.omg.CORBA.TypeCodePackage.BadKind, org.omg.CORBA.TypeCodePackage.Bounds {
        _logger.log(Level.FINE, "XMIImport packageModuleTemplate(" + package_.name() + ")");
        if (package_.visibility().equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
            org.omg.mof.Model.Class classes_ordered [] = mofHelper.allClassesOfPackageOrderedByInheritance(package_);            
            _logger.log(Level.FINE, "XMIImport produce");

			outputln("package " + javaHelper.javaPackage(package_) + ".xmi;");
            outputln();
            annotationTemplate(package_.annotation());
            outputln("public class " + class_name_ + " extends org.objectweb.modfact.corba.xmiio.importer.XMIImport {");
            classAttributes(package_, classes_ordered);
            constructor(class_name_, package_.name());
            initiateMethod(package_, classes_ordered);
            classTemplate(package_.name(), classes_ordered);
            outputln("} // end of class " + class_name_);
            outputln();
        }
    } 
    
    /**
    * XMIImport generation for the class attributes.
    * @param package_name_ The package name.
    * @param classes_ The classes of the package.
    */
   public void classAttributes(org.omg.mof.Model.Package package_, org.omg.mof.Model.Class classes_ []) throws MofError {
   	   String package_name = package_.name();
       _logger.log(Level.FINE, "XMIImport classAttributes(" + package_ + ")");
       outputln("// Factories Declaration");
       String model_package_type;
       if (mappingValid) {
           model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
       } else {
           model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
       }
       outputln("private " + model_package_type + " _" + idlHelper.format2(package_name) + "_package = null;");
       // Classes
       for (int i = 0; i < classes_.length; i++) {
        org.omg.mof.Model.Class cls = (org.omg.mof.Model.Class) classes_[i];
        String className;
        try {
        	className = idlHelper.className(cls);
        } catch (NotSet notSet) {
        	className = cls.name();
        }
        String javaName;
        try {
            javaName = javaHelper.javaPackage(org.omg.mof.Model.PackageHelper.narrow(cls.container())) + "." + className;
			outputln("private " + javaName + "Class _" + idlHelper.format2(javaHelper.javaQualifiedName(cls)) + "_class = null;");
        } catch (org.omg.mof.Reflective.NotSet notSet) {
            // Nothing to do.
        }
    }
    outputln();
   }

    /**
     * XMIImport generation for the initiate method.
     * @param package_ The package.
     * @param classes_ The classes of the package.
     */
    public void initiateMethod(org.omg.mof.Model.Package package_, org.omg.mof.Model.Class classes_ []) throws MofError {
    	String package_name = package_.name();
        _logger.log(Level.FINE, "XMIImport initiateMethod(" + package_name + ", " + classes_.length + ")");
        String model_package_type;
        if (mappingValid) {
            model_package_type = javaHelper.javaPackage(package_) + "._" + idlHelper.format1(package_name) + "Package";
        } else {
            model_package_type = javaHelper.javaPackage(package_) + "." + idlHelper.format1(package_name) + "Package";
        }
        outputln("public void initiate(" + model_package_type + " _model_package)");
        outputln("    throws org.omg.mof.Reflective.MofError {");
        outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"initiate(\" + _model_package + \")\");");
        outputln("// Factories creation");
        outputln("this._" + idlHelper.format2(package_name) + "_package = _model_package;");
        // Classes
        initiateMethodClasses(package_name, classes_);
        outputln("}");
        outputln();
    }
    
    /**
     * XMIImport generation for the "class attributes" of the initiate method.
     * @param package_name_ The package name.
     * @param classes_ The classes of the package.
     */
    public void initiateMethodClasses(String package_name_, org.omg.mof.Model.Class classes_ []) throws MofError {
        for (int i = 0; i < classes_.length; i++) {
            org.omg.mof.Model.Class cls = (org.omg.mof.Model.Class) classes_[i];
            String className = cls.name();
            String javaName;
            try {
				className = javaHelper.className(cls);
            	javaName = idlHelper.format2(javaHelper.javaQualifiedName(cls));
            } catch (NotSet notSet) {
            	javaName = idlHelper.format2(className);
            }
            String method = idlHelper.format2(className) + "_ref()";
            try {
                org.omg.mof.Model.ModelElement clsTmp = cls;
                while (!clsTmp.container().name().equals(package_name_)) {
                	String containerName = clsTmp.container().name();
                	if (clsTmp.container()._is_a(org.omg.mof.Model.ClassifierHelper.id())) {
                		org.omg.mof.Model.Classifier container = org.omg.mof.Model.ClassifierHelper.narrow(clsTmp.container());
                		containerName = javaHelper.javaClassifierName(container);
					}
                    method = idlHelper.format2(containerName) + "_ref()." + method;
                    clsTmp = clsTmp.container();
                }
            } catch (org.omg.CORBA.MARSHAL marshal) {
            	/* The marshal exception contains the NotSet Exception */
                if (!marshal.getMessage().equals("IDL:mof.omg.org/Reflective/NotSet:1.0"))
                    throw marshal;
            } catch (org.omg.mof.Reflective.NotSet notSet) {
                // Nothing to do
            }
            outputln("_" + javaName + "_class = _model_package." + method + ";");
        }
    }

    /**
     * XMIImport generation for the class template.
     * @param package_name_ The package name.
     * @param classes_ The classes of the package.
     */
    public void classTemplate(String package_name_, org.omg.mof.Model.Class classes_ []) throws org.omg.mof.Reflective.MofError {
        _logger.log(Level.FINE, "XMIImport classTemplate(" + package_name_ + ", " + classes_.length + ")");
        outputln("// ==================================================================");
        outputln("//  XXXTemplate with XXX an entity");
        outputln("// ==================================================================");
        // Classes
        for (int i = 0; i < classes_.length; i++) {
            org.omg.mof.Model.Class cls = (org.omg.mof.Model.Class) classes_[i];
            if (cls.is_abstract())
                classAbstractTemplate(cls);
            else
                classSpecificTemplate(cls);
            outputln();
        }
        outputln();
    }

    /**
      * XMIImport generation for the abstract class template.
      * @param cls The abstract class of the package.
      */
    public void classAbstractTemplate(org.omg.mof.Model.Class cls) throws org.omg.mof.Reflective.MofError {
        String clsName = cls.name();
        String clsName1 = idlHelper.format1(clsName);
        String clsName2 = idlHelper.format2(clsName);
        String javaName;
        try {
        	javaName = javaHelper.javaQualifiedName(cls);
        } catch (NotSet notSet) {
        	javaName = clsName;
        }
        String methodName = idlHelper.format1FirstMin(clsName);
        String returnType = javaName;
        String attributeName = "_" + clsName2 + "_element";
        annotationTemplate(cls.annotation());
        outputln("public org.objectweb.modfact.corba.xmiio.mofobject.MOFObjectList " + methodName + "Template(org.w3c.dom.Element " + attributeName + ")");
        outputln("    throws org.omg.mof.Reflective.MofError {");
        outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"" + methodName + "Template(\" + " + attributeName + " + \")\");");
        outputln("org.objectweb.modfact.corba.xmiio.mofobject.MOFObjectList list = new org.objectweb.modfact.corba.xmiio.mofobject.MOFObjectList();");
        outputln("// Get the " + clsName1 + " values");
        outputln("if (" + attributeName + " == null) return list;");
        outputln("if (" + attributeName + ".getChildNodes().getLength() == 0 && " + attributeName + ".hasAttribute(\"xmi.idref\")) {");
        outputln("// The " + clsName1 + " element is already read or it will be read (ref).");
        outputln("list.addXmiIdRef(" + attributeName + ".getAttribute(\"xmi.idref\"));");
        outputln("return list;");
        outputln("}");
        outputln("int i = 0;");
        outputln("org.w3c.dom.NodeList _child = " + attributeName + ".getChildNodes();");
        outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
        outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
        try {
            org.omg.mof.Model.Class[] subtypes = mofHelper.subClassesOfClass(cls);
            for (int j = 0; j < subtypes.length; j++) {
                outputln("if (_current != null && nodeNameCorrespondsTo(_current, \"" + idlHelper.format1(subtypes[j].name()) + "\"))");
                if (subtypes[j].is_abstract())
                    outputln(TABULATION + "list.addMOFObjectList(" + idlHelper.format1FirstMin(subtypes[j].name()) + "Template(_current));");
                else
                    outputln(TABULATION + idlHelper.format1FirstMin(subtypes[j].name()) + "Template(_current).addMOFObject(list);");
                output("else ");
            }
        } catch (org.omg.mof.Reflective.NotSet notSet) {
            // Nothing to do
        }
        outputln("if (_current != null && nodeNameCorrespondsTo(_current, \"" + idlHelper.format1(clsName) + "\"))");
        outputln(TABULATION + "list.addXmiIdRef(_current.getAttribute(\"xmi.idref\"));");
        outputln("else if (_current != null)");
        outputln(TABULATION + "System.err.println(\"" + idlHelper.format1(clsName) + " is an abstract type, it can't be instanciated (\" + _current.getNodeName() + \" encountered).\");");
        outputln("i++;");
        outputln("}");
        outputln("return list;");
        outputln("}");
    }

    /**
     * XMIImport generation for the specific class template.
     * @param cls The specific class of the package.
     */
    public void classSpecificTemplate(org.omg.mof.Model.Class cls) throws org.omg.mof.Reflective.MofError {
        String className;
		String javaName;
        try {
        	className = idlHelper.className(cls);
            javaName = javaHelper.javaPackage(org.omg.mof.Model.PackageHelper.narrow(cls.container())) + "." + className;
        } catch (NotSet notSet) {
        	className = cls.name();
            javaName = className;
        }
        String clsName = cls.name();
        try {
			clsName = idlHelper.className(cls);
        } catch (NotSet notSet) {
        	// Nothing to do.
        }
        String clsName1 = idlHelper.format1(clsName);
        String clsName2 = idlHelper.format2(clsName);
        String methodName = idlHelper.format1FirstMin(cls.name());
        String returnType = javaName;
        String returnMethod = "org.objectweb.modfact.corba.xmiio.mofobject.MOFObject";
        String attributeName = "_" + clsName2 + "_element";
        annotationTemplate(cls.annotation());
        outputln("public " + returnMethod + " " + methodName + "Template(org.w3c.dom.Element " + attributeName + ")");
        outputln("    throws org.omg.mof.Reflective.MofError {");
        outputln("_logger.log(org.objectweb.modfact.corba.logging.Level.FINE, \"" + methodName + "Template(\" + " + attributeName + " + \")\");");
        outputln("// Get the " + clsName1 + " values");
        outputln("if (" + attributeName + " == null)");
        outputln(TABULATION + "return new org.objectweb.modfact.corba.xmiio.mofobject.NullObject();");
        outputln("if (" + attributeName + ".getChildNodes().getLength() == 0 && " + attributeName + ".hasAttribute(\"xmi.idref\")) {");
        outputln("// The " + clsName1 + " is already read or it will be read (ref).");
        outputln("return new org.objectweb.modfact.corba.xmiio.mofobject.XMIIdRef(" + attributeName + ".getAttribute(\"xmi.idref\"));");
        outputln("}");
        outputln("// Default " + clsName1 + " creation");
        outputln(returnType + " _" + clsName2 + " = ");
        try {
        	output("    _" + idlHelper.format2(javaHelper.javaQualifiedName(cls)) + "_class.create_" + clsName2 + "(");
		} catch (NotSet notSet) {
			output("    null;");
		}
        org.omg.mof.Model.MofAttribute attributes [] = mofHelper.allAttributesOfClass(cls, org.omg.mof.Model.ScopeKind.instance_level, false);
        for (int j = 0; j < attributes.length; j++) {
            org.omg.mof.Model.MofAttribute attr = (org.omg.mof.Model.MofAttribute) attributes[j];
            if (attr.is_changeable()) {
                if (j > 0)
                    output(", ");
                if (attr.multiplicity().upper == -1) { // upper = n
                    try {
                        if (attr.type()._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
                            org.omg.CORBA.TypeCode typeCode = org.omg.mof.Model.DataTypeHelper.narrow(attr.type()).type_code();
                            if (typeCode.kind().value() == org.omg.CORBA.TCKind._tk_alias) {
                                if (typeCode.content_type().kind().value() == org.omg.CORBA.TCKind._tk_any)
                                    output("new " + typeCode.content_type().name() + " [0]");
                                else
                                    output("new " + xmiImpHelper.typeCode2Java(typeCode.content_type()) + " [0]");
                            } else {
                                output("new " + xmiImpHelper.typeCode2Java(typeCode) + " [0]");
                            }

                        }
                    } catch (org.omg.CORBA.TypeCodePackage.BadKind badKind) {
                        output(xmiImpHelper.defaultAttributeValue(attr.type()));
                    }
                } else
                    output(xmiImpHelper.defaultAttributeValue(attr.type()));
            }
        }
        outputln(");");
        outputln("_types.put(((java.lang.String)_" + clsName2 + "_element.getAttribute(\"xmi.id\")).trim(), " + "_" + clsName2 + ");");
        for (int j = 0; j < attributes.length ; j++) {
            org.omg.mof.Model.MofAttribute attr = (org.omg.mof.Model.MofAttribute) attributes[j];
            if (attr.multiplicity().lower == 0 && attr.multiplicity().upper != -1) {
                outputln("_" + clsName2 + ".unset_" + idlHelper.format2(attr.name()) + "();");
            }
        }
        outputln("int i = 0;");
        outputln("org.w3c.dom.NodeList _child = " + attributeName + ".getChildNodes();");
        outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
        outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
        String param = "_" + clsName2 + "_element, _" + clsName2 + ", _current";
        attributes = mofHelper.allAttributesOfClass(cls, org.omg.mof.Model.ScopeKind.instance_level, false);
        for (int j = 0; j < attributes.length ; j++) {
            org.omg.mof.Model.MofAttribute attr = (org.omg.mof.Model.MofAttribute) attributes[j];
            if (attr.is_changeable()) {
                try {
                    outputln("_xmi_import_attributes.read" + idlHelper.format1(attr.container().name()) + idlHelper.format1(attr.name()) + "(" + param + ");");
                } catch (org.omg.mof.Reflective.NotSet notSet) {
                    outputln("_xmi_import_attributes.read" + idlHelper.format1(attr.name()) + "(" + param + ");");
                }
            }
        }
        org.omg.mof.Model.Reference references [] = mofHelper.allReferencesOfClass(cls);
        for (int j = 0; j < references.length ; j++) {
            org.omg.mof.Model.Reference ref = (org.omg.mof.Model.Reference) references[j];
            org.omg.mof.Model.AssociationEnd assoEnd = ref.referenced_end();
            if (assoEnd.aggregation().value() == org.omg.mof.Model.AggregationKind._none) {
                try {
                    outputln("_xmi_import_attributes.read" + idlHelper.format1(ref.container().name()) + idlHelper.format1(ref.name()) + "(" + param + ");");
                } catch (org.omg.mof.Reflective.NotSet notSet) {
                    outputln("_xmi_import_attributes.read" + idlHelper.format1(ref.name()) + "(" + param + ");");
                }
            } else if (assoEnd.aggregation().value() == org.omg.mof.Model.AggregationKind._composite) {
                try {
                    org.omg.mof.Model.AssociationEnd otherEnd = mofHelper.oppositeAssociationEnd(assoEnd);
                    org.omg.mof.Model.Reference[] refs = mofHelper.referencesOfAssociation(org.omg.mof.Model.AssociationHelper.narrow(assoEnd.container()));
                    for (int k = 0; k < refs.length; k++) {
                        if (!refs[k]._is_equivalent(ref)) {
                            outputln("_xmi_import_attributes.read" + idlHelper.format1(refs[k].type().name()) + idlHelper.format1(assoEnd.name()) + "(" + param + ");");
                        }
                    }
                } catch (org.omg.mof.Reflective.NotSet notSet) {
                    // Nothing to do
                }
            }
        }
        outputln("i++;");
        outputln("}");
        outputln("return new org.objectweb.modfact.corba.xmiio.mofobject.BuiltObject(_" + clsName2 + ");");
        outputln("}");
    }

}
