/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.importer;

/**
 * An abstract implementation of an XMI Import.
 * This class provides common functions for all XMI importers,
 * and generic functions for parsing files thanks to the XMIImportUtils inheritance
 */
public abstract class XMIImport {

	/** The ORB. */
	protected org.omg.CORBA.ORB _orb;

	/**
	 * Set the ORB.
	 * @param orb The new ORB.
	 */
	public void setORB(org.omg.CORBA.ORB orb) {
		_orb = orb;
	}
	
	/**
	 * Read a boolean type from a String.
	 * @param value The String to read.
	 * @return The boolean read.
	 */
	public boolean readBooleanType(String value) {
		return value.trim().equalsIgnoreCase("true");
	}

	/**
	 * Read a String type from a String.
	 * @param value The String to read.
	 * @return The String read.
	 */
	public String readStringType(String value) {
		return value.trim();
	}

	/**
	 * Test if a node have a name which ends with the string in parameter (Ignore Case for the first character of the string).
	 * @param node The node.
	 * @param tag The name to compare to the name of the node.
	 * @return TRUE If the node's name ends with the tag.
	 */
	public boolean nodeNameCorrespondsTo(org.w3c.dom.Element node, String tag) {
		String nodeName = node.getNodeName().trim();
		int index = nodeName.lastIndexOf(".");
		if (index != -1) {
			nodeName = nodeName.substring(index + 1);
		} else {
			index = nodeName.lastIndexOf(":");
			if (index != -1)
				nodeName = nodeName.substring(index + 1);
		}
		return nodeName.equalsIgnoreCase(tag);
	}
	

}
