/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.xmiio.importer;

import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;


/**
 * An abstract implementation of an XMI Import.
 * This class provides common functions for all XMI importers,
 * and generic functions for parsing files thanks to the XMIImportUtils inheritance
 */
public abstract class XMIImportParser extends XMIImport {

	/** The XMI Document. */
	protected org.w3c.dom.Element _document;

	/**
	 * Parse the XMI File.
	 * @param file The XMI Filename.
	 * @return The package read.
	 */
	public java.lang.Object parse(String file)
		throws
			org.xml.sax.SAXException,
			javax.xml.parsers.ParserConfigurationException,
			java.io.FileNotFoundException,
			java.io.IOException,
			org.omg.mof.Reflective.MofError {
		// Load the document
		java.io.FileInputStream _xml_input_file = new java.io.FileInputStream(file);
		return parse(_xml_input_file);
	}

	/**
	 * Parse the XMI File.
	 * @param _xml_input_file The Input Stream of XMI File.
	 * @return The package read.
	 */
	public java.lang.Object parse(InputStream xml_input_file)
		throws
			org.xml.sax.SAXException,
			javax.xml.parsers.ParserConfigurationException,
			java.io.FileNotFoundException,
			java.io.IOException,
			org.omg.mof.Reflective.MofError  {

		// The package
		java.lang.Object _package = null;

		DocumentBuilderFactory _factory = DocumentBuilderFactory.newInstance();
		_factory.setNamespaceAware(true);
		_factory.setIgnoringComments(true);
		_factory.setIgnoringElementContentWhitespace(true);
		_factory.setCoalescing(true);

		DocumentBuilder _builder = _factory.newDocumentBuilder();

		// Load the document
		org.w3c.dom.Document doc = _builder.parse(xml_input_file);

		// Parse the document
		org.w3c.dom.Element _model = doc.getDocumentElement();
		_document = _model;
		// Delete spaces
		delSpaces(_model);

		// Get contents
		org.w3c.dom.NodeList _contents = _model.getChildNodes();
		for (int i = 0; i < _contents.getLength(); i++) {
			org.w3c.dom.Element _element = (org.w3c.dom.Element) _contents.item(i);
			String _name = _element.getNodeName().trim();
			// The package
			if (_name.endsWith("XMI.header")) {
				xmiHeaderTemplate(_element);
			} else if (_name.endsWith("XMI.content")) {
				_package = xmiContentTemplate(_element);
			} else {
				System.err.println("Unknown : " + _name);
			}
		}
		return _package;
	}

	/**
	 * Parse the header of the XMI File.
	 * @param _package_element The XML element representing the root package.
	 */
	public void xmiHeaderTemplate(org.w3c.dom.Element _package_element) {
		// The XMI header is not parsed.
	}

	/**
	 * Parse the content of the XMI File.
	 * @param _package_element The XML element representing the root package.
	 */
	public org.omg.CORBA.portable.IDLEntity xmiContentTemplate(
		org.w3c.dom.Element _package_element)
		throws org.omg.mof.Reflective.MofError {
		org.omg.CORBA.portable.IDLEntity _package = null;
		org.w3c.dom.NodeList _contents = _package_element.getChildNodes();

		for (int i = 0; i < _contents.getLength(); i++) {
			org.w3c.dom.Element _element = (org.w3c.dom.Element) _contents.item(i);
			// The package
			_package = rootPackageTemplate(_element);
		}
		completeImport();
		return _package;
	}

	/**
	 * The root package processing.
	 * @param _root_package_element The XML element representing the package.
	 */
	public abstract org.omg.CORBA.portable.IDLEntity rootPackageTemplate(
		org.w3c.dom.Element _root_package_element)
		throws org.omg.mof.Reflective.MofError ;

	/**
	 * This method is called at the end of the XMI importation.
	 */
	public abstract void completeImport ()
		throws org.omg.mof.Reflective.MofError;
		
	/**
	 * Delete the nodes with empty text.
	 * @param node The root node.
	 */
	public void delSpaces(org.w3c.dom.Node node) {
		try {
			org.w3c.dom.NodeList _child = node.getChildNodes();
			int i = 0;
			while (i < _child.getLength()) {
				org.w3c.dom.Node _temp = _child.item(i);
				if (_temp.getNodeType() == 3) {
					String _temp1 = _temp.getNodeValue();
					if (_temp1.trim().length() == 0)
						node.removeChild(_temp);
					else
						i++;
				} else {
					delSpaces(_temp);
					i++;
				}
			}
		} catch (org.w3c.dom.DOMException domException) {
			switch (domException.code) {
				case org.w3c.dom.DOMException.NO_MODIFICATION_ALLOWED_ERR :
					System.err.println(
						"You attempt to modify an object where modifications are not allowed.");
					break;
				case org.w3c.dom.DOMException.NOT_FOUND_ERR :
					System.err.println(
						"You attempt to reference a node in a context where it does not exist.");
					break;
			}
		}
	}

}
