/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.api;

import javax.jmi.model.Association;
import javax.jmi.model.AssociationEnd;
import javax.jmi.model.ModelElement;

import org.objectweb.modfact.jmi.generator.BracketGenerator;
import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.logging.ModFactLogger;


/**
 * @author Xavier
 *
 */
public class AssociationGenerator extends BracketGenerator {
	Association[] input;

	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new Association[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (Association) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}
	
	Association association;
	AssociationEnd[] ends;
	String classEnd0;
	String classEnd1;
	String end0;
	String end1;
	
	public void generate() {
		association = input[0];		
		outputln("package " +JMIProvider.qualifierOf(association) +";");

		outputln(
			"public interface "
				+ JMIProvider.jmiAssociationName(association)
				+ " extends javax.jmi.reflect.RefAssociation " 
				+ " { \n" );
		
		ends = MofHelper.associationEndsOfAssociation(association);
		classEnd0 = JMIProvider.jmiClassifierQualifiedName(ends[0].getType());
		classEnd1 = JMIProvider.jmiClassifierQualifiedName(ends[1].getType());
		end0 = ends[0].getName();
		end1 = ends[1].getName();
		
		existsTemplate();
		addTemplate();
		removeTemplate();
		queryTemplate(ends[0], end0, classEnd0, end1, classEnd1);
		queryTemplate(ends[1], end1, classEnd1, end0, classEnd0);
											
		outputln("}");
		flushFile();
	}
	
	
	void queryTemplate(AssociationEnd end, String endName, String classEndName,
			String otherEndName, String otherClassEndName) {
		//if associationEnd0 is single valued and isNavigable
		if (end.isNavigable() 
			&& isSingleValued(end) ) {					
			outputln(
				"public "
					+ classEndName
					+ " get"
					+ JMIProvider.jmiFormat1(endName)
					+ "("
					+ otherClassEndName
					+ " "
					+ otherEndName
					+ ") throws javax.jmi.reflect.JmiException ; ");
		}
		
		//if associationEnd0 is multivalued, and isNavigable
		if (end.isNavigable()
			&& !isSingleValued(end) ) {
			outputln(
				"public "
					+(end.getMultiplicity().isOrdered()?
						"java.util.List" : "java.util.Collection"
					 )
					+" get"
					+ JMIProvider.jmiFormat1(endName)
					+ "("
					+ otherClassEndName
					+ " "
					+ "end"
					+ ") throws javax.jmi.reflect.JmiException ; ");
		}	
	}
	
	
	void addTemplate() {
		//	if associationEnd0 and associationEnd1 isChangeable
			  if (ends[0].isChangeable() && ends[1].isChangeable()) {
				  outputln(
					  "public boolean add("
						  + classEnd0
						  + " "
						  + "end0"
						  + ","
						  + classEnd1
						  + " "
						  + "end1"
						  + ") throws javax.jmi.reflect.JmiException ; ");				
			  }				
	}
	
	void existsTemplate() {
		outputln(
			"public boolean exists("
				+ classEnd0
				+ " "
				+ "end0"
				+ ","
				+ classEnd1
				+ " "
				+ "end1"
				+ ") throws javax.jmi.reflect.JmiException ;");
	}
	
	void removeTemplate() {
		outputln(
			"public boolean remove("
				+ classEnd0
				+ " "
				+ "end0"
				+ ","
				+ classEnd1
				+ " "
				+ "end1"
				+ ") throws javax.jmi.reflect.JmiException "
				//+", javax.jmi.reflect.NotFoundException" 
				+";" );
	}
		
	
	
	

	boolean isSingleValued(AssociationEnd end) {
	  return end.getMultiplicity().getUpper() == 1;
	}

}
