/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.api;

import javax.jmi.model.Attribute;
import javax.jmi.model.Constant;
import javax.jmi.model.ModelElement;
import javax.jmi.model.MofClass;
import javax.jmi.model.Operation;
import javax.jmi.model.Reference;
import javax.jmi.model.ScopeKindEnum;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

/**
 * @author Xavier
 *
 */
public class InstanceInterfaceGenerator extends CommonGenerator {
	MofClass[] input;

    ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofClass[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofClass) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see fr.lip6.modfact.generation.Generator#generate()
	 */
	public void generate() {

		MofClass c = input[0];

        outputln(
			"package " + JMIProvider.qualifierOf(c) + ";" );

        output("public interface " + JMIProvider.jmiClassName(c));

		MofClass[] supertypes = (MofClass[])c.getSupertypes().toArray(new MofClass[0]);
		//if the class has no super-types
		if (supertypes.length == 0) {
            outputln(" extends javax.jmi.reflect.RefObject {");
		//else, for each super-class
		} else {
			output(" extends ");
			for (int i = 0; i < supertypes.length; i++) {
				if (i != 0) out.print(" ,");
                output(
					JMIProvider.jmiClassQualifiedName(supertypes[i]) );
			}
            outputln("{");
		}

		//for each constant, generate the appropriate code
		Constant[] constants = MofHelper.constantsOfClass(c);
		for (int i = 0; i < constants.length; i++) {
			constantTemplate(constants[i]);
		}

		//for each instance level attribute of this class, generate the appropriate code
		//MofAttribute[] attributes_d = MofHelper.attributesOfClass(c , ScopeKind.instance_level , true );
		//for (int i = 0 ; i < attributes_d.length ; i++) {
		//	attributeTemplate(attributes_d[i]);
		//}

		Attribute[] attributes_nd =
			MofHelper.attributesOfClass(
				c ,ScopeKindEnum.forName("instance_level"), false);
				
		for (int i = 0; i < attributes_nd.length; i++) {
			attributeTemplate(attributes_nd[i]);
		}

		//for each instance level reference of this class, generate the appropriate code if public
		Reference[] references = MofHelper.referencesOfClass(c, false);
		for (int i = 0; i < references.length; i++) {
			referenceTemplate(references[i]);
		}

		//for each operation, generate the appropriate code if public_vis
		Operation[] operations = MofHelper.operationsOfClass(c, ScopeKindEnum.INSTANCE_LEVEL, false );
		for (int i = 0; i < operations.length; i++) {
			if (operations[i].getVisibility().equals(VisibilityKindEnum.PUBLIC_VIS))
				operationTemplate(operations[i]);
		}

		//end
		outputln("}"); //end of interface
		flushFile();
	}
}
