/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.generator;

/**
 * An abstract implementation of a generator.
 * This class allows the generation of files organized with brackets ('{' and '}'),
 * for example, the Java class files.
 *
 * @author Pierre Carpentier
 */
public abstract class BracketGenerator extends PrintGenerator {

	/**
     * Print the output to the global PrintWriter (replace out.print(String)).
     * @param out The string to print.
     */
    protected void output(String outputString) {
        int index = outputString.indexOf("\n");
        if (index != -1 && index != outputString.length() - 1) {
        	// outputString contains at least '\n' which is not at the end
            java.util.StringTokenizer token = new java.util.StringTokenizer(outputString, "\n");
            while (token.hasMoreTokens()) {
                String next = token.nextToken();
                if (!token.hasMoreTokens()) {
                    if (outputString.endsWith("\n"))
                        outputln(next);
                    else
                        output(next);
                } else {
                    outputln(next);
                }
            }
        } else {
            if (outputString.trim().startsWith("}"))
                level--;
            if (newLine) {
                for (int j = 0; j < level; j++)
                    out.print(TABULATION);
            }
            if (outputString.trim().startsWith("}")) {
                out.print(outputString);
                if (outputString.trim().endsWith("{"))
                    level++;
            } else {
                out.print(outputString);
                for (int i = 0; i < outputString.length(); i++) {
                    String current = String.valueOf(outputString.charAt(i));
                    if (current.equals("{"))
                        level++;
                    else if (current.equals("}"))
                        level--;
                }
            }
            if (outputString.trim().endsWith("\n"))
                newLine = true;
            else
                newLine = false;
        }
    }
    
}
