/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.generator;

/**
 * An abstract implementation of a generator.
 * This class allows the generation of files with tags (like XML).
 *
 * @author Pierre Carpentier
 */
public abstract class TagGenerator extends PrintGenerator {

	/**
     * Print the output to the global PrintWriter (replace out.print(String)).
     * @param out The string to print.
     */
    protected void output(String outputString) {
		if (outputString.startsWith("</"))
			level--;
		if (newLine) {
			for (int i = 0; i < level; i++)
				out.print(TABULATION);
			newLine = false;
		}
		out.print(outputString);

		for (int i = 0; i < outputString.length(); i++) {
			switch (outputString.charAt(i)) {
				case '\n' :
					newLine = true;
					break;
				case '<' :
					if (i < outputString.length() - 1) {
						if (outputString.charAt(i + 1) != '/' && outputString.charAt(i + 1) != '?')
							level++;
					} else {
						level++;
					}
					break;
				case '/' :
					if (i < outputString.length() - 1) {
						if (outputString.charAt(i + 1) == '>')
							level--;
					}
					if (i > 0) {
						if (outputString.charAt(i - 1) == '<' && i != 1) // i!=1 for _out.startsWith("</")
							level--;
					}
					break;
			}
		}
    }
    
}
