/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.helper;

import javax.jmi.model.*;
import java.util.*;

public class MofHelper {
	
	public static MofPackage[] packagesOfPackage(MofPackage p){
		List l = filterContentsByClass(p, MofPackage.class ,false);
		return (MofPackage[]) l.toArray(new MofPackage[0]);			
	}
	
	public static MofClass[] classesOfPackage(MofPackage p) {
		List l = filterContentsByClass(p, MofClass.class ,false);
		return (MofClass[]) l.toArray(new MofClass[0]);			
	}
	
	public static Association[] associationsOfPackage(MofPackage p) {
		List l = filterContentsByClass(p, Association.class ,false);
		return (Association[]) l.toArray(new Association[0]);		
	}
	
	public static Import[] importsOfPackage(MofPackage p) {
		List l = filterContentsByClass(p, Import.class ,false);
		return (Import[]) l.toArray(new Import[0]);		
	}
		
	public static DataType[] datatypesOf(Namespace ns, boolean considerSupertypes) {
		List l = filterContentsByClass(ns, DataType.class ,considerSupertypes);
		return (DataType[]) l.toArray(new DataType[0]);			
	}
	
	public static AssociationEnd[] associationEndsOfAssociation(Association a) {
		List l = filterContentsByClass(a, AssociationEnd.class ,false);
		return (AssociationEnd[]) l.toArray(new AssociationEnd[0]);			  
	}

	
	public static Constant[] constantsOfClass(MofClass c) {
		List l = filterContentsByClass(c, Constant.class ,false);
		return (Constant[]) l.toArray(new Constant[0]);		
	}

	public static Reference[] referencesOfClass(MofClass c, boolean considerSupertypes) {
		List l = filterContentsByClass(c, Reference.class , considerSupertypes);
		return (Reference[]) l.toArray(new Reference[0]);		
	}
	
	public static Operation[] operationsOfClass(MofClass cl, ScopeKind sk, boolean considerSupertypes) {	
		List l = filterContentsByClass(cl, Operation.class ,considerSupertypes);
		List l2 = new Vector();
		Iterator it = l.iterator();
		while(it.hasNext()) {
		  Operation op = (Operation)it.next(); 
		  if(op.getScope().equals(sk)) {
			 l2.add(op);
		  }
		}
		return (Operation[]) l2.toArray(new Operation[0]);			
	}
		
	public static Attribute[] attributesOfClass(
			MofClass cl, ScopeKind sk, boolean considerSupertypes) {
				
		List l = filterContentsByClass(cl, Attribute.class ,considerSupertypes);
		List l2 = new Vector();
		Iterator it = l.iterator();
		while(it.hasNext()) {
		  Attribute att = (Attribute)it.next(); 
		  if(att.getScope().equals(sk)) {
		  	 l2.add(att);
		  }
		}		
		return (Attribute[]) l2.toArray(new Attribute[0]);		
	}
	
	public static StructureField[] structureFieldsOf(StructureType t) {
		List l = filterContentsByClass(t, StructureField.class ,false);
		return (StructureField[]) l.toArray(new StructureField[0]);	
	}
	
	
	public static Parameter[] parametersOf(Operation op) {
		List l = filterContentsByClass(op, Parameter.class ,false);
		return (Parameter[]) l.toArray(new Parameter[0]);	
	}
	
	public static MofException[] exceptionsOf(Operation op) {
	  List l = filterContentsByClass(op, MofException.class ,false);
	  return (MofException[]) l.toArray(new MofException[0]);
	}
	
	
	
	public static List filterContentsByClass
			(Namespace ns, Class c, boolean considerSupertypes) {
	  List r = new Vector();
	  if((ns instanceof GeneralizableElement) && considerSupertypes) {
		   Iterator it = ((GeneralizableElement)ns).getSupertypes().iterator();
		   while(it.hasNext()) {
			 Namespace sup = (Namespace) it.next();
			 Iterator it2 = filterContentsByClass(sup,c,true).iterator();
			 while(it2.hasNext()) {
			 	Object o = it2.next();
			 	if(!r.contains(o)) r.add(o);
			 }
		   }
	  }
	  Iterator it = ns.getContents().iterator();
	  while(it.hasNext()) {
		  Object o = it.next();
		  if(c.isInstance(o)) {
		     if(!r.contains(o))  r.add(o);
		  }
      }
	  return r;
	}
	
	
	public static boolean isSinglePrimitive(TypedElement elem) {
	  if(!(elem.getType() instanceof PrimitiveType)) return false;
	  if(elem instanceof StructuralFeature) {
		StructuralFeature f = (StructuralFeature)elem;
		if(f.getMultiplicity().getLower()!=1) return false;
		if(f.getMultiplicity().getUpper()!=1) return false; 
		return true;
	  }
	  if(elem instanceof Parameter) {
	  	Parameter p = (Parameter)elem;
		if(p.getMultiplicity().getLower()!=1) return false;
		if(p.getMultiplicity().getUpper()!=1) return false; 
		return true;
	  }
	  return true; // for StructureField
	}
	
	/*
	public static boolean isNullablePrimitive(TypedElement elem) {
		return (elem.getType() instanceof PrimitiveType)  
			  && (elem instanceof StructuralFeature)
			  && ((StructuralFeature)elem).getMultiplicity().getLower()==0 
			  && ((StructuralFeature)elem).getMultiplicity().getUpper()==1 ;
	}
	*/	

}
