/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.impl;

import javax.jmi.model.Attribute;
import javax.jmi.model.ModelElement;
import javax.jmi.model.MofClass;
import javax.jmi.model.Operation;
import javax.jmi.model.ScopeKindEnum;

import org.objectweb.modfact.jmi.helper.ImplHelper;
import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

/**
 * @author Xavier
 *
 */
public class ClassProxyImplementationGenerator
	extends CommonImplementationGenerator {

	MofClass[] input;

    ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofClass[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofClass) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate() {
		//Tag todo
		int c = 0;

		outputln("package " +ImplHelper.implPrefix + JMIProvider.shortQualifierOf(input[c]) + ";" );
		outputln("import " + JMIProvider.qualifierOf(input[c]) + ".*;" );		
		outputln("import org.objectweb.modfact.jmi.reflect.*;");
		outputln(
			"public class "
				+ JMIProvider.jmiClassName(input[c])
				+ "ClassImpl extends org.objectweb.modfact.jmi.reflect.RefClassImpl"
				+ " implements "
				+ JMIProvider.jmiClassQualifiedName(input[c]) +"Class"
				+ " {");
		
		creatorTemplate(input[c]);
		newObjectTemplate(input[c]);

		//for each StructType directly contained by the class (correction of the spec ?)
		dataTypeTemplates(input[c]);

		newEnumTemplate(input[c]);
		newStructTemplate(input[c]);

		//for each attribute and operation contained in this class or one of its
		//supertypes with "classifier-level" scope, generate the appropriate operations
		Attribute[] attributes = MofHelper.attributesOfClass(
				input[c],
				ScopeKindEnum.forName("classifier_level"), true );
		for (int i = 0; i < attributes.length; i++) {
			attributeTemplate(attributes[i]);
		}

		Operation[] operations =
			MofHelper.operationsOfClass(
				input[c],
				ScopeKindEnum.forName("classifier_level"), true );
		for (int i = 0; i < operations.length; i++) {
			operationTemplate(operations[i]);
		}

		outputln("}//end of interface");
		flushFile();
	}
	
	void creatorTemplate(MofClass cl) {
		//If isAbstract is not set to true, generate factory methods.		
		if (!cl.isAbstract()) {
			String javaType = JMIProvider.jmiClassQualifiedName(cl);
			outputln(
				"public "
					+ javaType
					+ " create"
					+ JMIProvider.jmiClassName(cl)
					+ "() throws javax.jmi.reflect.JmiException { ");
			 
			  outputln("return ("
				+javaType
				+") refCreateInstance(null);");			  		
			outputln("} \n");

			ArgList argList = new ArgList();
			Attribute[] attributes = MofHelper.attributesOfClass(cl,
					ScopeKindEnum.forName("instance_level"), true );
										
			java.util.List v = new java.util.Vector();
			for(int i =0; i<attributes.length; i++) {
			  if(!attributes[i].isDerived()) v.add(attributes[i]);		
			}
			attributes = (Attribute[]) v.toArray(new Attribute[0]);
								
			if (attributes.length != 0) {
				output(
					"public "
						+ JMIProvider.jmiClassQualifiedName(cl)
						+ " create"
						+ JMIProvider.jmiClassName(cl)
						+ "(");
				//for each non-derived direct or inherited attribute		
				for (int i = 0; i < attributes.length; i++) {	
				  	
					output(
						JMIProvider.typeTemplate(attributes[i]) +" " +attributes[i].getName()	
					);
					argList.add(attributes[i]);				
					if (i != attributes.length - 1)
						output(" , ");
				}
				outputln(") throws javax.jmi.reflect.JmiException { \n");
					output(argList.code);
					outputln("return ("
				  		+javaType
				  		+") refCreateInstance(list);");			  		
			  	outputln("} \n");
			}
		}
	}		
	
	void newObjectTemplate(MofClass cl) {
		outputln("public RefObjectImpl newObject() { ");
		  outputln("return new " +ImplHelper.implPrefix 
		  	+JMIProvider.shortQualifierOf(cl)
		  	+"."
		  	+JMIProvider.jmiClassName(cl)+"Impl();");
		outputln("}");
	}

}
