package org.objectweb.modfact.jmi.impl;

import javax.jmi.model.Attribute;
import javax.jmi.model.ModelElement;
import javax.jmi.model.MofClass;
import javax.jmi.model.Operation;
import javax.jmi.model.Reference;
import javax.jmi.model.ScopeKindEnum;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.helper.ImplHelper;
import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.logging.ModFactLogger;


/**
 * @author Xavier
 *
 */
public class InstanceImplementationGenerator extends CommonImplementationGenerator {
	MofClass[] input;

    ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofClass[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofClass) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate() {

		MofClass c = input[0];

		outputln("package " +ImplHelper.implPrefix + JMIProvider.shortQualifierOf(c) + ";" );
        outputln("import " + JMIProvider.qualifierOf(c) + ".*;" );
		
        outputln("import org.objectweb.modfact.jmi.reflect.*;");

        output("public class " + JMIProvider.jmiClassName(c) +"Impl"
			+" extends RefObjectImpl "
			+"implements "
			+JMIProvider.jmiClassQualifiedName(c)
			+" {" );


		//for each instance level attribute of this class, generate the appropriate code
		//MofAttribute[] attributes_d = MofHelper.attributesOfClass(c , ScopeKind.instance_level , true );
		//for (int i = 0 ; i < attributes_d.length ; i++) {
		//	attributeTemplate(attributes_d[i]);
		//}

		Attribute[] attributes_nd =
			MofHelper.attributesOfClass(
				c,
				ScopeKindEnum.forName("instance_level"), true);
				
		for (int i = 0; i < attributes_nd.length; i++) {
			attributeTemplate(attributes_nd[i]);
		}

		//for each instance level reference of this class, generate the appropriate code if public
		Reference[] references = MofHelper.referencesOfClass( c, true );
		for (int i = 0; i < references.length; i++) {
			referenceTemplate(references[i]);
		}

		//for each operation, generate the appropriate code if public_vis
		Operation[] operations = MofHelper.operationsOfClass(c, ScopeKindEnum.INSTANCE_LEVEL, true );
		for (int i = 0; i < operations.length; i++) {
			if (operations[i].getVisibility().equals(VisibilityKindEnum.PUBLIC_VIS))
				operationTemplate(operations[i]);
		}

		//end
        outputln("}"); //end of interface
		flushFile();
	}
}
