/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.impl.tayloredBased;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import org.objectweb.modfact.jmi.helper.*;
import org.objectweb.modfact.jmi.logging.Level;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

import javax.jmi.model.*;

import org.objectweb.modfact.jmi.generator.ZipGenerator;


/**
 * @author Xavier
 *
 */
public class JMI1_0ImplementationGenerator extends ZipGenerator {
	
	MofPackage[] input;
	
	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofPackage[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofPackage) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate()
			throws IOException {


		logger.log(Level.FINE, "Begin generation of JMI Interfaces");

		for (int p = 0; p < input.length; p++) {
			if (input[p]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {

				//Traitemement du Package
				PackageImplementationGenerator packageGenerator = new PackageImplementationGenerator();
				MofPackage[] param = new MofPackage[1];
				param[0] = input[p];
				packageGenerator.setInput(param);
				ByteArrayOutputStream entryOutput1 =
					new ByteArrayOutputStream();
				packageGenerator.setOutput(entryOutput1);
				packageGenerator.generate();
				
				String path = ImplHelper.implPrefix + JMIProvider.shortQualifierOf(input[p]);
				path = path.replace('.', '/') +"/";
				
				String entryName = path
					+JMIProvider.jmiPackageExtentName(input[p]) + "PackageImpl" + ".java";
					
				writeEntry(entryName, entryOutput1);
				entryOutput1.close();
				
				logger.log(Level.FINE, entryName+" is generated");

				//Traitement des Classes
				MofClass[] classes = MofHelper.classesOfPackage(input[p]);
				
				for (int i = 0; i < classes.length; i++) {
					if (classes[i].getVisibility()
						== VisibilityKindEnum.forName("public_vis")) {

						// Cette Partie gnre toutes les ClasseProxy
						ClassProxyImplementationGenerator classProxyGenerator = new ClassProxyImplementationGenerator();
						MofClass[] paramCP = new MofClass[1];
						paramCP[0] = classes[i];
						classProxyGenerator.setInput(paramCP);
						ByteArrayOutputStream entryOutputClassProxy =
							new ByteArrayOutputStream();
						classProxyGenerator.setOutput(entryOutputClassProxy);
						classProxyGenerator.generate();

						String entryNameClassProxy = path
							+JMIProvider.jmiClassName(classes[i])
								+ "ClassImpl"
								+ ".java";
						writeEntry(entryNameClassProxy, entryOutputClassProxy);
						entryOutputClassProxy.close();
						
						logger.log(Level.FINE, entryNameClassProxy+" is generated");

						// Cette partie gnre toutes les Classes 
						InstanceImplementationGenerator classImpl = new InstanceImplementationGenerator();
						MofClass[] paramCI = new MofClass[1];
						paramCI[0] = classes[i];
						classImpl.setInput(paramCI);
						ByteArrayOutputStream entryOutputInstance =	new ByteArrayOutputStream();
						classImpl.setOutput(entryOutputInstance);
						classImpl.generate();

						String entryNameInstance = 	path
							+JMIProvider.jmiClassName(classes[i]) + "Impl.java";
						writeEntry(entryNameInstance, entryOutputInstance);
						entryOutputInstance.close();
						
						logger.log(Level.FINE, entryNameInstance+" is generated");

						//Cette partie traite les DataType (Enum et Struct) des classes
						DataType[] datatypes =
							MofHelper.datatypesOf(classes[i], false);
						for (int j = 0; j < datatypes.length; j++) {
							
							if (datatypes[j] instanceof StructureType) {
								StructureTypeImplementationGenerator structGenerator =
									new StructureTypeImplementationGenerator();
								DataType[] inImpl = new DataType[1];
								inImpl[0] = datatypes[j];
								structGenerator.setInput(inImpl);
								structGenerator.setLogger(logger);

								ByteArrayOutputStream structEntryOutputImpl =
									new ByteArrayOutputStream();
								structGenerator.setOutput(
									structEntryOutputImpl);
								structGenerator.generate();

								String entryNameStruct = path
									+datatypes[j].getName() + "Impl.java";
								writeEntry(
									entryNameStruct,
									structEntryOutputImpl);
								structEntryOutputImpl.close();
								
								logger.log(Level.FINE, entryNameStruct+" is generated");
							}
						}

					}
				}

				//Traitement des Associations
				Association[] associations =
					MofHelper.associationsOfPackage(input[p]);
				for (int i = 0; i < associations.length; i++) {
					if (associations[i]
						.getVisibility()
						.equals(VisibilityKindEnum.forName("public_vis"))) {
						AssociationImplementationGenerator associationGenerator =
							new AssociationImplementationGenerator();
						Association[] paramA = new Association[1];
						paramA[0] = associations[i];
						associationGenerator.setInput(paramA);
						ByteArrayOutputStream entryOutputAssociation =
							new ByteArrayOutputStream();
						associationGenerator.setOutput(entryOutputAssociation);

						associationGenerator.generate();

						String entryNameAssociation = path+
							JMIProvider.jmiAssociationName(associations[i])
								+ "Impl.java";
						writeEntry(
							entryNameAssociation,
							entryOutputAssociation);
						entryOutputAssociation.close();
						
						logger.log(Level.FINE, entryNameAssociation+" is generated");
					}
				}

				//Traitement des DataTypes Enum et Struct
				DataType[] datatypes = MofHelper.datatypesOf(input[p], false);
				for (int i = 0; i < datatypes.length; i++) {
					if (datatypes[i] instanceof StructureType) {
						StructureTypeImplementationGenerator structureTypeInterfaceGenerator =
							new StructureTypeImplementationGenerator();
						DataType[] paramT = new DataType[1];
						paramT[0] = datatypes[i];
						structureTypeInterfaceGenerator.setInput(paramT);
						ByteArrayOutputStream entryOutputDataType =
							new ByteArrayOutputStream();
						structureTypeInterfaceGenerator.setOutput(
							entryOutputDataType);

						structureTypeInterfaceGenerator.generate();

						String entryNameStructure = path+
							datatypes[i].getName() + "Impl.java";
						writeEntry(entryNameStructure, entryOutputDataType);
						entryOutputDataType.close();
						
						logger.log(Level.FINE, entryNameStructure+" is generated");	
					}
				}

			}
		}
		out.finish();
		logger.log(Level.FINE, " Generation is completed without error");
	}

}
