/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.impl.tayloredBased;

import org.objectweb.modfact.jmi.helper.*;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

import javax.jmi.model.*;
import java.util.*;

/**
 * @author Xavier
 *
 */
public class PackageImplementationGenerator
	extends CommonImplementationGenerator {

	MofPackage[] input;

	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofPackage[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofPackage) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate() {
		//Tag Import todo
		MofPackage _package = input[0];
		//javadocsTemplate(_package);

		out.println("package " +ImplHelper.implPrefix + JMIProvider.shortQualifierOf(_package) + ";");
		out.println("import " + JMIProvider.qualifierOf(_package) + ".*;");
		
		out.println("import org.objectweb.modfact.jmi.reflect.*;");

		String pQName = JMIProvider.jmiPackageQualifiedName(_package);

		out.println(
			"public class "
				+ JMIProvider.jmiPackageExtentName(_package)
				+ "PackageImpl extends RefPackageImpl"
				+ " implements "
				+ pQName
				+ "Package { ");

		factoryTemplate(_package);

		//for each StructType 
		dataTypeTemplates(_package);

		/////////// Package import //////////////////
		Import[] imports = MofHelper.importsOfPackage(_package);
		for (int i = 0; i < imports.length; i++) {
			if (VisibilityKindEnum
				.forName("public_vis")
				.equals(imports[i].getVisibility())
				&& imports[i].isClustered()
				&& imports[i].getImportedNamespace() instanceof MofPackage) {
				MofPackage importedPackage =
					(MofPackage) imports[i].getImportedNamespace();
				out.println(
					"\tpublic "
						+ JMIProvider.jmiPackageQualifiedName(importedPackage)
						+ "Package get"
						+ JMIProvider.jmiImportName(imports[i])
						+ "() {");
				out.println(
					"\t\treturn "
						+ "("
						+ JMIProvider.jmiPackageQualifiedName(importedPackage)
						+ "Package)"
						+ "refPackage(\""
						+ importedPackage.getName()
						+ "\")"
						+ ";");

				out.println("\t}");
			}
		}

		//for each contained package where visibility = public_vis
		MofPackage[] containedPackages = MofHelper.packagesOfPackage(_package);
		for (int i = 0; i < containedPackages.length; i++) {
			if (containedPackages[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {

				out.println(
					"\tpublic "
						+ JMIProvider.jmiPackageQualifiedName(
							containedPackages[i])
						+ "Package get"
						+ JMIProvider.jmiPackageExtentName(containedPackages[i])
						+ "() {");
				out.println(
					"\t\treturn "
						+ "("
						+ JMIProvider.jmiPackageQualifiedName(
							containedPackages[i])
						+ "Package)"
						+ "refPackage(\""
						+ containedPackages[i].getName()
						+ "\")"
						+ ";");
				out.println("\t}");
			}
		}

		//for each contained class where visibility = public_vis
		MofClass[] containedClasses = MofHelper.classesOfPackage(_package);
		for (int i = 0; i < containedClasses.length; i++) {
			if (containedClasses[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {
				out.println(
					"\tpublic "
						+ JMIProvider.jmiClassQualifiedName(containedClasses[i])
						+ "Class get"
						+ JMIProvider.jmiClassName(containedClasses[i])
						+ "() {");
				out.println(
					"\t\treturn "
						+ "("
						+ JMIProvider.jmiClassQualifiedName(containedClasses[i])
						+ "Class)"
						+ "refClass(\""
						+ containedClasses[i].getName()
						+ "\")"
						+ ";");
				out.println("\t}");
			}
		}

		//for each contained association where visibility = public_vis
		Association[] containedAssociations =
			MofHelper.associationsOfPackage(_package);
		for (int i = 0; i < containedAssociations.length; i++) {
			if (containedAssociations[i]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))) {
				out.println(
					"\tpublic "
						+ JMIProvider.jmiAssociationQualifiedName(
							containedAssociations[i])
						+ " get"
						+ JMIProvider.jmiAssociationName(
							containedAssociations[i])
						+ "() {");
				out.println(
					"\t\treturn "
						+ "("
						+ JMIProvider.jmiAssociationQualifiedName(
							containedAssociations[i])
						+ ")"
						+ "refAssociation(\""
						+ containedAssociations[i].getName()
						+ "\")"
						+ ";");
				out.println("\t}");
			}
		}

		newPackageTemplate(_package);
		newClassTemplate(_package);
		newAssociationTemplate(_package);
		newEnumTemplate(_package);
		newStructTemplate(_package);

		out.println("}");
		out.flush();
	}

	void newPackageTemplate(MofPackage p) {
		out.println("\tpublic RefPackageImpl newPackage(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, MofPackage.class, true)
				.iterator();
		while (it.hasNext()) {
			MofPackage content = (MofPackage) it.next();
			out.println(
				"\t\t if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ JMIProvider.jmiPackageExtentName(content)
					+ "Package"
					+ "Impl"
					+ "();");
		}
		out.println("throw new RuntimeException(\"invalide type\");");
		out.println("\t}");
	}

	void newClassTemplate(MofPackage p) {
		out.println("\tpublic RefClassImpl newClass(String n) {");
		Iterator it =
			MofHelper.filterContentsByClass(p, MofClass.class, true).iterator();
		while (it.hasNext()) {
			MofClass content = (MofClass) it.next();
			out.println(
				"\t\t if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ JMIProvider.jmiClassName(content)
					+ "Class"
					+ "Impl"
					+ "();");
		}
		out.println("\t\t throw new RuntimeException(\"invalide Class: '\" + n + \"'\");");		
		out.println("\t}");
	}

	void newAssociationTemplate(MofPackage p) {
		out.println("\tpublic RefAssociationImpl newAssociation(String n) {");
		Iterator it =
			MofHelper
				.filterContentsByClass(p, Association.class, true)
				.iterator();
		while (it.hasNext()) {
			Association content = (Association) it.next();
			out.println(
				"\t\t if(n.equals(\""
					+ content.getName()
					+ "\")) "
					+ "return new " +ImplHelper.implPrefix
					+ JMIProvider.shortQualifierOf(content)
					+ "."
					+ JMIProvider.jmiAssociationName(content)
					+ "Impl"
					+ "();");
		}
		out.println("\t\t throw new RuntimeException(\"invalide Association: '\" + n + \"'\");");		
		out.println("\t}");
	}

	// factory for outer mostpackage	
	void factoryTemplate(MofPackage _package) {

		if (_package.getContainer() == null) {

			String pQName = JMIProvider.jmiPackageQualifiedName(_package);
			String pShortQName = JMIProvider.shortQualifierOf(_package) +"."
								+JMIProvider.jmiPackageExtentName(_package);

			out.println(
				"\t public static "
					+ pQName
					+ "Package"
					+ " create(javax.jmi.model.MofPackage meta) {");
			out.println(
				"\t\t "
					+ ImplHelper.implPrefix + pShortQName
					+ "PackageImpl p = new "
					+ ImplHelper.implPrefix + pShortQName
					+ "PackageImpl();");
			out.println("\t\t initRepository(p, meta);");
			out.println("\t\t return p;");
			out.println("\t } \n");

			out.println(
				"\t public static "
					+ pQName
					+ "Package"
					+ " create(String metaModelURL) throws Exception {");
			out.println(
				"\t\t javax.jmi.model.MofPackage meta=(javax.jmi.model.MofPackage)"
					+ "createMetaObject(\""
					+ _package.getName()
					+ "\", metaModelURL);");
			out.println("\t\t return create(meta);");
			out.println("\t }");
		}
	}

}
