package org.objectweb.modfact.jmi.reflect;

import javax.jmi.reflect.*;

import java.util.*;
import java.io.*;

// this class is used to generate a client that creates M3 in a repository

public class M3ClientGenerator {
	
	
	public static final String relpath = "org/objectweb/modfact/jmi/reflect";
	public String path = null;
	
	public static void main(String[] args) throws Exception {

		RefPackage p;

		p =
			RefPackageImpl
				.getM2Repository()
				.refMetaObject()
				.refImmediatePackage();

		M3ClientGenerator g = new M3ClientGenerator(p);
		g.run();
		System.out.println("M3Client generated to " + g.path);
	}

	public M3ClientGenerator(RefPackage p) {
		this.p = p;
	}

	RefPackage p;
	PrintWriter out, link_out;

	public void run() throws Exception {

		if(path==null)
		  path = ClassLoader
				.getSystemResource(relpath)
				.getPath();
				
		out = new PrintWriter(new FileOutputStream(path + "/M3Client.java"));

		link_out = new PrintWriter(new FileOutputStream(path +"/M3links.txt"));

		long timestamp = System.currentTimeMillis();
	
		out.println("//timestamp=" + timestamp );
		link_out.println("timestamp=");
		link_out.println(timestamp);

		out.println("package org.objectweb.modfact.jmi.reflect;");
		
		out.println("import javax.jmi.reflect.*;");
		out.println("import java.util.*;");
		out.println("class M3Client {");
		out.println("RefPackageImpl p;");
		out.println("Hashtable objs = new Hashtable();");
		out.println("//temp variables");
		out.println("List l, l2;");
		out.println("RefClassImpl cl;");
		out.println("RefAssociationImpl a;");
		out.println("RefStructImpl struct;");

		out.println("RefPackageImpl loadModel() {");
		out.println("loadDescription();");
		out.println("copyObjs();");
		out.println("copyAtts();");
		out.println("M3ClientHelper.loadLinks(this);");
		out.println("return p;");
		out.println("}");

		body();

		out.println("RefObjectImpl get(String key) {");
		out.println("\treturn (RefObjectImpl)objs.get(key);");
		out.println("}");
		
		out.println("void checkTimeStamp(long t) {");
		out.println("\t" 
			+"if(t!=" +timestamp
			+ ") throw new RuntimeException(\"" 
				+"invalide M3Link.txt, expecting timestamp="
				+timestamp 
			+"\")");
		out.println("}");		

		out.println("} //end class");
		out.close();

		link_out.close();
	}

	void body() {
		Iterator it;

		out.println("void loadDescription() {");
		packageDescription(p.refMetaObject());
		out.println("}");

		// copy all objs
		out.println("void copyObjs() {");
		it = p.refAllClasses().iterator();
		while (it.hasNext()) {
			RefClass c = (RefClass) it.next();
			String name = (String) c.refMetaObject().refGetValue("name");
			out.println("cl = (RefClassImpl) p.refClass(\"" + name + "\");");
			Iterator it2 = c.refAllOfClass().iterator();
			while (it2.hasNext()) {
				RefObject instance = (RefObject) it2.next();
				out.println(
					"objs.put(\"o"
						+ instance.hashCode()
						+ "\" ,cl._refCreateInstance(null));");
			}
		}
		out.println("}");

		// copy all attributes
		out.println("void copyAtts() {");
		it = p.refAllClasses().iterator();
		while (it.hasNext()) {
			RefClass c = (RefClass) it.next();
			RefObject meta = c.refMetaObject();
			Iterator it2 = c.refAllOfClass().iterator();
			while (it2.hasNext()) {
				RefObject instance = (RefObject) it2.next();
				Iterator it3 =
					RefBaseObjectImpl
						.getContentsInAllSupertypes(meta)
						.iterator();
				while (it3.hasNext()) {
					RefObject metaProp = (RefObject) it3.next();
					if (metaProp
						.refMetaObject()
						.refGetValue("name")
						.equals("Attribute")) {
						if (p
							.refGetEnum("ScopeKind", "instance_level")
							.equals(metaProp.refGetValue("scope"))) {
							String propName =
								(String) metaProp.refGetValue("name");
							Object propValue = instance.refGetValue(propName);

							String code = codeOf(propValue);
							out.println(
								codeOf(instance)
									+ ".setAttValue("
									+ "\""
									+ propName
									+ "\""
									+ ","
									+ code
									+ ");");
						} else {
							throw new RuntimeException("M3 should not have classifier_level attributes");
						}
					}
				}
			}
		}
		out.println("}");

		/////////// write link info ////////
		it = p.refAllAssociations().iterator();
		while (it.hasNext()) {
			RefAssociation a = (RefAssociation) it.next();
			link_out.println("Association=");
			link_out.println(a.refMetaObject().refGetValue("name"));
			Iterator it2 = a.refAllLinks().iterator();
			while (it2.hasNext()) {
				RefAssociationLink link = (RefAssociationLink) it2.next();
				link_out.println("o" + link.refFirstEnd().hashCode());
				link_out.println("o" + link.refSecondEnd().hashCode());
			}
		}

	}

	String codeOf(Object val) {
		if (val instanceof Collection) {
			out.println("l=new Vector();");
			Iterator it = ((Collection) val).iterator();
			while (it.hasNext()) {
				String code = codeOf(it.next());
				out.println("l.add(" + code + ");");
			}
			return "l";
		}
		if (val instanceof RefObject) {
			return "get(\"o" + val.hashCode() + "\")";
		}
		if (val instanceof RefStruct) {
			RefStruct v = (RefStruct) val;
			String code = codeOf(RefStructImpl.getAllValues(v));
			out.println(
				"struct=(RefStructImpl)p.refCreateStruct(\""
					+ getUnqualifiedName(v.refTypeName())
					+ "\","
					+ code
					+ ");");
			return "struct";
		}
		if (val instanceof RefEnum) {
			RefEnum v = (RefEnum) val;
			String containerName = getContainerName(v.refTypeName());
			if (containerName.equals("Model")) {
				return "p.refGetEnum(\""
					+ getUnqualifiedName(v.refTypeName())
					+ "\",\""
					+ v.toString()
					+ "\")";
			} else {
				return "p.refClass(\""
					+ containerName
					+ "\").refGetEnum(\""
					+ getUnqualifiedName(v.refTypeName())
					+ "\",\""
					+ v.toString()
					+ "\")";
			}
		}
		if (val instanceof Integer) {
			return "new Integer(" + val + ")";
		}
		if (val instanceof String) {
			String v = (String) val;
			if (v.indexOf("\n") >= 0)
				v = "";
			return "\"" + v + "\"";
		}
		if (val instanceof Boolean) {
			return "new Boolean(" + val + ")";
		}
		throw new RuntimeException("unknown type: " + val.getClass());
	}

	String getUnqualifiedName(List l) {
		return (String) l.get(l.size() - 1);
	}

	String getContainerName(List l) {
		return (String) l.get(l.size() - 2);
	}

	void packageDescription(RefObject meta) {
		// newPackage
		out.println("this.p=" + newPackage(meta) + ";");
		out.println("p.name=\"" + meta.refGetValue("name") + "\";");
		Iterator it;
		it = ((Collection) meta.refGetValue("contents")).iterator();
		int classCount, assoCount, structCount, enumCount;
		classCount = 0;
		assoCount = 0;
		structCount = 0;
		enumCount = 0;

		while (it.hasNext()) {
			RefObject content = (RefObject) it.next();
			String contentName = (String) content.refGetValue("name");
			String contentType =
				(String) content.refMetaObject().refGetValue("name");

			if (contentType.equals("Class")) {
				out.println("p.classNames.add(\"" + contentName + "\");");
				classCount++;
			} else if (contentType.equals("Association")) {
				out.println("p.associationNames.add(\"" + contentName + "\");");
				assoCount++;
			} else if (contentType.equals("StructureType")) {
				out.println("p.structNames.add(\"" + contentName + "\");");
				structCount++;
			} else if (contentType.equals("EnumerationType")) {
				out.println("p.enumTypeNames.add(\"" + contentName + "\");");
				enumCount++;
			}
		}
		out.println("p.classes = new RefClassImpl[" + classCount + "];");
		out.println("p.packages = new RefPackageImpl[" + 0 + "];");
		out.println(
			"p.associations = new RefAssociationImpl[" + assoCount + "];");
		out.println("p.structFieldNameLists = new List[" + structCount + "];");
		out.println("p.enumTypes = new Object[" + enumCount + "];");

		it = ((Collection) meta.refGetValue("contents")).iterator();
		classCount = 0;
		assoCount = 0;
		structCount = 0;
		enumCount = 0;

		List classNames = new Vector();

		while (it.hasNext()) {
			RefObject content = (RefObject) it.next();
			String contentName = (String) content.refGetValue("name");
			String contentType =
				(String) content.refMetaObject().refGetValue("name");

			if (contentType.equals("Class")) {

				/* newClass */
				out.println("cl=" + newClass(content) + ";");
				out.println("cl.container=p;");
				out.println("cl.name=\"" + contentName + "\";");

				///// class description /////////
				classDescription(
					RefBaseObjectImpl.findObjectByName(
						meta.refImmediatePackage().refClass("Class"),
						contentName));

				out.println("p.classes[" + classCount + "]=cl;");
				classCount++;
				classNames.add(contentName);
			} else if (contentType.equals("Association")) {
				Iterator it2 =
					((Collection) content.refGetValue("contents")).iterator();
				String[] endNames = new String[2];
				String[] typeNames = new String[2];
				boolean[] isOrdered = new boolean[2];
				int i = 0;
				while (it2.hasNext()) {
					RefObject assoContent = (RefObject) it2.next();
					if (assoContent
						.refMetaObject()
						.refGetValue("name")
						.equals("AssociationEnd")) {
						endNames[i] = (String) assoContent.refGetValue("name");
						RefObject type =
							(RefObject) assoContent.refGetValue("type");
						typeNames[i] = (String) type.refGetValue("name");
						RefStruct mul =
							(RefStruct) assoContent.refGetValue("multiplicity");
						isOrdered[i] =
							((Boolean) mul.refGetValue("isOrdered"))
								.booleanValue();
						i++;
					}
				}

				/* newAssociation() */
				out.println("a=" + newAssociation(content) + ";");
				out.println("a.container=p;");
				out.println("a.name=\"" + contentName + "\";");
				out.println("a.firstEndName=\"" + endNames[0] + "\";");
				out.println("a.secondEndName=\"" + endNames[1] + "\";");
				out.println("a.type1=\"" + typeNames[0] + "\";");
				out.println("a.type2=\"" + typeNames[1] + "\";");
				out.println("a.isEnd1Ordered=" + isOrdered[0] + ";");
				out.println("a.isEnd2Ordered=" + isOrdered[1] + ";");

				out.println("p.associations[" + assoCount + "]=a;");
				assoCount++;
			} else if (contentType.equals("StructureType")) {
				out.println("l = new Vector();");
				Iterator it2 =
					((Collection) content.refGetValue("contents")).iterator();
				while (it2.hasNext()) {
					RefObject cc = (RefObject) it2.next();
					String ccType =
						(String) cc.refMetaObject().refGetValue("name");
					if (ccType.equals("StructureField")) {
						out.println(
							"l.add(\"" + cc.refGetValue("name") + "\");");
					}
				}
				out.println("p.structFieldNameLists[" + structCount + "]=l;");
				structCount++;
			} else if (contentType.equals("EnumerationType")) {
				String code = codeOf(content.refGetValue("labels"));
				out.println("l2=" + code + ";");
				out.println("l=p.refTypeName();"); //qualified package name
				out.println("l.add(\"" + contentName + "\");");
				//enum type name

				/* newEnum */
				out.println(
					"p.enumTypes[" + enumCount + "]=" + newEnum(content) + ";");
				enumCount++;
			}
		}

		/*
		// supertype-subtype information
		RefClass metaClassProxy = meta.refImmediatePackage().refClass("Class");
		for (int i = 0; i < classCount; i++) {
			Iterator it2 =
				((Collection) RefBaseObjectImpl
					.findObjectByName(
						metaClassProxy,
						(String) classNames.get(i))
					.refGetValue("supertypes"))
					.iterator();
			while (it2.hasNext()) {
				RefObject supertypeMeta = (RefObject) it2.next();
				out.println(
					"cl=p.classes[p.classNames.indexOf(\""
						+ supertypeMeta.refGetValue("name")
						+ "\")];");
				out.println(
					"p.classes[" + i + "].allDirectSupertypes.add(cl);");
				out.println("cl.allDirectSubtypes.add(p.classes[" + i + "]);");
			}
		}
		*/
	}

	void classDescription(RefObject meta) {
		Iterator it;
		it = RefBaseObjectImpl.getContentsInAllSupertypes(meta).iterator();
		int attCount, refCount, enumCount;
		attCount = 0;
		refCount = 0;
		enumCount = 0;
		while (it.hasNext()) {
			RefObject content = (RefObject) it.next();
			String contentName = (String) content.refGetValue("name");
			String contentType =
				(String) content.refMetaObject().refGetValue("name");

			if (contentType.equals("Attribute")) {

				if (content
					.refImmediatePackage()
					.refGetEnum("ScopeKind", "instance_level")
					.equals(content.refGetValue("scope"))
					&& Boolean.FALSE.equals(content.refGetValue("isDerived"))) {
					out.println("cl.attNames.add(\"" + contentName + "\");");
					attCount++;
				}
			} else if (contentType.equals("Reference")) {
				out.println("cl.refNames.add(\"" + contentName + "\");");
				refCount++;
			} else if (contentType.equals("EnumerationType")) {
				out.println("cl.enumTypeNames.add(\"" + contentName + "\");");
				enumCount++;
			}
		}
		// Instance-level Attributes
		out.println("cl.attMultiplicities = new int[" + attCount + "];");

		// References
		out.println("cl.assoNames = new List[" + refCount + "];");
		out.println("cl.assoEndNames = new String[" + refCount + "];");
		out.println("cl.refMultiplicities = new int[" + refCount + "];");

		// Nested Enumerations
		out.println("cl.enumTypes = new Object[" + enumCount + "];");

		attCount = 0;
		refCount = 0;
		enumCount = 0;
		it = RefBaseObjectImpl.getContentsInAllSupertypes(meta).iterator();
		while (it.hasNext()) {
			RefObject content = (RefObject) it.next();
			String contentName = (String) content.refGetValue("name");
			String contentType =
				(String) content.refMetaObject().refGetValue("name");

			if (contentType.equals("Attribute")) {
				if (content
					.refImmediatePackage()
					.refGetEnum("ScopeKind", "instance_level")
					.equals(content.refGetValue("scope"))
					&& Boolean.FALSE.equals(content.refGetValue("isDerived"))) {
					out.println(
						"cl.attMultiplicities["
							+ (attCount++)
							+ "]="
							+ RefBaseObjectImpl.getMultiplicity(content)
							+ ";");
				}

			} else if (contentType.equals("Reference")) {
				out.println(
					"cl.refMultiplicities["
						+ (refCount)
						+ "]="
						+ RefBaseObjectImpl.getMultiplicity(content)
						+ ";");
				RefObject assoEnd =
					(RefObject) content.refGetValue("referencedEnd");
				out.println(
					"cl.assoEndNames["
						+ (refCount)
						+ "]=\""
						+ assoEnd.refGetValue("name")
						+ "\";");
				RefObject asso =
					(RefObject) meta
						.refImmediatePackage()
						.refAssociation("Contains")
						.refQuery("containedElement", assoEnd)
						.toArray()[0];
				String v = codeOf(asso.refGetValue("qualifiedName"));
				out.println(
					"cl.assoNames["
						+ (refCount)
						+ "]="
						+ v
						+ ";");
				refCount++;

			} else if (contentType.equals("EnumerationType")) {
				String code = codeOf(content.refGetValue("labels"));
				out.println("l2=" + code + ";");
				out.println("l=cl.refTypeName();"); //qualified package name
				out.println("l.add(\"" + contentName + "\");");
				//enum type name
				out.println(
					"cl.enumTypes[" + (enumCount++) + "]=new EnumType(l ,l2);");
			}
		}
	}

	public String newAssociation(RefObject a) {
		return "new RefAssociationImpl()";
	}

	public String newClass(RefObject c) {
		return "new RefClassImpl()";
	}

	public String newPackage(RefObject p) {
		return "new RefPackageImpl()";
	}

	public String newEnum(RefObject enum) {
		return "new EnumType(l, l2)";
	}

}
