package org.objectweb.modfact.jmi.reflect;

import javax.jmi.reflect.*;
import javax.jmi.xmi.XmiReader;
import org.objectweb.modfact.jmi.xmi.XmiReaderImpl;
import java.util.*;

public class RefPackageImpl extends DataTypeContainer 
							 implements RefPackage, java.io.Serializable  {
    
    // if this package is now in use to describe model instances
    // then its contents cannot be modified
    boolean canModify = true;
    
    List packageNames = new Vector();
    List classNames = new Vector();
    List associationNames = new Vector();
    
    RefPackageImpl[] packages;
    RefClassImpl[] classes;
    RefAssociationImpl[] associations;
    
    transient List packageList;
	transient List classList;
	transient List assoList;
        
    // used in serialization
    // should not be called by users
    public RefPackageImpl() {
    }
      
    private static Object lock = "";

    public static RefPackage getM2Repository() {
        synchronized(lock) {
            if(m3==null) {
                m3 = new M3Client().loadModel();
            }
        }
		RefPackageImpl p = new org.objectweb.modfact.jmi.repository.javax.jmi.model.ModelPackageImpl();        
		initRepository(p, (RefObjectImpl)m3.refMetaObject());
        return p;
    }
    
    public static RefPackage getM1Repository(RefObject metaObject) {
        RefPackageImpl p = new RefPackageImpl();
		initRepository(p, metaObject);
        return p;
    }
    
    public static void initRepository(RefPackageImpl p, RefObject metaObject) {
		p.metaObject = (RefObjectImpl)metaObject;
		p.name = (String)metaObject.refGetValue("name");
		p.readContentsFromMetaObject();      
    }
    
  // Helper  
  public static RefObject createMetaObject(String packageName, String metaModelURL) 
  			throws Exception {
	XmiReader reader = new XmiReaderImpl();
	RefPackage p = RefPackageImpl.getM2Repository();
	reader.read(metaModelURL,p);
	Iterator it = p.refClass("Package").refAllOfClass().iterator();
	while(it.hasNext()) {
	  RefObject o = (RefObject) it.next();
	  if( packageName.equalsIgnoreCase((String)o.refGetValue("name")) ) {
		  return o;
	  }
	}
	return null;
  }    
  
  //use outer most package          
  public static RefObject createMetaObject(String metaModelURL) 
			throws Exception {
	XmiReader reader = new XmiReaderImpl();
	RefPackage p = RefPackageImpl.getM2Repository();
	reader.read(metaModelURL,p);
	return ReflectHelper.getOuterMostPackage(p);
  }    
      
    public java.util.Collection refAllAssociations() {
        if(assoList==null) 
        	assoList = Collections.unmodifiableList(arrayToList(associations));
        return assoList;
    }
    
    public java.util.Collection refAllClasses() {
		if(classList==null) 
			classList = Collections.unmodifiableList(arrayToList(classes));
        return classList;
    }
    
    public java.util.Collection refAllPackages() {
		if(packageList==null) 
			packageList = Collections.unmodifiableList(arrayToList(packages));
        return packageList;
    }
    
    // method needs to be override with check
    public javax.jmi.reflect.RefAssociation refAssociation(String str) {
        int index = associationNames.indexOf(str);
        if(index<0) {
            throw new RuntimeException("No association named '" +str +"' in '" +name +"'");
        }
        return associations[index];
    }
    
    public javax.jmi.reflect.RefAssociation refAssociation(javax.jmi.reflect.RefObject refObject) {
        String name = (String) refObject.refGetValue("name");
        return refAssociation(name);
    }
    
    public javax.jmi.reflect.RefClass refClass(String str) {
        int index = classNames.indexOf( str );
        if(index==-1) {
            throw new RuntimeException(this +" No class named '" +str +"' in '" +name +"'");
        }
        return classes[index];
    }
    
    public javax.jmi.reflect.RefClass refClass(javax.jmi.reflect.RefObject refObject) {
        String name = (String) refObject.refGetValue("name");
        return refClass(name);
    }
    
    public javax.jmi.reflect.RefPackage refPackage(String str) {
		int index = packageNames.indexOf(str);
		if(index>=0)
        	return packages[index];
        	        	
        //find package import
        RefPackage meta = this.refMetaObject().refOutermostPackage();       
        Iterator it = meta.refClass("Package").refAllOfClass().iterator();
        while(it.hasNext()) {
        	RefObjectImpl o = (RefObjectImpl)it.next();
        	if(o.refGetValue("name").equals(str)) {
        		List qName = o._getQualifiedName();
        		return (RefPackage) resolveQualifiedNameInExtent(qName);
        	}
        }
        
        throw new RuntimeException("No Package named '" +str +"' in '" +name +"'");
    }
    
    public javax.jmi.reflect.RefPackage refPackage(javax.jmi.reflect.RefObject refObject) {
        String name = (String) refObject.refGetValue("name");
        return refPackage(name);
    }
 
    public void refDelete() {
        throw new RuntimeException("No implementation: RefPackage.refDelete");
    }
    
    public javax.jmi.reflect.RefObject refMetaObject() {
        if(metaObject!=null) return metaObject;
        return findObjectByName( this.refClass("Package"), name);
    }
    
    void readContentsFromMetaObject() {
        
		List packageList_ = new Vector();
		List classList_ = new Vector();
		List assoList_ = new Vector();        
        
        Iterator it = ((Collection)metaObject.refGetValue("contents")).iterator();
        while(it.hasNext()) {
            RefObjectImpl content = (RefObjectImpl)it.next();
            String contentName = (String) content.refGetValue("name");
            String contentType = (String) content.refMetaObject().refGetValue("name");
            if(contentType.equals("Class")) {
				classNames.add(contentName);

                RefClassImpl c = newClass(contentName);

                c.metaObject = content;
                c.container = this;
                c.name = contentName; 
                c.readContentsFromMetaObject();
  
                classList_.add(c);
                
            } else if(contentType.equals("Package")) {
				packageNames.add(contentName);

                RefPackageImpl p = newPackage(contentName);
                p.metaObject = content;
                p.container = this;
                p.name = contentName;
				p.readContentsFromMetaObject(); // bug fixed
				
				packageList_.add(p);	

            } else if(contentType.equals("Association")) {
				associationNames.add(contentName);

                Iterator it2 = ((Collection)content.refGetValue("contents")).iterator();
                String[] endNames = new String[2];
                String[] typeNames = new String[2];
                boolean[] isOrdered = new boolean[2];
                int i = 0;
                while(it2.hasNext()) {
                    RefObject assoContent = (RefObject) it2.next();
                    if(assoContent.refMetaObject().refGetValue("name").equals("AssociationEnd")) {
                        endNames[i] = (String)assoContent.refGetValue("name");
                        RefObject type = (RefObject)assoContent.refGetValue("type");
                        typeNames[i] = (String) type.refGetValue("name");
                        RefStruct mul = (RefStruct)assoContent.refGetValue("multiplicity");
                        isOrdered[i] = ((Boolean)mul.refGetValue("isOrdered")).booleanValue();
                        i++;
                    }
                }
                RefAssociationImpl a = newAssociation(contentName);
                a.metaObject = content;
                a.container = this;
                a.name = contentName;
                a.firstEndName = endNames[0];
                a.secondEndName = endNames[1];
                a.type1 = typeNames[0];
                a.type2 = typeNames[1];
                a.isEnd1Ordered = isOrdered[0];
                a.isEnd2Ordered = isOrdered[1];
                
                assoList_.add(a);
            } else if(contentType.equals("Import")) {
            	//to do 
            	
            }
        }
		
		classes = (RefClassImpl[]) classList_.toArray(new RefClassImpl[0]);
		packages = (RefPackageImpl[]) packageList_.toArray(new RefPackageImpl[0]);
		associations = (RefAssociationImpl[]) assoList_.toArray(new RefAssociationImpl[0]);
        
        readDataTypeContents();
        
    }
    
    
    /////////////// to be over writen ////////////////
    public RefPackageImpl newPackage(String name) {
      return new RefPackageImpl();
    
    }
    
    public RefClassImpl newClass(String name) {
      return new RefClassImpl();    
    }
    
    public RefAssociationImpl newAssociation(String name) {
      return new RefAssociationImpl();    
    }
}
