package org.objectweb.modfact.jmi.reflect;

import javax.jmi.reflect.*;
import java.util.*;

public class ReflectHelper {

	public static RefObject findObjectByName(RefClass c, String name) {
		Iterator it = c.refAllOfClass().iterator();
		while(it.hasNext()) {
			RefObject content = (RefObject) it.next();
			if(content.refGetValue("name").equals(name)) {
				return content;
			}
		}
		throw new RuntimeException("No element witn name='" +name +"' in " +c);  
	}
    
	public static List filterContentsByType(RefObject container
				, String type
				, boolean considerSupertypes) {
	  Collection c = (considerSupertypes)? 
							  getContentsInAllSupertypes(container)
							: (Collection) container.refGetValue("contents");
	  Iterator it = (c).iterator();
	  List r = new Vector();
	  while(it.hasNext()) {
		RefObject content = (RefObject) it.next();
		if(content.refMetaObject().refGetValue("name").equals(type)) {
			r.add(content);
		}
	  }
	  return r;    
	}
     
	public static List getContentsInAllSupertypes(RefObject container) {
	   List r = new DistinctList();
	   Iterator it = ((List) container.refGetValue("supertypes")).iterator();
	   while(it.hasNext()) {
		 RefObject sup = (RefObject) it.next();
		 r.addAll((List)getContentsInAllSupertypes(sup));
	   }
	   r.addAll((List)container.refGetValue("contents"));       
	   return r;
	}
	
	public static RefObject getOuterMostPackage(RefPackage metamodel) {
	  Iterator it =  metamodel.refClass("Class").refAllOfClass().iterator();
	  return (RefObject) ((RefObject)it.next()).refOutermostComposite();
	}
    
	static Collection queryInAllSupertypes(String assoName, String endName, RefObject end ) {
	  return queryInAllSupertypes(
			end.refImmediatePackage().refAssociation(assoName)
			, endName ,end );
	}
    
	// find all elements associated to this one and to any of its supertypes
	static Collection queryInAllSupertypes(RefAssociation a, String endName, RefObject end ) {
	   Collection r = new DistinctList();
	   Iterator it = ((Collection) end.refGetValue("supertypes")).iterator();
	   while(it.hasNext()) {
		 RefObject sup = (RefObject) it.next();
		 r.addAll( queryInAllSupertypes(a, endName, sup) );
	   }
	   r.addAll(a.refQuery(endName,end));       
	   return r;
	}    
    
    
	public static RefObject getOppositeAssociationEnd(RefObject end) {
	  RefObject asso = (RefObject) end.refGetValue("container");
	  Iterator it = ((Collection)asso.refGetValue("contents")).iterator();
	  while(it.hasNext()) {
		RefObject content = (RefObject) it.next();
		if( !content.equals(end)
		  && content.refMetaObject().refGetValue("name").equals("AssociationEnd")) 
			return content;
	  }
	  return null;
	}
  
    
	public static int getMultiplicity(RefObject meta) {
	  return ((Integer)((RefStruct)meta.refGetValue("multiplicity")).refGetValue("upper")).intValue();
	}    
    
	public static List arrayToList(Object[] array) {
	  List l = new Vector();
	  for(int i=0; i<array.length; i++) {
		l.add(array[i]);
	  }
	  return l;
	}
    
	public static String unqualifiedName(String dotList) {
		return dotList.substring(dotList.lastIndexOf(".")+1 ,dotList.length());
	}
    

}
