package org.objectweb.modfact.jmi.xmi;

import java.io.*;
import java.util.*;
import javax.jmi.reflect.*;

public class Writer {


	CharTranslator t = new CharTranslator();
	PrintStream out;

	Hashtable object_id = new Hashtable();
	
	Set writtenOuterMostComposites = new HashSet(); 
	Set referencedObjects = new HashSet(); 		

	// metaObject -> qualified name
	Hashtable qNameCach = new Hashtable();
	//metaObjectOwner -> properties
	Hashtable propCach = new Hashtable();
	
	XmiWriterImpl caller;

	Writer(XmiWriterImpl owner, OutputStream out) {
		this.caller = owner;
		this.out = new PrintStream(out);
	}

	void writeHeader() {
		out.println("<?xml version = '1.0' encoding = 'ISO-8859-1' ?>");
		out.println("<XMI xmi.version = '1.2'>");
		out.println("<XMI.header>");
		out.println("<XMI.documentation>");
		out.println("<XMI.exporter>ModFact</XMI.exporter>");
		out.println("<XMI.exporterVersion>1.0</XMI.exporterVersion>");
		out.println("</XMI.documentation>");
		out.println("</XMI.header>");
		out.println("<XMI.content>");
	}

	void writeEnd() {
		out.println("</XMI.content>");
		out.println("</XMI>");
	}

	void writePackage(RefPackage extent) {
		Iterator it = extent.refAllClasses().iterator();
		while (it.hasNext()) {
			RefClass c = (RefClass) it.next();
			Iterator it2 = c.refAllOfClass().iterator();
			while (it2.hasNext()) {
				RefObject o = (RefObject) it2.next();
				writeOuterMostCompositeOf(o);
			}
		}
		// write subpackages
		it = extent.refAllPackages().iterator();
		while (it.hasNext()) {
			RefPackage p = (RefPackage) it.next();
			writePackage(p);
		}		
	}
	
	
	void writeOuterMostCompositeOf(RefObject o) {
	   RefObject outer = (RefObject) o.refOutermostComposite();
	   if(writtenOuterMostComposites.contains(outer)) {
	   	 return;
	   }
	   
	   writtenOuterMostComposites.add(outer);
	   write(outer);
	}
	
	void writeReferencedObjects() {
		while(!writtenOuterMostComposites.containsAll(referencedObjects)) {
			RefObject[] objs = (RefObject[]) referencedObjects.toArray(new RefObject[0]);
			for(int i=0; i<objs.length; i++) {
				writeOuterMostCompositeOf(objs[i]);
			}
		}
	}
	

	void write(RefObject o) {
		String qName = qName(o.refMetaObject());
		out.print("<" + qName + " xmi.id='" + getId(o) + "'");
		writePrimitiveValues(o);
		out.println(">");
		writeComplexValues(o);
		out.println("</" + qName + ">");
	}

	int id_incrementer = 0; // use for generating xmi.id  

	String getId(RefObject o) {
		String id = (String) object_id.get(o);
		if (id != null)
			return id;
		id = String.valueOf(id_incrementer++);
		object_id.put(o, id);
		return id;
	}

	void writePrimitiveValues(RefObject o) {
		Iterator it = allProperties(o.refMetaObject()).iterator();
		while (it.hasNext()) {
			RefObject p = (RefObject) it.next();
			Object value = o.refGetValue(p);	
			if (value == null)
				continue;
			if (isOfPrimitiveType(value)) {	
				String vstring = value.toString();
				String encodedString = t.encode(vstring);		
				out.print(" " + p.refGetValue("name") + "='" + encodedString + "'");
			}
		}
	}

	boolean isOfPrimitiveType(Object value) {
		if (value instanceof RefObject)
			return false;
		if (value instanceof RefStruct)
			return false;
		if (value instanceof Collection)
			return false;
		return true;
	}

	void writeComplexValues(RefObject o) {
		Iterator it = allProperties(o.refMetaObject()).iterator();
		while (it.hasNext()) {
			RefObject prop = (RefObject) it.next();
			String qName = qName(prop);
			Object value = o.refGetValue(prop);
			if (value == null)
				continue;
			// Multi-valued type
			if ((value instanceof Collection)
				&& !((Collection) value).isEmpty()) {
				Object[] values = ((Collection) value).toArray();
				if (isOfPrimitiveType(values[0])) {
					for (int i = 0; i < values.length; i++) {
						out.println(
							"<"
								+ qName
								+ ">"
								+ values[i].toString()
								+ "</"
								+ qName
								+ ">");
					}
				} else if (values[0] instanceof RefStruct) {
					for (int i = 0; i < values.length; i++) {
						out.println("<" + qName + ">");
						writeStructFields((RefStruct) values[i]);
						out.println("</" + qName + ">");
					}
				} else if (values[0] instanceof RefObject) {
					writeObjectValues(prop, (Collection)value);
					
				}
				// single-valued type
			} else if (value instanceof RefObject) {			
				writeObjectValue(prop, (RefObject)value);
			
			} else if (value instanceof RefStruct) {
				out.println("<" + qName + ">");
				writeStructFields((RefStruct) value);
				out.println("</" + qName + ">");
			}
		}
	}
	
	
	void writeObjectValues(RefObject prop, Collection c) {
		String qName = qName(prop);		
		Iterator it = c.iterator();
		if("Attribute".equals(prop.refMetaObject().refGetValue("name"))) {
		  out.println("<" + qName + ">");	
		  while(it.hasNext()) {
		    RefObject v = (RefObject) it.next();
		    write(v);		    
		  }
		  out.println("</" + qName + ">");			
		} else {
		  RefObject end = (RefObject) prop.refGetValue("referencedEnd"); 	
		  if( end.refGetValue("aggregation").toString().equals("composite") ) {
			// do nothing
			// this is impossible (a component cannot have many composites)
		  } else if( oppositeEnd(end).refGetValue("aggregation").toString().equals("composite")  ) {
			out.println("<" + qName + ">");
			while(it.hasNext()) {
			  RefObject v = (RefObject) it.next();
			  write(v);		    
			}
			out.println("</" + qName + ">");
		  } else {
		  	// association without by-value aggregation semantic
			out.println("<" + qName + ">");
			while(it.hasNext()) {
			  RefObject v = (RefObject) it.next();
			  writeObjectReference(v);		    
			}
			out.println("</" + qName + ">");			
		  }
		}
	}
	
	
	void writeObjectValue(RefObject prop, RefObject v) {
		String qName = qName(prop);		
		if("Attribute".equals(prop.refMetaObject().refGetValue("name"))) {
		  out.println("<" + qName + ">");	
		  write(v);
		  out.println("</" + qName + ">");			
		} else {
		  RefObject end = (RefObject) prop.refGetValue("referencedEnd"); 	
		  if( end.refGetValue("aggregation").toString().equals("composite") ) {
		    // do nothing
		  } else if( oppositeEnd(end).refGetValue("aggregation").toString().equals("composite")  ) {
			out.println("<" + qName + ">");
		    write(v);	
			out.println("</" + qName + ">");
		  } else {
			//	association without by-value aggregation semantic
			out.println("<" + qName + ">");
		    writeObjectReference(v);
			out.println("</" + qName + ">");			
		  }
		}
	}
	
	
	RefObject oppositeEnd(RefObject end) {
		RefObject asso = (RefObject) end.refGetValue("container");
		Iterator it = ((List)asso.refGetValue("contents")).iterator();
		while(it.hasNext()) {
			RefObject o = (RefObject) it.next();
			if("AssociationEnd".equals(o.refMetaObject().refGetValue("name")) 
			  && !o.equals(end)
			) return o;
		}	
		return null;
	}
	

	void writeObjectReference(RefObject v) {
		out.println(
			"<"
				+ qName(v.refMetaObject())
				+ " xmi.idref='"
				+ getId(v)
				+ "'"
				+ "/>");
		referencedObjects.add(v.refOutermostComposite());		
	}

	void writeStructFields(RefStruct struct) {
		Iterator it = struct.refFieldNames().iterator();
		while (it.hasNext()) {
			Object value = struct.refGetValue((String) it.next());
			out.print("<XMI.field>");
			if (value instanceof RefObject) {
				writeObjectReference((RefObject) value);
			} else if (value instanceof RefStruct) {
				writeStructFields((RefStruct) value);
			} else {
				out.print(value);
			}
			out.println("</XMI.field>");
		}
	}

	List allProperties(RefObject metaObjectOwner) {
		List r = (List) propCach.get(metaObjectOwner);
		if (r != null)
			return r;
		r = _allProperties(metaObjectOwner);
		propCach.put(metaObjectOwner, r);
		return r;
	}

	List _allProperties(RefObject metaObjectOwner) {
		List r = new DistinctList();
		Iterator it =
			((List) metaObjectOwner.refGetValue("supertypes")).iterator();
		while (it.hasNext()) {
			RefObject supertype = (RefObject) it.next();
			r.addAll(allProperties(supertype));
		}
		it = ((List) metaObjectOwner.refGetValue("contents")).iterator();
		while (it.hasNext()) {
			RefObject content = (RefObject) it.next();			
			String type = (String) content.refMetaObject().refGetValue("name");
			if ((type.equals("Attribute")
				&& content.refGetValue("isDerived").equals(Boolean.FALSE))
				|| type.equals("Reference")) {
				r.add(content);
			}
		}
		return r;
	}

	String qName(RefObject meta) {
		String r = (String) qNameCach.get(meta);
		if (r != null)
			return r;
		r = _qName(meta);
		qNameCach.put(meta, r);
		return r;
	}

	String _qName(RefObject meta) {
		String name = (String) meta.refGetValue("name");
		RefObject container = (RefObject) meta.refGetValue("container");
		if (container == null)
			return name;
		if(    !caller.useFullyQualifiedName 
			&& "Package".equals(container.refMetaObject().refGetValue("name")))	
		    return name;
		return _qName(container) + "." + name;
	}

}
