package org.objectweb.modfact.jmi.xmi;

import javax.xml.parsers.*;
import org.w3c.dom.*;

import javax.jmi.reflect.*;
import java.util.*;
import java.io.*;



public class XmiParser {

    static DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    static DocumentBuilder builder;
    static {
       try {           
         builder = factory.newDocumentBuilder();
       } catch(Exception e) {
         System.err.println("Couldnot initialize DOM Parser");
         throw new RuntimeException(e);
       }
    }
    
 
    Document document;
    Node rootNode;
               
    // xmi.id to RefObject
    Hashtable id_object = new Hashtable();
    List unresolvedProps = new Vector();
    
    RefPackage extent;
    
    XmiReaderImpl caller;
    
	static Object lock = "";    
    XmiParser(XmiReaderImpl owner, RefPackage p, InputStream in) {
      this.caller = owner;	
      this.extent = p;        
      try {  
      	synchronized(lock) {                    
          document = builder.parse(in);
          rootNode = document.getDocumentElement().getElementsByTagName("XMI.content").item(0);
        }
      }   catch(Exception e) {
          throw new RuntimeException(e);
      }
    }
    
    private Set createdObjects = new HashSet();
    
    void run() {
    	
    	cacheAllFactories(extent);
    	
        Node xmi_content = document.getElementsByTagName("XMI.content").item(0);
        NodeList children = xmi_content.getChildNodes();
        for (int i=0; i < children.getLength(); i++) {
            Node node = children.item(i);
            if(node instanceof Element) {
              process1((Element) node);
            }
        } 

                       
        Iterator it = elements.iterator();
        while(it.hasNext()) {
          process2((Element)it.next());
        }
    }
    
    Collection getOuterMostObjects() {
      Collection r = new HashSet(); 
      Iterator it = createdObjects.iterator();
      while(it.hasNext()) {	
    	RefObject o = (RefObject) it.next();
    	RefBaseObject outer = o.refOutermostComposite();
    	if(outer instanceof RefObject) r.add(outer);	
      }
      return r;    	
    }
    
    
    
    Hashtable classProxies = new Hashtable();
    Hashtable structFactories = new Hashtable();
    Hashtable enumManagers = new Hashtable();
    Hashtable assoProxies = new Hashtable();
    
    // flatten package structure       
    void cacheAllFactories(RefPackage p) {
      Iterator it = ((Collection) p.refMetaObject().refGetValue("contents")).iterator();
      while(it.hasNext()) {
      	RefObject content = (RefObject) it.next();
      	String name = (String) content.refGetValue("name");
      	String type = (String) content.refMetaObject().refGetValue("name");
      	if("Class".equals(type)) {
      	  classProxies.put(name, p.refClass(name));	      
      	  cacheAllFactories(p.refClass(name));	  
      	} else if("StructureType".equals(type)) {
      	  structFactories.put(name, p);
	  	} else if("EnumerationType".equals(type)) {
          enumManagers.put(name, p);
		} else if("Association".equals(type)) {
		  assoProxies.put(name, p.refAssociation(name));	
		} else if("Package".equals(type)) {
		  cacheAllFactories(p.refPackage(name));		  	
      	}	  	      	
      }
    }
    
    void cacheAllFactories(RefClass c) {
		Iterator it = ((Collection) c.refMetaObject().refGetValue("contents")).iterator();
		while(it.hasNext()) {
		  RefObject content = (RefObject) it.next();
		  String name = (String) content.refGetValue("name");
		  String type = (String) content.refMetaObject().refGetValue("name");
		  if("StructureType".equals(type)) {
			structFactories.put(name, c);
		  } else if("EnumerationType".equals(type)) {
			enumManagers.put(name, c);
		  }	      	
		}    	    	
    }
   
    // list of empty element to assign properties' values to
    List elements = new Vector();    
    
    // create corresponding object
    // and map xmi.id to it
    void process1(Element elem) {

		String xmi_id = elem.getAttribute("xmi.id");		
		String elemName = unqualifiedName(elem.getTagName());
		RefObject o;
		        
        if(!"".equals(xmi_id)) {
          // support for XMI 1.0, 1.1
		  o = doCorbaType(xmi_id, elemName, elem);
          if(o==null) {     
		    if(caller.verbose) System.err.println("creating object '" +elem.getTagName() +"' xmi.id=" +xmi_id );        		  
		    o = createObject(elem);
            if(o!=null) {
			  id_object.put(xmi_id, o);
			  elements.add(elem); // to be processed later;             	
            }
          }         
        }

        NodeList children = elem.getChildNodes();
        for (int i=0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if(child instanceof Element) {
              try {
                  process1((Element) child);
              } catch(RuntimeException e) {
                 if(!caller.ignoreError) throw e;        
              }
            }
        } 
    }
    
    
    RefObject createObject(Element elem) {
		String xmi_id = elem.getAttribute("xmi.id");		
		String elemName = unqualifiedName(elem.getTagName());
		
		RefClass c = (RefClass)classProxies.get(elemName);
		if(c==null) {
		  if(caller.ignoreError) return null;	
		  throw new RuntimeException("class '"+elemName +"' undefined at xmi.id=" +xmi_id);
		}
		RefObject o = c.refCreateInstance(null);
		createdObjects.add(o);
		return o;    
    }
    
    // Set values of all properties
    void process2(Element elem) {

		String xmi_id = elem.getAttribute("xmi.id");
		RefObject o = (RefObject) id_object.get(xmi_id);     
		
		if(caller.verbose) 
			System.err.println("setting properties for object '" 
				+elem.getNodeName() +"' xmi.id=" +xmi_id 
			);                
		try {
		  setProperties(elem ,o);
		} catch(RuntimeException e) {          
		  RuntimeException e2 = new RuntimeException("at xmi.id="+xmi_id, e);
		  throw e2;
		}    		  
		
    }
    
    void setProperties(Element elem, RefObject o) {

		  NamedNodeMap attrs = elem.getAttributes();
           
          NodeList children = elem.getChildNodes();
          for (int i=0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if(child instanceof Element)
              setValue(o, child );
          }
        
          for (int i=0; i < attrs.getLength(); i++) {
            Attr attr = (Attr) attrs.item(i);
            if(!attr.getName().equals("xmi.id"))
              setValue(o, attr );
          }
            
    }
    
    
    
    Hashtable typeCach = new Hashtable();
    
    void setValue(RefObject owner, Node propNode) {
        RefObject metaObject = owner.refMetaObject();
        String propName = propNode.getNodeName();
        
        // ignore xmi extention tag
        if( propName.toLowerCase().indexOf("xmi.")>=0) {
          if(caller.verbose)
              System.err.println("\t\t\t*** ignore node '" +propName +"'");
          return;
        }
        
        String unqPropName = unqualifiedName(propNode.getNodeName());
        TypeInfo type;
        try {
          type = findTypeFor(metaObject, unqPropName);
        } catch(Exception e) {

			String err = "property '" +propName +"' undefined in '" +metaObject.refGetValue("qualifiedName") +"'";
			if(caller.ignoreError) {
				if(caller.verbose)  System.err.println("***" +err);
				return;
			} else  { 
                throw new RuntimeException( err ,e );
			}
        }
        try {
          if(type.isMultiValued) {
            Collection values = (propNode instanceof Attr)?
                  createValuesFrmAttr(type, (Attr) propNode)
                : createValuesFrmElement(type, (Element)propNode);  
            if(caller.verbose) System.err.println("    set property '" 
                +propName +"' =" +values
            );    
            ((Collection)owner.refGetValue(unqPropName)).addAll(values);        
          } else {
            Object value = (propNode instanceof Attr)?
                  createValueFrmAttr(type, (Attr)propNode)
                : createValueFrmElement(type, (Element)propNode); 
            if(caller.verbose) System.err.println("    set property '" 
                  +propName +"' =" +value
            );
            owner.refSetValue(unqPropName, value );
          } 
        } catch(RuntimeException e) {
            String err = "Could not set property '" +propName +"' from Node "+ propNode;
            if(caller.ignoreError) {
              if(caller.verbose)  System.err.println("***" +err);
            } else  { 
              throw new RuntimeException(err, e);
            }			  
        }
    }
    
    TypeInfo findTypeFor(RefObject metaObjectOwner, String unqPropName ) throws RefException {
        List args = new Vector();
        args.add( unqPropName );
        RefObject propMeta = (RefObject) metaObjectOwner.refInvokeOperation("lookupElementExtended", args);            
        return findTypeFor(propMeta);
    }
    
    // propMetaObject describes the property
    // propMetaObject.name = property name
    // propMetaObject.type = property type meta object
    // propMetaObject.type.name = type name
    TypeInfo findTypeFor(RefObject propMetaObject) {
       return findTypeFor(propMetaObject, false); 
    }
    
    TypeInfo findTypeFor(RefObject propMetaObject, boolean isStructField) {
        // find in Cache
        TypeInfo typeInfo = (TypeInfo)typeCach.get(propMetaObject);
        if(typeInfo!=null)  return typeInfo;
        // find type info in M2 (to load M1 XMI data) 
        //             or in M3 (to load M2 XMI data) 
        typeInfo = new TypeInfo();
        if(!isStructField) {
          typeInfo.isMultiValued = (
            ((Integer)((RefStruct)propMetaObject.refGetValue("multiplicity")).refGetValue("upper")).intValue()!=1
          );
        }
        RefObject type= (RefObject)propMetaObject.refGetValue("type");
        typeInfo.name = (String) type.refGetValue("name");
        String typeKind = (String) type.refMetaObject().refGetValue("name");
        if(typeKind.equals("PrimitiveType")) {
          typeInfo.isPrimitive = true;
        } else if(typeKind.equals("EnumerationType")) {
          typeInfo.isEnum = true;
          typeInfo.enumUnprefix = findTagValue(type,"org.omg.xmi.enumerationUnprefix");
          
        } else if(typeKind.equals("StructureType")) {
          typeInfo.isStruct = true;
          List fieldTypes = new Vector();
		  List fieldNames = new Vector();
          Iterator it = ((List)type.refGetValue("contents")).iterator();
          while(it.hasNext()) {
            RefObject content = (RefObject)it.next();
            if(content.refMetaObject().refGetValue("name").equals("StructureField")) {
              fieldTypes.add(findTypeFor(content, true));
			  fieldNames.add(content.refGetValue("name"));
            }
          }
          typeInfo.structFieldTypes = (TypeInfo[]) fieldTypes.toArray(new TypeInfo[0]);
		  typeInfo.structFieldNames = (String[])   fieldNames.toArray(new String[0]);
        } 
        // put in cache
        typeCach.put(propMetaObject, typeInfo);
        return typeInfo;
    }
    
   
   String findTagValue(RefObject o, String tagId) {
	try {
	  Iterator it = findTagsFor(o).iterator();	
	  while(it.hasNext()) {
		RefObject t = (RefObject) it.next();
		if(tagId.equalsIgnoreCase((String)t.refGetValue("tagId"))) {
		  // System.out.println(t.refGetValue("values"));	
		  return (String) ((Collection)t.refGetValue("values")).toArray()[0];
		} 
	  }
	} catch(Exception e) {
	  e.printStackTrace();
	}
	return null;
   }
   
   
   Collection findTagsFor(RefObject o) {
	 RefAssociation asso = (RefAssociation) o.refOutermostPackage().refAssociation("AttachesTo");   
     return asso.refQuery("modelElement",o);
   }
    
    
   Object createValueFrmAttr(TypeInfo type, Attr propNode) {
       return createValue(type, propNode.getNodeValue());
   }
   
   Object createValueFrmElement(TypeInfo type, Element propNode) {
        if(type.isStruct) {            
          return createStruct(type, propNode);
        }
        
        if(propNode.hasAttribute("xmi.value")) {
          return createValue(type, propNode.getAttribute("xmi.value"));        
        }
        
        String xmi_idref = null;
        NodeList children = ((Element)propNode).getChildNodes();
        for(int i=0; i<children.getLength(); i++) {
           Node child = children.item(i);
           if(child instanceof Element) {
              Element childElem = (Element)child;
              if(childElem.hasAttribute("xmi.id")) {
                return createValue(type, childElem.getAttribute("xmi.id"));

              } else if(childElem.hasAttribute("xmi.idref")) {
                return createValue(type, childElem.getAttribute("xmi.idref"));

			  } else if(childElem.getTagName().equalsIgnoreCase("XMI.any")) {
				return createValue(type, childElem.getFirstChild().getNodeValue());

              } else if( !type.isEnum && !type.isPrimitive 
                         && classProxies.containsKey( unqualifiedName(childElem.getTagName()) )
                     ) {  // if type is object but the XML element has no xmi.id 
                     	  // then create an object now
              	RefObject o = createObject(childElem);
              	setProperties(childElem, o);
                return o;
              } 
              // otherwise ignore this element
              System.err.println("\t\t\t*** ignore element '" +childElem +"'");
           } else if(child instanceof Text) {
              // avoid empty text nodes mixed with child elements 
              if(child.getNodeValue().trim().length() > 0)                
                return createValue( type, child.getNodeValue() );
           
           } else if(child instanceof Comment) {
           
           } else {
             // otherwise ignore 
             System.err.println("\t\t\t*** ignore node '" +child +"'");
           }
        }
        return createValue(type ,"");
    }
    
    Collection createValuesFrmAttr(TypeInfo type, Attr propNode) {
          List r = new Vector();
          // parse space-seperated string
          StringTokenizer t = new StringTokenizer(propNode.getNodeValue());
          while(t.hasMoreTokens()) {
            String v = t.nextToken();
            r.add(createValue(type,v));
          }
          return r;
    }
    
    Collection createValuesFrmElement(TypeInfo type, Element propNode) {    
        List r = new Vector();
        if(type.isStruct) {
          r.add(createStruct(type, propNode));
          return r;
        }
        if(propNode.hasAttribute("xmi.value")) {
          r.add(createValue(type, propNode.getAttribute("xmi.value")));
          return r;
        }        
        NodeList list = ((Element)propNode).getChildNodes();
        for(int i=0; i<list.getLength(); i++) {
          Node child = list.item(i);
          if(child instanceof Element) {
             Element childElem = (Element) child;
             if(childElem.hasAttribute("xmi.id")) {
               r.add(createValue(type, childElem.getAttribute("xmi.id")));
             } else if(childElem.hasAttribute("xmi.idref")) {
               r.add(createValue(type, childElem.getAttribute("xmi.idref")));
               
             } else if(childElem.getTagName().equalsIgnoreCase("XMI.any")) {
               r.add(createValue(type, childElem.getFirstChild().getNodeValue()));
            

			 } else if( !type.isEnum && !type.isPrimitive 
						&& classProxies.containsKey( unqualifiedName(childElem.getTagName()) )
					) {  // if type is object but the XML element has no xmi.id 
						 // then create an object now             
			   RefObject o = createObject(childElem);
			   setProperties(childElem, o);
			   r.add(o);
             
             } else {
               // otherwise ignore this element
               System.err.println("\t\t\t*** ignore element '" +childElem +"'");
             }
          
          } else if(child instanceof Text) {
             // avoid empty text nodes mixed with child elements 
             if(child.getNodeValue().trim().length() > 0) 
                 r.add(createValue(type, child.getNodeValue()));
          
          } else if(child instanceof Comment) {
          
          } else {
            // otherwise ignore 
            System.err.println("\t\t\t*** ignore node '" +child +"'");
          }
        }
        return r;
    }    

    Object createValue(TypeInfo type, String value) {
        if(type.isPrimitive) {
          return parsePrimitiveValue(type.name, value);
        }
        if(type.isEnum) {           
          return getEnum(type, value);
        }
        if(type.isStruct) {            
          throw new RuntimeException("Struct value should not be encoded as attribute");
        }
        // type is RefObject
        String xmi_idref = value; //the same action even if value comes from xmi.id
        Object r = id_object.get(xmi_idref);
        if(r==null) {
           throw new RuntimeException("unresolved xmi.idref '" +xmi_idref +"'");
        }
        return r;
    }    
    
    Object parsePrimitiveValue(String type, String value) {
      if(type.equalsIgnoreCase("string")) {
        return value;
      } else if(type.equalsIgnoreCase("long")
        || type.equalsIgnoreCase("integer") ) {
        return new Integer(Integer.parseInt(value));
      } else if(type.equalsIgnoreCase("double")
        || type.equalsIgnoreCase("float")  ) {
        return new Double(Double.parseDouble(value));  
      }  else if(type.equalsIgnoreCase("boolean")) {
        return Boolean.valueOf(value);
      }
      throw new RuntimeException("unknown primitive type '" + type +"'"); 
    }
    
    
    RefEnum getEnum(TypeInfo type, String label) {
        
       Object m = enumManagers.get(type.name);
	   if(m==null) throw new RuntimeException("enumtype '" +type.name +"' undefined");
       String prefixLabel = label; 
       if(type.enumUnprefix!=null) { 
       	 if(!label.startsWith(type.enumUnprefix)) {
		    prefixLabel = type.enumUnprefix + label;
       	 }
       } 
              
       if(m instanceof RefPackage) {
         return ((RefPackage)m).refGetEnum(type.name, prefixLabel);
       }        
       return ((RefClass)m).refGetEnum(type.name, prefixLabel);
            
    }

   /*
    format 1:
                 <Model.StructuralFeature.multiplicity>
                    <XMI.field>1</XMI.field>
                    <XMI.field>1</XMI.field>
                    <XMI.field>false</XMI.field>
                    <XMI.field>false</XMI.field>
                 </Model.StructuralFeature.multiplicity>
    format 2:
       <Model:StructuralFeature.multiplicity>
            <Model:MultiplicityType lower="1" upper="1" is_ordered="false" is_unique="false" /> 
       </Model:StructuralFeature.multiplicity>  
      
    todo: Test with nested Structures (Structure of Structures)
    */
    RefStruct createStruct(TypeInfo type, Element elem) {
      if(isStructFormat1(elem)) {
        return createStruct_Format1(type, elem);
      } 
      return createStruct_Format2(type, elem);
    }
    
    boolean isStructFormat1(Element elem) {
      return getChildFromLocalName(elem, "XMI.field")!=null;
    }
    
    Element getChildFromLocalName(Element elem, String name) {
      NodeList list = elem.getChildNodes();
      for(int i=0; i<list.getLength() ; i++) {
        Node child = list.item(i);
        if((child instanceof Element) && child.getNodeName().endsWith(name) )   
            return (Element)child;
      }
      return null;
    }
        
    RefStruct createStruct_Format1(TypeInfo type, Element elem) {
        List args = new Vector();
        NodeList list = elem.getChildNodes();
        int j = 0;
        for(int i=0; i<list.getLength(); i++) {
          Node child = list.item(i);
          String nodeName = child.getNodeName();
          if((child instanceof Element) && nodeName.endsWith("XMI.field")) {
            args.add( createValueFrmElement(type.structFieldTypes[j], (Element)child) );
            j++;
          }
        }
        return createStruct(type.name, args);
    }
    
    RefStruct createStruct_Format2(TypeInfo type, Element elem) {
        Element inner = getChildFromLocalName(elem, type.name);
        List args = new Vector();
        for(int i=0; i<type.structFieldTypes.length; i++) {
          try {  
            String value = inner.getAttribute(type.structFieldNames[i]);
            args.add(
                createValue(type.structFieldTypes[i], value )
            );
          } catch(RuntimeException e) {
            System.err.println("CreateStruct: Error at att[" +i +"] "
            	+type.structFieldNames[i]            	
            );
            throw e;
          }
    	}
        return createStruct(type.name, args);
    }    
    
    RefStruct createStruct(String name, List args) {
      Object f = structFactories.get(name);
      if(f==null) throw new RuntimeException("structype '" +name +"' undefined");
      if(f instanceof RefPackage) {
      	return ((RefPackage) f).refCreateStruct(name, args);
      }
      return ((RefClass) f).refCreateStruct(name, args);    
    }
    
    
    // support for XMI 1.0 / 1.1
    // translate corba typecode to PrimitiveType
    // return false if this element does not have corba typecode
    RefObject doCorbaType(String xmi_id, String elemName, Element elem) {
        
        if(!elemName.equals("DataType")) { 
            return null;
        }        
		Element tc = findTypeCodeInDataType(elem); 
		if(tc==null) return null;
		       
        String name = null;
        RefObject type; // object to be create

		//resolve alias type
		while(tc.getNodeName().equalsIgnoreCase("XMI.CorbaTcAlias")) {
		  tc = findTypeCodeInDataType(tc);
		}
             
        if(tc.getNodeName().equalsIgnoreCase("XMI.CorbaTcEnum")) {
          if(caller.verbose) System.err.println(
            "creating EnumerationType '" +name +"' xmi.id=" +xmi_id
          );                  
          type = extent.refClass("EnumerationType").refCreateInstance(null);
		  name = tc.getAttribute("xmi.tcName");
		  if(name.length()==0) {
			throw new RuntimeException("Expecting xmi.tcName in " +tc.getNodeName());
		  }
		  type.refSetValue("name", name);		  
		  NodeList labels = tc.getElementsByTagName("XMI.CorbaTcEnumLabel");
          List type_labels = (List)type.refGetValue("labels");
          for(int i=0; i<labels.getLength() ; i++) {
            Element label = (Element) labels.item(i);
            type_labels.add(label.getAttribute("xmi.tcName"));
          }


        } else {          
		  //handle primitive type
		  if(caller.verbose) System.err.println(
			"creating PrimitiveType '" +name +"' xmi.id=" +xmi_id
		  );         
          type = extent.refClass("PrimitiveType").refCreateInstance(null);
          name = getTcName(tc);
          // normalize the first letter of type name to upper case
          // ex, 'integer' to 'Integer'
          char firstChar = name.charAt(0);
          name = Character.toUpperCase(firstChar) + name.substring(1);
		  type.refSetValue("name", name);  
		          
        }
        type.refSetValue("name", name);
        id_object.put(xmi_id, type);
		createdObjects.add(type);        
        return type;
    }
    
    
    // find child element with tag XMI.CorbaTc...
    // example : XMI.CorbaTcString XMI.CorbaTcBoolean
    Element findTypeCodeInDataType(Element dataType) {
		NodeList typeCode = dataType.getElementsByTagName("XMI.CorbaTypeCode");		  
		if(typeCode.getLength()==0) {
		  return null;
		}          
		Element tcElem = (Element) typeCode.item(0);     
		NodeList tcContentList = tcElem.getChildNodes();
		for(int i = 0; i<tcContentList.getLength(); i++) {
		  Node n = tcContentList.item(i); 
		  if(n instanceof Element) {
			Element tcContent = (Element)n;
			String s = tcContent.getNodeName().toLowerCase();
			if(s.indexOf("xmi.corbatc")>=0) {
			  return tcContent;
			}
		  }
		}		          
		throw new RuntimeException("Cannot handle CorbaTypeCode " + tcElem);
    }
    
    String getTcName(Element tc) {
      String n = tc.getNodeName().toLowerCase();
      return n.replaceAll("xmi.corbatc","");      
    }
    
        
    static String unqualifiedName(String dotList) {
        dotList = dotList.substring(
            dotList.lastIndexOf(":")+1 ,dotList.length()
        );
        return dotList.substring(
            dotList.lastIndexOf(".")+1 ,dotList.length()
        );
    }

    static class TypeInfo {
        String name;
        boolean isMultiValued = false;
        boolean isEnum = false;
        String enumUnprefix = null; //correspond to tag "xmi.enumerationUnprefix"
        boolean isStruct = false;
        boolean isPrimitive = false;
        TypeInfo[] structFieldTypes = null;
        String[] structFieldNames = null;
    }

    // no use for now
    static String getQualifiedName(RefObject modelElement) {
      String r = (String) modelElement.refGetValue("name");
      RefObject namespace = (RefObject) modelElement.refGetValue("container");
      if(namespace!=null) {
          r = getQualifiedName(namespace) +"." + r;
      }
      return r;
    }   
    
}
