/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.importer;

import java.util.Iterator;
import java.util.Vector;

import javax.jmi.model.GeneralizableElement;
import javax.jmi.model.MofPackage;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.XMIIOHelper;
import org.objectweb.modfact.jmi.xmiio.common.AbstractXMIIOGenerator;

/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImport extends AbstractXMIIOGenerator {

	/**
	 * XMIImport generation.
	 * @param package_ The root package.
	 */
	public void generateXMIImport(MofPackage package_) throws java.io.IOException {
		String className = getClassName(JMIProvider.jmiPackageExtentName(package_));
		packageModuleTemplate(package_);
		out.flush();
	}

	/**
	 * Get the class name.
	 * @param package_name_ The root package name.
	 * @return The class name.
	 */
	protected String getClassName(String package_name_) {
		String packageName1 = JMIProvider.jmiFormat1(package_name_);
		String className = packageName1 + "XMIImport";
		return className;
	}

	/**
	 * XMIImport generation for constructor.
	 * @param class_name The class name.
	 */
	public void constructor(String class_name, String package_name) {
		outputln("/** The XMIImportAssociations class. */");
		outputln(package_name + "XMIImportAssociations _xmi_import_associations;");
		outputln();
		outputln("/** The XMIImportAttributes class. */");
		outputln(package_name + "XMIImportAttributes _xmi_import_attributes;");
		outputln();
		outputln("/** The XMIImportClasses class. */");
		outputln(package_name + "XMIImportClasses _xmi_import_classes;");
		outputln();
		outputln("/**");
		outputln(" * " + class_name + " Constructor");
		outputln(" */");
		outputln("public " + class_name + "() {");
		outputln(
			"_xmi_import_associations = new " + package_name + "XMIImportAssociations (_types);");
		outputln("_xmi_import_attributes = new " + package_name + "XMIImportAttributes (_types);");
		outputln("_xmi_import_classes = new " + package_name + "XMIImportClasses (_types);");
		outputln("_xmi_import_classes.setXMIImportAttributes(_xmi_import_attributes);");
		outputln("_xmi_import_associations.setXMIImportClasses(_xmi_import_classes);");
		outputln("_xmi_import_attributes.setXMIImportClasses(_xmi_import_classes);");
		outputln("}");
		outputln();
	}

	java.util.Vector classes = new java.util.Vector();
	java.util.Vector associations = new java.util.Vector();

	private void processContents(MofPackage package_) {
		java.util.List contentsList = package_.getContents();
		Iterator it = contentsList.iterator();
		while (it.hasNext()) {
			javax.jmi.model.ModelElement contents = (javax.jmi.model.ModelElement) it.next();
			if (contents instanceof javax.jmi.model.MofPackage)
				processContents((javax.jmi.model.MofPackage) contents);
			else if (contents instanceof javax.jmi.model.Import)
				processContents((javax.jmi.model.MofPackage)((javax.jmi.model.Import) contents).getImportedNamespace());
			else if (contents instanceof javax.jmi.model.MofClass)
				classes.addElement((javax.jmi.model.MofClass) contents);
			else if (contents instanceof javax.jmi.model.Association)
				associations.addElement((javax.jmi.model.Association) contents);
		}
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 */
	public void packageModuleTemplate(MofPackage package_) {
		if (package_.getVisibility() == VisibilityKindEnum.forName("public_vis")) {
			processContents(package_);
			java.util.Vector classes_ordered = new java.util.Vector();
			while (classes.size() != 0) {
				javax.jmi.model.MofClass courant = null;
				for (int i = 0; i < classes.size(); i++) {
					courant = (javax.jmi.model.MofClass) classes.elementAt(i);
					java.util.List supertypes = courant.getSupertypes();
					boolean is_top = true;
					Iterator stypesIt = supertypes.iterator();
					while (stypesIt.hasNext()) {
						GeneralizableElement supertype = (GeneralizableElement) stypesIt.next();
						if (classes.contains(supertype))
							is_top = false;
					}
					if (is_top) {
						classes_ordered.addElement(courant);
						classes.removeElement(courant);
					}
				}
			}

			String packageName = JMIProvider.jmiPackageExtentName(package_);
			String className = getClassName(packageName);
            String packageQualifiedName = JMIProvider.qualifierOf(package_);
            outputln("package " + packageQualifiedName + ".xmi;");
			outputln();
			annotationTemplate(package_.getAnnotation());
			outputln(
				"public class "
					+ className
					+ " extends org.objectweb.modfact.jmi.xmiio.importer.XMIImportParser {");
			classAttributes(packageName, package_);
			constructor(className, packageName);
			associationAttributes(packageName, associations);
			initiateMethod(packageName, package_, classes_ordered, associations);
			rootPackage(packageName, classes_ordered, associations);
			completeImportMethod(associations);
			outputln("} // end of class " + className);
			outputln();
		}
	}

	/**
	* XMIImport generation for the class attributes.
	* @param package_name_ The package name.
	 * @param associations_ The associations of the package.
	*/
	public void classAttributes(String package_name_, MofPackage package_) {
		outputln("// Factories Declaration");
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln(
			"private "
				+ model_package_type
				+ " _"
				+ JMIProvider.jmiFormat2(package_name_)
				+ "_package = null;");
		outputln();
	}

	/**
	 * XMIImport generation for the association attributes.
	 * @param package_name_ The package name.
	 * @param associations_ The associations of the package.
	 */
	public void associationAttributes(String package_name_, Vector associations_) {
		outputln("// Hashtables for associations (Model / Key Vector)");
		outputln();
		outputln("/** Hashtables for types (Key / Model) */");
		outputln("private java.util.Hashtable _types = new java.util.Hashtable();");
		outputln();
	}

	/**
	 * XMIImport generation for the initiate method.
	 * @param package_name_ The package name.
	 * @param classes_ The classes of the package.
	 * @param associations_ The associations of the package.
	 */
	public void initiateMethod(
		String package_name_,
		MofPackage package_,
		Vector classes_,
		Vector associations_) {
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln("public void initiate(" + model_package_type + " _model_package) {");
		outputln("// Factories creation");
		//outputln("org.omg." + model_package_type + " _model_package = (org.omg." + model_package_type + ") _model_ref_package;");
		outputln("this._" + JMIProvider.jmiFormat2(package_name_) + "_package = _model_package;");
		outputln("_xmi_import_classes.initiate(_model_package);");
		outputln("_xmi_import_associations.initiate(_model_package);");
		outputln("_xmi_import_attributes.initiate(_model_package);");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the root package.
	 * @param package_name_ The package name.
	 * @param classes_ The classes of the package.
	 * @param associations_ The associations of the package.
	 */
	public void rootPackage(String package_name_, Vector classes_, Vector associations_) {
		annotationTemplate(" The root package processing");
		outputln("public java.lang.Object rootPackageTemplate(org.w3c.dom.Element _root_package_element) {");
		outputln("java.lang.String prefix_element = new java.lang.String();");
		outputln("java.lang.String namespace = _root_package_element.getNamespaceURI();");
		outputln("if (namespace != null && namespace.length() > 0)");
		outputln("    prefix_element = _root_package_element.getPrefix() + \":\";");
		outputln("org.w3c.dom.Element _current = _root_package_element;");
		outputln("String _current_name = _root_package_element.getNodeName().trim();");

		for (int i = 0; i < classes_.size(); i++) {
			javax.jmi.model.MofClass cls = (javax.jmi.model.MofClass) classes_.elementAt(i);
			String className = JMIProvider.jmiFormat1(JMIProvider.jmiClassName(cls));
			String classNameMin = className.substring(0, 1).toLowerCase() + className.substring(1);
			outputln("if (_current_name.endsWith(prefix_element + \"" + className + "\")) {");
			outputln("_xmi_import_classes." + classNameMin + "Template(_current);");
			if (i == classes_.size() - 1 && associations_.size() == 0)
				outputln("}");
			else
				output("} else ");
		}
		for (int i = 0; i < associations_.size(); i++) {
			javax.jmi.model.Association asso =
				(javax.jmi.model.Association) associations_.elementAt(i);
			String assoName = JMIProvider.jmiFormat1(JMIProvider.jmiAssociationName(asso));
			String assoNameMin = assoName.substring(0, 1).toLowerCase() + assoName.substring(1);
			outputln("if (_current_name.endsWith(prefix_element + \"" + assoName + "\")) {");
			outputln("_xmi_import_associations." + assoNameMin + "Template(_current);");
			if (i == associations_.size() - 1)
				outputln("}");
			else
				output("} else ");
		}
		outputln("return _" + JMIProvider.jmiFormat2(package_name_) + "_package;");
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the completeImport() method.
	 * @param associations_ The associations of the package.
	 */
	public void completeImportMethod(Vector associations_) {
		outputln("public void completeImport () {");
		for (int i = 0; i < associations_.size(); i++) {
			javax.jmi.model.Association asso =
				(javax.jmi.model.Association) associations_.elementAt(i);
			try {
				javax.jmi.model.Reference[] refs = XMIIOHelper.getReferences(asso);
				for (int j = 0; j < refs.length; j++) {
					javax.jmi.model.AssociationEnd assoEnd = refs[j].getReferencedEnd();
					if (assoEnd.getAggregation()
						== javax.jmi.model.AggregationKindEnum.forName("none")
						&& assoEnd.isChangeable())
						outputln(
							"_xmi_import_attributes.put"
								+ JMIProvider.jmiFormat1(refs[j].getContainer().getName())
								+ JMIProvider.jmiFormat1(refs[j].getName())
								+ "();");
				}
			} catch (NullPointerException notSet) {
				// Nothing to do
			}
		}
		outputln("}");
		outputln();
	}




	

}
