/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.importer;

import java.util.Iterator;
import java.util.Vector;

import javax.jmi.model.GeneralizableElement;
import javax.jmi.model.MofPackage;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.helper.JMIProvider;
import org.objectweb.modfact.jmi.helper.MofHelper;
import org.objectweb.modfact.jmi.helper.XMIIOHelper;
import org.objectweb.modfact.jmi.xmiio.common.AbstractXMIIOGenerator;

/**
 * The XMI Import generator.
 * This class allows the generation of the XMI importer (into Java).
 */
public class GenerationXMIImportClasses extends AbstractXMIIOGenerator {

	/** The generated association attributes methods. */
	private java.util.List _associationAttributes;

	/** The classes of the package. */
	private java.util.Vector classes = new java.util.Vector();

	/**
	 * Default Constructor.
	 */
	public GenerationXMIImportClasses() {
		super();
		_associationAttributes = new java.util.ArrayList();
	}

	/**
	 * XMIImport generation.
	 * @param package_ The root package.
	 * @param class_name_ The class name of the generated class.
	 */
	public void generateXMIImportClasses(MofPackage package_, String class_name_)
		throws java.io.FileNotFoundException {
		packageModuleTemplate(package_, class_name_);
		out.flush();
	}

	/**
	 * XMIImport generation for constructor.
	 * @param class_name The class name.
	 * @param package_name The package name.
	 */
	public void constructor(String class_name, String package_name) {
		outputln("/** The XMIImportAttributes class. */");
		outputln(package_name + "XMIImportAttributes _xmi_import_attributes;");
		outputln();
		outputln("/** Hashtables for types (Key / Model) */");
		outputln("private java.util.Hashtable _types;");
		outputln();
		outputln("/**");
		outputln(" * Default " + class_name + " Constructor");
		outputln(" * @param types The defined types.");
		outputln(" */");
		outputln("public " + class_name + "(java.util.Hashtable types) {");
		outputln("_types = types;");
		outputln("}");
		outputln();
		outputln("public void setXMIImportAttributes(" + package_name + "XMIImportAttributes xmi_import_attributes) {");
		outputln("_xmi_import_attributes = xmi_import_attributes;");
		outputln("}");
		outputln();
	}

	/**
	 * Parse the content of the package.
	 * @param package_ The root package.
	 */
	private void processContents(MofPackage package_) {
		java.util.List contentsList = package_.getContents();
		Iterator it = contentsList.iterator();
		while (it.hasNext()) {
			javax.jmi.model.ModelElement contents = (javax.jmi.model.ModelElement) it.next();
			if (contents instanceof javax.jmi.model.MofPackage)
				processContents((javax.jmi.model.MofPackage) contents);
			else if (contents instanceof javax.jmi.model.Import)
				processContents((javax.jmi.model.MofPackage)((javax.jmi.model.Import) contents).getImportedNamespace());
			else if (contents instanceof javax.jmi.model.MofClass)
				classes.addElement((javax.jmi.model.MofClass) contents);
		}
	}

	/**
	 * Package processing.
	 * @param package_ The package to process.
	 * @param class_name_ The class name of the generated class.
	 */
	public void packageModuleTemplate(MofPackage package_, String class_name_) {
		if (package_.getVisibility() == VisibilityKindEnum.forName("public_vis")) {
			processContents(package_);
			java.util.Vector classes_ordered = new java.util.Vector();
			while (classes.size() != 0) {
				javax.jmi.model.MofClass courant = null;
				for (int i = 0; i < classes.size(); i++) {
					courant = (javax.jmi.model.MofClass) classes.elementAt(i);
					java.util.List supertypes = courant.getSupertypes();
					boolean is_top = true;
					Iterator stypesIt = supertypes.iterator();
					while (stypesIt.hasNext()) {
						GeneralizableElement supertype = (GeneralizableElement) stypesIt.next();
						if (classes.contains(supertype))
							is_top = false;
					}
					if (is_top) {
						classes_ordered.addElement(courant);
						classes.removeElement(courant);
					}
				}
			}

			String packageName = JMIProvider.jmiPackageExtentName(package_);
            String packageQualifiedName = JMIProvider.qualifierOf(package_);
            outputln("package " + packageQualifiedName + ".xmi;");
			outputln();
			annotationTemplate(package_.getAnnotation());
			outputln("public class " + class_name_ + " extends org.objectweb.modfact.jmi.xmiio.importer.XMIImport {");
			classAttributes(packageName, package_, classes_ordered);
			constructor(class_name_, packageName);
			initiateMethod(packageName, package_, classes_ordered);
			classTemplate(packageName, classes_ordered);
			outputln("} // end of class " + class_name_);
			outputln();
		}
	}

	/**
	 * XMIImport generation for the class attributes.
	 * @param package_name_ The package name.
	 * @param package_ The package.
	 * @param classes_ The classes of the package.
	 */
	public void classAttributes(String package_name_, MofPackage package_, Vector classes_) {
		outputln("// Factories Declaration");
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln("private " + model_package_type + " _" + JMIProvider.jmiFormat2(package_name_) + "_package = null;");

		// Classes
		for (int i = 0; i < classes_.size(); i++) {
			javax.jmi.model.MofClass cls = (javax.jmi.model.MofClass) classes_.elementAt(i);
			String className = JMIProvider.jmiClassName(cls);
			String javaName;
			try {
				javaName = JMIProvider.qualifierOf((MofPackage) cls.getContainer()) + "." + className;
			} catch (NullPointerException notSet) {
				javaName = className;
			}
			outputln(
				"private "
					+ javaName
					+ "Class _"
					+ JMIProvider.jmiFormat2(JMIProvider.jmiClassQualifiedName(cls))
					+ "_class = null;");
		}
		outputln();
	}

	/**
	 * XMIImport generation for the initiate method.
	 * @param package_name_ The package name.
	 * @param package_ The package.
	 * @param classes_ The classes of the package.
	 */
	public void initiateMethod(String package_name_, MofPackage package_, Vector classes_) {
		String model_package_type;
		model_package_type = JMIProvider.jmiPackageQualifiedName(package_) + "Package";
		outputln("public void initiate(" + model_package_type + " _model_package) {");
		outputln("// Factories creation");
		outputln("this._" + JMIProvider.jmiFormat2(package_name_) + "_package = _model_package;");
		// Classes
		initiateMethodClasses(package_, classes_);
		outputln("}");
		outputln();
	}

	/**
	 * XMIImport generation for the "class attributes" of the initiate method.
	 * @param package_ The package.
	 * @param classes_ The classes of the package.
	 */
	public void initiateMethodClasses(MofPackage package_, Vector classes_) {
		for (int i = 0; i < classes_.size(); i++) {
			javax.jmi.model.MofClass cls = (javax.jmi.model.MofClass) classes_.elementAt(i);
			String javaName = JMIProvider.jmiFormat2(JMIProvider.jmiClassQualifiedName(cls));
			String method = "get" + (JMIProvider.jmiClassName(cls)) + "()";
            
			try {
				javax.jmi.model.ModelElement clsTmp = cls;
				while (!clsTmp.getContainer().equals(package_)) {
					method = "get" + JMIProvider.jmiPackageExtentName((MofPackage)clsTmp.getContainer()) + "()." + method;
					clsTmp = clsTmp.getContainer();
				}
			} catch (NullPointerException notSet) {
				// Nothing to do
			}
			outputln("_" + javaName + "_class = _model_package." + method + ";");
		}
	}

	/**
	 * XMIImport generation for the class template.
	 * @param package_name_ The package name.
	 * @param classes_ The classes of the package.
	 */
	public void classTemplate(String package_name_, Vector classes_) {
		outputln("// ==================================================================");
		outputln("//  XXXTemplate with XXX an entity");
		outputln("// ==================================================================");
		// Classes
		for (int i = 0; i < classes_.size(); i++) {
			javax.jmi.model.MofClass cls = (javax.jmi.model.MofClass) classes_.elementAt(i);
			if (cls.isAbstract())
				classAbstractTemplate(cls);
			else
				classSpecificTemplate(cls);
			outputln();
		}
		outputln();
	}

	/**
	 * XMIImport generation for the abstract class template.
	 * @param cls The abstract class of the package.
	 */
	public void classAbstractTemplate(javax.jmi.model.MofClass cls) {
		String clsName = JMIProvider.jmiClassName(cls);
		String clsName1 = JMIProvider.jmiFormat1(clsName);
		String clsName2 = JMIProvider.jmiFormat2(clsName);
		String javaName = JMIProvider.jmiClassQualifiedName(cls);
		String methodName = clsName1.substring(0, 1).toLowerCase() + clsName1.substring(1);
		String returnType = javaName;
		try {
			returnType = JMIProvider.qualifierOf((MofPackage) cls.getContainer()) + "." + clsName;
		} catch (NullPointerException notSet) {
			returnType = clsName;
		}
		String attributeName = "_" + clsName2 + "_element";
		annotationTemplate(cls.getAnnotation());
		outputln(
			"public org.objectweb.modfact.jmi.xmiio.mofobject.MOFObjectList "
				+ methodName
				+ "Template(org.w3c.dom.Element "
				+ attributeName
				+ ") {");
		outputln("org.objectweb.modfact.jmi.xmiio.mofobject.MOFObjectList list = new org.objectweb.modfact.jmi.xmiio.mofobject.MOFObjectList();");
		outputln("// Get the " + clsName1 + " values");
		outputln("if (" + attributeName + " == null) return list;");
		outputln(
			"if ("
				+ attributeName
				+ ".getChildNodes().getLength() == 0 && "
				+ attributeName
				+ ".hasAttribute(\"xmi.idref\")) {");
		outputln("// The " + clsName1 + " element is already read or it will be read (ref).");
		outputln("list.addXmiIdRef(" + attributeName + ".getAttribute(\"xmi.idref\"));");
		outputln("return list;");
		outputln("}");
		outputln("int i = 0;");
		outputln("org.w3c.dom.NodeList _child = " + attributeName + ".getChildNodes();");
		outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
		outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
		try {
			javax.jmi.model.MofClass[] subtypes = XMIIOHelper.subClasses(cls);
			for (int j = 0; j < subtypes.length; j++) {
				String subTypeXMLName = JMIProvider.jmiFormat1(subtypes[j].getName());
				String subTypeName = JMIProvider.jmiFormat1(JMIProvider.jmiClassName(subtypes[j]));
				String subTypeTemplateMethod = subTypeName.substring(0, 1).toLowerCase() + subTypeName.substring(1);
				outputln("if (_current != null && nodeNameCorrespondsTo(_current, \"" + subTypeXMLName + "\"))");
				if (subtypes[j].isAbstract())
					outputln(TABULATION + "list.addMOFObjectList(" + subTypeTemplateMethod + "Template(_current));");
				else
					outputln(TABULATION + subTypeTemplateMethod + "Template(_current).addMOFObject(list);");
				output("else ");
			}
		} catch (NullPointerException notSet) {
			// Nothing to do
		}
		outputln(
			"if (_current != null && nodeNameCorrespondsTo(_current, \"" + JMIProvider.jmiFormat1(clsName) + "\"))");
		outputln(TABULATION + "list.addXmiIdRef(_current.getAttribute(\"xmi.idref\"));");
		outputln("else if (_current != null)");
		outputln(
			TABULATION
				+ "System.err.println(\""
				+ JMIProvider.jmiFormat1(clsName)
				+ " is an abstract type, it can't be instanciated (\" + _current.getNodeName() + \" encountered).\");");
		outputln("i++;");
		outputln("}");
		outputln("return list;");
		outputln("}");
	}

	/**
	 * XMIImport generation for the specific class template.
	 * @param cls The specific class of the package.
	 */
	public void classSpecificTemplate(javax.jmi.model.MofClass cls) {
		String className = JMIProvider.jmiClassName(cls);
		String javaName;
		try {
			javaName = JMIProvider.jmiPackageQualifiedName((MofPackage) cls.getContainer()) + "." + className;
		} catch (NullPointerException notSet) {
			javaName = className;
		}
		String clsName = JMIProvider.jmiClassName(cls);
		String clsName1 = JMIProvider.jmiFormat1(clsName);
		String clsName2 = JMIProvider.jmiFormat2(clsName);
		String methodName = XMIIOHelper.format1FirstMin(clsName1);
		String returnType = javaName;
		try {
			returnType = JMIProvider.qualifierOf((MofPackage) cls.getContainer()) + "." + clsName;
		} catch (NullPointerException notSet) {
			returnType = clsName;
		}
		String returnMethod = "org.objectweb.modfact.jmi.xmiio.mofobject.MOFObject";
		String attributeName = "_" + clsName2 + "_element";
		annotationTemplate(cls.getAnnotation());
		outputln("public " + returnMethod + " " + methodName + "Template(org.w3c.dom.Element " + attributeName + ") {");
		outputln("// Get the " + clsName1 + " values");
		outputln("if (" + attributeName + " == null)");
		outputln(TABULATION + "return new org.objectweb.modfact.jmi.xmiio.mofobject.NullObject();");
		outputln(
			"if ("
				+ attributeName
				+ ".getChildNodes().getLength() == 0 && "
				+ attributeName
				+ ".hasAttribute(\"xmi.idref\")) {");
		outputln("// The " + clsName1 + " is already read or it will be read (ref).");
		outputln(
			"return new org.objectweb.modfact.jmi.xmiio.mofobject.XMIIdRef("
				+ attributeName
				+ ".getAttribute(\"xmi.idref\"));");
		outputln("}");
		outputln("// Default " + clsName1 + " creation");
		outputln(returnType + " _" + clsName2 + " = ");
		output(
			"    _"
				+ JMIProvider.jmiFormat2(JMIProvider.jmiClassQualifiedName(cls))
				+ "_class.create"
				+ clsName1
				+ "(");

		javax.jmi.model.Attribute[] attributes =
			MofHelper.attributesOfClass(cls, javax.jmi.model.ScopeKindEnum.forName("instance_level"), true);
		outputln(");");
		outputln(
			"_types.put(((java.lang.String)_"
				+ clsName2
				+ "_element.getAttribute(\"xmi.id\")).trim(), "
				+ "_"
				+ clsName2
				+ ");");
		outputln("int i = 0;");
		outputln("org.w3c.dom.NodeList _child = " + attributeName + ".getChildNodes();");
		outputln("while (i<_child.getLength() || (_child.getLength()==0 && i==0)) {");
		outputln("org.w3c.dom.Element _current = (org.w3c.dom.Element) _child.item(i);");
		String param = "_" + clsName2 + "_element, _" + clsName2 + ", _current";
		attributes = MofHelper.attributesOfClass(cls, javax.jmi.model.ScopeKindEnum.forName("instance_level"), true);
		for (int j = 0; j < attributes.length; j++) {
			javax.jmi.model.Attribute attr = (javax.jmi.model.Attribute) attributes[j];
			if (attr.isChangeable()) {
				String attr_name = JMIProvider.findSubstituteName(attr);
				if (attr_name == null)
					attr_name = attr.getName();
				try {
					String attr_container_name = JMIProvider.findSubstituteName(attr.getContainer());
					if (attr_container_name == null)
						attr_container_name = attr.getContainer().getName();
					outputln(
						"_xmi_import_attributes.read"
							+ JMIProvider.classNameFormat(attr_container_name)
							+ JMIProvider.classNameFormat(attr_name)
							+ "("
							+ param
							+ ");");
				} catch (NullPointerException notSet) {
					outputln("_xmi_import_attributes.read" + JMIProvider.classNameFormat(attr_name) + "(" + param + ");");
				}
			}
		}
		javax.jmi.model.Reference[] references = MofHelper.referencesOfClass(cls, true);
		for (int j = 0; j < references.length; j++) {
			javax.jmi.model.Reference ref = (javax.jmi.model.Reference) references[j];
			javax.jmi.model.AssociationEnd assoEnd = ref.getReferencedEnd();
			if (assoEnd.getAggregation() == javax.jmi.model.AggregationKindEnum.forName("none")) {
				String ref_name = JMIProvider.findSubstituteName(ref);
				if (ref_name == null)
					ref_name = ref.getName();
				try {
					String ref_container_name = JMIProvider.findSubstituteName(ref.getContainer());
					if (ref_container_name == null)
						ref_container_name = ref.getContainer().getName();
					outputln(
						"_xmi_import_attributes.read"
							+ JMIProvider.classNameFormat(ref_container_name)
							+ JMIProvider.classNameFormat(ref_name)
							+ "("
							+ param
							+ ");");
				} catch (NullPointerException notSet) {
					outputln("_xmi_import_attributes.read" + JMIProvider.classNameFormat(ref_name) + "(" + param + ");");
				}
			} else if (assoEnd.getAggregation() == javax.jmi.model.AggregationKindEnum.forName("composite")) {
				try {
					javax.jmi.model.AssociationEnd otherEnd = XMIIOHelper.otherAssociationEnd(assoEnd);
					javax.jmi.model.Reference[] refs =
						XMIIOHelper.getReferences((javax.jmi.model.Association) assoEnd.getContainer());
					for (int k = 0; k < refs.length; k++) {
						if (refs[k] != ref) {
							String ref_type_name = JMIProvider.findSubstituteName(refs[k].getType());
							if (ref_type_name == null)
								ref_type_name = refs[k].getType().getName();
							String asso_end_name = JMIProvider.findSubstituteName(assoEnd);
							if (asso_end_name == null)
								asso_end_name = assoEnd.getName();
							outputln(
								"_xmi_import_attributes.read"
									+ JMIProvider.classNameFormat(ref_type_name)
									+ JMIProvider.classNameFormat(asso_end_name)
									+ "("
									+ param
									+ ");");
						}
					}
				} catch (NullPointerException notSet) {
					// Nothing to do
				}
			}
		}
		outputln("i++;");
		outputln("}");
		outputln("return new org.objectweb.modfact.jmi.xmiio.mofobject.BuiltObject(_" + clsName2 + ");");
		outputln("}");
	}

}
