/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.importer;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import org.objectweb.modfact.jmi.logging.Level;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

import javax.jmi.model.ModelElement;
import javax.jmi.model.MofPackage;
import javax.jmi.model.VisibilityKindEnum;

import org.objectweb.modfact.jmi.generator.ZipGenerator;
import org.objectweb.modfact.jmi.helper.JMIProvider;

/**
 * An abstract implementation of an XMI Import application.
 * This class provides methods for all XMI importers.
 */
public class JMI1_0ImportGenerator extends ZipGenerator {
	
	MofPackage[] input;
	
	ModFactLogger logger;

	/**
	 * Set Input
	 */
	public void setInput(ModelElement[] elt) {
		input = new MofPackage[elt.length];
		for (int i = 0; i < input.length; i++) {
			input[i] = (MofPackage) elt[i];
		}
	}

	/**
	 * Set Trace
	 */
	public void setLogger(ModFactLogger log) {
		logger = log;
	}

	public JMI1_0ImportGenerator() {
	}
	
	/**
	 * @see org.objectweb.modfact.generation.Generator#generate()
	 */
	public void generate()
			throws IOException {
		logger.log(Level.FINE, "Begin generation of JMI Importer");
		
		for (int p = 0; p < input.length; p++) {
			if (input[p]
				.getVisibility()
				.equals(VisibilityKindEnum.forName("public_vis"))
				&& input[p].refGetValue("container") == null) {

				//Traitemement du Package
				MofPackage[] param = new MofPackage[1];
				param[0] = input[p];
				
				String path = JMIProvider.qualifierOf(input[p]);
				path = path.replace('.', '/') +"/";
				String className = JMIProvider.jmiPackageExtentName(input[p]) + "XMIImport";
				String baseFileName = path + className;

				GenerationXMIImport importGenerator = new GenerationXMIImport();
				ByteArrayOutputStream entryOutput1 = new ByteArrayOutputStream();
				importGenerator.setOutput(entryOutput1);
				importGenerator.generateXMIImport(param[0]);
				writeEntry(baseFileName + ".java", entryOutput1);
				entryOutput1.close();
				

				// XMIImportAttributes 
				GenerationXMIImportAttributes xmiImportAttributes = new GenerationXMIImportAttributes();
				entryOutput1 = new ByteArrayOutputStream();
				xmiImportAttributes.setOutput(entryOutput1);
				xmiImportAttributes.generateXMIImportAttributes(param[0], className + "Attributes");
				writeEntry(baseFileName + "Attributes.java", entryOutput1);
				entryOutput1.close();
				
				// XMIImportClasses 
				GenerationXMIImportClasses xmiImportClasses = new GenerationXMIImportClasses();
				entryOutput1 = new ByteArrayOutputStream();
				xmiImportClasses.setOutput(entryOutput1);
				xmiImportClasses.generateXMIImportClasses(param[0], className + "Classes");
				writeEntry(baseFileName + "Classes.java", entryOutput1);
				entryOutput1.close();
				
				// XMIImportAssociations 
				GenerationXMIImportAssociations xmiImportAssociations = new GenerationXMIImportAssociations();
				entryOutput1 = new ByteArrayOutputStream();
				xmiImportAssociations.setOutput(entryOutput1);
				xmiImportAssociations.generateXMIImportAssociations(param[0], className + "Associations");
				writeEntry(baseFileName + "Associations.java", entryOutput1);
				entryOutput1.close();
				
				// XMIImportApplication 
				GenerationXMIImportApplication xmiImportApplication = new GenerationXMIImportApplication();
				entryOutput1 = new ByteArrayOutputStream();
				xmiImportApplication.setOutput(entryOutput1);
				xmiImportApplication.generateXMIImportApplication(param[0], className, className + "Application");
				writeEntry(baseFileName + "Application.java", entryOutput1);
				entryOutput1.close();
				
				logger.log(Level.FINE, "Importer is generated");
			}
		}
		out.finish();
		logger.log(Level.FINE, " Generation is completed without error");
	}

}
