/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.mofobject;

/**
* This class represents an object built from the XMI file.
*/
public class BuiltObject implements MOFObject {

	/** The Object read from the XMI file. */
	protected Object object;

	/**
	 * Constructor for a BuiltObject.
	 * @param xmiid The Object read from the XMI file.
	 */
	public BuiltObject(Object object) {
		this.object = object;
	}

	/**
	 * Add a MOFObject to a list of MOFObject (in the Objects list).
	 * @param list The list of MOFObject.
	 */
	public void addMOFObject(MOFObjectList list) {
		list.addObject(object);
	}

	/**
	 * Get the object built from the XMI File.
	 * @return The encapsulate object.
	 */
	public Object get() {
		return object;
	}

	/**
	 * Test if the MOFObject represents a real object.
	 * @return TRUE if the MOFObject is a real object.
	 */
	public boolean isAnObject() {
		return true;
	}

	/**
		* Test if the MOFObject is valid, i.e it represents a real object or a XMI IdRef.
		* @return TRUE if the MOFObject is valid.
		*/
	public boolean isValid() {
		return true;
	}

}
