package org.objectweb.modfact.qvt.service;

import java.io.*;
import java.net.URL;

public abstract class LocalClient {

  static String getSimpleTrlUrl() {
    
    String resourceURL = System.getProperty("org.objectweb.modfact.resourceURL");
    if(resourceURL!=null) { 
      return resourceURL +"/metamodel_SimpleTRL.xml";
    } 
    try {
      return ClassLoader.getSystemResource("org/objectweb/modfact/qvt/metamodel_SimpleTRL.xml").toString();
    } catch(Exception e) {
      throw new RuntimeException("Please set system property 'org.objectweb.modfact.resourceURL'");
    }
  }
 
  /* example  
    fr.lip6.qvt.service.LocalClient.run(
      "dtd_xs/xmi/dtd_to_xs.xml",
      "dtd_xs/xmi/dtd_mm.xml", "dtd_xs/xmi/dtd_instance.xml",
      "dtd_xs/xmi/xs_mm.xml", "dtd_xs/xmi"
    );
  */    
  public static void run(
    String rulePath,
    String srcMmPath, String srcPath,
    String targetMmPath, String outputPath
  ) throws Exception {
      
    Model rules, src, target;
    
    rules = new ModelImpl( 
    	getSimpleTrlUrl()
        ,ClassLoader.getSystemResource(rulePath).toString()
    );
    src = new ModelImpl(
         ClassLoader.getSystemResource(srcMmPath).toString()
        ,ClassLoader.getSystemResource(srcPath).toString()
    );
    target = new ModelImpl( 
         ClassLoader.getSystemResource(targetMmPath).toString()
    );
    
    System.setProperty("fr.lip6.modfact.qvt.engine.verbose", "true");
    
    TransformationService service = new TransformationService();
    
    service.transform( rules, src, target  );
    System.err.println("---------- Transformation finished -----------");

    File f = new File( 
      ClassLoader.getSystemResource(outputPath).getPath()
      +"/"  +"_instance.xml"
    );
    FileOutputStream out = new FileOutputStream(f);
    target.writeModelTo(out);
    out.close();
    System.err.println("Result written to " 
       +f.getAbsolutePath()
    );    
  }
  
  // use Simple TRL parser
  public static void runFromTextFormat(
	InputStream ruleStream,
	String srcPackageName, InputStream srcMmStream, InputStream srcStream,
	String targetPackageName, InputStream targetMmStream, OutputStream output
  ) throws Exception {  
	Model rules, src, target;
    
	rules = new ModelImpl(new java.net.URL(getSimpleTrlUrl()).openStream());
	
	//rules.getModel().

	src = new ModelImpl( srcMmStream
		,srcStream
	);
	target = new ModelImpl( targetMmStream );
    
	System.setProperty("fr.lip6.modfact.qvt.engine.verbose", "true");    
    
	TransformationService service = new TransformationService();
    
	service.transform( rules, src, target  );

	target.writeModelTo(output);
	  
  }


  public static void run(
    InputStream ruleStream,
    InputStream srcMmStream, InputStream srcStream,
    InputStream targetMmStream, OutputStream output
  ) throws Exception {
  
    Model rules, src, target;
    
    rules = new ModelImpl(
         new java.net.URL(getSimpleTrlUrl()).openStream()
        ,ruleStream
    );

    src = new ModelImpl(
         srcMmStream
        ,srcStream
    );
    target = new ModelImpl( 
         targetMmStream
    );
    
    System.setProperty("fr.lip6.modfact.qvt.engine.verbose", "true");    
    
    TransformationService service = new TransformationService();
    
    service.transform( rules, src, target  );

    target.writeModelTo(output);
  }  
  
  public static void main(String[] args) throws Exception {
	FileOutputStream out = new FileOutputStream(args[4]);
	run(
	  	 new URL(args[0]).openStream()  // rules
		,new URL(args[1]).openStream()  // source MM
		,new URL(args[2]).openStream()	// source M	
		,new URL(args[3]).openStream()	// target MM
		,out	
	);
	out.close(); 	
  }
}